FUNCTION GET_BEAM_STRUCT, type, $
                          beamsize=beamsize, sizeunits=sizeunits, $
                          ell=ell, bl=bl, lmax=lmax, $
                          instr=instr, version=version, channel=channel, $
                          dim=dim, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'GET_BEAM_STRUCT: function to get a structure describing a PSM beam'
     PSM_INFO_MESSAGE, 'SYNTAX: result = GET_BEAM_STRUCT(type, beamsize=, sizeunits=, ell=, bl=, instr=, channel=, lmax=, dim=)'
     PSM_INFO_MESSAGE, "'type' can be 'GAUSSIAN' or 'TABULATED' or 'INSTR' or 'NO_BEAM' or 'UNKNOWN'"
     PSM_INFO_MESSAGE, "'beamsize' and 'sizeunits' are inputs for the 'GAUSSIAN' case"
     PSM_INFO_MESSAGE, "'beamsize' is the size of the gaussian beam, in units of 'sizeunits'"
     PSM_INFO_MESSAGE, "'sizeunits' can be 'arcmin' or 'deg' or 'rad' or 'arcsec'"
     PSM_INFO_MESSAGE, "'ell' and 'bl' are input arrays for the 'TABULATED' case"
     PSM_INFO_MESSAGE, "'lmax' is the max value of ell for the 'TABULATED' case"
     PSM_INFO_MESSAGE, "'instr', 'version' and 'channel' are used for the 'INSTR' case"
     PSM_INFO_MESSAGE, "--- WMAP case ------------------------------------------"
     PSM_INFO_MESSAGE, "'version' can be '5yr' or '7yr'"
     PSM_INFO_MESSAGE, "'channel' can be any of [K, Ka, Q, Q1, Q2, V, V1, V2, W, W1, W2, W3, W4]"
     PSM_INFO_MESSAGE, "--- DIRBE case -----------------------------------------"
     PSM_INFO_MESSAGE, "'version' can be 'from-omega'"
     PSM_INFO_MESSAGE, "'channel' can be any of ['12um','25um','60um','100um','140um','240um']"
     PSM_INFO_MESSAGE, "--------------------------------------------------------"
     PSM_INFO_MESSAGE, "set 'dim = 3' for polarisation"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  IF SIZE(type,/type) LE 5 THEN BEGIN
     ;; Assume type is actually the size of a gaussian beam

     IF DEFINED(beamsize) THEN IF beamsize NE type THEN STOP

     IF NOT DEFINED(sizeunits) THEN sizeunits='arcmin' ; arcminutes by default
     beam = { type:'GAUSSIAN', $
              size:FLOAT(type), $
              sizeunits:sizeunits $
            }
     
  ENDIF ELSE BEGIN

     IF SIZE(type,/type) NE 7 THEN STOP
     
     CASE type OF 
        'GAUSSIAN': BEGIN
           IF NOT DEFINED(beamsize) THEN size=0. ELSE size=FLOAT(beamsize)
           IF NOT DEFINED(sizeunits) THEN sizeunits='arcmin'
           beam = { type:'GAUSSIAN', $
                    size:size, $
                    sizeunits:sizeunits $                  
                  }
        END
        'TABULATED': BEGIN
           IF NOT DEFINED(bl) THEN BEGIN
              IF NOT KEYWORD_SET(dim) THEN dim=1
              IF NOT KEYWORD_SET(ell) THEN BEGIN
                 IF NOT KEYWORD_SET(lmax) THEN lmax=2500
                 ell = INDGEN(lmax+1)  
                 IF dim EQ 1 THEN bl = DBLARR(lmax+1) ELSE bl = DBLARR(lmax+1,dim) 
              ENDIF ELSE BEGIN
                 IF dim EQ 1 THEN bl = DBLARR(N_ELEMENTS(ell)) ELSE bl = DBLARR(N_ELEMENTS(ell),dim)
              ENDELSE
           ENDIF ELSE BEGIN
              IF NOT KEYWORD_SET(ell) THEN BEGIN
                 dimens = SIZE(bl,/dimensions)
                 ell=INDGEN(dimens[0])
              ENDIF
           ENDELSE
           beam = { type:'TABULATED', $
                    ell:ell, $
                    bl:bl $
                  }
        END
        'INSTR': BEGIN
           IF NOT KEYWORD_SET(instr) THEN instr=''
           IF NOT KEYWORD_SET(channel) THEN channel=''
           IF NOT KEYWORD_SET(version) THEN version=''
           beam = { type:'INSTR', $
                    instr:instr, $
                    version:version, $
                    channel:channel $
                  }
        END
        'NO_BEAM': BEGIN
           beam = {type:'NO_BEAM'}
        END
        'UNKNOWN': BEGIN
           beam = {type:'UNKNOWN'}
        END
        ELSE: MESSAGE, 'Beam type not implemented'
     ENDCASE
  ENDELSE

  RETURN, beam

END
