; Copyright 2008 CNRS

; This file is part of the Planck Sky Model.
;
; The Planck Sky Model is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; version 2 of the License.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY, without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model. If not, write to the Free Software
; Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

;+
; Get b_ell values for a given beam
; 
; @param inname  {in}{required}{type=string} Name of the beam (can also be an output parameter). 
; WMAP beams are implemented. For these beams, beam_name can be WMAP3yr- or WMAP5yr- + name of the channel.
; The name of the channel is in the list [K, Ka, Q, Q1, Q2, V, V1, V2, W, W1, W2, W3, W4 ]
; So that possible WMAP beam names, are, e.g. WMAP5yr-Ka, ... 
; The K, Ka, Q, V, W channels are noise-weighted averages of individual differencing assemblies.
; GAUSSIAN beams are also implemented. For these beams, beam_name can be GAUSSIAN- + beam width in arcminutes.
; So that possible Gaussian beam names are, e.g. GAUSSIAN-5-arcmin (for a 5 arcmin gaussian beam), GAUSSIAN-2.3-deg ... 
; (for a 2.3 degree beam), ... 
; The 'NO_BEAM' beam is implemented: it returns a vector of 1
; The instrument and channel keywords overwrite the definition of beam_name, if any. 
; @keyword lmax {in}{optional}{type=int} the value of lmax of the returned beam b_ell's (default 4000)
; @keyword double {in}{optional}{type=boolean} If set the output is in double precision, simple precision otherwise.
; @keyword dim {in}{optional}{type=int} the dimension of the output: 
;     <p> if absent or set to 0 or 1, the output has size (lmax+1) and is the temperature beam </p>
;     <p> if set to >= 2 <= 4, the output has size (lmax+1,dim) and contains in that order the TEMPERATURE beam,
;          the GRAD/ELECTRIC polarisation beam, the CURL/MAGNETIC polarisation beam, the TEMPERATURE*GRAD beam
; @uses READCOL (astron) GAUSSBEAM (HEALPix)
; @author Jacques Delabrouille, 
; @history <p>28/03/2008: First version, Jacques Delabrouille </p>
; @history <p>21/01/2009: Most keywords supressed (now call only using beam_name), 'NO_BEAM' option added, Jacques Delabrouille </p>
; @history <p>04/02/2009: dim keyword added, Jacques Delabrouille, </p>
; @copyright 2008 Jacques Delabrouille, CNRS
;-

FUNCTION PSM_GET_BL, input_beam_name, lmax=lmax, double=double, dim=dim, help=help
;, instrument=instrument, channel=channel, gaussian=gaussian, resol=resol, units=units, lmax=lmax

  IF ((KEYWORD_SET(help) NE 0) OR (N_PARAMS() NE 1)) THEN BEGIN
     PRINT, ''
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'PSM_GET_BL: function which returns bl for a beam'
     PSM_INFO_MESSAGE, 'SYNTAX: result = PSM_GET_BL(input_beam_name, lmax=, double=, dim=)'
     PSM_INFO_MESSAGE, "input_beam_name can be a FWHM in arcminutes, or a beam structure created by GET_BEAM_STRUCT,"
     PSM_INFO_MESSAGE, "or a string of the form 'WMAP7yr-Ka', where 'WMAP7yr' can be replaced by 'WMAP3yr' or 'WMAP5yr'"
     PSM_INFO_MESSAGE, "and 'Ka' can be replaced by any of [K, Ka, Q, Q1, Q2, V, V1, V2, W, W1, W2, W3, W4]."
     PSM_INFO_MESSAGE, "lmax is 4000 by default. Precision is simple unless /double is set."
     PSM_INFO_MESSAGE, "dim keyword is 1 by default (temperature only). Set to 3 for bl(T,E,B)"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

IF KEYWORD_SET(dim) EQ 0 THEN dim=1
IF NOT KEYWORD_SET(lmax) then lmax=4000
IF NOT KEYWORD_SET(double) THEN beam = FLTARR(lmax+1,dim) ELSE beam = DBLARR(lmax+1,dim)

beamnametype = SIZE(input_beam_name, /type) 

IF beamnametype EQ 7 THEN beam_name = input_beam_name 
IF beamnametype LE 5 THEN beam_name = 'GAUSSIAN-'+NUMBER2STRING(input_beam_name)+'-arcmin'
IF beamnametype EQ 8 THEN BEGIN
   CASE input_beam_name.type OF
      'GAUSSIAN': BEGIN
         beam_name = 'GAUSSIAN-'+NUMBER2STRING(input_beam_name.size * CONVERSION_FACTOR(input_beam_name.sizeunits,'arcmin'))+'-arcmin'
      END
      'TABULATED': BEGIN
         wh = WHERE(input_beam_name.ell LE lmax, nwh)
         ellvalues = input_beam_name.ell[wh]
         beam[ellvalues] = input_beam_name.bl[ellvalues]
         IF NOT KEYWORD_SET(double) THEN beam=FLOAT(beam)
         RETURN, beam
      END
      'INSTR': BEGIN
         CASE input_beam_name.instr OF
            'WMAP': BEGIN
               beam = PSM_GET_BL(input_beam_name.instr+input_beam_name.version+'-'+ input_beam_name.channel, lmax=lmax, double=double, dim=dim)
               RETURN, beam
            END
            'DIRBE': BEGIN
               listch = ['12um','25um','60um','100um','140um','240um']
               bandnumber = ['05','06','07','08','09','10']
               CASE input_beam_name.version OF
                  'from-omega': BEGIN
                     omegabeam = [1.427,1.456,1.512,1.425,1.385,1.323]*1e-4 ;steradian
                                ;from explanatory supplement, chap.4
                     beamsize = SQRT(4*omegabeam*alog(2)/!pi) ;radian
                     beamsize = 60*180L/!pi*beamsize          ;arcminutes
                     wh = WHERE(listch EQ input_beam_name.channel, nwh)
                     IF nwh EQ 1 THEN beam = PSM_GET_BL(beamsize[wh[0]], lmax=lmax, double=double, dim=dim) ELSE MESSAGE, 'Channel not recognised for DIRBE beam'
                     RETURN, beam
                  END
                  ELSE: MESSAGE, 'Version not recognised for DIRBE beam'
               ENDCASE
            END
         ENDCASE
      END
      'NO_BEAM': beam_name = 'GAUSSIAN-0-arcmin'
      ELSE: MESSAGE, 'Case not implemented'
   ENDCASE
ENDIF

CASE beam_name OF

   'NO_BEAM': BEGIN
      beam[*]=1.
   END
   ELSE: BEGIN
      inst_chan = STR_SEP(beam_name,'-')
      IF N_ELEMENTS(inst_chan) LT 2 THEN BEGIN
         PRINT, "PSM_GET_BL: beam_name not supported. ABORTING"
         GOTO, closing
      ENDIF ELSE BEGIN
         inst = inst_chan[0]
         chan = inst_chan[1]
      ENDELSE

      ; wmap beams
      CASE STRMID(inst,0,4) OF

         'WMAP': BEGIN
         ;;;; IF dim GT 1 THEN MESSAGE, 'beams for'
            listch = ['Q1','Q2','V1','V2','W1','W2','W3','W4']
                                ;dirbeams = !PSMROOT+'data/instrument/'+inst
            CASE inst OF
               'WMAP3yr': BEGIN
                  dirbeams = FIX_SEPARATOR(!PSMROOT)+'Soft/datafiles/WMAP/beams/wmap_ampl_bl_3yr_v2/'
                  noiserms = [2.2449,2.1347,3.3040,2.9458,5.8833,6.532,6.8849,6.7441]*1D0
                  poststring = '_ampl_bl_3yr_v2.txt'
               END
               'WMAP5yr': BEGIN
                  dirbeams = FIX_SEPARATOR(!PSMROOT)+'Soft/datafiles/WMAP/beams/wmap_ampl_bl_5yr_v3/'
                  noiserms = [2.254,2.141,3.314,2.953,5.899,6.565,6.926,6.761]*1D0
                  poststring = '_ampl_bl_5yr_v3.txt'
               END
               'WMAP7yr': BEGIN
                  dirbeams = FIX_SEPARATOR(!PSMROOT)+'Soft/datafiles/WMAP/beams/wmap_ampl_bl_7yr_v4/'
                  noiserms = [2.254,2.140,3.319,2.955,5.906,6.572,6.941,6.778]*1D0
                  poststring = '_ampl_bl_7yr_v4.txt'
               END
               ELSE: MESSAGE, 'beam name is not valid'
            ENDCASE
            
            doavg = ((chan EQ 'Q') OR (chan EQ 'V') OR(chan EQ 'W'))
            IF chan EQ 'K' THEN chan='K1'
            IF chan EQ 'Ka' THEN chan='Ka1'
            
            IF doavg THEN BEGIN
               wh = WHERE(STRMID(listch,0,1) EQ chan,nwh)
               useweights = 1D0/noiserms(wh)^2
               FOR i=0, nwh-1 DO BEGIN
                  file2read = FIX_SEPARATOR(dirbeams) + 'wmap_'+chan+STRTRIM(STRING(i+1),2)+poststring
                  READCOL, file2read, c1, c2, COMMENT='#'
                  nel = N_ELEMENTS(c2)
                  uselmax = (nel-1)<lmax
                  FOR j=0, dim-1 DO beam(0:uselmax,j) = beam(0:uselmax,j) + useweights(i)*c2(0:uselmax)
               ENDFOR   
               beam = beam/TOTAL(useweights)
            ENDIF ELSE BEGIN
               CASE chan OF
                  'NILC': BEGIN
                     wmap_nilc_bl_file = GETPSMDIR('psmdata')+'ancillary/observations/WMAP/NILC-CMB-5yr/beam_bl.txt'
                     READCOL, wmap_nilc_bl_file, ell5, bln5, blw5, comment='#', format='L,F,D'
                     nel = N_ELEMENTS(bln5)
                     uselmax = (nel-1)<lmax
                     FOR j=0, dim-1 DO beam(0:uselmax,j) = bln5(0:uselmax)
                  END
                  'NILCwiener': BEGIN
                     wmap_nilc_bl_file = GETPSMDIR('psmdata')+'ancillary/observations/WMAP/NILC-CMB-5yr/beam_bl.txt'
                     READCOL, wmap_nilc_bl_file, ell5, bln5, blw5, comment='#', format='L,F,D'
                     nel = N_ELEMENTS(bln5)
                     uselmax = (nel-1)<lmax
                     FOR j=0, dim-1 DO beam(0:uselmax,j) = blw5(0:uselmax)
                  END
                  ELSE: BEGIN
                     file2read = FIX_SEPARATOR(dirbeams) + 'wmap_'+chan+poststring
                     READCOL, file2read, c1, c2, COMMENT='#'
                     nel = N_ELEMENTS(c2)
                     uselmax = (nel-1)<lmax
                     FOR j=0, dim-1 DO beam(0:uselmax,j) = c2(0:uselmax)
                  END
               ENDCASE
            ENDELSE   
         END

         ELSE: BEGIN
      
            ;; gaussian beams
            IF inst EQ 'GAUSSIAN' THEN BEGIN
               resolution = DOUBLE(chan)
               angle_units = inst_chan[2]
               IF angle_units EQ 'deg' THEN resolution = 60D0*resolution
               beam[*] = GAUSSBEAM(resolution, lmax, dim)
            ENDIF ELSE MESSAGE, 'Instrument '+inst+' not implemented' 

     
         END
      ENDCASE
   END
   
ENDCASE

IF NOT KEYWORD_SET(double) THEN beam=FLOAT(beam)
RETURN, beam
   
CLOSING:

END

