; Copyright 2008 ---- Jacques Delabrouille, CNRS

; This file is part of the Planck Sky Model.
;
; The Planck Sky Model is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; version 2 of the License.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY, without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model. If not, write to the Free Software
; Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

;+
;-
FUNCTION WMAP_NOISE, channel, version=version, pix=pix, polar=polar, help=help
;; output in mK_CMB

  IF (KEYWORD_SET(help) NE 0) OR (N_PARAMS() NE 1) THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'WMAP_NOISE: function which returns a random simulation of WMAP noise'
     PSM_INFO_MESSAGE, 'SYNTAX: result = WMAP_NOISE(channel, version=, pix=, polar=)'
     PSM_INFO_MESSAGE, 'channel is the WMAP channel to be considered - 1 or 2 character string among the following:' 
     PSM_INFO_MESSAGE, "['K1','Ka1','Q1','Q2','V1','V2','W1','W2','W3','W4','K','Ka','Q','V','W']"
     PSM_INFO_MESSAGE, "version sets the version of the instrument, which can be '7yr' only at present"
     PSM_INFO_MESSAGE, "pix is an input structure describing the polarisation, which should contain"
     PSM_INFO_MESSAGE, "the following tags: NSIDE, NPIX, ORDERING, COORDSYS."
     PSM_INFO_MESSAGE, "If pix is not defined, the default is NESTED nside=512, 'G'."
     PSM_INFO_MESSAGE, "Set /polar for polarised noise maps (I,Q,U)"
     PSM_INFO_MESSAGE, "The output is in 'mK_CMB'"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  CASE version OF
     '7yr': BEGIN
        ;; sigma0 of the noise in WMAP maps
        ;; Pixel noise in units of mK may be evaluated from Nobs with the expression sigma = sigma0 / sqrt(Nobs) where
        band = ['K1',	'Ka1',	'Q1',	'Q2',	'V1',	'V2',	'W1',	'W2',	'W3',	'W4',	'K',	'Ka',	'Q',	'V',	'W']
        sigT = ['1.437','1.470','2.254','2.140','3.319','2.955','5.906','6.572','6.941','6.778','1.437','1.470','2.197','3.137','6.549']
        sigP = ['1.456','1.490','2.290','2.164','3.348','2.979','5.940','6.612','6.983','6.840','1.456','1.490','2.227','3.164','6.594']
        wh = WHERE(band EQ channel, nwh)
        IF nwh NE 1 THEN MESSAGE, 'Error, unrecognised WMAP channel' ELSE BEGIN
           sigT = sigT(wh[0])
           sigP = sigP(wh[0])
        ENDELSE
     END
     ELSE: MESSAGE, 'Case not implemented' 
  ENDCASE

  hits = WMAP_WEIGHTARRAYS(channel, version=version)

  IF NOT DEFINED(pix) THEN BEGIN
     pix = GET_PIX_STRUCT('HEALPIX', hpx_nside=512L, hpx_ordering='NESTED', hpx_coordsys='G')
  ENDIF
  IF pix.coordsys NE 'G' THEN MESSAGE, 'Case not implemented'

  ;; Multiplicative factor for sigT and sigP, due to number of pixels
  ;; if the number of pixels of the final map is <512, the map will be
  ;; UD_GRADED later
  pixratio = (pix.nside/512L) > 1L

  ;; Get to the appropriate seed chain
  PSM_SEEDSTATUS, verbosity=0, exists=seedchainexists, seedchain=previousseedchain
  SET_PSM_SEEDCHAIN, 'wmap_noise'

  ;; Generate the noise at nside=512 or larger
  ;;-------------------------------------------
  IF NOT KEYWORD_SET(polar) THEN BEGIN
     noisereanested = RANDOMN(GETSEED(),pix.npix>NSIDE2NPIX(512L))
     nrand = pixratio^2         ; number of random numbers to be generated with same noise variance (or covariance matrix)
                                ; can be 1, 4, 16, 64, ...
     noisereanested[*] = noisereanested[*] * (sigT*double(pixratio))
     FOR i=0L, NSIDE2NPIX(512L)-1L DO BEGIN & $
        noisereanested[i*nrand:(i+1L)*nrand-1L] = noisereanested[i*nrand:(i+1L)*nrand-1L] / SQRT(hits[i].n_obs) & $
     ENDFOR
  ENDIF ELSE BEGIN

     ;; T noise
     ;;---------
     noisereanested = RANDOMN(GETSEED(),pix.npix>NSIDE2NPIX(512L),3)
     nrand = pixratio^2         ; number of random numbers to be generated with same noise variance (or covariance matrix)
                                ; can be 1, 4, 16, 64, ...
     noisereanested[*,0] = noisereanested[*,0] * (sigT*double(pixratio))
     FOR i=0L, NSIDE2NPIX(512L)-1L DO BEGIN
        noisereanested[i*nrand:(i+1L)*nrand-1L,0] = noisereanested[i*nrand:(i+1L)*nrand-1L,0] / SQRT(hits[i].n_obs)
     ENDFOR

     ;; Q and U noise
     ;;---------------
     covar = FLTARR(2,2)
     diag = INDGEN(2) * 3L
     newsigP = (sigP*double(pixratio))
     starttime = systime(1)
     FOR i=0L, NSIDE2NPIX(512L)-1L DO BEGIN
        pp=i
        IF pp/(314573L) EQ pp/(314573.) THEN BEGIN
           print, 'Generating '+version+' WMAP polarised noise maps for channel '+channel+': ', STRTRIM(STRING(10L*pp/314573L),2)+'% completed in '+STRING(systime(1) -starttime)+ ' seconds'
        ENDIF
        covar[0,0] = hits[i].qq
        covar[0,1] = hits[i].qu
        covar[1,0] = covar[0,1]
        covar[1,1] = hits[i].uu
        covar = INVERT(covar) * newsigP^2
        epsilon = FLTARR(nrand,2)
        epsilon[*,*] = noisereanested[i*nrand:(i+1L)*nrand-1L,1:2]
        A = covar               ; store covariance into dummy variable for input into TRIRED
        CHOLDC, A, P, /double   ; do Cholesky decomposition
        FOR j = 0, 1 DO FOR k = j, 1 do A[k,j] = 0d
        A[diag] = P
        ;; Transform standard normal deviates so they have covariance matrix COVAR
        noisereanested[i*nrand:(i+1L)*nrand-1L,1:2] = A ## epsilon
     ENDFOR 
  ENDELSE

  IF seedchainexists THEN SET_PSM_SEEDCHAIN, previousseedchain

  ;; If the final map is at lower nside then udgrade the map
  ;;---------------------------------------------------------

  IF pix.nside LT 512 THEN BEGIN
     UD_GRADE, noisereanested, nrn, nside_out=pix.nside, order_in='NESTED', order_out=pix.ordering
     RETURN, nrn
  ENDIF ELSE BEGIN
     CASE pix.ordering OF
        'RING': BEGIN
           noisereanested = REORDER(TEMPORARY(noisereanested),/n2r)
           RETURN, noisereanested
        END
        'NESTED': RETURN, noisereanested
     ENDCASE
  ENDELSE


END
