; -----------------------------------------------------------------------------
;
; Copyright (C) 2010-2012
;   Guillaume Castex
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	ELLIPTIC_SOURCE_CONVOLVER
;
; PURPOSE:
;	Software being developed, not to be used.
;
;-
; Add point sources patterns to map with a non-symetric gaussian beam
; (gaussian in 2sin(theta/2))
;
; @categories ps
;
; @param flux_jy {in}{required}{type=dblarr} vector of flux (in jansky)
; @param theta_gal {in}{required}{type=dblarr} corresponding theta
; coordinates (galactic colatitude in radian)
; @param phi_gal {in}{required}{type=dblarr} corresponding phi
; coordinates (galactic longitude in radian)
; @param map {in}{required}{type=pointer} Pointer towards the map to
; complete
; @param map_unit {in}{required}{type=string} Unit of the map
; @param freq {in}{required}{type=double} frequency of the map
; @param fwhm_arcmin {in}{required}{type=float} FWHM of the gaussian
; beam in arcminutes
; 
; @history <p> First version (circular version), March 23 2007, Marc Betoule </p>
; @history <p> Elliptic version, July 2 2009, Guillaume Castex </p>



pro vector_get_angle, vector1, vector2, angle

  cos_angle=float((transpose(vector2)#vector1))/sqrt(((transpose(vector1)#vector1)*(transpose(vector2)#vector2)))

  angle=acos(cos_angle)

end

pro vector_rotation, vector, rotation_vector, angle, new_vector

  temp_arr=transpose(rotation_vector)#rotation_vector
  squa_length=temp_arr[0]
  rotation_vector=rotation_vector*1/sqrt(float(squa_length))
  
  cos_angle=cos(angle)
  sin_angle=sin(angle)

  new_vector=fltarr(3)

  temp_arr=(1-cos_angle)*(transpose(rotation_vector)#vector)
  second_term=temp_arr[0]
  first_term=cos_angle[0]
  third_term=sin_angle[0]
  new_vector[*]= first_term*vector+second_term*rotation_vector+third_term*crossp(rotation_vector,vector)

end


pro elliptic_source_convolver, flux, theta_gal, phi_gal, map, fwhm_max_arcmin, b_on_a, angle, freq=freq,polar_percent=polar, psi=psi, nest=nest, method=method, debug=debug

MESSAGE, 'Check this code, debug if necessary!'

  IF NOT KEYWORD_SET(method) THEN BEGIN
     method='ANGLE'
  ENDIF
  
  fwhm=fwhm_max_arcmin * !DtoR / 60d
  sigma2 = fwhm^2 / (8d*alog(2))
  a = 1d/sigma2
  
  rmax = 3.5*fwhm
  ang2vec, theta_gal, phi_gal, pos ;help, pos = Array[nsources, 3]
  
  nside = sqrt(n_elements((*map)[*,0])/12l)
  
  
  
  
  for isource = 0l, n_elements(flux)-1l do begin
     IF KEYWORD_SET(debug) NE 0 THEN BEGIN    
        print, 'source '+string(isource)
     ENDIF
     position=pos[isource,*]
     query_disc , nside, position, rmax, listpix, /INCLUSIVE
     if keyword_set(nest) then begin
        nested=1    
        pix2vec_nest, nside, listpix, vector
     endif else begin
        nested=0
        pix2vec_ring, nside, listpix, vector 
     endelse                    ;help, vector = Array[npix, 3]
     
     oldvector=vector
     
     if keyword_set(nest) then begin
        ang2pix_nest, nside, 0,0, north_pix
        pix2vec_nest, nside, north_pix, north_vec
     endif else begin
        ang2pix_ring, nside, 0,0, north_pix
        pix2vec_ring, nside, north_pix, north_vec
     endelse
     if (total(crossp(north_vec,position)) ne 0) then begin
        vec_axe_b=crossp(north_vec,position)
     endif else begin
        ang2vec, !pi/2, 0, ref_vec
        vec_axe_b=crossp(north_vec,ref_vec)
     endelse
     vec_axe_a=crossp(position,vec_axe_b)
     
     temp_vector=fltarr(3)
     vector_rotation, vec_axe_b, transpose(position), angle[isource], temp_vector
     vec_axe_b=temp_vector
     vector_rotation, vec_axe_a, transpose(position), angle[isource], temp_vector
     vec_axe_a=temp_vector
     
     IF KEYWORD_SET(debug) NE 0 THEN BEGIN
        angold=dblarr(n_elements(listpix))
        angnew=dblarr(n_elements(listpix))
     ENDIF
     
     for i=0L,n_elements(listpix)-1 do begin
        temp_vector[*]=vector[i,*]
        
        coef=total(temp_vector*vec_axe_b)
        n_temp_vector=temp_vector-coef*vec_axe_b
                                ;n_temp_vector=temp_vector
        vector_get_angle, n_temp_vector, vec_axe_a, temp_angle
        ang_plane=!pi/2-temp_angle
        
        if (transpose(vec_axe_a)#temp_vector ge 0) then begin
           vector_rotation, temp_vector, vec_axe_b, ang_plane*(1-double(b_on_a[isource])), new_vector
        endif else begin
           vector_rotation, temp_vector, vec_axe_b, ang_plane*(1-double(b_on_a[isource])), new_vector
        endelse
        

        IF KEYWORD_SET(debug) NE 0 THEN BEGIN
           angold[i]=ang_plane
           coef=total(new_vector*vec_axe_b)
           n_temp_vector=new_vector-coef*vec_axe_b
           
           vector_get_angle, n_temp_vector, vec_axe_a, temp_angle
           ang_plane2=!pi/2-temp_angle
           angnew[i]=ang_plane2
           print, ang_plane2, ang_plane, ang_plane2/ang_plane
           print, '---------'
        ENDIF

        
        vector[i,*]=new_vector
        
        
     endfor
     
     
     cthetas = vector#transpose(position)
     cthetas = temporary(cthetas)*(a)
     cthetas = temporary(cthetas)-a
     
     
     
     
     
;factor of 0.096006 obtained numerically
     
                                ; B = double(flux)*nside2npix(nside)/(0.096006*(fwhm_max_arcmin^2)*4*!pi)

     temp_src=exp(cthetas)
     coeff_amp=flux[isource]/total(temp_src)
  
                                ; (*map)[listpix,0] = (*map)[listpix,0]+B[isource]*exp(cthetas)
  
     (*map)[listpix,0] = (*map)[listpix,0]+coeff_amp*exp(cthetas)

  IF max(*map) EQ 0 THEN STOP
     vector=oldvector
     


     IF method eq 'ANGLE' then begin
        if keyword_set(polar) then begin
           
           vec2ang,vector, theta_p, phi_p
           POLAR_NEARBY_ANGLE_ON_SPHERE, theta_gal[isource], phi_gal[isource], psi[isource], theta_p, phi_p, angle_out
           
           Q = polar[isource]/100d * B[isource] * cos(2d*angle_out)
           U = polar[isource]/100d * B[isource] * sin(2d*angle_out)
           stokesloc = dblarr(n_elements(listpix),2,/noz)
           stokesloc[*,0] = Q
           stokesloc[*,1] = U
           
           (*map)[listpix,1] = (*map)[listpix,1]+stokesloc[*,0]*exp(cthetas)
           (*map)[listpix,2] = (*map)[listpix,2]+stokesloc[*,1]*exp(cthetas)
           
           
        endif
        
     endif else begin
      
        if keyword_set(polar) then begin
           mat = euler_matrix_new(0,!pi/2d - theta_gal[isource], -phi_gal[isource],/y)
           vectorloc = rotate_coord(vector,eul=mat)
           Q = polar[isource]/100d * B[isource] * cos(2d*psi[isource])
           U = polar[isource]/100d * B[isource] * sin(2d*psi[isource])
           stokesloc = dblarr(n_elements(listpix),2,/noz)
           stokesloc[*,0] = Q
           stokesloc[*,1] = U
           
           mat = euler_matrix_new(phi_gal[isource],-!pi/2d + theta_gal[isource],0,/y)
           
           vectorloc = rotate_coord(vectorloc, eul=mat, sto=stokesloc)
           
           
           (*map)[listpix,1] = (*map)[listpix,1]+stokesloc[*,0]*exp(cthetas)
           (*map)[listpix,2] = (*map)[listpix,2]+stokesloc[*,1]*exp(cthetas)
           
           
        endif
     endelse

     listpix = 0
     vector = 0
     cthetas = 0
  endfor
 

end
