; -----------------------------------------------------------------------------
;
; Copyright (C) 2007-2012
;   Marc Betoule
;   Guillaume Castex
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	SYMMETRIC_SOURCE_CONVOLVER
;
; PURPOSE:
;	Software being developed, not to be used.
;
;-
; Copyright 2007 C.N.R.S., Observatoire de Paris

; This file is part of the Planck Sky Model.
;
; The Planck Sky Model is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; version 2 of the License.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY, without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model. If not, write to the Free Software
; Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

; Add point sources patterns to map with a non-symetric gaussian beam
; (gaussian in 2sin(theta/2))
;
; @categories ps
;
; @param flux_jy {in}{required}{type=dblarr} vector of flux (in jansky)
; @param theta_gal {in}{required}{type=dblarr} corresponding theta
; coordinates (galactic colatitude in radian)
; @param phi_gal {in}{required}{type=dblarr} corresponding phi
; coordinates (galactic longitude in radian)
; @param map {in}{required}{type=pointer} Pointer towards the map to
; complete
; @param map_unit {in}{required}{type=string} Unit of the map
; @param freq {in}{required}{type=double} frequency of the map
; @param fwhm_arcmin {in}{required}{type=float} FWHM of the gaussian
; beam in arcminutes
; 
; @history <p> First version (circular version), March 23 2007, Marc Betoule </p>

pro symmetric_source_convolver, flux, theta_gal, phi_gal, map, fwhm_max_arcmin, polar_percent=polar, psi=psi, nest=nest, method=method


MESSAGE, 'Check this code, debug if necessary!'

 IF NOT KEYWORD_SET(method) THEN BEGIN
    method='ANGLE'
 ENDIF

  fwhm=fwhm_max_arcmin * !DtoR / 60d
  sigma2 = fwhm^2 / (8d*alog(2))
  a = 1d/sigma2

  rmax = 3.5*fwhm
  ang2vec, theta_gal, phi_gal, pos ;;help, pos = Array[nsources, 3]
  
  nside = sqrt(n_elements((*map)[*,0])/12l)

;factor of 0.096006 obtained numerically

  B = double(flux)*nside2npix(nside)/(0.096006*(fwhm_max_arcmin^2)*4*!pi)


   PSM_DEBUG_MESSAGE,'Clean up the two methods for computing polarisation', routine='symmetric_source_convolver'


for isource = 0l, n_elements(flux)-1l do begin
   IF isource/10000L EQ isource/10000. THEN print, 'source '+string(isource)
   position=pos[isource,*]


   QUERY_DIST, nside, position, rmax, listpix, nlist, distpix, /INCLUSIVE

   if keyword_set(nest) then begin
      nested=1    
      pix2vec_nest, nside, listpix, vector
   endif else begin
      nested=0
      pix2vec_ring, nside, listpix, vector 
   endelse

   cthetas = -distpix*distpix*(a/2)

   (*map)[listpix,0] = (*map)[listpix,0]+B[isource]*exp(cthetas)
   
 

   IF method eq 'ANGLE' then begin
      if keyword_set(polar) then begin
         
         vec2ang,vector, theta_p, phi_p
         POLAR_NEARBY_ANGLE_ON_SPHERE, theta_gal[isource], phi_gal[isource], psi[isource], theta_p, phi_p, angle_out
         
         Q = polar[isource]/100d * B[isource] * cos(2d*angle_out)
         U = polar[isource]/100d * B[isource] * sin(2d*angle_out)
         stokesloc = dblarr(n_elements(listpix),2,/noz)
         stokesloc[*,0] = Q
         stokesloc[*,1] = U
 
         (*map)[listpix,1] = (*map)[listpix,1]+stokesloc[*,0]*exp(cthetas)
         (*map)[listpix,2] = (*map)[listpix,2]+stokesloc[*,1]*exp(cthetas)
         
         
      endif
      
   endif else begin
      
      if keyword_set(polar) then begin
         mat = euler_matrix_new(0,!pi/2d - theta_gal[isource], -phi_gal[isource],/y)
         vectorloc = rotate_coord(vector,eul=mat)
         Q = polar[isource]/100d * B[isource] * cos(2d*psi[isource])
         U = polar[isource]/100d * B[isource] * sin(2d*psi[isource])
         stokesloc = dblarr(n_elements(listpix),2,/noz)
         stokesloc[*,0] = Q
         stokesloc[*,1] = U
         
         mat = euler_matrix_new(phi_gal[isource],-!pi/2d + theta_gal[isource],0,/y)
         
         vectorloc = rotate_coord(vectorloc, eul=mat, sto=stokesloc)
         
         
         (*map)[listpix,1] = (*map)[listpix,1]+stokesloc[*,0]*exp(cthetas)
         (*map)[listpix,2] = (*map)[listpix,2]+stokesloc[*,1]*exp(cthetas)
         

      endif
   endelse

   listpix = 0
   vector = 0
   cthetas = 0
endfor

end
