PRO RING_ON_SKY, theta_spin, phi_spin, theta_pointing, phi_pointing, v, dv, anglescanning, $
	theta_open=theta_open, psi_phase=psi_phase, refpixcenter=refpixcenter, nside=nside, visu=visu

;+
; NAME:
;     RING_ON_SKY
; ONE LINE HELP:
;     Compute coordinates of ring points on the sky
; PURPOSE:
;     Compute coordinates of ring points on the sky, given spin axis, opening angle  and phase
; CALLING SEQUENCE:
;     RING_ON_SKY, theta_spin, phi_spin, theta_pointing, phi_pointing, $
;		theta_open=theta_open, psi_phase=psi_phase
; INPUT:
;     theta_spin, phi_spin : spin axis spherical coordinates (radians)
; OUTPUT:
;     theta_pointing, phi_pointing : pointing spherical coordinates
;     (radians)
;     v, dv : pointing vector and derivative (tangent to scanning
;     direction, normalised)
;     anglescanning : angle between scanning direction and meridian
; KEYWORDS:
;     theta_open : opening angle of pointing (radians)
;     psi_phase : phase along the ring (radians) measured from closest
;     approach to North
;     set refpixcenter to measure angle with reference at pix center
;     (safer for polar pixels)
; EXAMPLES:
;     
; ALGORITHM:
;     
; SIDE EFFECTS:
;     
; RESTRICTIONS:
;     Report any bugs to Jacques Delabrouille
;     delabrouille@apc.univ-paris7.fr
; PROCEDURES CALLED:
;     
; REVISION HISTORY
;     Written, Jacques Delabrouille 01/07/2009
;-

  IF ( N_PARAMS() EQ 0 ) THEN BEGIN
     print,'  CALL IS : RING_ON_SKY, theta_spin, phi_spin, theta_pointing, phi_pointing, v, dv, anglescanning'
     print,'  Keywords : theta_open : opening angle of pointing (radians)'
     print,'             psi_phase : phase along the ring (radians)'
     print,'             refpixcenter : to use the pixel center coord for defining the reference for the scanning angle'
     print,'  all angles in radians' 
     GOTO, closing
  ENDIF
  

  IF NOT KEYWORD_SET(psi_phase) THEN psi_phase=findgen(360)*!dtor
  IF NOT KEYWORD_SET(theta_open) THEN theta_open=85.*!dtor

  twopi = 2*!dpi

;; Ring around the north pole
;;----------------------------

  ;;x = SIN(theta_open)*COS(psi_phase+!pi)                ; x
  ;;y = SIN(theta_open)*SIN(psi_phase+!pi)                ; y

  ;same thing slightly faster
  x = -SIN(theta_open)
  y = x*SIN(psi_phase)
  x = TEMPORARY(x)*COS(psi_phase)

  z = COS(theta_open)                                   ; z
  z = REPLICATE(z,N_ELEMENTS(psi_phase))

;; Ring around (theta=theta_spin, phi=0)
;;---------------------------------------
  costhetaspin = COS(theta_spin)
  sinthetaspin = SIN(theta_spin)

  xprime = x*costhetaspin + z*sinthetaspin              ; x
  xprime = (xprime<1) > (-1)
                                                        ; y unchanged
  zprime = z*costhetaspin - x*sinthetaspin              ; z
  zprime = (zprime<1) > (-1)


;; theta and phi of ring around (theta=theta_spin, phi=0) 
;;--------------------------------------------------------
  theta_pointing = ACOS(zprime)
  phi_pointing = ATAN(y,xprime)

;; theta and phi of ring around (theta=theta_spin, phi=phi_spin) 
;;---------------------------------------------------------------
  phi_pointing = phi_pointing + phi_spin

;; vector
;;--------
  IF N_PARAMS() GE 5 THEN BEGIN
     v = [[SIN(theta_pointing)],[SIN(phi_pointing)],[zprime]]
     v[*,1] = v[*,1]*v[*,0]
     v[*,0] = v[*,0]*COS(phi_pointing)
     
     IF KEYWORD_SET(visu) NE 0 THEN BEGIN
        WINDOW, /free, title='POINTING VECTOR', xsize=800, ysize=800
        PLOT_3DBOX, v[*,0], v[*,1], v[*,2], /xy_plane, /xz_plane, /yz_plane, $
                    xystyle=2, xzstyle=2, yzstyle=2, $
                    xcharsize=1.5, ycharsize=1.5, zcharsize=1.5, $
                    xrange=[-1,1], yrange=[-1,1], zrange=[-1,1], $
                    xstyle=1, ystyle=1, zstyle=1, $
                    xtitle='X', ytitle='Y', ztitle='Z', $
                    charsize=2
     ENDIF
  ENDIF

;; tangent
;;---------
  IF N_PARAMS() GE 6 THEN BEGIN
     dv = [[SHIFT(v[*,0],-1)-SHIFT(v[*,0],1)],[SHIFT(v[*,1],-1)-SHIFT(v[*,1],1)],[SHIFT(v[*,2],-1)-SHIFT(v[*,2],1)]]
     norm = SQRT(TOTAL(dv^2,2))
     FOR i=0, 2 DO dv[*,i] = dv[*,i] / norm[*]
     
     IF KEYWORD_SET(visu) NE 0 THEN BEGIN
        WINDOW, /free, title='SCANNING DIRECTION VECTOR', xsize=800, ysize=800
        PLOT_3DBOX, dv[*,0], dv[*,1], dv[*,2], /xy_plane, /xz_plane, /yz_plane, $
                    xystyle=2, xzstyle=2, yzstyle=2, $
                    xcharsize=1.5, ycharsize=1.5, zcharsize=1.5, $
                    xrange=[-1,1], yrange=[-1,1], zrange=[-1,1], $
                    xstyle=1, ystyle=1, zstyle=1, $
                    xtitle='X', ytitle='Y', ztitle='Z', $
                    charsize=2
     ENDIF
  ENDIF

;; anglescanning
;;---------------
  IF N_PARAMS() GE 6 THEN BEGIN
     IF NOT KEYWORD_SET(refpixcenter) THEN BEGIN
        ref = [[COS(theta_pointing)],[SIN(phi_pointing)],[-SIN(theta_pointing)]]
        ref[*,1] = ref[*,1] * ref[*,0]
        ref[*,0] = ref[*,0] * COS(phi_pointing)
        xx = TOTAL(dv*ref,2)
        ref2 = [[-SIN(phi_pointing)],[COS(phi_pointing)],REPLICATE(0,N_ELEMENTS(psi_phase))]
        yy =  TOTAL(dv*ref2,2)
        anglescanning = ATAN(yy,xx)
     ENDIF ELSE BEGIN
        ANG2PIX_RING, nside, theta_pointing, (phi_pointing MOD twopi), p
                                ;stop
        PIX2ANG_RING, nside, p, theta, phi
        ref = [[COS(theta)],[SIN(phi)],[-SIN(theta)]]
        ref[*,1] = ref[*,1] * ref[*,0]
        ref[*,0] = ref[*,0] * COS(phi)
        xx = TOTAL(dv*ref,2)
        ref2 = [[-SIN(phi)],[COS(phi)],REPLICATE(0,N_ELEMENTS(psi_phase))]
        yy =  TOTAL(dv*ref2,2)
        anglescanning = ATAN(yy,xx)
     ENDELSE
  ENDIF
  
closing:
END
