; Copyright 2009 Jacques Delabrouille, CNRS

; This file is part of the Planck Sky Model.
;
; The Planck Sky Model is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; version 2 of the License.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY, without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model. If not, write to the Free Software
; Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

;+
; Reads a file containing CMB C_ell, returns a standard structure with tags {ell,ctt} or {ell,ctt,cee,cbb,cte}
;
; @categories cmb
;
; @param file {in}{required}{type=string} file to be read
; @param cl {out}{required}{type=structure} output C_ell
; @keyword units {out}{optional}{array of string} units of the C_ell (if FITS file) 
; @uses FILE_EXTENSION, NUMBER2STRING (PSM) MRDFITS, READCOL, SXPAR (ASTRON)
; @author Jacques Delabrouille
; @history <p>19/01/2009: First version, Jacques Delabrouille</p>
; @history <p>28/09/2009: Modified to match cl structure format as produced by cl_structure.pro, help added, Jacques Delabrouille</p>
; @copyright Jacques Delabrouille, CNRS
;-
PRO READ_CL_ANYFORMAT, file, cl, camb=camb, lmax=lmax, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'READ_CL_ANYFORMAT: procedure to read Cl from a fits or text file as produced by CAMB or ANAFAST'
     PSM_INFO_MESSAGE, 'SYNTAX: READ_CL_ANYFORMAT, file, cl, /camb'
     PSM_INFO_MESSAGE, 'Output cl is a structure as created with cl_structure.pro'
     PSM_INFO_MESSAGE, 'Set /camb if the file contains l(l+1)cl/2pi and not directly cl'
     PSM_INFO_MESSAGE, "Set lmax to the maximum value of l requested (dafault: all available l's)"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF

ext = FILE_EXTENSION(file)
units = STRARR(5)

CASE STRMID(STRLOWCASE(ext),0,3) OF

   ;---- FITS FILE ------------------------------------------------------------------------
   'fit': BEGIN
      c_ell = MRDFITS(file,1,hdr)
      ok = CHECK_CL_STRUCT(c_ell)

      ;; If the data read has the format of a valid PSM cl structure 
      ;;-------------------------------------------------------------
      IF ok THEN cl=c_ell


      ;; If the data read does NOT have a valid format
      ;;-----------------------------------------------
      IF NOT ok THEN BEGIN         

         ntags = N_TAGS(c_ell)
         nnn = N_ELEMENTS(c_ell)
         ;;cl = REPLICATE({ell:0,ctt:0.,cee:0.,cbb:0.,cte:0.},nnn)
         
         tags = TAG_NAMES(c_ell)
         l_col = WHERE((tags EQ 'L') OR (tags EQ 'ELL'), ncol)
         IF ncol EQ 0 THEN usel = LINDGEN(nnn) 
         IF ncol EQ 1 THEN usel = c_ell.(l_col)
         
         IF ncol GT 1 THEN BEGIN
            PRINT, 'READ_CL_FILE: AMBIGUOUS VALUE OF ELL IN '+file
            GOTO, closing
         ENDIF
         maxl = MAX(usel)
         ell = LINDGEN(maxl+1)
         
         ctt = DBLARR(maxl+1)
         tt_col = WHERE((tags EQ 'TEMPERATURE') OR (tags EQ 'CTT') OR (tags EQ 'TEMPERATURE_C_L'), ncol)
         IF ncol EQ 1 THEN BEGIN
            ctt(usel) = c_ell.(tt_col)
            units[1] = SXPAR(hdr,'TUNIT'+NUMBER2STRING(tt_col[0]+1))
            UNIT2PSMUNIT, units[1], unit_tt
         ENDIF
         IF ncol GT 1 THEN BEGIN
            PRINT, 'READ_CL_FILE: CANNOT READ  VALUE OF CTT IN '+file
         ENDIF
         
         cee = DBLARR(maxl+1)
         ee_col = WHERE((tags EQ 'GRADIENT') OR (tags EQ 'CEE') OR (tags EQ 'E_MODE_C_L'), ncol)
         IF ncol EQ 1 THEN BEGIN
            cee(usel) = c_ell.(ee_col)
            units[2] = SXPAR(hdr,'TUNIT'+NUMBER2STRING(ee_col[0]+1))
            UNIT2PSMUNIT, units[2], unit_ee
         ENDIF
         IF ncol GT 1 THEN BEGIN
            PRINT, 'READ_CL_FILE: CANNOT READ  VALUE OF CEE IN '+file
         ENDIF       
         
         cbb = DBLARR(maxl+1)
         bb_col = WHERE((tags EQ 'CURL') OR (tags EQ 'CBB') OR (tags EQ 'B_MODE_C_L'), ncol)
         IF ncol EQ 1 THEN BEGIN
            cbb(usel) = c_ell.(bb_col)
            units[3] = SXPAR(hdr,'TUNIT'+NUMBER2STRING(bb_col[0]+1))
            UNIT2PSMUNIT, units[3], unit_bb
         ENDIF
         IF ncol GT 1 THEN BEGIN
            PRINT, 'READ_CL_FILE: CANNOT READ  VALUE OF CBB IN '+file
         ENDIF
         
         cte = DBLARR(maxl+1)
         te_col = WHERE((tags EQ 'G_T') OR (tags EQ 'CTE') OR (tags EQ 'T_E_CROSS_CORR_') OR (tags EQ 'E_T_CROSS_CORR_'), ncol)
         IF ncol EQ 1 THEN BEGIN
            cte(usel) = c_ell.(te_col)
            units[4] = SXPAR(hdr,'TUNIT'+NUMBER2STRING(te_col[0]+1))
         ENDIF
         IF ncol NE 1 THEN BEGIN
            PRINT, 'READ_CL_FILE: CANNOT READ VALUE OF CTE IN '+file
         ENDIF       
         
         ;;tnot0 = (MAX(ctt) NE 0D0)
         ;;enot0 = (MAX(cee) NE 0D0)
         ;;bnot0 = (MAX(cee) NE 0D0)

         IF DEFINED(lmax) EQ 0 THEN lmax=maxl
         cl = CL_STRUCTURE(fieldnames=['T','E','B'], lmin=0, lmax=lmax, /double)
         lll = ell[WHERE(ell LE lmax)]
         cl.c_1_1[*] = 'NAN'
         cl.c_1_2[*] = 'NAN'
         cl.c_2_2[*] = 'NAN'
         cl.c_3_3[*] = 'NAN' 
         cl.c_1_1[lll] = ctt[lll]
         cl.c_1_2[lll] = cte[lll]
         cl.c_2_2[lll] = cee[lll]
         cl.c_3_3[lll] = cbb[lll] 

         ;; dealing with units
         fieldunit = STRARR(3)
         tok = CHECKUNIT(unit_tt, prefix, core, exponent)
         IF tok THEN fieldunit[0] = prefix+core ELSE fieldunit[0] = 'unknown'
         eok = CHECKUNIT(unit_ee, prefix, core, exponent)
         IF eok THEN fieldunit[1] = prefix+core ELSE fieldunit[1] = 'unknown'
         bok = CHECKUNIT(unit_bb, prefix, core, exponent)
         IF bok THEN fieldunit[2] = prefix+core ELSE fieldunit[2] = 'unknown'

         cl.units = fieldunit
      ENDIF
   END
      
   ;---- TEXT FILE ------------------------------------------------------------------------
   'txt': BEGIN
      READCOL, file, ell, ctt, cee, cbb, cte
      IF NOT DEFINED(lmax) THEN lmax = MAX(ell)
      cl = CL_STRUCTURE(fieldnames=['T','E','B'], lmin=0, lmax=lmax, /double)
         lll = ell[WHERE(ell LE lmax)]
         cl.c_1_1[*] = 'NAN'
         cl.c_1_2[*] = 'NAN'
         cl.c_2_2[*] = 'NAN'
         cl.c_3_3[*] = 'NAN' 
         cl.c_1_1[lll] = ctt[lll]
         cl.c_1_2[lll] = cte[lll]
         cl.c_2_2[lll] = cee[lll]
         cl.c_3_3[lll] = cbb[lll] 
      cl.units[*] = 'unknown'
   END

   ;---- DAT FILE -------------------------------------------------------------------------
   'dat': BEGIN
      ;; Check number of columns
      GET_LUN, unit
      OPENR, unit, file
      var = ''
      READF, unit, var
      CLOSE, unit
      FREE_LUN, unit
      var = STRTRIM(var,2)
      var = STRCOMPRESS(var)
      res = STRSPLIT(var)
      ncol = N_ELEMENTS(res)

      
      ;; If 5 columns
      IF ncol EQ 5 THEN BEGIN
         READCOL, file, ell, ctt, cee, cbb, cte
         IF NOT DEFINED(lmax) THEN lmax = MAX(ell)
         cl = CL_STRUCTURE(fieldnames=['T','E','B'], lmin=0, lmax=lmax, /double)
         lll = ell[WHERE(ell LE lmax)]
         cl.c_1_1[*] = 'NAN'
         cl.c_1_2[*] = 'NAN'
         cl.c_2_2[*] = 'NAN'
         cl.c_3_3[*] = 'NAN' 
         cl.c_1_1[lll] = ctt[lll]
         cl.c_1_2[lll] = cte[lll]
         cl.c_2_2[lll] = cee[lll]
         cl.c_3_3[lll] = cbb[lll] 
         cl.units[*] = 'unknown'
      ENDIF

      ;; If 4 columns or more than 5
      IF ((ncol EQ 4) OR (ncol GT 5)) THEN BEGIN
         READCOL, file, ell, ctt, cee, cte
         IF NOT DEFINED(lmax) THEN lmax = MAX(ell)
         cl = CL_STRUCTURE(fieldnames=['T','E'], lmin=0, lmax=lmax, /double)
         lll = ell[WHERE(ell LE lmax)]
         cl.c_1_1[*] = 'NAN'
         cl.c_1_2[*] = 'NAN'
         cl.c_2_2[*] = 'NAN'
         cl.c_1_1[lll] = ctt[lll]
         cl.c_1_2[lll] = cte[lll]
         cl.c_2_2[lll] = cee[lll]
         cl.units[*] = 'unknown'
      ENDIF
      
   END

   ;---- OTHER FILE -----------------------------------------------------------------------
   ELSE: BEGIN
      PSM_INFO_MESSAGE, 'READ_CL_FILE: ' + ext + ' extention not implemented'
      cl = 0
      GOTO, closing
   END

ENDCASE


IF KEYWORD_SET(camb) NE 0 THEN BEGIN

   ;; Assume we are reading l (l + 1) Cl / 2pi in uK^2
   DEFSYSV, 'phyconst', exists=exists
   ;IF NOT exists THEN DEFINE_PHYSICAL_CONSTANTS
   tcmb = CMB_TEMPERATURE()
   factor = 2D0*!DPI
   ncl = N_ELEMENTS(cl.field)
   ell = cl.l
   FOR i=1,ncl DO BEGIN
      is = NUMBER2STRING(i)
      FOR j=i,ncl DO BEGIN
         js=NUMBER2STRING(j)
         ok = EXECUTE('cl.c_'+is+'_'+js+'[ell] = cl.c_'+is+'_'+js+ '[ell] * factor/ell/(ell+1)')
      ENDFOR
   ENDFOR
   cl.units[*] = 'uK_CMB'
   
ENDIF
   
IF ((PSM_CAREFULNESS() GE 1) OR (PSM_VERBOSITY() GE 1)) THEN PSM_INFO_MESSAGE, 'File read : '+file
IF PSM_CAREFULNESS() GE 1 THEN BEGIN
   PSM_INFO_MESSAGE, 'Warning, READ_CL_ANYFORMAT makes some assumptions about input format, ordering of columns, units etc.'
   PSM_INFO_MESSAGE, 'Check your Cls !'
   IF PSM_CAREFULNESS() GE 2 THEN STOP
ENDIF

PSM_DEBUG_MESSAGE, 'Need to make routine more robust', routine='READ_CL_ANYFORMAT', infolimit=2, stoplimit=3


closing:
END

