; -----------------------------------------------------------------------------
;
; Copyright (C) 2012-
;   Jacques Delabrouille
;   Jean-Baptiste Melin
;   Matthieu Roman
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	BDELTA_C
;
; PURPOSE:
;	Compute the value of the critical density contrast Delta_c at redshift z.
;	Developed for including cosmological models with quintessence
;	and w between -1 and -0.4.
;
; CATEGORY:
;	Sky model (Cosmo)
;
; CALLING SEQUENCE:
;	Result = BDELTA_C(Z_array, COSMO=)
;
; INPUTS:
;	Z_array:	A redshift, or an array of redshift values.
;
; KEYWORD PARAMETERS:
;	COSMO:		A structure of cosmological parameters
;
; OUTPUTS:
;	Result is the value of the critical density contrast at the
;	specified redshift values.
;
; RESTRICTIONS:
;	Results should be OK for a dark energy equation of state
;	parameter w between -1 and -0.4, probably gradually worsening
;	for values well outside that range.
;
;	Report any bugs to Jean-Baptiste Melin
;	jean-baptiste.melin@cea.fr
;	and
;	Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;	Based on a fit done for the following range : 
;	0.2 LE omega_m LE 1
;	0   LE omega_q LE 0.9
;	-1  LE    w    LE -0.4
;
; EXAMPLE:
;	... 
;
; MODIFICATION HISTORY:
; 	dd/04/2012:	Matthieu Roman, First version merging
; 	 	 	FAST_BIG_DELTA_C and FAST_BIG_DELTA_C_Q by Jean-Baptiste Melin.
; 	22/09/2012:	Jacques Delabrouille, Doc and licence header updated.
;-

FUNCTION BDELTA_C, z_array, cosmo=cosmo

; Curve fits depending on curvature
; Made with delta_c_q and big_deltac_fit.scr

; 0.2 LE omega_m LE 1
; 0   LE omega_q LE 0.9
; -1  LE    w    LE -0.4

omega_m = cosmo.omega_m
omega_q = cosmo.omega_de
h = cosmo.h
w = cosmo.w_dark_energy
bdelta_eds = 18d0*!dpi^2
omega_m_array = omega_m*(1+z_array)^3*(100*h/HUBBLE_AT_Z(z_array,cosmo=cosmo))^2
omega_c_tab = [-0.6, -0.5, -0.4, -0.3, -0.2, -0.1, 0., 0.1, 0.2, 0.3, 0.4, 0.5, 0.6]

IF omega_m EQ 1 AND omega_q EQ 0 THEN BEGIN
  big_delta_c = REPLICATE(178d0, N_ELEMENTS(z_array))
ENDIF ELSE BEGIN
  IF omega_m LT 1 AND omega_q EQ 0 THEN BEGIN
     omega_m_z = omega_m*(1+z_array)^3/(HUBBLE_AT_Z(z_array,cosmo=cosmo)/100./h)^2
     xz = 2./omega_m_z-1
     eta_f_z = ALOG(xz+SQRT(xz^2-1))
     big_delta_c = 4d0*!dpi^2*(COSH(eta_f_z)-1)^3/(SINH(eta_f_z)-eta_f_z)^2*omega_m_z
  ENDIF ELSE BEGIN
     omega_c = 1-(omega_m+omega_q)
     ok = MIN(ABS(omega_c_tab - omega_c), whc)
     ;print, omega_c_tab[whc]
   CASE omega_c_tab[whc] OF
      0.  : BEGIN
              a = 0.220+0.151*(w+1)+0.504*(w+1)^2+0.014*omega_m-0.004*omega_m*(w+1)+0.025*omega_m^2
              b = 0.895+0.033*(w+1)+0.024*(w+1)^2+0.125*omega_m-0.054*omega_m*(w+1)-0.021*omega_m^2
              theta = 1/omega_m_array-1>0
              big_delta_c = bdelta_eds*(1+a*theta^b)*omega_m_array
            END
      0.1 : BEGIN
              a = 0.368+0.012*(w+1)+0.410*(w+1)^2-0.251*omega_m+0.186*omega_m*(w+1)+0.229*omega_m^2
              b = 0.758+0.347*(w+1)-0.101*(w+1)^2+0.044*omega_m+0.046*omega_m*(w+1)-0.006*omega_m^2
              theta = 1/omega_m_array-1>0
              big_delta_c = bdelta_eds*(1+a*theta^b)*omega_m_array
            END
      0.2 : BEGIN
              a = 0.453-0.046*(w+1)+0.334*(w+1)^2-0.346*omega_m+0.166*omega_m*(w+1)+0.400*omega_m^2
              b = 0.778+0.269*(w+1)+0.035*(w+1)^2-0.073*omega_m-0.022*omega_m*(w+1)+0.200*omega_m^2
              theta = 1/omega_m_array-1>0
              big_delta_c = bdelta_eds*(1+a*theta^b)*omega_m_array
            END
      0.3 : BEGIN
              a = 0.508-0.086*(w+1)+0.267*(w+1)^2-0.353*omega_m+0.159*omega_m*(w+1)+0.473*omega_m^2
              b = 0.815+0.163*(w+1)+0.118*(w+1)^2-0.125*omega_m-0.032*omega_m*(w+1)+0.329*omega_m^2
              theta = 1/omega_m_array-1>0
              big_delta_c = bdelta_eds*(1+a*theta^b)*omega_m_array
            END
      0.4 : BEGIN
              a = 0.540-0.094*(w+1)+0.196*(w+1)^2-0.316*omega_m+0.144*omega_m*(w+1)+0.495*omega_m^2
              b = 0.845+0.097*(w+1)+0.144*(w+1)^2-0.128*omega_m-0.058*omega_m*(w+1)+0.420*omega_m^2
              theta = 1/omega_m_array-1>0
              big_delta_c = bdelta_eds*(1+a*theta^b)*omega_m_array
            END
      0.5 : BEGIN
              a = 0.559-0.088*(w+1)+0.132*(w+1)^2-0.260*omega_m+0.139*omega_m*(w+1)+0.472*omega_m^2
              b = 0.871+0.047*(w+1)+0.135*(w+1)^2-0.096*omega_m-0.050*omega_m*(w+1)+0.442*omega_m^2
              theta = 1/omega_m_array-1>0
              big_delta_c = bdelta_eds*(1+a*theta^b)*omega_m_array
            END
      0.6 : BEGIN
              a = 0.567-0.067*(w+1)+0.080*(w+1)^2-0.199*omega_m+0.114*omega_m*(w+1)+0.443*omega_m^2
              b = 0.889+0.023*(w+1)+0.119*(w+1)^2-0.038*omega_m-0.110*omega_m*(w+1)+0.482*omega_m^2
              theta = 1/omega_m_array-1>0
              big_delta_c = bdelta_eds*(1+a*theta^b)*omega_m_array
            END
     -0.1 : BEGIN
              a = -0.860+0.280*(w+1)+0.277*(w+1)^2+0.962*omega_m-0.392*omega_m*(w+1)-0.128*omega_m^2
              ;b = 0.232-0.596*(w+1)+0.184*(w+1)^2-0.131*omega_m+0.317*omega_m*(w+1)+0.009*omega_m^2
              b =  0.223-0.673*(w+1)+0.430*(w+1)^2 $
                  -0.074*omega_m+0.380*omega_m*(w+1)-0.587*omega_m*(w+1)^2 $
                  -0.051*omega_m^2+0.092*omega_m^2*(w+1)+0.249*omega_m^2*(w+1)^2
              c = -0.389+1.627*(w+1)-1.225*(w+1)^2+2.512*omega_m-2.157*omega_m*(w+1)+0.145*omega_m^2
              ;d = 0.400-2.860*(w+1)+2.928*(w+1)^2+2.984*omega_m-2.587*omega_m*(w+1)-0.997*omega_m^2
              d = -0.206+0.852*(w+1)-0.975*(w+1)^2 $
                  +5.416*omega_m-17.198*omega_m*(w+1)+15.013*omega_m*(w+1)^2 $
                  -2.996*omega_m^2+11.848*omega_m^2*(w+1)-11.964*omega_m^2*(w+1)^2
              z = z_array
              big_delta_c = bdelta_eds*(1+(a+b*z)/(1+c*z+d*z^2))
            END
     -0.2 : BEGIN
              a = -0.879+0.310*(w+1)+0.295*(w+1)^2+0.919*omega_m-0.389*omega_m*(w+1)-0.099*omega_m^2
              ;b = 0.364-0.887*(w+1)+0.269*(w+1)^2-0.157*omega_m+0.417*omega_m*(w+1)-0.006*omega_m^2
              b =  0.349-0.940*(w+1)+0.506*(w+1)^2 $
                  -0.079*omega_m+0.300*omega_m*(w+1)-0.327*omega_m*(w+1)^2 $
                  -0.081*omega_m^2+0.272*omega_m^2*(w+1)-0.052*omega_m^2*(w+1)^2
              c = -0.625+2.078*(w+1)-1.331*(w+1)^2+2.505*omega_m-2.114*omega_m*(w+1)+0.131*omega_m^2
              ;d = 0.405-2.588*(w+1)+2.450*(w+1)^2+2.612*omega_m-2.313*omega_m*(w+1)-0.722*omega_m^2
              d = -0.186+0.746*(w+1)-0.695*(w+1)^2 $
                  +4.682*omega_m-13.940*omega_m*(w+1)+10.907*omega_m*(w+1)^2 $
                  -2.311*omega_m^2+8.893*omega_m^2*(w+1)-8.304*omega_m^2*(w+1)^2
              z = z_array
              big_delta_c = bdelta_eds*(1+(a+b*z)/(1+c*z+d*z^2))
            END
     -0.3 : BEGIN
              a = -0.903+0.342*(w+1)+0.315*(w+1)^2+0.892*omega_m-0.390*omega_m*(w+1)-0.082*omega_m^2
              ;b = 0.480-1.120*(w+1)+0.308*(w+1)^2-0.177*omega_m+0.489*omega_m*(w+1)-0.014*omega_m^2
              b =  0.455-1.097*(w+1)+0.446*(w+1)^2 $
                  -0.074*omega_m+0.112*omega_m*(w+1)+0.081*omega_m*(w+1)^2 $
                  -0.103*omega_m^2+0.455*omega_m^2*(w+1)-0.368*omega_m^2*(w+1)^2
              c = -0.827+2.453*(w+1)-1.400*(w+1)^2+2.438*omega_m-2.049*omega_m*(w+1)+0.167*omega_m^2
              ;d = 0.440-2.447*(w+1)+2.170*(w+1)^2+2.307*omega_m-2.121*omega_m*(w+1)-0.513*omega_m^2
              d = -0.171+0.696*(w+1)-0.381*(w+1)^2 $
                  +4.218*omega_m-12.006*omega_m*(w+1)+8.115*omega_m*(w+1)^2 $
                  -1.883*omega_m^2+7.126*omega_m^2*(w+1)-5.905*omega_m^2*(w+1)^2
              z = z_array
              big_delta_c = bdelta_eds*(1+(a+b*z)/(1+c*z+d*z^2))
            END
     -0.4 : BEGIN
              a = -0.933+0.379*(w+1)+0.330*(w+1)^2+0.884*omega_m-0.395*omega_m*(w+1)-0.077*omega_m^2
              ;b = 0.593-1.327*(w+1)+0.334*(w+1)^2-0.208*omega_m+0.545*omega_m*(w+1)-0.009*omega_m^2
              b =  0.580-1.405*(w+1)+0.612*(w+1)^2 $
                  -0.145*omega_m+0.431*omega_m*(w+1)-0.222*omega_m*(w+1)^2 $
                  -0.068*omega_m^2+0.278*omega_m^2*(w+1)-0.188*omega_m^2*(w+1)^2
              c = -1.017+2.728*(w+1)-1.415*(w+1)^2+2.407*omega_m-1.933*omega_m*(w+1)+0.168*omega_m^2
              ;d = 0.538-2.377*(w+1)+1.974*(w+1)^2+1.919*omega_m-1.961*omega_m*(w+1)-0.264*omega_m^2
              d = -0.078+0.204*(w+1)+0.758*(w+1)^2 $
                  +3.668*omega_m-9.415*omega_m*(w+1)+3.743*omega_m*(w+1)^2 $
                  -1.440*omega_m^2+5.086*omega_m^2*(w+1)-2.690*omega_m^2*(w+1)^2
              z = z_array
              big_delta_c = bdelta_eds*(1+(a+b*z)/(1+c*z+d*z^2))
            END
     -0.5 : BEGIN
              a = -0.944+0.399*(w+1)+0.353*(w+1)^2+0.836*omega_m-0.386*omega_m*(w+1)-0.051*omega_m^2
              b = 0.684-1.491*(w+1)+0.338*(w+1)^2-0.205*omega_m+0.572*omega_m*(w+1)-0.017*omega_m^2
              c = -1.363+3.050*(w+1)-1.435*(w+1)^2+2.758*omega_m-1.904*omega_m*(w+1)-0.051*omega_m^2
              d = 0.681-2.320*(w+1)+1.810*(w+1)^2+1.469*omega_m-1.833*omega_m*(w+1)+0.008*omega_m^2
              z = z_array
              big_delta_c = bdelta_eds*(1+(a+b*z)/(1+c*z+d*z^2))
            END
     -0.6 : BEGIN
              a = -0.984+0.431*(w+1)+0.371*(w+1)^2+0.856*omega_m-0.386*omega_m*(w+1)-0.064*omega_m^2
              b = 0.803-1.656*(w+1)+0.344*(w+1)^2-0.274*omega_m+0.601*omega_m*(w+1)+0.017*omega_m^2
              c = -1.313+3.214*(w+1)-1.353*(w+1)^2+2.229*omega_m-1.778*omega_m*(w+1)+0.226*omega_m^2
              d = 0.664-2.380*(w+1)+1.679*(w+1)^2+1.478*omega_m-1.618*omega_m*(w+1)-0.019*omega_m^2
              z = z_array
              big_delta_c = bdelta_eds*(1+(a+b*z)/(1+c*z+d*z^2))
            END
     ELSE : BEGIN
             PRINT, 'BE CAREFUL : strange omega_c = 1-(omega_m+omega_q) value'
             big_delta_c = REPLICATE(bdelta_eds, N_ELEMENTS(z_array))
            END
  ENDCASE
ENDELSE
ENDELSE


RETURN, big_delta_c

closing:
END
