;+
; NAME:
;    FOF2DELTA
; ONE LINE HELP:
;    Convert the Friend-Of-Friend parameter b into an overdensity big_delta given a NFW profile
; PURPOSE:
;    Convert the Friend-Of-Friend parameter b into an overdensity big_delta given a NFW profile
; CALLING SEQUENCE:
;    FOF2DELTA, fof_b, big_delta_in, z_in, c_nfw, cosmo=cosmo, reverse=reverse
; INPUT:
;    fof_b: the FOF b parameter
;      OR
;    big_delta_in (If reverse is set): the average overdensity of the cluster (compared to rho_crit)
;    z_in: the cluster redshift
;    c_nfw: the NFW concentration parameter (useless when using the /gao keyword)
;            c_nfw=r200/rs where r200 is defined compared to rho_crit (see Navarro, Frenk and White 1996)
;            the procedure uses c_nfw=5 if not defined
;    mass_in: the cluster "FOF mass"
; OUTPUT:
;    big_delta_in: the average overdensity of the cluster (compared to rho_crit)
;      OR
;    fof_b (If reverse is set): the FOF b parameter
;    c_nfw: when using the /gao keyword; concentration corresponding to the input mass (following Gao et al. 2008)
; KEYWORDS:
;    cosmo: the structure containing the cosmological parameters
;    reverse: set this keyword if you want to compute b from big_delta
;    gao: determine the concentration value corresponding to the input mass (following Gao et al. 2008)
;    five_hundred : if set c_nfw is r500/rs and not r200/rs
; EXAMPLES:
;
; RESTRICTIONS:
;    Not fully tested yet. Report any bugs to Jean-Baptiste Melin
;    jean-baptiste.melin@cea.fr
;
; PROCEDURES CALLED:
;
; REVISION HISTORY
;    Written, Jean-Baptiste Melin, June 22 2006
;    bug in the isodensity contour definition fixed, JBM, June 30 2006
;    "compared to rho_crit" added in the header, JBM, June 30 2006
;    Bug fixed on the fof_b computation, Antoine Chamballu, December 4 2007
;    /gao keyword added (not for the /reverse case), AC, September 9 2008
;    five_hundred keyword added, JBM, December 11 2008
;-

PRO FOF2DELTA, fof_b, big_delta_in, z_in, c_nfw, mass_in, cosmo=cosmo, reverse=reverse, gao=gao, five_hundred=five_hundred

IF KEYWORD_SET(reverse) EQ 0 THEN BEGIN

   if keyword_set(gao) eq 0 then begin

      nclu = N_ELEMENTS(fof_b)

      IF NOT DEFINED(c_nfw) THEN BEGIN
        IF KEYWORD_SET(five_hundred) THEN BEGIN
          PRINT, 'NFW concentration parameter is not defined (c_nfw=r500/rs). I take c_nfw=3.3'
          c_nfw = REPLICATE(3.3, nclu)
        ENDIF ELSE BEGIN
          PRINT, 'NFW concentration parameter is not defined (c_nfw=r200/rs). I take c_nfw=5.'
          c_nfw = REPLICATE(5., nclu)
        ENDELSE
      ENDIF
      
      use_c_nfw = FLOAT(c_nfw)
      
      omega_m_z = cosmo.omega_m*(1+z_in)^3.*(HUBBLE_AT_Z(z_in, cosmo=cosmo)/100./cosmo.h)^(-2.)
      
      IF KEYWORD_SET(five_hundred) THEN BEGIN
        ; Compute the normalisation of the NFW profile given c_nfw=r500/rs
        delta_c = 500/3.*use_c_nfw^3/(ALOG(1+use_c_nfw)-use_c_nfw/(1+use_c_nfw))
      ENDIF ELSE BEGIN    
        ; Compute the normalisation of the NFW profile given c_nfw=r200/rs
        delta_c = 200/3.*use_c_nfw^3/(ALOG(1+use_c_nfw)-use_c_nfw/(1+use_c_nfw))
      ENDELSE

      xtab = FLTARR(nclu)
      
      FOR clu_num=0l, nclu-1 DO BEGIN
         
         roots = FZ_ROOTS([-delta_c[clu_num]*fof_b[clu_num]^3/omega_m_z[clu_num]*2.*!dpi/3., 1., 2., 1.])
         whr = WHERE(IMAGINARY(roots) EQ 0, nwhr)
         IF nwhr EQ 1 THEN BEGIN
            xtab[clu_num] = roots[whr]
         ENDIF ELSE BEGIN
            ; just in case...
            PRINT, 'Be careful ! Three real roots for x...'
         ENDELSE
      ENDFOR
      
      big_delta_in = delta_c*(ALOG(1+xtab)-xtab/(1+xtab))*3./xtab^3.

   endif else begin
      ; fit from Gao et al. 2008
      ztab = [0., 0.5, 1., 2., 3.]
      atab = [-0.138,-0.125,-0.092,-0.031,-0.004]
      btab = [2.646,2.372,1.891,0.985,0.577]
      
      a_in = INTERPOL(atab,ztab,z_in)
      b_in = INTERPOL(btab,ztab,z_in)
      
      delta_mass_out = 1d-3 ; 0.1% accuracy in mass computation
      
      nclu = n_elements(mass_in)
      if not defined(c_nfw) then c_nfw = fltarr(nclu)
      if not defined(big_delta_in) then big_delta_in = fltarr(nclu)
      delta_c_out = fltarr(nclu)

      omega_m_z = cosmo.omega_m*(1+z_in)^3.*(HUBBLE_AT_Z(z_in, cosmo=cosmo)/100./cosmo.h)^(-2.)

      for clu_num=0l, nclu-1 do begin ; loop over clusters

         old_mass_out = [1d-5]
         count = 0 
         while count ne 2 do begin ; find the good c for the given mass_in
            ; using fit from Gao et al. 2008 (except for initial value)
            if count eq 0 then use_c_200 = 5. else use_c_200 = 10^(a_in[clu_num]*ALOG10(old_mass_out*1d15)+b_in[clu_num])
            count = 1

            IF KEYWORD_SET(five_hundred) THEN BEGIN
              ; Compute the normalisation of the NFW profile given c_nfw=r500/rs
              delta_c = 500/3.*use_c_nfw^3/(ALOG(1+use_c_nfw)-use_c_nfw/(1+use_c_nfw))
            ENDIF ELSE BEGIN    
              ; Compute the normalisation of the NFW profile given c_nfw=r200/rs
              delta_c = 200/3.*use_c_nfw^3/(ALOG(1+use_c_nfw)-use_c_nfw/(1+use_c_nfw))
            ENDELSE

            roots = FZ_ROOTS([-delta_c*fof_b[clu_num]^3/omega_m_z[clu_num]*2.*!dpi/3., 1., 2., 1.])
            whr = WHERE(IMAGINARY(roots) EQ 0, nwhr)
            IF nwhr EQ 1 THEN BEGIN
               xtab = roots[whr]
            ENDIF ELSE BEGIN
               ; just in case...
               PRINT, 'Be careful ! Three real roots for x...'
            ENDELSE
      
            big_delta = delta_c*(ALOG(1+xtab)-xtab/(1+xtab))*3./xtab^3.

            fast_convert_mass_radius, mass_in[clu_num], z_in[clu_num], big_delta, 200., $
                                      use_c_200, r_in, r_200, M_200, cosmo=cosmo ; compute M_200
            
            if abs((old_mass_out-M_200)/old_mass_out) le delta_mass_out then count = 2
            old_mass_out = M_200
         endwhile
         
         c_nfw[clu_num] = use_c_200
         big_delta_in[clu_num] = big_delta
      endfor
   endelse

ENDIF ELSE BEGIN
      
   nclu = N_ELEMENTS(big_delta_in)
  
   IF NOT DEFINED(c_nfw) THEN BEGIN
      PRINT, 'NFW concentration parameter is not defined. I take c_nfw=5.'
      c_nfw = REPLICATE(5., nclu)
   ENDIF

   use_c_nfw = FLOAT(c_nfw)
   
   omega_m_z = cosmo.omega_m*(1+z_in)^3.*(HUBBLE_AT_Z(z_in, cosmo=cosmo)/100./cosmo.h)^(-2.)
  
   IF KEYWORD_SET(five_hundred) THEN BEGIN
     ; Compute the normalisation of the NFW profile given c_nfw=r500/rs
     delta_c = 500/3.*use_c_nfw^3/(ALOG(1+use_c_nfw)-use_c_nfw/(1+use_c_nfw))
   ENDIF ELSE BEGIN    
     ; Compute the normalisation of the NFW profile given c_nfw=r200/rs
     delta_c = 200/3.*use_c_nfw^3/(ALOG(1+use_c_nfw)-use_c_nfw/(1+use_c_nfw))
   ENDELSE

   xtab = FLTARR(nclu)

   ; x=r/rs
   x = (FINDGEN(1d5)+1)/1d3

   FOR clu_num=0, nclu-1 DO BEGIN

      ; Compte x_in=r_in/rs
      fx = big_delta_in[clu_num]/3.*x^3-delta_c[clu_num]*(ALOG(1+x)-x/(1+x))
      whp = WHERE(fx GE 0, nwhp)
      xtab[clu_num] = x[MIN(whp)]
      
   ENDFOR
   
   fof_b = (xtab*(1+xtab)^2*omega_m_z/delta_c*3*!pi/2)^(1/3.)
   
ENDELSE

END
