;FUNCTION SMALL_DELTA_C, z_array, cosmo=cosmo
;+
; NAME:
;    SMALL_DELTA_C
; ONE LINE HELP:
;    Compute the value of the critical linear density contrast delta_c at redshift z.
; PURPOSE:
;    Compute the value of the critical linear density contrast delta_c
;    at redshift z with a precision >0.1%
; CALLING SEQUENCE:
;    delta_c = SMALL_DELTA_C(z_array, cosmo=cosmo)
; INPUT:
;    z_array : an array of redshift values
;
; OUTPUT:
;    delta_c : the array of critical linear density contrast corresponding to z_array
;              (precision >0.1%)
; KEYWORDS:
;    cosmo : the structure containing the cosmological parameters
; EXAMPLE:
;    
; RESTRICTIONS:
;    Not fully tested yet. Report any bugs to Jacques Delabrouille
;    j.delabrouille@cdf.in2p3.fr
;    
;    BE CAREFUL : coded for LambdaCDM only !
;
; PROCEDURES CALLED:
;
; REVISION HISTORY
;    Written, Jean-Baptiste Melin, November 2004
;-

;-----------------------------------------------------------------
FUNCTION SUB_SMALL_DELTA_C_1, s
;------------------------------
  COMMON SUB_SMALL_DELTA_C_S, cosmo_s, zero_s
  fs = (s^2.*(HUBBLE_AT_Z(1./s-1.,cosmo=cosmo_s)/100./cosmo_s.h)^2.-zero_s/s)^(-1./2.)
  RETURN, fs
END  
;-----------------------------------------------------------------

;-----------------------------------------------------------------
FUNCTION SUB_SMALL_DELTA_C_2, s
;------------------------------
  COMMON SUB_SMALL_DELTA_C_S, cosmo_s, zero_s
  fs = s*(SUB_SMALL_DELTA_C_1(s))^(-2.)
  RETURN, fs
END  
;-----------------------------------------------------------------

;-----------------------------------------------------------------
FUNCTION SUB_SMALL_DELTA_C_3, s
;------------------------------
  COMMON SUB_SMALL_DELTA_C_S, cosmo_s, zero_s
  RETURN, (SUB_SMALL_DELTA_C_1(s))^3.
END  
;-----------------------------------------------------------------

FUNCTION SMALL_DELTA_C, z_array, cosmo=cosmo

IF ( N_PARAMS() EQ 0 ) THEN BEGIN
   print,'CALL IS : delta_c = SMALL_DELTA_C(z_array)'
   print,'Keyword : cosmo=cosmo'
   GOTO, closing
ENDIF

IF MAX(z_array) GT 11 THEN BEGIN
   print, 'MAX(z_array) must be lower than 10 !'
   GOTO, closing
ENDIF

omega_m = cosmo.omega_m*1d
lambda = cosmo.omega_de*1d
h = cosmo.h*1d
d_cosmo = DEFINE_COSMO(h=h, omega_m=omega_m, lambda=lambda, omega_b=cosmo.omega_b, /silent)

z_inf = 0.
z_sup = 10.
z_step = 0.1
z = (DINDGEN(CEIL((z_sup-z_inf)/z_step)+1)*z_step)+z_inf
n_z = N_ELEMENTS(z)

COMMON SUB_SMALL_DELTA_C_S, cosmo_s, zero_s

f_tab = DBLARR(n_z)
a_tab = 1./(1+z)
cosmo_s = d_cosmo
zero_s = 0.
FOR i=0, n_z-1 DO BEGIN
  f_tab[i] = QROMO('sub_small_delta_c_1',0.,a_tab[i],/MIDSQL,/DOUBLE)
ENDFOR

zi = 12d ;100d
hi = HUBBLE_AT_Z(zi, cosmo=d_cosmo)/100./h
omegai = omega_m*(1+zi)^3./hi^2.
lambdai = lambda/hi^2.

deltai_min = 1d-1 ;1d-2
deltai_max = 1d2 ;1d0
step_log10 = 100.
log10_deltai = $
(FINDGEN(CEIL((ALOG10(deltai_max)-ALOG10(deltai_min))*step_log10)+1)/step_log10)+ALOG10(deltai_min)
deltai_tab = EXP(ALOG(10)*log10_deltai)
n_deltai = N_ELEMENTS(deltai_tab)
g_tab = DBLARR(n_deltai)
g_tab[*] = -1

FOR i=0, n_deltai-1 DO BEGIN

;print, i, n_deltai
deltai = deltai_tab[i]

cosmoi_s = DEFINE_COSMO(h=d_cosmo.h, omega_m=omegai*(1+deltai), lambda=lambdai, omega_b=d_cosmo.omega_b, /silent)

root = -1.
IF lambdai EQ 0 THEN BEGIN
  root = -omegai*(1+deltai)/(1-omegai*(1+deltai))
  zero_s = 0.
  cosmo_s = cosmoi_s
ENDIF ELSE BEGIN
  zero_s = 0.
  cosmo_s = cosmoi_s
  x = [1d-5,1d-4,1d-3]
  iroot = FX_ROOT(x,'sub_small_delta_c_2', /DOUBLE, STOP=1, TOL=1d-5)
  IF ABS(IMAGINARY(iroot)) EQ 0 THEN BEGIN
    root = DOUBLE(iroot)
    new_zero_s = SUB_SMALL_DELTA_C_2(root)
    zero_s = new_zero_s
  ENDIF
ENDELSE

IF root GT 0 THEN BEGIN
  xc = root/2.
  g_tab[i] = 2./hi*(QROMO('sub_small_delta_c_1',0.,xc, /MIDSQL, /DOUBLE)+ $
             QROMO('sub_small_delta_c_1',xc,root, /MIDSQU, /DOUBLE))
ENDIF

ENDFOR

whp = WHERE(g_tab GT 0, nwhp)
deltai_tab = deltai_tab[whp]
g_tab = g_tab[whp]

;plot, deltai_tab, g_tab, /xlog, /ylog
;oplot, deltai_tab, g_tab, psym=4

old_deltai_tab = deltai_tab
old_g_tab = g_tab
deltai_coll_min = DBLARR(n_z)
deltai_coll_max = DBLARR(n_z)

epsi = 1d4 ; precision requiered for deltai_coll

FOR j=0, n_z-1 DO BEGIN
 ;print, j, n_z
 whn = WHERE(old_g_tab-f_tab[j] LE 0, nwhn)
 deltai_min = old_deltai_tab[MIN(whn)-1]
 deltai_max = old_deltai_tab[MIN(whn)]
 step_log10 = epsi
 log10_deltai = $
 (FINDGEN(CEIL((ALOG10(deltai_max)-ALOG10(deltai_min))*step_log10)+1)/step_log10)+ALOG10(deltai_min)
 deltai_tab = EXP(ALOG(10)*log10_deltai)
 n_deltai = N_ELEMENTS(deltai_tab)
 g_tab = DBLARR(n_deltai)

 FOR i=0, n_deltai-1 DO BEGIN
   ;print, i, n_deltai
   deltai = deltai_tab[i]

   cosmoi_s = DEFINE_COSMO(h=d_cosmo.h, omega_m=omegai*(1+deltai), lambda=lambdai, omega_b=d_cosmo.omega_b, /silent)

   root = -1.
   IF lambdai EQ 0 THEN BEGIN
     root = -omegai*(1+deltai)/(1-omegai*(1+deltai))
     zero_s = 0.
     cosmo_s = cosmoi_s
   ENDIF ELSE BEGIN
     zero_s = 0.
     cosmo_s = cosmoi_s
     x = [1d-5,1d-4,1d-3]
     iroot = FX_ROOT(x,'sub_small_delta_c_2', /DOUBLE, STOP=1, TOL=1d-5)
     IF ABS(IMAGINARY(iroot)) EQ 0 THEN BEGIN
       root = DOUBLE(iroot)
       new_zero_s = SUB_SMALL_DELTA_C_2(root)
       zero_s = new_zero_s
     ENDIF
   ENDELSE

   IF root GT 0 THEN BEGIN
     xc = root/2.
     g_tab[i] = 2./hi*(QROMO('sub_small_delta_c_1',0.,xc, /MIDSQL, /DOUBLE)+ $
                QROMO('sub_small_delta_c_1',xc,root, /MIDSQU, /DOUBLE))
   ENDIF

 ENDFOR
 whn = WHERE(g_tab-f_tab[j] LE 0, nwhn)
 deltai_coll_min[j] = deltai_tab[MIN(whn)-1]
 deltai_coll_max[j] = deltai_tab[MIN(whn)]
ENDFOR

;FOR i=0, n_z-1 DO BEGIN
; print, i
; whn = WHERE(g_tab-f_tab[i] LE 0, nwhn)
; deltai_coll_min[i] = deltai_tab[MIN(whn)-1]
; deltai_coll_max[i] = deltai_tab[MIN(whn)]
; ;plot, deltai_tab, g_tab, /xlog, /ylog
; ;oplot, deltai_tab, g_tab, psym=4
; ;oplot, deltai_tab, REPLICATE(f_tab[i], N_ELEMENTS(deltai_tab)), line=5
; ;wait, 1.
;ENDFOR

;plot, z, deltai_coll_min, line=5
;oplot, z, deltai_coll_max, lin=1
;print, deltai_coll_max-deltai_coll_min

h_tab_min = FLTARR(n_z)
h_tab_max = FLTARR(n_z)
FOR i=0, n_z-1 DO BEGIN
  h_tab_min[i] = 3/5.*((1-omega_m-lambda)/omega_m+ $
                   (omega_m*(1+(1+zi)*deltai_coll_min[i])+lambda-1)/omega_m/(1+deltai_coll_min[i])^(2/3.))
  h_tab_max[i] = 3/5.*((1-omega_m-lambda)/omega_m+ $
                   (omega_m*(1+(1+zi)*deltai_coll_max[i])+lambda-1)/omega_m/(1+deltai_coll_max[i])^(2/3.))
ENDFOR

d_tab = DBLARR(n_z)
cosmo_s = d_cosmo
zero_s = 0.
FOR i=0, n_z-1 DO BEGIN
  d_tab[i] = 5*omega_m/2./a_tab[i]/sub_small_delta_c_1(a_tab[i])* $
             QROMO('sub_small_delta_c_3',0.,a_tab[i],/MIDSQL,/DOUBLE)
ENDFOR

delta_c_min = h_tab_min*d_tab
delta_c_max = h_tab_max*d_tab
delta_c = (delta_c_min+delta_c_max)/2.
eps = MAX((delta_c_max-delta_c_min)/delta_c)*100.

delta_c_array = INTERPOL(delta_c, z, z_array)

RETURN, delta_c_array

closing:
END
