;FUNCTION TINKER, delta_c, sigma, z, cosmo=cosmo, delta=delta, mean=mean
;
;+
; NAME:
;  JENKINS
; ONE LINE HELP:
;  give M/rho0*dNclu/(d_covolume*dln(sigma^(-1))) using the Tinker et al. 2008 fitting
; PURPOSE:
;  give M/rho0*dNclu/(d_covolume*dln(sigma^(-1))) using the Tinker et al. 2008 fitting
; CALLING SEQUENCE:
;  result = TINKER(delta_c, sigma, z, cosmo=cosmo, delta=delta)
;
; INPUT:
;  delta_c : the critical density contrast at redshift z
;  sigma : fluctuation amplitude for scale M and redshift z
;  z : the redshift of computation
;
; OUTPUT:
;  M/rho0*dNclu/(d_covolume*dln(sigma^(-1)))
;    
; KEYWORDS:
;  cosmo : the structure containing the cosmological parameters
;  delta : overdensity of the considered halo in rho_crit (default: 500)
;  mean : set this keyword if delta is given in rho_mean instead of rho_crit
; EXAMPLES:
;  
; RESTRICTIONS:
;  Not fully tested yet. Report any bugs to Jean-Baptiste Melin
;  jean-baptiste.melin@cea.fr
;
; PROCEDURES CALLED:
;  No procedure called 
; REVISION HISTORY
;  Written, Jean-Baptiste Melin, July 24 2009
;  spline interpolation & mass is now assumed to be in 500 rho_crit, July 29 2009
;-

; fitting formula given by Tinker et al. 2008

FUNCTION TINKER, delta_c, sigma, z, cosmo=cosmo, delta=delta, mean=mean

IF ( N_PARAMS() EQ 0 ) THEN BEGIN
    print,'  CALL IS : TINKER(delta_c, sigma, z)'
    print,'  KEYWORD : cosmo=cosmo, delta=delta, mean=mean'
    GOTO, closing
ENDIF

IF NOT KEYWORD_SET(delta) THEN use_delta_crit=500. ELSE use_delta_crit=delta

;use_delta_crit=200. ; to fix delta_crit to 200

IF NOT KEYWORD_SET(mean) THEN BEGIN
  omega_m_z = cosmo.omega_m*(1+z)^3.*(HUBBLE_AT_Z(z, cosmo=cosmo)/100./cosmo.h)^(-2.)
  use_delta_mean = use_delta_crit/omega_m_z
ENDIF ELSE BEGIN
  use_delta_mean = REPLICATE(use_delta_crit, N_ELEMENTS(z))
ENDELSE

;use_delta_mean=200. ; to fix delta_mean to 200

; Values given in Table 2
delta_mean = [200., 300., 400., 600., 800., 1200., 1600., 2400., 3200.]
ga_tab = [0.186, 0.200, 0.212, 0.218, 0.248, 0.255, 0.260, 0.260, 0.260]
a_tab = [1.47, 1.52, 1.56, 1.61, 1.87, 2.13, 2.30, 2.53, 2.66]
b_tab = [2.57, 2.25, 2.05, 1.87, 1.59, 1.51, 1.46, 1.44, 1.41]
c_tab = [1.19, 1.27, 1.34, 1.45, 1.58, 1.80, 1.97, 2.24, 2.44]

; Values given in Table 3
d2_ga_tab = [0.00, 0.50, -1.56, 3.05, -2.95, 1.07, -0.71, 0.21, 0.00]
d2_a_tab = [0.00, 1.19, -6.34, 21.36, -10.95, 2.59, -0.85, -2.07, 0.00]
d2_b_tab = [0.00, -1.08, 12.61, -20.96, 24.08, -6.64, 3.84, -2.09, 0.00]
d2_c_tab = [0.00, 0.94, -0.43, 4.61, 0.01, 1.21, 1.43, 0.33, 0.00]

ga0 = SPL_INTERP(ALOG10(delta_mean), ga_tab, d2_ga_tab, ALOG10(use_delta_mean))
a0 = SPL_INTERP(ALOG10(delta_mean), a_tab, d2_a_tab, ALOG10(use_delta_mean))
b0 = SPL_INTERP(ALOG10(delta_mean), b_tab, d2_b_tab, ALOG10(use_delta_mean))
c0 = SPL_INTERP(ALOG10(delta_mean), c_tab, d2_c_tab, ALOG10(use_delta_mean))

gaz = ga0*(1.+z)^(-0.14)
az = a0*(1+z)^(-0.06)
alpha = 10^(-(0.75/ALOG10(use_delta_mean/75.))^1.2)
bz = b0*(1+z)^(-alpha)

RETURN, gaz*((sigma/bz)^(-az)+1.)*EXP(-c0/sigma^2.)

closing:
END
