; Copyright 2009 ---- Jacques Delabrouille, CNRS

; This file is part of the Planck Sky Model.
;
; The Planck Sky Model is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; version 2 of the License.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY, without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model. If not, write to the Free Software
; Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

;+
; Generate a cmb dipole
;
; @categories pipe
;   
; @keyword test {in}{optional}{type=boolean} If set no computation is
; done, the routine only test the validity of the parameters it receives
; @keyword debug {in}{optional}{type=boolean} If set no catch on error
; done, subdirectory components/dipole is not removed when an error occurs
; 
; @history <p>22/09/2009: First version, Jacques Delabrouille </p>
;-

PRO DIPOLE_PIPE, test=test, debug=debug

COMMON sky, model
COMMON psm, param

  ;; error handling
  ;;----------------
  IF NOT KEYWORD_SET(debug) THEN BEGIN
     CATCH, error_status
     IF error_status NE 0 THEN BEGIN
        PSM_INFO_MESSAGE, 'An error was encountered while computing CMB dipole'
        CATCH, /CANCEL
        MESSAGE, !ERR_STRING
     ENDIF
  ENDIF 

  double = PRECISION_ISDOUBLE()
  bl = PSM_GET_BL(SKY_RESOLUTION(), lmax=1, double=double) 

  ;; File for writing dipole alm
  ;;-----------------------------
  dir = GETPSMDIR('dipole')
  IF NOT FILE_TEST(dir) THEN FILE_MKDIR, dir
  filename = param.names.dipole_basename + '_alm.fits'
  dipole_alm_file = dir + filename

  ;; Create dipole component structure
  ;;-----------------------------------
  PSM_ID_KEYGEN, dipole_id
  dipole = COMPONENT_STRUCT('cmb',name=param.names.dipole_basename,id=dipole_id)
  dipole.polarised = 0B
  dipole.e1.ampl.unit = model.units.cmb
  *dipole.e1.ampl.value = FILE_BASENAME(dipole_alm_file)
  *dipole.e1.numin.value = 0.
  *dipole.e1.numax.value = FLOAT('INF')

  dip = model.dipole

  CASE dip.model OF 
     'prediction': BEGIN
        compute = 1
        ampl = dip.ampl * CONVERSION_FACTOR('mK_CMB',model.units.cmb)
        glat = dip.glat
        glon = dip.glon
     END
     'generic': BEGIN
        compute = 1
        SET_PSM_SEEDCHAIN, 'cmb-dipole'
        rnd = RANDOMN(GETSEED(),3)
        ampl = (dip.ampl + dip.ampl_error * rnd[0]) * CONVERSION_FACTOR('mK_CMB',model.units.cmb)
        glat = (dip.glat + dip.glat_error * rnd[1])
        glon = (dip.glon + dip.glon_error * rnd[2])
     END
     ELSE: MESSAGE, 'Dipole model not allowed'
  ENDCASE

  IF compute THEN BEGIN
     IF double THEN BEGIN
        alm = DBLARR(3,2)
        dtor = !DPI/180D0
     ENDIF ELSE BEGIN
        alm = FLTARR(3,2)
        dtor = !dtor
     ENDELSE

     ;; set alm values
     ;;----------------
     alm[0,*] = 0D0
     alm[1,0] = ampl*SIN(glat*dtor) * bl[1] * SQRT(4D0*!dpi/3)     ;; normalisation SUM_m <Ylm^2> = (2l+1)/4pi
     alm[1,1] = 0D0
     alm[2,0] = ampl*COS(glat*dtor)*COS(glon*dtor)/SQRT(2D0) * bl[1] * SQRT(4D0*!dpi/3)
     alm[2,1] = ampl*COS(glat*dtor)*SIN(glon*dtor)/SQRT(2D0) * bl[1] * SQRT(4D0*!dpi/3)
     
     ;; apply window function if needed
     ;;---------------------------------
     CASE model.mapping.sky_pixwindow OF
        1: BEGIN
           result = HEALPIXWINDOW(model.mapping.pix.nside)
           alm[1,0] = alm[1,0] * result[1,0]
           alm[2,0] = alm[2,0] * result[1,0]
           alm[2,1] = alm[2,1] * result[1,0]
        END
        ELSE: MESSAGE, 'Case not implemented'
     ENDCASE

     ;; Write the alm file
     ;;--------------------
     WRITE_DIPOLE_ALM, alm, dipole_alm_file, dipole

     ;; Write the map file if needed
     ;;------------------------------
     CASE model.dipole.save_map OF
        'yes': BEGIN
           dir = GETPSMDIR('dipole')
           IF NOT FILE_TEST(dir) THEN FILE_MKDIR, dir
           filename = param.names.dipole_basename + '_map.fits'
           dipole_map_file = dir + filename
           PSM_ALM2MAP, dipole_alm_file, dipole_map_file, ordering='RING', hpx_nside=SKY_NSIDE(), double=double
        END
     ENDCASE

     ;; Make visu if needed
     ;;---------------------
     IF PSM_VISU() GT 0 THEN BEGIN
        CASE model.dipole.save_map OF
           'no': BEGIN
              dipole_map_file = PSM_TMPFILENAME(extension='.fits')
              PSM_ALM2MAP, dipole_alm_file, dipole_map_file, ordering='RING', hpx_nside=SKY_NSIDE(), double=double
              erasemap = 1B
           END
           'yes': erasemap = 0B
        ENDCASE
        
        CASE param.run.output_visu OF
           'screen': BEGIN
              VISU_MAP, dipole_map_file, title = 'CMB dipole', /moll
           END
           'png': BEGIN
              VISU_MAP, dipole_map_file, title = 'CMB dipole', /moll, png=GETPSMDIR('visu')+param.names.dipole_basename + '_map.png'
           END
           'ps': BEGIN
              VISU_MAP, dipole_map_file, title = 'CMB dipole', /moll, ps=GETPSMDIR('visu')+param.names.dipole_basename + '_map.ps'
           END
        ENDCASE
                
        IF erasemap THEN FILE_DELETE, dipole_map_file

     ENDIF

     ;; Erase the dipole structure (free memory and pointers)
     ;;-------------------------------------------------------
     SAVE_COMP, dipole
     PRTSTRUCT, dipole, name='DIPOLE', file=GETPSMDIR('dipole')+'dipole.txt'
     DESTROY_DATA, dipole

  ENDIF
     
END
