FUNCTION FIRB_NU_INU, nu, lagache1999=lagache1999, fixsen1998=fixsen1998, bestfit2012=bestfit2012, randomize=randomize, megajysr=megajysr, plot=plot, help=help

  IF ((KEYWORD_SET(help) NE 0) OR (N_PARAMS() NE 1)) THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'FIRB_NU_INU: function to compute nu.Inu of integrated FIRB emission, in W/m2/sr'
     PSM_INFO_MESSAGE, 'SYNTAX: result = FIRB_NU_INU(nu, lagache1999=, fixsen1998=, bestfit2012=, /randomize, /megajysr, /plot)'
     PSM_INFO_MESSAGE, 'Set /lagache1999 to use the model of Lagache et al., 1999'
     PSM_INFO_MESSAGE, 'Set /fixsen1998 to use the model of Fixsen et al., 1998'
     PSM_INFO_MESSAGE, 'Set /bestfit2012 to use the model that fits FIRAS and Spitzer data as of 2012'
     PSM_INFO_MESSAGE, 'Set /randomize to get a random SED that is compatible with FIRAS within errors in Fixsen et al. 1999'
     PSM_INFO_MESSAGE, 'Set /megajysr to get SED in MJy/sr.Hz instead of W/m2/sr'
     PSM_INFO_MESSAGE, 'Set /plot to plot the data and the various models'
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  ON_ERROR, 2

  DEFSYSV, '!phyconst', exists=exists
  IF NOT exists THEN DEFINE_PHYSICAL_CONSTANTS

  nu0 = 100 * (100 * !phyconst.c)

  lag = KEYWORD_SET(lagache1999)
  fix = KEYWORD_SET(fixsen1998)
  fit = KEYWORD_SET(bestfit2012)

  IF lag+fix+fit NE 1 THEN MESSAGE, 'Select only one of the lagache1999, fixsen1998, or bestfit2012 models'

  IF KEYWORD_SET(lagache1999) THEN model='lagache1999'
  IF KEYWORD_SET(fixsen1998) THEN model='fixsen1998'
  IF KEYWORD_SET(bestfit2012) THEN model='bestfit2012'

  CASE model OF

     ;; If lagache model selected
     ;;---------------------------
     'bestfit2012': BEGIN
        freq = [217e9,353e9,545e9,857e9,!phyconst.c/160e-6,!phyconst.c/100e-6]
        flux = [3.4e-2,.13,.37,.71, 0.77, 0.24]       ;MJy/sr
        dflux = [1.1e-2,.04,.12,.23,SQRT(0.12^2+0.04^2), SQRT(0.08^2+0.04^2)]
        nuref = freq[4]
        FIT_GREYBODY, freq, flux, amplitude, specind, temp, nuref=nuref, dflux=dflux
        a = GREYBODY(nu0,[nuref,amplitude,specind,temp]) / PLANCK_BNU(nu0, temp=temp, /megajysr)
        alpha = specind
     END
     'lagache1999': BEGIN
        a = 8.8e-5
        alpha = 1.4
        temp = 13.6
     END
     'fixsen1998': BEGIN
        a = 1.3e-5
        alpha = 0.64
        temp = 18.5
        IF KEYWORD_SET(randomize) NE 0 THEN BEGIN
           da = .4e-5
           dalpha = .12
           dtemp = 1.2
           ;; Draw a reasonable random realisation (cutting off the tails)
           ok = 0
           WHILE ok EQ 0 DO BEGIN
              randomdraw = RANDOMN(seed,3)
              ok = (randomdraw[0] GE -1) AND (ABS(randomdraw[1]) LE 2) AND (ABS(randomdraw[2]) LE 1)
           ENDWHILE
           a = a + da*randomdraw[0]
           alpha = alpha + dalpha*randomdraw[1]
           temp = temp + dtemp*randomdraw[2]
           ;; print, a, alpha, temp
        ENDIF 
     END
  ENDCASE
  f = a * (nu/nu0)^alpha * PLANCK_BNU(nu, temp=temp) * nu
  IF KEYWORD_SET(megajysr) NE 0 THEN f = f*1e20

  IF KEYWORD_SET(plot) NE 0 THEN BEGIN
     ff = (DINDGEN(490)+11)*1e10
     fix = FIRB_NU_INU(ff, /fixsen1998, megajysr=megajysr)
     lag = FIRB_NU_INU(ff, /lagache1999, megajysr=megajysr)
     fit = FIRB_NU_INU(ff, /bestfit2012, megajysr=megajysr)
     IF NOT KEYWORD_SET(megajysr) THEN units='W/M!U2!N/sr' ELSE units = 'Hz.MJy/sr'
     PLOT, [ff,ff,ff,nu]/1e9, [fix,lag,fit,f], /xlog, /ylog, /nodata, title='CIB SED nu.Inu', charsize=1.8, $
           xtitle='Frequency (GHz)', ytitle='nu.Inu ('+units+')', xthick=2, ythick=2, charthick=2
     OPLOT, ff/1e9, fix, color=GET_COLOR('cyan'), thick=2
     OPLOT, ff/1e9, lag, color=GET_COLOR('pink'), thick=2
     OPLOT, ff/1e9, fit, color=GET_COLOR('grey'), thick=2
     OPLOT, nu/1e9, f, psym=1, thick=2, symsize=1.5
     freq = [217e9,353e9,545e9,857e9,!phyconst.c/160e-6,!phyconst.c/100e-6]
     flux = [3.4e-2,.13,.37,.71, 0.77, 0.24] ;MJy/sr
     dflux = [1.1e-2,.04,.12,.23,SQRT(0.12^2+0.04^2), SQRT(0.08^2+0.04^2)]
     rescale=1 & IF NOT KEYWORD_SET(megajysr) THEN rescale=1d-20
     OPLOT, freq/1e9, flux*rescale*freq, psym=4, thick=2, symsize=2
     FOR i=0, N_ELEMENTS(freq)-1 DO OPLOT, [freq[i],freq[i]]/1e9, [flux[i]-dflux[i],flux[i]+dflux[i]]*rescale*freq[i]
  ENDIF

  RETURN, f
  
END
