PRO make_ff_map, halpha, dust, free_free, FREQ_GHZ= freq_ghz, F_D=f_d, LIMIT=limit, DUSTCOL = dustcol, EBVSCALE = ebvscale, T_e=T_e

; Procedure to take the raw Halpha map and convert free-free emission
; at a known frequency frequency in GHz (freq_ghz) 
; and assuming some kind of dust correction f_d (0 to 1)
; from the SFD extinction map at Halpha
;
; INPUTS: 
; 
; halpha        Sky map of H-Alpha intensity in Rayleighs
;
; dust          Sky map of 100um dust intensity in MJy/sr
;
; OPTIONAL INPUTS:
;
; freq_ghz 	Radio frequency (between 1 and 1000 GHz) with which
;		to convert the Halpha values to free-free emission (mK).
;		If this is set to 0, then no conversion is made
;
; F_D		The fraction of the total dust to be used in calculating
;		the Halpha absorption (0=no absorption, 1=full extragalactic
;		absorption). Default=0
;
; limit		Limit to which the dust absorption correction is made	
;		(in mag.)
;
; ebvscale      conversion factor from optical to R-band extinction
;               [WMAP use 2.2 in Bennett et al. 2003]
;
;
; KEYWORDS:
;
; dustcol       If set, the routine assumes that the input template map
;               given in the dust variable is the dust column density, and
;               defines the scaling factor appropriately. Otherwise the
;               template is assumed to be an extinction map E(B-V).
;
; OUTPUTS:
;
; free_free     Sky map of free-free emission at requested frequency [if given].
;

; Initial draft C. Dickinson (Jodrell Bank) April 2002.
; Modified by A.J. Banday (MPA) June 2002 for small error in computation of correction.
; Modified by A.J. Banday (MPA) August 2002 to use Ebv extinction map instead of
;          dust column density.
; Modified by M.A. Miville-Deschenes (IAS) July 2007 : add T_e keyword

; ------------------------------------------------------------------------

; check consistency of input sky maps
npix_ha = n_elements(halpha)
npix_dust = n_elements(dust)
IF (npix_dust ne npix_ha) THEN begin
  print, '*** MAPS HAVE DIFFERENT NSIDE !!! ***' 
  goto, Exit
ENDIF

; Extinction map E(B-V) or Dust Column density D(T)?
if keyword_set(dustcol) then begin
  scale = 0.0462 
endif else begin
  if n_elements(ebvscale) eq 0 then ebvscale = 2.51
  scale = ebvscale
endelse

; only alter good values
good_values = where( (halpha ne -1.6375e30) and (dust ne -1.6375e30) )

; Convert the dust intensity map into absorption at Halpha 
; following Dickinson et al. (2002) assuming T=18.3 K
; and knowing how much dust is in front of Halpha i.e. f_d
abs_mag = fltarr(npix_dust)  - 1.6375e30
abs_mag(good_values) = (Scale) * dust(good_values)

; apply a limit to the dust absorption if neccessary
if n_elements(limit) eq 0 then begin
  lim = double(1.0)
endif else begin
  lim=double(limit)
endelse

; convert to real absorption
if n_elements(f_d) gt 0 then begin
  f_d=double(f_d) 
;  print, '*** Dust fraction absorbing, f_d=', f_d
;  print, '*** Absorption cut-off limit    =', lim
endif else begin
  f_d = 0.0d0
;  print, ' '
;  print, '*** No dust absorption correction is made (f_d=0) ***'
endelse

abs_correct = fltarr(npix_dust) - 1.6375e30
abs_correct(good_values) = abs_mag(good_values) * f_d
vals = where(abs_correct gt lim,ct)
if(ct gt 0)then abs_correct[vals] = lim
abst = fltarr(npix_dust)  - 1.6375e30 
abst(good_values) = 10.0^(abs_correct(good_values) / 2.5)

; Multiply up the Halpha map for dust extinction 
halpha_correct = fltarr(npix_ha) - 1.6375e30
halpha_correct(good_values) = halpha(good_values) * abst(good_values)

; Convert the Halpha map to free-free at that frequency in units of mK
free_free = fltarr(npix_ha) - 1.6375e30
IF n_elements(freq_ghz) gt 0 THEN BEGIN
  tb_ha = r2ff(freq_ghz, T_e=T_e)
  free_free(good_values) = halpha_correct(good_values) * tb_ha 
ENDIF ELSE BEGIN
  free_free(good_values) = halpha_correct(good_values)
  print, '*** No conversion is made to free-free ***'
ENDELSE

; end of processing
;print, ' '
;print, '*** COMPLETED PROCESSING ***'
;print, ' '

Exit:
return
END









