; -----------------------------------------------------------------------------
;
; Copyright (C) 2009-2012
;   Jacques Delabrouille
;   Clive Dickinson
;   Marc-Antoine Miville-Deschenes
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	READ_GAL_PARAM
;
; PURPOSE:
;	Read from a table the specific parameters used for the
;	generation of the galaxy in the PSM, and store them in the
;	form of a (galaxy-specific) structure of parameters that
;	define the model used, the components included, the way they
;	are modelled, and where the data is written.
;
; CATEGORY:
;	Sky model (galaxy).
;
; CALLING SEQUENCE:
;	READ_GAL_PARAM, Table
;
; INPUTS:
;	Table:	A string array of pairs [keyword, value] of dim (n,2)
;	for a list of n keywords, read from the PSM configuration file.
;
; KEYWORD PARAMETERS:
;	CAREFULNESS:	Defines the carefulness used in handling
;	                missing or repeated keywords in the table 
;	                (see PSM_CAREFULNESS).
;	RESTORE:	If set, restore galactic parameters previously
;	                saved in the current output directory, instead
;	                of reading them in an input table.
;
; OUTPUTS:
;	The result is a structure that defines the galactic model and
;	its parameters.
;
; COMMON BLOCKS:
;	None
;
; RESTRICTIONS:
;       Report any bugs to Jacques Delabrouille
;       delabrouille@apc.univ-paris7.fr
;
; MODIFICATION HISTORY:
; 	20/04/2009:	Jacques Delabrouille and Marc-Antoine Miville Deschenes, First version.
; 	28/04/2009:	JD, Removed unnecessary intermediate 'model' structure.
; 	22/05/2009:	JD, Keyword for carefulness added, improved
; 	                handling of missing or repeated keywords. 
; 	18/09/2009:	JD, Now is a function + restore keyword added.
; 	18/12/2009:	JD, Restore globally all the sky model, not
; 	                components individually.
; 	01/06/2010:	JD, Change free-free model number for MAMD2009.
; 	                galaxy.
; 	06/07/2010:	JD, Added CO molecular line emission.
; 	28/03/2011:	JD, Small bug fix for model case where there is no galaxy'no_galaxy'.
; 	31/03/2011:	JD, Corrected small bug for the parameters of the CO model.
; 	06/05/2011:	JD and Clive Dickinson, added user-defined
; 	                composition for spinning dust emission law, +
; 	                7th component in the composition. 
; 	07/01/2012:	JD, doc and PSM license header.
;-
FUNCTION READ_GAL_PARAM, table, carefulness=carefulness, restore=restore

  ;; File where parameters will be written
  ;;---------------------------------------
  file = GETPSMDIR('private',/create)+'gal_parameters.sav'

  IF NOT KEYWORD_SET(restore) THEN BEGIN
     
     which_gal = {gal_model: 'no_galaxy'}  
     FILL_STRUCT, table, which_gal, tags=['GAL_MODEL'], carefulness=carefulness
     
     temppar = {i100:'SFDnoHII'}
     FILL_STRUCT, table, temppar, tags = tag_names(temppar), carefulness=carefulness
     CASE temppar.i100 OF
        'SFD':thdust=1 
        'SFDnoHII':thdust=4
        'FFP6-JD':thdust=5
     ENDCASE
     
     what_to_include = {include_synchrotron:'yes',include_freefree:'yes',include_spindust:'yes',include_thermaldust:'yes',include_co:'yes'}
     FILL_STRUCT, table, what_to_include, tags = tag_names(what_to_include), carefulness=carefulness
     
     smallscale = {gal_smallscale:'no'}
     CASE which_gal.gal_model OF
        'prediction': ;; do nothing
        'simulation': smallscale.gal_smallscale='yes'
        'no_galaxy': ;; do nothing
     ENDCASE
     
     ;; Galactic polarisation parameters
     ;;----------------------------------
     dummy = {gal_polar_model:'mamd2008'}
     FILL_STRUCT, table, dummy, tags=['gal_polar_model']

     CASE dummy.gal_polar_model OF
        'fauvet2011': BEGIN
           galpol = {gal_polar_model:dummy.gal_polar_model,gal_bfield_pitch_angle:-30.,gal_bfield_turb_ampl:0.2}
           FILL_STRUCT, table, galpol, tags=['gal_bfield_pitch_angle','gal_bfield_turb_ampl']
        END
        'mamd2008': BEGIN
           galpol = {gal_polar_model:dummy.gal_polar_model}
        END
     ENDCASE

     ;; Synchrotron parameters
     ;;------------------------
     IF ((what_to_include.include_synchrotron NE 'no') AND (which_gal.gal_model NE 'no_galaxy')) THEN BEGIN
        sync_emlaw = {synchrotron_emission_law:'powerlaw',synchrotron_index_model:'mamd2008',synchrotron_curv_freq:FLOAT('NAN'),synchrotron_curv_ampl:FLOAT('NAN')}
        FILL_STRUCT, table, sync_emlaw, tags=['synchrotron_emission_law','synchrotron_index_model']
        IF ((sync_emlaw.synchrotron_emission_law EQ 'curvpowerlaw') ) THEN BEGIN
           sync_emlaw.synchrotron_curv_freq = 23.
           sync_emlaw.synchrotron_curv_ampl = -0.3
           FILL_STRUCT, table, sync_emlaw, tags=['synchrotron_curv_freq','synchrotron_curv_ampl']
           ;; convert frequency from GHz to Hz
           sync_emlaw.synchrotron_curv_freq = sync_emlaw.synchrotron_curv_freq*1e9 
        ENDIF
        synchrotron = {model:0, smallscale:smallscale.gal_smallscale, emlaw:sync_emlaw}
     ENDIF ELSE BEGIN
        synchrotron = {model:0}
     ENDELSE

     ;; Free-free parameters
     ;;----------------------
     IF ((what_to_include.include_freefree NE 'no') AND (which_gal.gal_model NE 'no_galaxy')) THEN BEGIN
        freefree = {model:0, smallscale:smallscale.gal_smallscale, freefree_template:'mamd2008', freefree_e_temp:7000.}
        FILL_STRUCT, table, freefree, tags=['freefree_template','freefree_e_temp']
     ENDIF ELSE BEGIN
        freefree = {model:0}
     ENDELSE

     ;; Spinning dust parameters
     ;;--------------------------
     IF ((what_to_include.include_spindust NE 'no') AND (which_gal.gal_model NE 'no_galaxy')) THEN BEGIN
        spindust_emlaw = {spindust_emission_law:'dl98',composition:[0,0.962,0,0,0,0.038,0]}
        FILL_STRUCT, table, spindust_emlaw, tags=['spindust_emission_law']
        IF spindust_emlaw.spindust_emission_law EQ 'dl98composition' THEN BEGIN
           c = {spd_cnm:0.0,spd_wnm:0.962,spd_wim:0.0,spd_mol:0.0,spd_drk:0.0,spd_rn:0.038,spd_extra:0}
           FILL_STRUCT, table, c, tags=['SPD_CNM','SPD_WNM','SPD_WIM','SPD_MOL','SPD_DRK','SPD_RN','SPD_EXTRA']
           spindust_emlaw.composition = [c.spd_cnm,c.spd_wnm,c.spd_wim,c.spd_mol,c.spd_drk,c.spd_rn,c.spd_extra]
        ENDIF
        spindust = {model:0, smallscale:smallscale.gal_smallscale, emlaw:spindust_emlaw}
     ENDIF ELSE BEGIN
        spindust = {model:0}
     ENDELSE

     ;; Thermal dust parameters
     ;;-------------------------
     IF ((what_to_include.include_thermaldust NE 'no') AND (which_gal.gal_model NE 'no_galaxy')) THEN BEGIN
        thermaldust = {model:0, smallscale:smallscale.gal_smallscale, i100:temppar.i100, dust_intrinsic_pol:0.15}
        FILL_STRUCT, table, thermaldust, tags=['dust_intrinsic_pol']
        IF ((thermaldust.dust_intrinsic_pol LT 0) OR (thermaldust.dust_intrinsic_pol GT 1)) THEN MESSAGE, 'DUST_INTRINSIC_POL should be between 0 and 1'
     ENDIF ELSE BEGIN
        thermaldust = {model:0}
     ENDELSE

     ;; CO parameters
     ;;---------------
     IF ((what_to_include.include_co NE 'no') AND (which_gal.gal_model NE 'no_galaxy')) THEN BEGIN
        co = {model:0, smallscale:'no'}
     ENDIF ELSE BEGIN
        co = {model:0}
     ENDELSE
     
     galp = { galpol:galpol, $
              synchrotron:synchrotron, $
              freefree:freefree, $
              thermaldust:thermaldust, $
              spindust:spindust, $
              co:co $
            }
     
     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
     ;; Galaxy pipeline parameters
     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
     PSM_DEBUG_MESSAGE, 'Fix CO map and small scales for CO', routine='READ_GAL_PARAM', infolimit=2, stoplimit=3

     CASE which_gal.gal_model OF

        ;; NO GALAXY
        ;;-----------
        'no_galaxy': BEGIN
           galp.synchrotron.model = 0
           galp.freefree.model = 0
           galp.thermaldust.model = 0
           galp.spindust.model = 0
           galp.co.model = 0
           ;; galp.synchrotron.smallscale = 'no'
           ;; galp.freefree.smallscale = 'no'
           ;; galp.thermaldust.smallscale = 'no'
           ;; galp.spindust.smallscale = 'no'
           ;; galp.co.smallscale = 'no'
        END

     ;;    'mamd2009_curvsync': BEGIN
     ;;       galp.synchrotron.model = 7
     ;;       galp.freefree.model = 5
     ;;       galp.thermaldust.model = thdust
     ;;       galp.spindust.model = 4
     ;;    END 
     ;;   'mamd2009': BEGIN


        ;; GALAXY MODELED
        ;;----------------
        ELSE: BEGIN 

           ;; Synchrotron
           ;;-------------
           IF (what_to_include.include_synchrotron NE 'no') THEN BEGIN
              CASE synchrotron.emlaw.synchrotron_index_model OF
                 'mamd2008': BEGIN
                    galp.synchrotron.model = 6
                 END
                 'giardino2002': BEGIN
                    galp.synchrotron.model = 3
                 END
                 'uniform': BEGIN
                    galp.synchrotron.model = 5
                 END
                 ELSE: MESSAGE, 'Model not allowed for synchrotron spectral index'
              ENDCASE
           ENDIF

           ;; Free-free
           ;;-----------
           IF (what_to_include.include_freefree NE 'no') THEN BEGIN
              CASE freefree.freefree_template OF
                 'mamd2008': BEGIN
                    galp.freefree.model = 6
                 END
                 'wmap_mem': BEGIN
                    galp.freefree.model = 3
                 END
                 'dickinson_h_alpha': BEGIN
                    galp.freefree.model = 1
                 END
                 ELSE: MESSAGE, 'Model not allowed for free-free emission template'
              ENDCASE
           ENDIF

           ;; Thermal dust
           ;;--------------
           IF (what_to_include.include_thermaldust NE 'no') THEN BEGIN
              galp.thermaldust.model = thdust
           ENDIF

           ;; Spinning dust
           ;;---------------
           IF (what_to_include.include_spindust NE 'no') THEN BEGIN
              CASE galp.spindust.emlaw.spindust_emission_law OF
                 'dl98': galp.spindust.model = 4
                 'dl98composition': galp.spindust.model = 5
              ENDCASE
           ENDIF

           ;; CO lines
           ;;----------
           IF (what_to_include.include_co NE 'no') THEN BEGIN
              galp.co.model = 1
           ENDIF
           
        END 

     ENDCASE 
     
     ;; IF (which_gal.gal_model NE 'no_galaxy') THEN BEGIN
     ;;    IF what_to_include.include_synchrotron EQ 'no' THEN BEGIN
     ;;       galp.synchrotron.model = 0
     ;;       galp.synchrotron.smallscale = 'no'
     ;;    ENDIF
     ;;    IF what_to_include.include_freefree EQ 'no' THEN BEGIN
     ;;       galp.freefree.model = 0
     ;;       galp.freefree.smallscale = 'no'
     ;;    ENDIF
     ;;    IF what_to_include.include_spindust EQ 'no' THEN BEGIN
     ;;       galp.spindust.model = 0
     ;;       galp.spindust.smallscale = 'no'
     ;;    ENDIF
     ;;    IF what_to_include.include_thermaldust EQ 'no' THEN BEGIN
     ;;       galp.thermaldust.model = 0
     ;;       galp.thermaldust.smallscale = 'no'
     ;;    ENDIF
     ;;    IF what_to_include.include_co EQ 'no' THEN BEGIN
     ;;       galp.co.model = 0
     ;;       galp.co.smallscale = 'no'
     ;;    ENDIF      
     ;; ENDIF
     
     ;; Write the parameters
     ;;----------------------
     SAVE, filename=file, galp
  
  ENDIF ELSE BEGIN
     RESTORE, filename=file
  ENDELSE
  

RETURN, galp
END


