; -----------------------------------------------------------------------------
;
; Copyright (C) 2007-2012
;   Jonathan Aumont
;   Jacques Delabrouille
;   Francois-Xavier Desert
;   Lauranne Fauvet
;   Juan Macias-Perez
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	MAKE_TEMPLATES_POLAR
;
; PURPOSE:
;	Make templates of galactic synchrotron and dust intensity and
;	polarisation, based on a 3D model of the ISM and of the
;	galactic magnetic field.
;
; CATEGORY:
;	Galaxy
;
; CALLING SEQUENCE:
;	MAKE_TEMPLATES_POLAR, Nside, B_turb, A, P, Sync_ref, Dust_ref, G_sync, G_d
;
; INPUTS:
;	Nside:	Healpix nside for the output maps (warning, the code
;		is slow for nside > 32).
;
;	B_turb:	a realisation of the turbulent part of the galactic
;		magnetic field (a cube of 512^3 pixels).
;
;	A:	Relative amplitude of the turbulent field as compared
;		to the regular field (typically 0.2).
;
;	P:	Pitch angle of the spiral arms in degrees (typically 30 degrees).
;
; OPTIONAL INPUTS:
;	none
;	
; KEYWORD PARAMETERS:
;	none
;
; OUTPUTS:
;	sync_ref:	The reference synchrotron emission as
;		        modelled. It comes in the form of a structure.
;
;	dust_ref:	The reference dust emission as
;		        modelled. It comes in the form of a structure.
;
;	g_sync: 	The geometrical depolarisation factor for the synchrotron.
;
;	g_sync: 	The geometrical depolarisation factor for the dust.
;
;
; OPTIONAL OUTPUTS:
;	None
;
; COMMON BLOCKS:
;	None
;
; SIDE EFFECTS:
;	...
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;	...
;
; EXAMPLE:
;	... 
;
; MODIFICATION HISTORY:
; 	15/03/2012:	Lauranne Fauvet and Jacques Delabrouille,
; 	                First version, adapted from original code
; 	                described in Fauvet et al. 2011.
;-
PRO MAKE_TEMPLATES_POLAR, nside, B_turb, A, p, sync_ref, dust_ref, g_sync, g_d
  
  ;; Galaxy
  ;;--------
  kpc=1.
  dsun=8.*kpc

  ;; Map
  ;;-----
  npix = NSIDE2NPIX(nside)
  ip = LINDGEN(npix)
  pix2ang_ring,nside,ip,theta,phi
  l=phi
  b=!dpi/2.0-theta

  ;; Line of sight
  ;;---------------
  npts = 10000l                 ; number of points along LOS for regular magnetic field
  Rlos = 25.                    ; kpc
  los = dblarr(3,npts)
  n = dindgen(npts)/npts*Rlos

  ;; parameters of the regular magnetic field
  ;;------------------------------------------
  B0 = 1.0d0                    ; amplitude in microgauss (not important because of renormalisation later on)
  r0 = 7.1d0*kpc                ; 
  Beta = 1./tan(p)              
  ki0 = 22.4d0/!radeg
  ki_height_scale = 1.0d0*kpc   ; scale height (perpendicularly to galactic plane)

  ;; Array of magnetic field along the line of sight (will be updated for each LOS)
  Bnlt = dblarr(3,npts)

  ;; Electron density
  ;;------------------
  n0_e = 1.0
  n_e = dblarr(npts)
  ;;n_e_radial_scale=5.0*kpc
  n_e_radial_scale=8.0*kpc
  n_e_height_scale=1.0*kpc
  
  ;; Synchrotron Polarisation
  ;;--------------------------
  U_sync = dblarr(npix)
  Q_sync = dblarr(npix)
  gamma_sync = dblarr(npix)

  ;; Synchrotron Geometric suppression
  ;;-----------------------------------
  p_sync = 3.
  g_sync = dblarr(npix)
  PI_sync = (p_sync+1.)/(p_sync+7./3.)
  I_sync = dblarr(npix)
  
  ;; Synchrotron Q,U model
  ;;-----------------------
  Qratio=0.7

  U_sync_model=dblarr(npix)
  Q_sync_model=dblarr(npix)

  ;; Dust density
  ;;--------------
  n0_d = 1.
  n_d = dblarr(npts)
  n_d_radial_scale = 3.0*kpc
  n_d_height_scale = 0.1*kpc

  ;; Dust Polarisation
  ;;-------------------
  U_d = dblarr(npix)
  Q_d = dblarr(npix)
  gamma_d = dblarr(npix)

  ;; Dust Geometric suppression
  ;;----------------------------
  PI_d = 0.10
  g_d = dblarr(npix)
  I_d = dblarr(npix)

  ;; Dust Q,U model
  ;;----------------
  U_d_model = dblarr(npix)
  Q_d_model = dblarr(npix)

  Q_d_xav = Q_d
  U_d_xav = U_d
  I_d_xav = I_d
  Q_sync_xav = Q_d
  U_sync_xav =U_d
  I_sync_xav = I_d

  ;; Faraday rotation
  RM = dblarr(npix)



  ;;------------------- calculation ------------------------

  sync_ref = {i:0.0d0,q:0.0d0,u:0.0d0,p:0.0d0,gamma:0.0d0}
  dust_ref = replicate(sync_ref,npix)
  sync_ref = replicate(sync_ref,npix)
  
  for j=0l,npix-1 do begin      

 
     ;; Line of sight
     los(0,*)=dindgen(npts)/npts*Rlos*cos(b[j])*cos(l[j])
     los(1,*)=dindgen(npts)/npts*Rlos*cos(b[j])*sin(l[j])
     los(2,*)=dindgen(npts)/npts*Rlos*sin(b[j])
     
     ;; Magnetic field
        
     los_bfield_bss_turb,kpc,l[j],b[j],los,r0,B0,Beta,p,ki0,ki_height_scale,B_turb, A,Bnlt 

     ;; Synchrotron electron density
     ;; n_e=n0_e*exp(-sqrt(((los(0,*)-dsun)*(los(0,*)-dsun))+(los(1,*)*los(1,*)))/n_e_radial_scale)/(cosh(los(2,*)/kpc)*cosh(los(2,*)/n_e_height_scale))
     n_e = n0_e*exp(-sqrt((los(0,*)-dsun)*(los(0,*)-dsun)+(los(1,*)*los(1,*)))/n_e_radial_scale-sqrt(los(2,*)*los(2,*))/n_e_height_scale)
    
     ;; Synchrotron polarisation
     U_sync[j]=INT_TABULATED(n,(-2.)*n_e*Bnlt(1,*)*Bnlt(2,*),/double)
     Q_sync[j]=INT_TABULATED(n,n_e*(Bnlt(1,*)*Bnlt(1,*)-Bnlt(2,*)*Bnlt(2,*)),/double)
     gamma_sync[j]=0.5*Atan(-U_sync[j],Q_sync[j])
     
     ;; Synchrotron Geometric suppression
     I_sync[j]=INT_TABULATED(n,n_e*(Bnlt(1,*)*Bnlt(1,*)+Bnlt(2,*)*Bnlt(2,*)),/double)
     ;; g_sync[j]=(sqrt(Q_sync[j]*Q_sync[j]+U_sync[j]*U_sync[j]))/(PI_sync*I_sync[j])
  
     ;; Dust density
     n_d=n0_d*exp(-sqrt(((los(0,*)-dsun)*(los(0,*)-dsun))+(los(1,*)*los(1,*)))/n_d_radial_scale)/(cosh(los(2,*)/n_d_height_scale)*cosh(los(2,*)/n_d_height_scale))
    
     
     ;; Normalization of angles .....
     ;;------------------------------
     Bperpnorm = Bnlt(1,*)*Bnlt(1,*)+Bnlt(2,*)*Bnlt(2,*)
     lp = where( Bperpnorm eq 0.0d0, countperp)
     if countperp gt 0 then Bperpnorm[lp]=1.0d0
     Bnorm =    Bnlt(0,*)*Bnlt(0,*)+Bnlt(1,*)*Bnlt(1,*)+Bnlt(2,*)*Bnlt(2,*)
     ln = where( Bnorm eq 0.0d0, countn)
     if countn gt 0 then Bnorm[ln]=1.0d0
     cos2gamma=  (Bnlt(1,*)*Bnlt(1,*)-Bnlt(2,*)*Bnlt(2,*))/Bperpnorm
     sin2gamma = (-2.)*Bnlt(1,*)*Bnlt(2,*)/Bperpnorm
     ;; my2gamma =  Atan(-(-2.)*Bnlt(1,*)*Bnlt(2,*),Bnlt(1,*)*Bnlt(1,*)-Bnlt(2,*)*Bnlt(2,*))
     ;; cos2gamma = cos(my2gamma)
     ;; sin2gamma = sin(my2gamma)
     sinalpha = sqrt(1.0-Bnlt(0,*)*Bnlt(0,*)/Bnorm)
     ;; Changed by Xavier's suggestion
     dI_dust = n_d
     I_d_xav[j]=INT_TABULATED(n,dI_dust,/double)
     Q_d_xav[j]=INT_TABULATED(n,dI_dust*cos2gamma*sinalpha*sinalpha*sinalpha,/double)
     U_d_xav[j]=INT_TABULATED(n,dI_dust*sin2gamma*sinalpha*sinalpha*sinalpha,/double)
     g_d[j]=(sqrt(Q_d_xav[j]*Q_d_xav[j]+U_d_xav[j]*U_d_xav[j]))/(PI_d*I_d_xav[j])
     ;; Changed by Xavier's suggestion
     ;; sinalpha already included in dI_sync
     dI_sync = n_e*(Bnlt(1,*)*Bnlt(1,*)+Bnlt(2,*)*Bnlt(2,*))
     I_sync_xav[j]=INT_TABULATED(n,dI_sync,/double)
     Q_sync_xav[j]=INT_TABULATED(n,dI_sync*cos2gamma,/double)
     U_sync_xav[j]=INT_TABULATED(n,dI_sync*sin2gamma,/double)
     
     g_sync[j]=(sqrt(Q_sync_xav[j]*Q_sync_xav[j]+U_sync_xav[j]*U_sync_xav[j]))/(PI_sync*I_sync_xav[j])
    
  endfor

  sync_ref.i =  I_sync_xav
  sync_ref.q =  Q_sync_xav
  sync_ref.u =  U_sync_xav
  sync_ref.p = sqrt(sync_ref.q*sync_ref.q+sync_ref.u*sync_ref.u)/sync_ref.i
  sync_ref.gamma = 0.5*Atan(-sync_ref.u,sync_ref.q)
  
  dust_ref.i =  I_d_xav
  dust_ref.q =  Q_d_xav
  dust_ref.u =  U_d_xav
  dust_ref.p = sqrt(dust_ref.q*dust_ref.q+dust_ref.u*dust_ref.u)/dust_ref.i
  dust_ref.gamma = 0.5*Atan(-dust_ref.u,dust_ref.q) 
  
  
end
