FUNCTION dl98_spectrum, freq, composition=composition, nuref=nuref, amp=amp

;+
; NAME:
;    dl98_spectrum
; CATEGORY:
;    Planck Sky Model
; PURPOSE:
;    Computes spinning dust grain emission at given frequencies
;    using standard spinning dust models from Draine & Lazarian (1998)
; CALLING SEQUENCE:
;    vec=dl98_spectrum(freq, composition=composition, nuref=nuref)
; INPUT:
;    freq    = frequency at which calculation is to be done (GHz)
; OUTPUT:
;    vec     = spinning dust grain contribution at given pixels and
;              frequency (in MJy/sr)
; OPTIONAL INPUT:
;  Composition =   Fraction of each dust grain component consisting of
;                  CNM, WNM, WIM, MOL, DRK, RN, EXTRA based on the
;                  emissitivity per H.
;                  Default = [0.0,0.35,0,0.0,0,0,0]
;  nuref =  reference frequency at which the spectrum is normalized 
;             Default = 23 GHz
;
;  amp = amplitude of the spectrum at the reference frequency
;        Default = 1
;
; INPUT KEYWORDS:
;    None
; OUTPUT KEYWORDS:
;    None
; SIDE EFFECTS:
;    None
; EXAMPLES:
;     vec=dl98_spectrum([23,41,61,94.], nuref=23.)
; RESTRICTIONS:
;    The sky_model_init routine must have been executed before
; PROCEDURES CALLED:
;    None
; REVISION HISTORY:
;    Created by M.A. Miville-Deschenes (22/04/09) based on the 
;    dl98_spin_dust.pro function of C. Dickinson 
;    Possible bug fixed, linked to the fact that composition can have
;    either 6 or 7 elements, JD, 12/01/2012
;-
;------------------------------------------------------------------
   freq = float(freq)

   ;; DUST COMPOSITION
   ;;------------------
   IF (defined(composition) eq 1) THEN BEGIN
      IF GET_VERBOSITY() GE 2 THEN print, 'Composition is defined'
      IF ((n_elements(composition) NE 7) AND (n_elements(composition) NE 6)) THEN composition=[0.0,0.35,0,0.0,0,0,0]
   ENDIF
   IF (defined(composition) eq 0) THEN BEGIN
      composition = [0.0,0.35,0,0.0,0,0,0]
   ENDIF
   IF GET_VERBOSITY() GE 2 THEN IF (n_elements(composition) EQ 7) THEN print, 'Composition = ' + strcompress(string(composition, format='(f5.3,f5.3,f5.3,f5.3,f5.3,f5.3,f5.3)'))
   IF GET_VERBOSITY() GE 2 THEN IF (n_elements(composition) EQ 6) THEN print, 'Composition = ' + strcompress(string(composition, format='(f5.3,f5.3,f5.3,f5.3,f5.3,f5.3)'))
   
   ;; Get Draine & Lazarian spectra
   ;;-------------------------------
   DEFSYSV, '!sky_model', exists=exists
   IF exists THEN BEGIN
      dir = GETPSMDIR('psmsoft')+'datafiles/spindust/'
      okdl98 = PTR_VALID(!sky_model)
      IF okdl98 THEN okdl98 = DEFINED((*!sky_model).spinning_dust.dl98)
      ;; Frequencies
      ;;-------------
      IF okdl98 THEN BEGIN
         ok = PTR_VALID((*!sky_model).spinning_dust.dl98.jnu_freqs)
      ENDIF ELSE ok=0
      IF ok THEN $
         jnu_freqs = (*(*!sky_model).spinning_dust.dl98.jnu_freqs) $
      ELSE BEGIN
         file=dir+'emit4.jnu.cnm_a'
         READCOL, file, jnu_freqs, jnu_cnm, /silent
      ENDELSE
      ;; Cold neutral medium
      ;;---------------------
      IF okdl98 THEN BEGIN
         ok = PTR_VALID((*!sky_model).spinning_dust.dl98.jnu_cnm)
      ENDIF ELSE ok=0
      IF ok THEN $
         jnu_cnm = (*(*!sky_model).spinning_dust.dl98.jnu_cnm) $    
      ELSE BEGIN
         file=dir+'emit4.jnu.cnm_a'
         READCOL, file, jnu_freqs, jnu_cnm, /silent
      ENDELSE
      ;; Warm neutral medium
      ;;---------------------
      IF okdl98 THEN BEGIN
         ok = PTR_VALID((*!sky_model).spinning_dust.dl98.jnu_wnm)
      ENDIF ELSE ok=0
      IF ok THEN $
         jnu_wnm = (*(*!sky_model).spinning_dust.dl98.jnu_wnm) $   
      ELSE BEGIN
         file=dir+'emit4.jnu.wnm_a'
         READCOL, file, freq_wnm, jnu_wnm, /silent
      ENDELSE
      ;; Warm ionized medium
      ;;---------------------
      IF okdl98 THEN BEGIN
         ok = PTR_VALID((*!sky_model).spinning_dust.dl98.jnu_wim)
      ENDIF ELSE ok=0
      IF ok THEN $      
         jnu_wim = (*(*!sky_model).spinning_dust.dl98.jnu_wim) $
      ELSE BEGIN
         file=dir+'emit4.jnu.wim_a'
         READCOL, file, freq_wim, jnu_wim, /silent
      ENDELSE
      ;; Molecules
      ;;-----------
      IF okdl98 THEN BEGIN
         ok = PTR_VALID((*!sky_model).spinning_dust.dl98.jnu_mol)
      ENDIF ELSE ok=0
      IF ok THEN $      
         jnu_mol = (*(*!sky_model).spinning_dust.dl98.jnu_mol) $
      ELSE BEGIN
         file=dir+'emit4.jnu.mol_a'
         READCOL, file, freq_mol, jnu_mol, /silent
      ENDELSE
      ;; drk
      ;;-----
      IF okdl98 THEN BEGIN
         ok = PTR_VALID((*!sky_model).spinning_dust.dl98.jnu_drk)
      ENDIF ELSE ok=0
      IF ok THEN $      
         jnu_drk = (*(*!sky_model).spinning_dust.dl98.jnu_drk) $
      ELSE BEGIN
         file=dir+'emit4.jnu.drk_a'
         READCOL, file, freq_drk, jnu_drk, /silent
      ENDELSE
      ;; rn
      ;;----
      IF okdl98 THEN BEGIN
         ok = PTR_VALID((*!sky_model).spinning_dust.dl98.jnu_rn)
      ENDIF ELSE ok=0
      IF ok THEN $      
         jnu_rn = (*(*!sky_model).spinning_dust.dl98.jnu_rn) $  
      ELSE BEGIN
         file=dir+'emit4.jnu.rn_a'
         READCOL, file, freq_rn, jnu_rn, /silent
      ENDELSE
      ;; extra
      ;;-------
      ;; An extra component, for testing purposes
      IF okdl98 THEN BEGIN
         ok = PTR_VALID((*!sky_model).spinning_dust.dl98.jnu_extra)
      ENDIF ELSE ok=0
      IF ok THEN BEGIN      
         ;; jnu_extra = (*(*!sky_model).spinning_dust.dl98.jnu_extra)   
         PSM_DEBUG_MESSAGE, 'Fix this', infolimit=3, stoplimit=3, routine='DL98_SPECTRUM'
         file=dir+'emit4.jnu.extra_a'
         readcol,file,freq_extra,jnu_extra,/silent
      ENDIF ELSE BEGIN
         file=dir+'emit4.jnu.extra_a'
         READCOL, file, freq_extra, jnu_extra, /silent
      ENDELSE
   ENDIF ELSE BEGIN
      dir = GETPSMDIR('psmsoft')+'datafiles/spindust/'
      file=dir+'emit4.jnu.cnm_a'
      readcol,file,jnu_freqs,jnu_cnm,/silent
      file=dir+'emit4.jnu.drk_a'
      readcol,file,freq_drk,jnu_drk,/silent
      file=dir+'emit4.jnu.mol_a'
      readcol,file,freq_mol,jnu_mol,/silent
      file=dir+'emit4.jnu.wim_a'
      readcol,file,freq_wim,jnu_wim,/silent
      file=dir+'emit4.jnu.wnm_a'
      readcol,file,freq_wnm,jnu_wnm,/silent
      file=dir+'emit4.jnu.rn_a'
      readcol,file,freq_rn,jnu_rn,/silent
      file=dir+'emit4.jnu.extra_a'
      readcol,file,freq_extra,jnu_extra,/silent
   ENDELSE
   
; Make up the spinning dust and scale to give micro K 
   jnu_model = (composition[0]*jnu_cnm) + (composition[1]*jnu_wnm) + (composition[2]*jnu_wim) + $
              (composition[3]*jnu_mol) + (composition[4]*jnu_drk) + (composition[5]*jnu_rn) 
   IF DEFINED(jnu_extra) THEN jnu_model_extra = (composition[6]*jnu_extra)

; get the dust emissivity for the required frequency by interpolation
   model_dust = interpol(jnu_model,jnu_freqs,freq)
   IF DEFINED(jnu_extra) THEN model_dust = model_dust + interpol(jnu_model_extra,freq_extra,freq)

; normalise spectrum to reference frequency
  if keyword_set(nuref) then begin
     if not keyword_set(amp) then amp=1.
     ref = interpol(jnu_model, jnu_freqs, nuref)
     IF DEFINED(jnu_extra) THEN ref = ref + interpol(jnu_model_extra, freq_extra, nuref)
     model_dust = model_dust*amp/ref[0]
  endif

  RETURN, model_dust

END

;=====================================================

