; -----------------------------------------------------------------------------
;
; Copyright (C) 2009-2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	RESTORE_SKY_MODEL
;
; PURPOSE:
;	This procedure is an I/O tool for restoring to memory a saved
;	sky model, hiding the technical detail of the format of the
;	model representation, as well as the format and location of
;	the file saved. It uses at some point (either directly, or in
;	called procedures and functions) the PSM and SKY COMMON
;	parameters (which must be defined). The saved model must have
;	been previously written to file with SAVE_SKY_MODEL.
;
; CATEGORY:
;	SKY_MODEL
;
; CALLING SEQUENCE:
;	RESTORE_SKY_MODEL
;
; INPUTS:
;	None
;
; COMMON BLOCKS:
;	SKY:	model
;
; SIDE EFFECTS:
;	Replaces any existing current 'SKY' common block by the saved one.
;
; RESTRICTIONS:
;       Report any bugs to Jacques Delabrouille
;       delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;
; MODIFICATION HISTORY:
; 	29/10/2009:	Jacques Delabrouille, First version.
; 	18/12/2009:	JD, Added some verification steps,
; 	                specifically for the case of possible model updates.
; 	04/01/2010:	JD, Refinements in verifications.
; 	22/01/2011:	JD, Some cleaning.
; 	06/01/2012:	JD, doc and PSM license header.
;-
PRO RESTORE_SKY_MODEL, model, output_dir=output_dir

  file = GETPSMDIR('components', psm_output_dir=output_dir)+'sky_model.sav'
  ok = FILE_TEST(file)
  IF NOT ok THEN STOP
  RESTORE, file

  DEFSYSV, '!SKY_RESOLUTION', FLOAT(model.global.sky_resolution)

    IF PSM_CAREFULNESS() GE 1 THEN BEGIN
       ;; Component parameters
       ;;----------------------
       dipole = READ_DIPOLE_PARAM(carefulness=psm_carefulness(), /restore)
       cmbp = READ_CMB_PARAM(carefulness=psm_carefulness(), /restore)
       galp = READ_GAL_PARAM(carefulness=psm_carefulness(), /restore)
       szp = READ_SZ_PARAM(carefulness=psm_carefulness(), /restore)
       psp = READ_PS_PARAM(carefulness=psm_carefulness(), /restore)
       firbp = READ_FIRB_PARAM(carefulness=psm_carefulness(), /restore)
       zlep = {zle_model:'no_zle'}
       
       parameters = {global:model.global, mapping:model.mapping, cosmo:model.cosmo, dipole:dipole, cmb:cmbp, gal:galp, sz:szp, ps:psp, firb:firbp, zle:zlep, units:model.units}

       isok = STRUCT_EQUAL(model,parameters)
       IF NOT isok THEN BEGIN
          PSM_INFO_MESSAGE, 'Saved sky model incompatible with individual components'
          IF PSM_CAREFULNESS() GE 2 THEN STOP
       ENDIF
       model=parameters
       
    ENDIF


END
