; -----------------------------------------------------------------------------
;
; Copyright (C) 2007
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;     SKY_ROTATION_MATRIX
; PURPOSE:
;     Computes the matrix to transform between Galactic, celestial, and ecliptic coordinates.
; EXPLANATION:
;     Uses EULER.PRO from ASTRON 
;
; CALLING SEQUENCE:
;     result = SKY_ROTATION_MATRIX(select, fk4=fk4, double=double)
;
; INPUTS:
;      SELECT - Integer (1-6) specifying type of coordinate transformation.  
;
;      SELECT   From          To        |   SELECT      From            To
;       1     RA-Dec (2000)  Galactic   |     4       Ecliptic      RA-Dec    
;       2     Galactic       RA-DEC     |     5       Ecliptic      Galactic  
;       3     RA-Dec         Ecliptic   |     6       Galactic      Ecliptic  
;
;      Celestial coordinates (RA, Dec) should be given in equinox J2000 
;      unless the /FK4 keyword is set.
; OUTPUTS:
;      matrix : a 3x3 rotation matrix
;      matrix#[x,y,z] gives the coordinates of the vector in the new coordinate system
;
; INPUT KEYWORD:
;      /FK4 - If this keyword is set and non-zero, then input and output 
;             celestial and ecliptic coordinates should be given in equinox 
;             B1950.
; NOTES:
;       
; REVISION HISTORY:
;       Written J. Delabrouille,  December 2007
;-
FUNCTION SKY_ROTATION_MATRIX, select, fk4=fk4, double=double

IF NOT KEYWORD_SET(double) THEN isdouble=1. ELSE isdouble=1d0
matrice = FLTARR(3,3)*isdouble

;spherical coordinates (longitude and latitude) of X and Y and Z 
x = [0.,0.]*isdouble
y = [90.,0.]*isdouble
z = [0.,90.]*isdouble

;compute matrix elements
EULER, x(0), x(1), lon, lat, select, fk4=fk4
lon = lon*!DTOR & lat=lat*!DTOR
matrice(0,0) = COS(lat)*COS(lon)
matrice(1,0) = COS(lat)*SIN(lon)
matrice(2,0) = SIN(lat)

EULER, y(0), y(1), lon, lat, select, fk4=fk4
lon = lon*!DTOR & lat=lat*!DTOR
matrice(0,1) = COS(lat)*COS(lon)
matrice(1,1) = COS(lat)*SIN(lon)
matrice(2,1) = SIN(lat)

EULER, z(0), z(1), lon, lat, select, fk4=fk4
lon = lon*!DTOR & lat=lat*!DTOR
matrice(0,2) = COS(lat)*COS(lon)
matrice(1,2) = COS(lat)*SIN(lon)
matrice(2,2) = SIN(lat)

RETURN, matrice
END
