; Copyright 2007, Jacques Delabrouille, Sam Leach, CNRS

; This file is part of the Planck Sky Model.
;
; The Planck Sky Model is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; version 2 of the License.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY, without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model. If not, write to the Free Software
; Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

;+
; Group all the postprocessing done to the Ultra Compact H-II region catalog provided by Giovanna Giardino, Grazia Umana, Roberta Paladini.
;   - Conversion of coordinates from degrees to radians
;   - Production of the ancillary data catalog
;   - Fit of the observed data with the sum of a powerlaw and a greybody
; 
; @categories ps
; -----------------------------------------------------------------------------
;
; Copyright (C) 2007-2012
;   Jacques Delabrouille
;   Marc Betoule
;   Guillaume Castex
;   Joaquin Gonzalez-Nuevo
;   Samuel Leach
;   Maude Le Jeune
;   Roberta Paladini
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; @param param {in}{required}{type=structure} Parameters of the PSM run
;
; @history <p>13/07/2007: First version, Jacques Delabrouille</p>
; @history <p>17/10/2007: Replacing call to obsolete str_sep.pro with strsplit.pro + making check on wh_keep to see if it is != -1, Sam Leach</p>
; @history <p>26/10/2007: Bug fixed in reading the input catalogue + add tag name for type in structure, Jacques Delabrouille</p>
; @history <p>22/04/2009: Major rewriting for PSM v1.7, Jacques Delabrouille</p>
; @history <p>30/06/2009: Bug fix in writing the parameters of the fit, Jacques Delabrouille & Guillaume Castex</p>
; @copyright Jacques Delabrouille, Sam Leach, CNRS, 2009
;-

PRO INTERPOLATE_HII_CATALOG

  COMMON psm, param

  runp = param.run
  names = param.names

;; Read the input catalog
;;------------------------
  ;; warning:
  ;; this is valid only for catalogue ultracompact_hii_regions_v2.0.txt
  ;; it assumes the ordering and number of columns used in that catalogue


  dir = GETPSMDIR('psmdata') 
  catfile = FIX_SEPARATOR(dir)+'ancillary/observations/HII-regions/ultracompact_hii_regions_v2.0.txt'
  
  ok = TEST_PSM_DATA(catfile,/get)

  line=''
  nsource=0
  ncomment=0
  GET_LUN, UNIT
  OPENR, UNIT, catfile
  WHILE NOT EOF(UNIT) DO BEGIN
     READF, UNIT, line
     line = STRCOMPRESS(line)
     line = STRTRIM(line,1)
     IF STRMID(line,0,1) EQ '#' THEN ncomment = ncomment+1
     IF STRMID(line,0,1) NE '#' THEN nsource = nsource+1
     IF nsource EQ 1 THEN BEGIN
        entries = STRSPLIT(line,' ',/extract)
        nentry = N_ELEMENTS(entries)
     ENDIF
  ENDWHILE
  CLOSE, UNIT
  entries = STRARR(nentry,nsource)
  s=0
  OPENR, UNIT, catfile
  WHILE NOT EOF(UNIT) DO BEGIN
     READF, UNIT, line
     IF (STRMID(line,0,1) NE '#') AND (STRLEN(STRTRIM(line,1)) NE 0) THEN BEGIN
        line = STRCOMPRESS(line)
        entries(*,s) = STRSPLIT(line,' ',/extract)
        s = s+1
     ENDIF
  ENDWHILE
  CLOSE, UNIT
  FREE_LUN, UNIT
  
  freqlist = [30,44,70,100,143,217,353,545,857,!phyconst.c/1e-4/1e9,!phyconst.c/6e-5/1e9,5.,5.,1.4]

;; Compact H-II regions are assumed unpolarised (because of beam averaging)
;;--------------------------------------------------------------------------
  polarised=0

;; Create an ancillary data catalogue from available observations (IRAS, GB6, Giveon et al. 2005, and NVSS)
;;----------------------------------------------------------------------------------------------------------

  ;; bands (assumed ideal Dirac)
  bands = 'DIRAC-'+['1.4','4.85','5']+'GHz'
  inst = LOAD_INSTRUMENT('IRAS_IDEAL')

  band_list = PTRARR(5,/allocate_heap)
  radiofreq = [1.4,4.85,5.]*1e9
  FOR i=0, 2 DO *band_list[i] = GET_BAND_STRUCT('DIRAC', nu_c=radiofreq[i])
  FOR i=3, 4 DO *band_list[i] = CHANNELPARAM(inst, 'BAND', i-2)

  DESTROY_DATA, inst

  newcat = CREATE_PS_DATA_CAT(nsource, band_list, name='UCHII_Ancillary_Cat', id=id, polarised=polarised)

  ;; Fill in flux values from the entries of the input file
  ;; NVSS, GB6, Giveon et al., IRAS 100-micron, IRAS 60-micron

  indexbands = [16,14,15,12,13]                       

  FOR obsnum = 1, 3 DO BEGIN
     ok = EXECUTE("*newcat.obs"+NUMBER2STRING(obsnum)+".iflux = REFORM(FLOAT(entries[indexbands[obsnum-1],*]))")            ; already in mJy (THIS IS ASSUMED LATER ON)
     ok = EXECUTE("*newcat.obs"+NUMBER2STRING(obsnum)+".ierr = REFORM(FLOAT(entries[indexbands[obsnum-1],*])) * .1")        ; assume 10% error
     ok = EXECUTE("newcat.obs"+NUMBER2STRING(obsnum)+".unit = 'mJy'")
  ENDFOR
  FOR obsnum = 4, 5 DO BEGIN
     ok = EXECUTE("*newcat.obs"+NUMBER2STRING(obsnum)+".iflux = REFORM(FLOAT(entries[indexbands[obsnum-1],*])) * 1000.")    ; converted from Jy to mJy (THIS IS ASSUMED LATER ON)
     ok = EXECUTE("*newcat.obs"+NUMBER2STRING(obsnum)+".ierr = REFORM(FLOAT(entries[indexbands[obsnum-1],*])) * 100.")      ; assume 10% error
     ok = EXECUTE("newcat.obs"+NUMBER2STRING(obsnum)+".unit = 'mJy'")
  ENDFOR
  

  ;; Fill in names and coordinates
  dtor = !DPI/180D0
  *newcat.objectname = REFORM(entries[0,*])
  *newcat.lon.value = REFORM(DOUBLE(entries(1,*))) * dtor
  *newcat.lat.value = REFORM(DOUBLE(entries(2,*))) * dtor

  
  ;; We will not model radio for sources with flux at 4.85 or 5 GHz lower than 0.5 Jy
  ;; nor with flux at 1.4 GHz lower than 1 Jy  
  wh = WHERE(*newcat.obs1.iflux LT 1000, nwh)
  IF nwh NE 0 THEN BEGIN
     (*newcat.obs1.iflux)[wh] = FLOAT('NAN')
     (*newcat.obs1.ierr)[wh] = FLOAT('NAN')
  ENDIF
  wh = WHERE(*newcat.obs2.iflux LT 500, nwh)
  IF nwh NE 0 THEN BEGIN
     (*newcat.obs2.iflux)[wh] = FLOAT('NAN')
     (*newcat.obs2.ierr)[wh] = FLOAT('NAN')
  ENDIF
  wh = WHERE(*newcat.obs3.iflux LT 500, nwh)
  IF nwh NE 0 THEN BEGIN
     (*newcat.obs3.iflux)[wh] = FLOAT('NAN')
     (*newcat.obs3.ierr)[wh] = FLOAT('NAN')
  ENDIF

  ;; Write the catalog if needed
  dir = GETPSMDIR('ancillary')
  dir_exists = FILE_TEST(dir, /directory)
  IF NOT dir_exists THEN FILE_MKDIR, dir
  filename = FIX_SEPARATOR(dir)+'uchii_ancil_cat.sav'
  WRITE_CATALOG, newcat, filename, /sav

  entries=0

;; Create the ultra compact H-II region model catalog
;;----------------------------------------------------
  
  ;; We assume that each source is described by the sum of a powerlaw and a greybody
  ;; The powerlaw has a spectral index around -2.15 (free-free emission from ionised gas)
  ;; The thermal emission has a spectral index of 1.5

  modelcat = CREATE_PS_MODEL_CAT(nsource, ['POWERLAW','GREYBODY'], name='strong'+names.uchii_basename, id=id, polarised=0)

  modelcat.e1.ampl.unit = 'mJy'
  modelcat.e2.ampl.unit = 'mJy'

  *modelcat.objectname = *newcat.objectname
  *modelcat.lon.value = *newcat.lon.value
  *modelcat.lat.value = *newcat.lat.value
  *modelcat.e1.nuref.value = 3e9

  *modelcat.e2.nuref.value = (*newcat.obs4.band).nu_c
  nuc = FLTARR(5)
  FOR k=1,5 DO ok = EXECUTE('nuc[k-1] = (*newcat.obs'+NUMBER2STRING(k)+'.band).nu_c')

  *modelcat.e1.numin.value = 0.
  *modelcat.e1.numax.value = FLOAT('INF')
  *modelcat.e2.numin.value = 0.
  *modelcat.e2.numax.value = FLOAT('INF')

  *modelcat.e1.ampl.value = FLTARR(modelcat.nsources)
  *modelcat.e1.specind.value = FLTARR(modelcat.nsources)
  *modelcat.e2.ampl.value = FLTARR(modelcat.nsources)
  *modelcat.e2.specind.value = FLTARR(modelcat.nsources)
  *modelcat.e2.temp.value = FLTARR(modelcat.nsources)

  windowcreated = 0B

  FOR s=0L, nsource-1 DO BEGIN
     sourceflux = FLTARR(5)
     sourceerr = FLTARR(5)
     FOR k=1,5 DO ok = EXECUTE('sourceflux[k-1] = (*newcat.obs'+NUMBER2STRING(k)+'.iflux)[s]')
     FOR k=1,5 DO ok = EXECUTE('sourceerr[k-1] = (*newcat.obs'+NUMBER2STRING(k)+'.ierr)[s]')
     whok = WHERE(FINITE(sourceflux),nwhok)
     nu = nuc(whok)
     flux = sourceflux(whok)
     err = sourceerr(whok)
     emission_names = ['POWERLAW','GREYBODY']
     par_name = ['EM1_'+EMLAW_PARAMLIST('POWERLAW'),'EM2_'+EMLAW_PARAMLIST('GREYBODY')]
     par = {value:0.,fixed:0B,limited:[0B,0B],limits:[0.,0.],parname:'',relstep:1e-3,mpside:0,mpmaxstep:0}
     parinfo = REPLICATE(par,7)
     IF MIN(nu) LE 1e10 THEN BEGIN    ; Data exists below 10 GHz 
        parinfo.value = [3e9, flux[0], -0.15, nuc[3], sourceflux[3], 1.5, 35.]
        parinfo.fixed = [1,0,1,1,0,1,0]
        truc=1
     ENDIF ELSE BEGIN                 ; Data does not exist below 10 GHz 
        parinfo.value = [3e9, 0., -0.15, nuc[3], sourceflux[3], 1.5, 35.]
        parinfo.fixed = [1,1,1,1,0,1,0]  
        truc=0     
     ENDELSE

     parms = MPFITFUN('PWLAW_AND_GREYBODY', nu, flux, err, ftol=1.e-8, parinfo=parinfo, status=status, /quiet)  ;
     
     (*modelcat.e1.ampl.value)[s] = parms[1]
     (*modelcat.e1.specind.value)[s] = parms[2]
     (*modelcat.e2.ampl.value)[s] = parms[4]
     (*modelcat.e2.specind.value)[s] = parms[5]
     (*modelcat.e2.temp.value)[s] = parms[6]
     
     IF 0 THEN BEGIN
     ;IF truc EQ 1 THEN BEGIN
     ;IF runp.visu GE 2 THEN BEGIN
        IF NOT windowcreated THEN BEGIN
           IF runp.output_visu NE 'ps' THEN WINDOW, /free, title = 'VISU H-II REGIONS -- DATA AND MODEL'
           windowcreated = 1B
        ENDIF
        vnu = (FINDGEN(10000)+1)*1e9
        PLOT, vnu/1e9, PWLAW_AND_GREYBODY(vnu,parms), /xlog, /ylog, $
              title = 'H-II region ' + (*modelcat.objectname)[s], xtitle='FREQUENCY (GHz)', ytitle='FLUX (mJy)'
        OPLOT, nu/1e9, flux, psym=4
        FOR k=0, N_ELEMENTS(nu)-1 DO OPLOT, [nu[k]/1e9,nu[k]/1e9], [flux[k]-err[k],flux[k]+err[k]]
        WAIT, .2
        ;;stop
     ENDIF
  ENDFOR

  PSM_DEBUG_MESSAGE, 'Implement visu for UCHII regions', infolimit=2, stoplimit=3

  PSM_DEBUG_MESSAGE, 'Some H-II regions are improperly modeled (radio data not fully compatible with model)', infolimit=2, stoplimit=3

;; Write the model catalog
;;-------------------------

  IF runp.verbosity GE 1 THEN PSM_INFO_SUBTITLE, "WRITING CATALOGUE OF ULTRACOMPACT H-II REGIONS"
  filename = GETPSMDIR('ps')+'strong'+names.uchii_basename+'.sav'
  WRITE_CATALOG, modelcat, filename, /sav, name='comp'
  IF runp.verbosity GE 1 THEN PSM_INFO_MESSAGE, "Catalogue of ultracompact H-II regions written: "+names.uchii_basename+'.sav'


;remove sources at galactic latitudes above 10 degrees
;these are unlikely to be H-II regions

;;   wh_keep = WHERE(ABS(multifreq_cat.glat_deg) LE 10.,nwh)
;;   if nwh ne 0 then begin
;;      multifreq_cat = multifreq_cat(wh_keep)
;;   endif


END


