; -----------------------------------------------------------------------------
;
; Copyright (C) 2007-2008, 2011
;   Jacques Delabrouille
;   Maude le Jeune
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
;-
PRO PSM_DATACAT2LS, datacatlist, lsfitsfile, overwrite=overwrite, help=help

  ncat = N_ELEMENTS(datacatlist)

  IF NOT DEFINED(lsfitsfile) THEN BEGIN
     IF ncat EQ 1 THEN lsfitsfile=PSM_STR_REPLACE(datacatlist[0],'.sav','-LSformat.fits') ELSE BEGIN
        PSM_INFO_MESSAGE, 'PSM_DATACAT2LS: Need to define the output catalogue name (no default if more than one input)'
        GOTO, closing
     ENDELSE
  ENDIF

  exists = FILE_TEST(lsfitsfile)
  IF exists THEN BEGIN
     IF NOT KEYWORD_SET(overwrite) THEN BEGIN
        PSM_INFO_MESSAGE, 'PSM_DATACAT2LS: output file already exists! Set /overwrite to erase it'
        GOTO, closing
     ENDIF ELSE BEGIN
        FILE_DELETE, lsfitsfile
     ENDELSE
  ENDIF


  ;; Create the catalogue table
  ;;----------------------------
  nfield = 29                   ; 2 + 9x3

  nsources = 0L
  FOR i=0, ncat-1 DO BEGIN & $
     datacat = datacatlist[i] & $
     READ_CATALOG, datacat, cat & $
     nsources = nsources+cat.nsources & $
     DESTROY_DATA, cat & $
  ENDFOR

  levelscat = FLTARR(nsources,nfield)
  allnames = STRARR(nsources)

  ;; Loop over catalogues
  ;;----------------------
  firstsource = 0               ; initialisation of the index of the first source for the next catalogue

  FOR i=0, ncat-1 DO BEGIN
     datacat = datacatlist[i]
     READ_CATALOG, datacat, cat
     IF cat.nobs NE 1 THEN MESSAGE, 'Case not implemented, sorry!'
     
     ;; Find channel name (e.g. '_30GHz')
     ;;-----------------------------------
     planck_frequencies='_'+['030','044','070','100','143','217','353','545','857']
     findfreq = INTARR(9)
     
     FOR j=0, 8 DO findfreq[j] = STR_MATCH(FILE_BASENAME(datacat),planck_frequencies[j])
     lsfreqnames='_'+['30','44','70','100','143','217','353','545','857']+'GHz'
     wh = WHERE(findfreq EQ 1, nwh)
     IF nwh NE 1 THEN MESSAGE, 'Unexpected frequency!' ELSE channelname = lsfreqnames(wh[0])
     
     ;; Write the catalogue information into useful arrays
     ;;----------------------------------------------------
     allnames[firstsource:firstsource+cat.nsources-1L] = *cat.objectname
     glon = *cat.lon.value * CONVERSION_FACTOR(cat.lon.unit,'deg')
     glat = *cat.lat.value * CONVERSION_FACTOR(cat.lon.unit,'deg')
     
     ;; Building also the corresponding location information in ecliptic coordinates
     ;;------------------------------------------------------------------------------
     EULER, glon, glat, elon, elat, 6
     theta_ecl = (90d - elat) * !dpi / 180d
     phi_ecl   = elon * !dpi / 180d
     
     ;; Computation of reference system rotation angle "alpha" for polarisation
     ;;-------------------------------------------------------------------------
     IF (cat.polarised EQ 1) THEN BEGIN
        
        ;; theta and phi in galactic coordinates
        theta_gal = (90d - glat) * !dpi / 180d
        phi_gal = glon * !dpi / 180d
        
        ;; rotation matrix to convert from galactic to ecliptic
        matr = SKY_ROTATION_MATRIX(6,/double)
        
        ;; components of spherical unit vectors in galactic coordinates
        e_thetagal = [[cos(theta_gal)*cos(phi_gal)],[cos(theta_gal)*sin(phi_gal)],[-sin(theta_gal)]]
        e_phigal = [[-sin(phi_gal)],[cos(phi_gal)],[REPLICATE(0.,N_ELEMENTS(phi_gal))]]
        e_thetagal = TRANSPOSE(temporary(e_thetagal))
        e_phigal = TRANSPOSE(temporary(e_phigal))
        xyz_ecl_of_e_thetagal = matr # e_thetagal
        xyz_ecl_of_e_phigal = matr # e_phigal
        
        ;; components of unit vector along theta in ecliptic coordinates
        e_thetaecl = [[cos(theta_ecl)*cos(phi_ecl)],[cos(theta_ecl)*sin(phi_ecl)],[-sin(theta_ecl)]]
        ;;e_phiecl = [[-sin(phi_ecl)],[cos(phi_ecl)],[REPLICATE(0.,N_ELEMENTS(phi_ecl))]]
        
        cosalpha = xyz_ecl_of_e_thetagal * TRANSPOSE(e_thetaecl)   
        cosalpha = TOTAL(cosalpha,1)
        sinalpha = xyz_ecl_of_e_phigal * TRANSPOSE(e_thetaecl)     
        sinalpha = -TOTAL(sinalpha,1)
        
        ;; angle between reference vector in ecliptic and in galactic (to be added to polarisation angle)
        ;; = angle position of the e_theta_gal vector in the e_theta_ecl, e_phi_ecl system 
        alpha = ATAN(sinalpha,cosalpha) 
        
        ;; polarised intensity p in PSM catalogue units and polarisation angle ang in radians
        QU2POL, *cat.obs1.qflux, *cat.obs1.uflux, p, ang 
        
        ;; polarisation fraction
        p = TEMPORARY(p) / *cat.obs1.iflux
        
        ;; polarisation angle in ecliptic coordinates (in radians)
        ang = TEMPORARY(ang) + alpha
        
        ;; conversion to degrees
        ang = TEMPORARY(ang) * CONVERSION_FACTOR('rad','deg')
        ang = (ang+360) MOD 180
        
     ENDIF ELSE BEGIN
        p = REPLICATE(0., cat.nsources)
        ang = REPLICATE(0., cat.nsources)
     ENDELSE
     
     ;; Coordinates
     ;;-------------
     levelscat[firstsource:firstsource+cat.nsources-1L,0] = theta_ecl
     levelscat[firstsource:firstsource+cat.nsources-1L,1] = phi_ecl
     
     ;; Flux in intensity and polarisation
     ;;------------------------------------
     i_col = 2+3*wh[0]
     a_col = i_col+1
     p_col = a_col+1
     
     IF cat.obs1.unit EQ 'mJy' THEN factor = 1/1000d0 ELSE MESSAGE, 'Input units not recognised'
     
     levelscat[firstsource:firstsource+cat.nsources-1L,i_col] = *cat.obs1.iflux * factor ; converted from mJy to Jy
     levelscat[firstsource:firstsource+cat.nsources-1L,a_col] = ang                      ; angle in degrees
     levelscat[firstsource:firstsource+cat.nsources-1L,p_col] = p * 100.                 ; multiplied by 100 to get percent values
     
     firstsource = firstsource + cat.nsources

     DESTROY_DATA, cat          ; Clean-up catalogue, freeing pointers
  ENDFOR

  ;; Tags and units
  ;;----------------
  tags = STRARR(nfield)
  units = tags
  tags[0] = 'theta_ecl'
  tags[1] = 'phi_ecl'
  units[0:1]   = ['rad','rad']
  FOR i=0, 8 DO BEGIN
     tags[2+i*3] = 'flux'+lsfreqnames[i]
     units[2+i*3] = 'Jy'
     tags[2+i*3+1] = 'polangle'+lsfreqnames[i]
     units[2+i*3+1] = 'deg'
     tags[2+i*3+2] = 'polpercent'+lsfreqnames[i]
     units[2+i*3+2] = 'percent'
  ENDFOR

  ;; Write the catalogue
  ;;---------------------

  ;; Minimal header
  WRITEFITS, lsfitsfile, 0
  h0 = HEADFITS(lsfitsfile)
  CHECK_FITS, image, h0, /UPDATE, /FITS
  
  ;; Update date to Y2K
  fdate = today_fits()
  SXADDPAR,h0,'DATE',fdate,' Creation date (CCYY-MM-DD) of FITS header'

  ;; Open the file, write the header and the image if any, and close it
  WRITEFITS, lsfitsfile, image, h0

  ;; Create the minimal extension header
  nrow=n_elements(levelscat[*,0])
  ncol=n_elements(levelscat[0,*])
  FXBHMAKE, xthdr, nrow

  ;; Update the header
  FOR i=0, ncol-1 DO FXBADDCOL_NOUPCASE, col, xthdr, levelscat[0,i], tags[i], TUNIT=units[i]

  ;; Write everything
  maxstringlength = max(strlen(allnames))
  format='(I'+strtrim(maxstringlength,2)+')'
  FXBADDCOL_NOUPCASE, col, xthdr, string('',format=format), 'source_name', TUNIT='none'
  sxaddpar, xthdr, 'EXTNAME', 'xtension', 'name of this binary table extension'
  objtype = 'catalog.LS_pointsource_catalog_pol'
  sxaddhist, "HIERARCH objType = '"+objtype+"'", xthdr, /blank

  xtn_id=2
  FXBCREATE, unit, lsfitsfile, xthdr, xtn_id
  
  FOR i=0L, ncol-1 DO FXBWRITM, unit,tags[i], levelscat[*,i]
  FXBWRITM, unit,'source_name', allnames  
  FXBFINISH, unit
  
  closing:
  IF DEFINED(cat) THEN DESTROY_DATA, cat

END
