; -----------------------------------------------------------------------------
;
; Copyright (C) 2009-2012
;   Jacques Delabrouille
;   Guillaume Castex
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	READ_PS_PARAM
;
; PURPOSE:
;	Read from a table the specific parameters used for the
;	generation of point sources in the PSM, and store them in the
;	form of a (PS-specific) structure of parameters that
;	define the model used, the components included, the way they
;	are modelled, and where the data is written.
;
; CATEGORY:
;	Sky model (point sources).
;
; CALLING SEQUENCE:
;	READ_PS_PARAM, Table
;
; INPUTS:
;	Table:	A string array of pairs [keyword, value] of dim (n,2)
;	for a list of n keywords, read from the PSM configuration file.
;
; KEYWORD PARAMETERS:
;	CAREFULNESS:	Defines the carefulness used in handling
;	                missing or repeated keywords in the table 
;	                (see PSM_CAREFULNESS).
;	RESTORE:	If set, restore galactic parameters previously
;	                saved in the current output directory, instead
;	                of reading them in an input table.
;
; OUTPUTS:
;	The result is a structure that defines the point source model and
;	its parameters.
;
; COMMON BLOCKS:
;	None
;
; RESTRICTIONS:
;       Report any bugs to Jacques Delabrouille
;       delabrouille@apc.univ-paris7.fr
;
; MODIFICATION HISTORY:
; 	06/02/2009:	Jacques Delabrouille, First version.
; 	28/04/2009:	JD, Removed unnecessary intermediate 'model' structure.
; 	29/04/2009:	JD, Directory and file names for ps included here now.
; 	19/05/2009:	Jean-Baptiste Melin, Bug in the no_ps model fixed.
; 	22/05/2009:	JD, Keyword for carefulness added, improved
; 	                handling of missing or repeated keywords. 
; 	18/09/2009:	JD, Now is a function + restore keyword added.
; 	15/11/2010:	JD, Changed way to handle separation between
; 	                faint and strong sources.
; 	22/04/2011:	Guillaume Castex, added prediction model.
; 	28/04/2011:	GC, added tags in preparation of ERCSC input.
; 	15/09/2011:	JD, changed some default poarameters.
; 	18/11/2011:	JD, Improved way to handle separation between
; 	                faint and strong sources (now at user-defined frequencies).
; 	07/01/2012:	JD, doc and PSM license header.
;-
FUNCTION READ_PS_PARAM, table, carefulness=carefulness, restore=restore


  ;; File where parameters will be written
  ;;---------------------------------------
  file = GETPSMDIR('private',/create)+'ps_parameters.sav'

  which_ps = {ps_model: 'no_ps'}  


  IF NOT KEYWORD_SET(restore) THEN BEGIN
     FILL_STRUCT, table, which_ps, tags=['PS_MODEL'], carefulness=carefulness
     
     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
     ;; PS pipeline parameters
     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

     limits = { $
              strong_ps_limit_freq_ghz:FLTARR(200), $ ;
              strong_ps_limit_flux_jy:FLTARR(200) $
              }
     tags = ['STRONG_PS_LIMIT_FREQ_GHZ','STRONG_PS_LIMIT_FLUX_JY']
     FILL_STRUCT, table, limits, tags=tags, carefulness=carefulness

     wh = WHERE(limits.strong_ps_limit_freq_ghz NE 0, nwh)
     IF nwh NE 0 THEN BEGIN
        limfreq = limits.strong_ps_limit_freq_ghz[wh]
        limflux = limits.strong_ps_limit_flux_jy[wh]
     ENDIF ELSE BEGIN
        limfreq = [20.,1000.]
        limflux = [0.1, 0.5]
     ENDELSE

     CASE which_ps.ps_model OF
        'simulation': BEGIN

           psp = { $
                 model:which_ps.ps_model, $               ;
                 include_radio_sources:'yes', $           ;
                 include_ir_sources:'yes', $              ;
                 include_uchii_sources:'yes', $           ;
                 include_cold_cores:'no', $               ;
                 include_wmap_sources:'no', $             ;
                 include_ercsc_sources:'no', $            ;
                 mean_ir_polar_degree:0.01, $             ;
                 strong_ps_limit_freq_ghz:limfreq, $      ;
                 strong_ps_limit_flux_jy:limflux, $       ;
                 ;limit_strong_radiops_jy_20GHz:.1, $      ;
                 ;limit_strong_irps_jy_1THz:.5, $          ;
                 use_real_sources:'yes' $                 ;
                 }
           tags = ['INCLUDE_RADIO_SOURCES','INCLUDE_IR_SOURCES','INCLUDE_UCHII_SOURCES','INCLUDE_COLD_CORES','INCLUDE_WMAP_SOURCES','INCLUDE_ERCSC_SOURCES', $
                   'MEAN_IR_POLAR_DEGREE']
           FILL_STRUCT, table, psp, tags=tags, carefulness=carefulness


        END
        'prediction': BEGIN

           psp = { $
                 model:which_ps.ps_model, $               ;
                 include_radio_sources:'no', $            ;
                 include_ir_sources:'yes', $              ;
                 include_uchii_sources:'yes', $           ;
                 include_cold_cores:'no', $               ;
                 include_wmap_sources:'no', $             ;
                 include_ercsc_sources:'no', $           ;
                 mean_ir_polar_degree:0.01, $             ;
                 strong_ps_limit_freq_ghz:limfreq, $      ;
                 strong_ps_limit_flux_jy:limflux, $       ;
                 ;limit_strong_radiops_jy_20GHz:.1, $      ;
                 ;limit_strong_irps_jy_1THz:.5, $          ;
                 use_real_sources:'yes' $                 ;
                 }
;           tags = ['INCLUDE_RADIO_SOURCES','INCLUDE_IR_SOURCES','INCLUDE_UCHII_SOURCES','INCLUDE_COLD_CORES', $
;                   'MEAN_IR_POLAR_DEGREE', 'LIMIT_STRONG_RADIOPS_JY_20GHZ', 'LIMIT_STRONG_IRPS_JY_1THZ']
           tags = ['INCLUDE_IR_SOURCES','INCLUDE_UCHII_SOURCES', $
                   ;;'INCLUDE_ERCSC_SOURCES', $
                   'MEAN_IR_POLAR_DEGREE']
           FILL_STRUCT, table, psp, tags=tags, carefulness=carefulness


        END
        'no_ps': BEGIN
           psp = { $
                 model:which_ps.ps_model $ ;
                 }
        END
        ELSE: MESSAGE, 'POINT SOURCE MODEL NOT ALLOWED'
     ENDCASE

     ;; Write the parameters
     ;;----------------------
     SAVE, filename=file, psp

  ENDIF ELSE BEGIN
     RESTORE, filename=file
  ENDELSE
     
  RETURN, psp

END
