; -----------------------------------------------------------------------------
;
; Copyright (C) 2001, 2004-2009
;   Jacques Delabrouille
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;  CLUSTER_CATALOG
; ONE LINE HELP:
;  Create a catalog of clusters using a mass_function
; PURPOSE:
;  Create a catalog of clusters using a mass_function (default : Press_Schechter function)
; CALLING SEQUENCE:
;  result = CLUSTER_CATALOG(solid_angle, M_inf, z_list=z_list, deltarho_rho=deltarho_rho,
;                           cosmo=cosmo, cluster=cluster, mass_function=mass_function, iseed=iseed, cm500=cm500)
; INPUT:
;  solid_angle : the total solid angle observed (steradians)
;  M_inf : the minimum mass of a cluster on the map in 10^15 solar mass units
;          M_inf must be greater than 0.01 (i.e. 10^13*M_sun)
;          and lower than 10 (i.e 10^16*M_sun)
;
; OUTPUT:
;  The catalog of clusters with
;      result[i].z : the redshift of the cluster i
;      result[i].mass : the mass (in 10^15 solar mass units) of the cluster i
;
; KEYWORDS:
;  z_list : a list of z values at which clusters will be located
;  deltarho_rho : average density contrast at the above z_list
;  cosmo : the structure containing the cosmological parameters.
;  cluster : the structure containing the structure formation parameters (cf. DEFINE_CLUSTER)
;  mass_function : the choosen mass function
;                   (e.g. Sheth-Tormen, Jenkins or Press-Schechter (default))
;  iseed : set this keyword to the value of the seed used for random generation
;  cm500 : the c_nfw_500 for the matter profile (needed if mass_function is Jenkins to convert M_Jenkins to Mvir)
; EXAMPLES:
;
; RESTRICTIONS:
;  Report any bugs to Jacques Delabrouille
;  delabrouille@apc.univ-paris7.fr
;
; PROCEDURES CALLED:
;  GROWTH_FACTOR, COVOLUME,
;  PK_SIGMA
; REVISION HISTORY
;  Written from PRESS_SCH_CATALOG, JBM, September 10 2001 
;      Written, Jean-Baptiste Melin April & May 2001
;      Made compatible with IDL 5.3 by using LONG instead of L64,
;      remove /double in call to RANDOMU, JD, may 2001
;      Fix bug in transverse velocity too small by factor SQRT(2), JD, may 2001
;      Remove velocities JD and JBM may 2001
;      Update header JD 29 may 2001
;      Masses in 10^15 solar mass units, JBM, May 30 2001
;      Keyword cosmo introduced, JBM, Sept 27 2001
;      Step in mass and redshift changed, JBM, October 1 2001
;      SMALL_DELTA_C included, JBM, November 24 2004
;      SMALL_DELTA_C removed and cluster structure added, JBM, February 2006
;      z_inf changed from 0.001 to 0.01, JBM, March 2007
;      cm500 keyword added, JBM, Dec 11 2008
;      mass conversion for tinker mass function added, JBM, July 29 2009
;-

FUNCTION CLUSTER_CATALOG, solid_angle, M_inf, $
                            z_list=z_list, deltarho_rho=deltarho_rho, $
                            cosmo=cosmo, cluster=cluster, mass_function = mass_function, iseed=iseed, cm500=cm500


IF ( N_PARAMS() EQ 0 ) THEN BEGIN
 PRINT,'  CALL IS : result = CLUSTER_CATALOG(solid_angle, M_inf)'
 PRINT,'  Keywords : cosmo=cosmo, cluster=cluster, mass_function=mass_function, iseed=iseed, cm500=cm500'
 PRINT,'  z_list=z_list, deltarho_rho=deltarho_rho'
 PRINT,'         (list of z and corresponding average drho/rho)'
 GOTO, closing0
ENDIF

IF NOT KEYWORD_SET(iseed) THEN iseed = LONG(systime(1))

IF DEFINED(M_inf) EQ 0 THEN M_inf = 0.1

IF (M_inf LT 0.01) THEN BEGIN
  print, 'M_inf is too small'
  GOTO, closing0
ENDIF

IF (M_inf GE 10) THEN BEGIN
  print, 'M_inf is too large'
  GOTO, closing0
ENDIF

; default mass_function
IF NOT KEYWORD_SET(mass_function) THEN BEGIN
  PRINT, 'Using the Press-Schechter function...'
  mass_function = 'press_schechter'
ENDIF

; choose the max and min values for z in the simulation
z_sup=5.
;z_inf = 0.001
z_inf = 0.01

IF NOT KEYWORD_SET(z_list) THEN BEGIN
	; chose the step for z
	delta_z = 0.01
	; z_array building
	z_array = (FINDGEN(CEIL((z_sup-z_inf)/delta_z)+1)*delta_z)+z_inf
        nb_elts_z_array = N_ELEMENTS(z_array)
        correction_factor = REPLICATE(1,nb_elts_z_array)
	delta_z = REPLICATE(delta_z,nb_elts_z_array)
ENDIF ELSE BEGIN
	wh = WHERE(z_list LE z_sup AND z_list GE z_inf)
	z_array = z_list(wh)
	correction_factor = deltarho_rho(wh)+1.
	order = SORT(z_array)
	z_array = z_array(order)
	correction_factor = correction_factor(order)
	keep = REPLICATE(1B,N_ELEMENTS(z_array))
	FOR i=1, N_ELEMENTS(z_array)-1 DO $
          IF z_array(i) EQ z_array(i-1) THEN keep(i)=0
	indices = WHERE(keep)
	z_array = z_array(indices)
	correction_factor = correction_factor(indices)
	nb_elts_z_array = N_ELEMENTS(z_array)
	delta_z = z_array(1:nb_elts_z_array-1) - z_array(0:nb_elts_z_array-2)
	delta_z = [delta_z,delta_z(nb_elts_z_array-2)]
ENDELSE

; compute the critical linear over-density
nb_elts_z_array = N_ELEMENTS(z_array)
;delta_c_array = DBLARR(nb_elts_z_array)
;delta_c_array[*] = 1.68 ; very small variations with z and the cosmology
;delta_c_array = SMALL_DELTA_C(z_array, cosmo=cosmo)
delta_c_array = INTERPOL(cluster.small_delta_c_array, cluster.z_array, z_array)

; compute the linear growth factor
GROWTH_FACTOR, z_array, D_g_array, f_array, cosmo=cosmo

;compute the covolumes
cov_array = COVOLUME(z_array, cosmo=cosmo)

; choose the step in log10 for M (e.g. 100. steps between 10^43 and 10^44)
step_log10 = 200.
; compute the step in ln for M
delta_lnM = ALOG(10)/step_log10
; max value for M in M_array
M_sup = 10. ; in 10^15 solar mass units
; compute the number of elements for M_array
log10_m = (FINDGEN(CEIL((ALOG10(M_sup)-ALOG10(M_inf))*step_log10)+1)/step_log10)+ALOG10(M_inf)
M_array = EXP(ALOG(10)*log10_m)
nb_elts_M_array = N_ELEMENTS(M_array)
; compute sigma and the slope for the M values of M_array
sigma_slope_array = DBLARR(2,nb_elts_M_array)
mass_func = DBLARR(nb_elts_z_array, nb_elts_M_array)

FOR i=0,nb_elts_M_array-1 DO BEGIN
  PK_SIGMA, M_array[i], sigma, alpha, cosmo=cosmo, cluster=cluster, /slope
  sigma_slope_array[0,i] = sigma
  sigma_slope_array[1,i] = alpha
  ; compute the mass function for z and mass
  mass_func[*,i] = CALL_FUNCTION(mass_function,delta_c_array,sigma_slope_array[0,i]*D_g_array, z_array, cosmo=cosmo)
ENDFOR

; format the result
max_cluster_number = 10000L + LONG(150*solid_angle/(!dtor)^2)
result = DBLARR(2,ROUND(max_cluster_number))
cluster_nb = 0

; compute rho0
Mpc2m = 3.0856775807d+22 ; in m
rho0 = Mpc2m^3*cosmo.omega_m*cosmo.h^2*1.88*1d-26 ; in kg/Mpc^3

M_sun = 1.9889*1d30 ; in kg
fact = rho0/(1d15*M_sun)*solid_angle

z_i = 0.001
z_s = 5. ; be careful redondancy with z_sup in DO_DENSITY_CONTRAT_BOX
z_st = 0.1
z_l = (FINDGEN(CEIL((z_s-z_i)/z_st)+1)*z_st)+z_i
nz = N_ELEMENTS(z_l)
fm = REPLICATE(1.,nz)
fz = REPLICATE(1.,nb_elts_M_array)
z = z_l#TRANSPOSE(fz)
m = fm#TRANSPOSE(M_array)

bias = GET_CLUSTERING_BIAS(m, z, cosmo=cosmo, cluster=cluster)

bias = REFORM(bias, nz, nb_elts_M_array)

cf = FLTARR(nb_elts_z_array, nb_elts_M_array)
b = FLTARR(nb_elts_z_array, nb_elts_M_array)

FOR i=0, nb_elts_M_array-1 DO BEGIN
  b = INTERPOL(bias[*,i],z_l,z_array)
  cf[*,i] = 1.+b*(correction_factor-1.)>0.
  ;if i/100 eq i/100. then print, i
ENDFOR

mf_limit = 1d-50

; create the catalog
FOR k = 0, nb_elts_z_array-1 DO BEGIN   ; loop on redshift
    i=0
    lim=10.
   WHILE (i LT nb_elts_M_array) AND (lim GT mf_limit) DO BEGIN ; loop on cluster masses
      ; compute the average number of clusters using the mass function
      lim = mass_func[k,i]*fact/M_array[i]* $
                   sigma_slope_array[1,i]*cov_array[k]*delta_z[k]*delta_lnM
      average_nb = lim
      average_nb = average_nb*cf[k,i] ;correction_factor[k]
       ;Pb in randomu(poisson) (underflow) if map and the are large too many clusters
       ;There are then too many loops but the result seems valid
      ;compute the real number of clusters using a Poisson law
      IF average_nb LE 0 THEN BEGIN
      	real_nb = 0
      ENDIF ELSE BEGIN
             IF !version.release GE 5.4 THEN BEGIN
	 res = EXECUTE('real_nb = RANDOMU(iseed, POISSON = average_nb,/double)')
	      ENDIF ELSE BEGIN
	      	real_nb = RANDOMU(iseed, POISSON = average_nb)
	      ENDELSE
      ENDELSE
      ; write the cluster caracteristics in the result array
      IF (real_nb GE 1) THEN BEGIN
        FOR j=0,real_nb-1 DO BEGIN
          IF (cluster_nb GT max_cluster_number) THEN BEGIN
            print, 'More than'
            print, STRING(max_cluster_number)
            print, 'clusters'
            GOTO, closing1
          ENDIF
          result[0,cluster_nb+j] = z_array[k]
          result[1,cluster_nb+j] = M_array[i]
        ENDFOR
      ENDIF
      ; increase the number of clusters
      cluster_nb = cluster_nb + real_nb
    i=i+1
    ENDWHILE
ENDFOR
closing1:
cluster_nb = MIN([ROUND(cluster_nb), ROUND(max_cluster_number)])
IF (cluster_nb GT 0) THEN BEGIN
amas = {name:'',z:0.,mass:0.}
catalog = REPLICATE(amas,cluster_nb)
catalog.z = REFORM(result[0,0:cluster_nb-1])
mass = REFORM(result[1,0:cluster_nb-1])

IF (mass_function EQ 'jenkins') THEN BEGIN
  ; mass is mass_jenkins
  ; It has be converted to mass_vir
  IF KEYWORD_SET(cm500) THEN BEGIN
    nz = N_ELEMENTS(cluster.z_array)
    fof_b = REPLICATE(0.2, nz)
    c_nfw = REPLICATE(cm500, nz)
    FOF2DELTA, fof_b, big_delta_in_z, cluster.z_array, c_nfw, cosmo=cosmo, /five_hundred
    big_delta_in = INTERPOL(big_delta_in_z, cluster.z_array, catalog.z)
    big_delta_out = INTERPOL(cluster.big_delta_c_array, cluster.z_array, catalog.z)
    c_nfw = REPLICATE(cm500, cluster_nb)
    FAST_CONVERT_MASS_RADIUS, mass, catalog.z, big_delta_in, big_delta_out, c_nfw, $
                              r_in, r_out, mass_out, $
                              cosmo=cosmo, /five_hundred
    catalog.mass = mass_out
  ENDIF ELSE BEGIN
    ; cm500 not given, no conversion
    catalog.mass = mass
  ENDELSE
ENDIF ELSE BEGIN
  IF (mass_function EQ 'tinker') THEN BEGIN
    ; mass is mass500crit
    ; It has be converted to mass_vir
    IF KEYWORD_SET(cm500) THEN BEGIN
      big_delta_in = REPLICATE(500., cluster_nb)
      big_delta_out = INTERPOL(cluster.big_delta_c_array, cluster.z_array, catalog.z)
      c_nfw = REPLICATE(cm500, cluster_nb)
      FAST_CONVERT_MASS_RADIUS, mass, catalog.z, big_delta_in, big_delta_out, c_nfw, $
                                r_in, r_out, mass_out, $
                                cosmo=cosmo, /five_hundred
      catalog.mass = mass_out
    ENDIF ELSE BEGIN
      ; cm500 not given, no conversion
      catalog.mass = mass
    ENDELSE
  ENDIF ELSE BEGIN
    catalog.mass = mass
  ENDELSE
ENDELSE

RETURN,catalog
ENDIF ELSE PRINT,'No cluster'

closing0:
END
