; -----------------------------------------------------------------------------
;
; Copyright (C) 2009
;   Jacques Delabrouille
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; Function to create a catalogue of sources (with data in a set of bands), with its header
; 
; @categories sz
; 
; @param name_cat {in}{required}{type=string} Name pf the catalogue
; @param nclusters {in}{required}{type=long64} Number of clusters in the catalogue 
; @keyword model {in}{required}{type=string} Model used to describe clusters (can be 'xmm','chandra','beta')
; @keyword usenan {in}{optional}{type=boolean} Set this keyword to use IEEE 'NaN' and 'Inf' as initialisation values (default = use 0 values)
; @keyword header {out}{optional}{type=structure} Header of the catalogue
;
; @history <p>14/04/2009: First version, Jacques Delabrouille and Jean-Baptiste Melin</p>
; @history <p>17/04/2009: TYPE tag changed to DATATYPE, and put in front of the structure, to match convention for other PSM data types, Jacques Delabrouille</p>
; @history <p>11/02/2010: structure made compatible with all the cases, Jean-Baptiste Melin</p>
; @copyright Jacques Delabrouille, Jean-Baptiste Melin, CNRS, CEA (2009)
;-
FUNCTION CREATE_SZ_MODEL_CAT, name_cat, nclusters, szp=szp, usenan=usenan, header=header

  model = szp.cluster_profile
  bounds = szp.profile_bounds
  norm = szp.norm_profile

  IF NOT KEYWORD_SET(usenan) THEN BEGIN
   db = 0d0 
   fl = 0.
  ENDIF ELSE BEGIN
   db = DOUBLE('NaN')
   fl = FLOAT('NaN')
  ENDELSE

  CASE STRLOWCASE(model) OF
     'beta': useparam='beta'
     'xmm': useparam='alphabetagamma'
     'chandra': useparam='alphabetagamma'
  ENDCASE

CASE useparam OF
   'beta': BEGIN
      cluster = { $
                name:'', $
                glon_rad:db, $
                glat_rad:db, $
                z:fl, $
                mass:fl, $
                v_radial:fl, $
                v_transverse:fl, $
                v_angle_rad:fl, $
                theta_c:fl, $
                theta_v:fl, $
                cy:fl $
                }

      field_name = ['Name', 'Galactic longitude', 'Galactic latitude', 'Redshift', 'Virial mass', 'Radial velocity', 'Transverse velocity', 'Angle of transverse velocity', 'Core radius', 'Virial radius', 'Compton parameter Y(r<rvir)']
      units_name = ['unitless', 'rad', 'rad', 'unitless', 'M_sun', 'km/s', 'km/s', 'rad', 'arcmin', 'arcmin', '(arcmin)**2']

      IF norm EQ 'yes' THEN BEGIN
        header = {datatype:'CLUSTER_MODEL_CAT',cat_name:name_cat,profile:szp.cluster_profile,norm:szp.norm_profile,bounds:szp.profile_bounds,fields:field_name,units:units_name}
      ENDIF ELSE BEGIN
        header = {datatype:'CLUSTER_MODEL_CAT',cat_name:name_cat,profile:szp.cluster_profile,norm:szp.norm_profile,bounds:szp.profile_bounds,t_star:szp.cluster_t_star,fields:field_name,units:units_name}
      ENDELSE

   END
   'alphabetagamma': BEGIN

      IF bounds EQ 'rvir' THEN BEGIN

      cluster = { $
                name:'', $
                glon_rad:db, $
                glat_rad:db, $
                z:fl, $
                mass:fl, $
                v_radial:fl, $
                v_transverse:fl, $
                v_angle_rad:fl, $
                theta_s:fl, $
                theta_v:fl, $
                cy:fl $
                }

      field_name = ['Name', 'Galactic longitude', 'Galactic latitude', 'Redshift', 'Virial mass', 'Radial velocity', 'Transverse velocity', 'Angle of transverse velocity', 'Scale radius', 'Virial radius', 'Compton parameter Y(r<rvir)']
      units_name = ['unitless', 'rad', 'rad', 'unitless', 'M_sun', 'km/s', 'km/s', 'rad', 'arcmin', 'arcmin', '(arcmin)**2']

      IF norm EQ 'yes' THEN BEGIN
        header = {datatype:'CLUSTER_MODEL_CAT',cat_name:name_cat,profile:szp.cluster_profile,norm:szp.norm_profile,bounds:szp.profile_bounds,fields:field_name,units:units_name}
      ENDIF ELSE BEGIN
        header = {datatype:'CLUSTER_MODEL_CAT',cat_name:name_cat,profile:szp.cluster_profile,norm:szp.norm_profile,bounds:szp.profile_bounds,t_star:szp.cluster_t_star,fields:field_name,units:units_name}
      ENDELSE

       ENDIF ELSE BEGIN
            
       cluster = { $
                 name:'', $
                 glon_rad:db, $
                 glat_rad:db, $
                 z:fl, $
                 mass_500:fl, $
                 v_radial:fl, $
                 v_transverse:fl, $
                 v_angle_rad:fl, $
                 theta_s:fl, $
                 theta_500:fl, $
                 cy_5r500:fl $
                 }

      field_name = ['Name', 'Galactic longitude', 'Galactic latitude', 'Redshift', 'Mass M500', 'Radial velocity', 'Transverse velocity', 'Angle of transverse velocity', 'Scale radius', 'Radius r500', 'Compton parameter Y(r<5*r500)']
      units_name = ['unitless', 'rad', 'rad', 'unitless', 'M_sun', 'km/s', 'km/s', 'rad', 'arcmin', 'arcmin', '(arcmin)**2']

      IF norm EQ 'yes' THEN BEGIN
        header = {datatype:'CLUSTER_MODEL_CAT',cat_name:name_cat,profile:szp.cluster_profile,norm:szp.norm_profile,bounds:szp.profile_bounds,fields:field_name,units:units_name}
      ENDIF ELSE BEGIN
        header = {datatype:'CLUSTER_MODEL_CAT',cat_name:name_cat,profile:szp.cluster_profile,norm:szp.norm_profile,bounds:szp.profile_bounds,t_star:szp.cluster_t_star,fields:field_name,units:units_name}
      ENDELSE

       ENDELSE

   END
ENDCASE

RETURN, REPLICATE(cluster,nclusters)

END
