; -----------------------------------------------------------------------------
;
; Copyright (C) 2001, 2003, 2004, 2006-2011
;   Alexandre Boucaud
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;    CLUSTER_MODEL
;
; PURPOSE:
;    Compute the temperatures, the central densities, the core radii, the scale radii
;    the central optical depth or the central Compton parameter
;    from z and M arrays
; CALLING SEQUENCE:
;    catalog = CLUSTER_MODEL(Z_array, M_array)
;
; INPUTS:
;    Z_array : the array containing cluster redshifts z
;    M_array : the array containing cluster masses M (in 10^15 solar mass units)
;
; KEYWORDS:
;    cosmo : The structure containing the cosmological parameters
;    cluster : the structure containing the structure formation parameters (cf. DEFINE_CLUSTER)
;    Te : set this keyword if you want the cluster temperatures (keV)
;    ne0 : set this keyword if you want the cluster central densities(cm^(-3))
;    rc : set this keyword if you want the cluster core radii (Mpc) [valid only for beta-profile modeling]
;    tau0 : set this keyword if you want the cluster central optical depth
;           setting this keyword compute automatically ne0 and rc [for beta-profile modeling] or ne0 and rs [for nfw-profile modeling]
;    y0 : set this keyword if you want the cluster central Compton parameter
;         setting this keyword compute automatically Te and tau0
;         (so ne0 and rc [for beta-profile modeling] or ne0 and rs [for nfw-profile modeling])
;    rs : set this keyword if you want the cluster scale radii (Mpc)
;    r_cut : the radius (in number of rc) where you stop your beta model [for beta-profile modeling]
;            default : r_cut = 10.
;            the radius (in number of rs) where you stop your nfw profile [for nfw-profile modeling]
;            default : r_cut = r_vir/rs
;    beta : the factor beta in the beta model (e.g. 0.66)
;           default : beta = 2./3
;    chandra : set this keyword if you want to use the nfw-profile fitted on chandra observations (Nagai et al 2007, ApJ 668:1-14)
;              There are some bugs in that paper : for example, P0=12.2 and not 3.3 for Chandra Obs. in Table A1 -> see erratum
;    xmm : set this keyword if you want the to use teh nfw-profile fitted on xmm observations (Arnaud et al. 2008)
;              Fits XMM, courtesy of M. Arnaud on the REXCESS sample 
;    norm : set this keyword if you want to normalise the profile on the chandra or xmm observations and not on the model.
;    nfw_a, nfw_b, nfw_c : parameters of the NFW generalized profile (you must set xmm or chandra keyword to use these keywords)
;    c500 : concentration rs/r500 of the pressure profile (you must set xmm or chandra keyword to use this keyword)
;    P0 : normalisation of the pressure profile (you must set xmm or chandra keyword to use this keyword)
;    cm500 : concentration rs/r500 of the matter profile (you must set xmm or chandra keyword to use this keyword)
;    nstd : non standard NFW profile (valid if xmm keyword is set)
;    cco : cool core profile (if nstd keyword is set)
;    mdi : morphologically disturbed profile (if nstd keyword is set)
;
; OUTPUTS:
;    catalog : the catalog of cluster carateristics you asked for
;
; EXAMPLES:
;
; RESTRICTIONS:
;    Report any bugs to Jacques Delabrouille
;    j.delabrouille@cdf.in2p3.fr
;
; REVISION HISTORY
;    Written, Jean-Baptiste Melin April&May 2001
;    Masses in 10^15 solar mass units, Te in keV, JBM, May 30 2001
;    ne0 in cm^(-3), JBM, May 31 2001
;    mu reintroduced in ne0 computation, JBM, August 14 2001
;    r_cut and beta introduced in ne0 computation, Sept 19 2001
;    omega_m and density contrast introduced in Te computation, Sept 19 2001
;    case when z_array and M_array are not arrays implemented, Sept 20 2001
;    lambda removed, Sept 24 2001
;    Keyword cosmo introduced, JBM, Sept 27 2001
;    gaz_fraction deleted, now in cosmo structure, JBM, Oct 3 2003
;    BIG_DELTA_C added, JBM, Nov 24 2004
;    cluster keyword added, JBM, Feb 3 2006
;    rv (virial radius field) added when rc field computed, JBM, July 17 2007
;    rs, chandra and norm added to make the model coherent with observations, JBM & Alexandre Boucaud June 6 2008
;    xmm added, JBM & AB June 26 2008
;    cosmetic and velocity improvements, JBM, June 27 2008
;    bug in call of cluster_mass2dens with /norm fixed, JBM, July 1 2008
;    Bug in P500(electron) which is mu/me_e=0.52 times P500(gas) fixed in cluster_mass2dens, JBM, July 10 2008
;    New value for c_nfw_m introduced for XMM profile, JBM, September 23 2008
;    New values for c_nfw_p and alpha, beta, gamma following the Nagai erratum, JBM, Sept. 24 2008
;    c500 and P0 keywords added, JBM, Sept. 30 2008
;    bug in P0 keyword fixed, cm500 keyword added, JBM, Oct. 2 2008
;    Updated values for the REXCESS profiles, JBM, March 19 2009
;    Final values for the REXCESS profile implemented, JBM, July 6 2009
;    Last changes in the REXCESS profile, JBM, Oct 9 2009
;    nstd keyword added, JBM, Sept 9 2010
;    cco and mdi keyword added, JBM, Oct 20 2010
;    CLUSTER_MASS2TEMP modified : M500_array, xmm, chandra, norm keywords added, JBM, Feb 1st 2011
;-

;+
; NAME:
;	CLUSTER_MASS2TEMP
;
; PURPOSE:
;	...
;
;-
;------------------------------------------------------------------------------
FUNCTION CLUSTER_MASS2TEMP, z_array, M_array, cosmo=cosmo, cluster=cluster, M500_array=M500_array, xmm=xmm, chandra=chandra, norm=norm
;------------------------------------------------------------------------------
; Compute the central temperatures (keV) of the clusters from z and M arrays
;------------------------------------------------------------------------------

IF ((KEYWORD_SET(chandra) OR KEYWORD_SET(xmm)) AND KEYWORD_SET(norm)) THEN BEGIN

  ; Form Arnaud, Pointecouteau et Pratt 2005 [Table 2]
  a500 = 0.41 ; 0.410 pm 0.019
  alpham = 1.49 ; 1.49 pm 0.15
  hz = HUBBLE_AT_Z(z_array, cosmo=cosmo)/100./cosmo.h
  cluster_temp = 5.*(hz*M500_array/a500)^(1./alpham) ; keV

ENDIF ELSE BEGIN

; taken from Bartlett 1997

;density_contrast = 178.  ; very small variations with the cosomlogy
;Not very precise for LCDM cosmologies
;COMMON CONTRAST_PARAMETER, density_contrast

;beta=1.3
beta = cluster.t_star
density_contrast = INTERPOL(cluster.big_delta_c_array, cluster.z_array, z_array)

cluster_temp = beta*178.^(1./3)*(M_array)^(2./3)*(cosmo.h)^(2./3)*(density_contrast/178.)^(1./3)* $
               (HUBBLE_AT_Z(z_array, cosmo=cosmo)/100./cosmo.h)^(2./3)

;cluster_temp = 6.8*(cosmo.h)^(2./3)*(M_array)^(2./3)* $
;               (cosmo.omega_m*density_contrast/178.)^(1./3)*(1+z_array)

ENDELSE

RETURN, cluster_temp
END
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
FUNCTION CLUSTER_MASS2RS, M_array, z_array, cosmo=cosmo, cluster=cluster, M500_array=M500_array, xmm=xmm, c500=c500, cm500=cm500, nstd=nstd, cco=cco, mdi=mdi
;------------------------------------------------------------------------------

; Chandra is the default case here
five_hundred = 1.
c_nfw_m = 2.8 ; c500 = r500/rs for the Chandra matter profile (weighted mean of c500 given in table 4
              ; Vikhlinin et al. 2006, ApJ 640:691-709) 
IF KEYWORD_SET(xmm) THEN c_nfw_m = 3.0 ; c500 for the XMM matter profile (weighted mean and conversion
                                       ; from c200 given in table 2 Pointecouteau et al. 2005, AA 435:1-7)
;c_nfw_p = 1.8  ; c500 = r500/rs for the Chandra pressure profile (Nagai et al. 2007)
;IF KEYWORD_SET(xmm) THEN c_nfw_p = 1./0.60977117 ; c500 = r500/rs ~ 1.6 for the XMM pressure profile (Arnaud et al. 2008)
c_nfw_p = 1.3  ; c500 = r500/rs for the Chandra pressure profile (Nagai et al. 2007, erratum)
;IF KEYWORD_SET(xmm) THEN c_nfw_p = 1./0.72028693 ; c500 = r500/rs ~ 1.4 for the XMM pressure profile (Arnaud et al. 2008)
;IF KEYWORD_SET(xmm) THEN c_nfw_p = 1./0.63827873 ; c500 = r500/rs ~ 1.6 for the XMM pressure profile (March 19 2009)
;IF KEYWORD_SET(xmm) THEN c_nfw_p = 1./0.67485218 ; c500 = r500/rs ~ 1.6 for the XMM pressure profile (April 17 2009)
;IF KEYWORD_SET(xmm) THEN c_nfw_p = 1./0.67220352 ; c500 = r500/rs ~ 1.6 for the XMM pressure profile (April 17 2009)
;IF KEYWORD_SET(xmm) THEN c_nfw_p = 1./0.70198483 ; c500 = r500/rs ~ 1.6 for the XMM pressure profile (July 6 2009)
;IF KEYWORD_SET(xmm) THEN c_nfw_p = 1./0.864942 ; c500 for the XMM pressure profile (October 9 2009) see B2 formula in Arnaud et al. 2009

IF KEYWORD_SET(xmm) THEN BEGIN
  IF KEYWORD_SET(nstd) THEN BEGIN
    c_nfw_p = 1./0.84942723
    IF KEYWORD_SET(cco) THEN c_nfw_p = 1.128
    IF KEYWORD_SET(mdi) THEN c_nfw_p = 1.083
  ENDIF ELSE $
    c_nfw_p = 1./0.864942 ; see Rocco's notes in you notebook (p. 160-162)
ENDIF

IF KEYWORD_SET(c500) THEN c_nfw_p = c500 ELSE c500=c_nfw_p
IF KEYWORD_SET(cm500) THEN c_nfw_m = cm500 ELSE cm500=c_nfw_m

big_delta_in = INTERPOL(cluster.big_delta_c_array,cluster.z_array,z_array)
big_delta_out = REPLICATE(500.,N_ELEMENTS(M_array))

FAST_CONVERT_MASS_RADIUS, M_array, z_array, big_delta_in, big_delta_out, REPLICATE(c_nfw_m,N_ELEMENTS(M_array)), $
                          r_in, r500, m_500, r_s=r_s, $
                          cosmo=cosmo, five_hundred=five_hundred

M500_array=m_500

cluster_rs = r500/c_nfw_p

RETURN, cluster_rs

END
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
FUNCTION CLUSTER_MASS2P500,  M500_array, z_array, cosmo=cosmo, nstd=nstd
;------------------------------------------------------------------------------
; P500(gas) from Nagai et al. 2007
; P500(electron) is given by mu/mu_e*P500(gas) and is computed in cluster_mass2dens

Ez = HUBBLE_AT_Z(z_array, cosmo=cosmo)/(100*cosmo.h)
;cluster_P500 = 1.45d-11 * (cosmo.h)^(2/3.) * M500_array^(2/3.) * Ez^(8/3.) ; in erg.cm-3
fact_h = cosmo.h/0.7
cluster_P500 = 1.45d-11 * (cosmo.h)^(2/3.) * M500_array^(2/3.) * Ez^(8/3.) * fact_h^2. ; in erg.cm-3

IF KEYWORD_SET(nstd) THEN cluster_P500=cluster_P500*(M500_array/(0.3/fact_h))^0.115865

RETURN, cluster_P500

END
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
FUNCTION CLUSTER_MASS2VIR_RAD, z_array, M_array, $
                               cosmo=cosmo, cluster=cluster
;----------------------------------------------------------
; Compute the virial radii (Mpc) of the clusters from z and M arrays
;------------------------------------------------------------------------------

; taken from Bartlett 2000

;density_contrast = 178  ; very small variations with the cosomlogy
;Not very precise for LCDM cosmologies
;COMMON CONTRAST_PARAMETER, density_contrast

density_contrast = INTERPOL(cluster.big_delta_c_array, cluster.z_array, z_array)

;cluster_vir_rad = 1.69/(cosmo.h)^(2./3)*(M_array)^(1./3)/ $
;                  (1+z_array)/(cosmo.omega_m)^(1./3)*(178/density_contrast)^(1./3)
cluster_vir_rad = 1.69/(cosmo.h)^(2./3)*(M_array)^(1./3)/ $
                  (HUBBLE_AT_Z(z_array, cosmo=cosmo)/100./cosmo.h)^(2./3)*(178/density_contrast)^(1./3)
RETURN, cluster_vir_rad
END
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
FUNCTION VIR_RAD2CORE_RAD, z_array, M_array, cluster_vir_rad
;-----------------------------------------------------------
; Compute the core radii (Mpc) from the virial radii (Mpc)
;------------------------------------------------------------------------------

; taken from Bartlett 2000
cluster_core_rad = 0.1*cluster_vir_rad ; independant of M for the moment
;cluster_core_rad = 1./20.*cluster_vir_rad ; independant of M for the moment

RETURN, cluster_core_rad
END
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
FUNCTION SUBFUNC_MASS2DENS, x
;----------------------------
COMMON MASS2DENS_PARAMETER, mass2dens_beta
RETURN, (1+x^2)^(-3*mass2dens_beta/2.)*x^2
END
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
FUNCTION SUBFUNC_MASS2DENS_NFW, x
;------------------------------------
COMMON MASS2DENS_NFW_PARAMETER, mass2dens_a, mass2dens_b, mass2dens_c
RETURN, (1+x^(mass2dens_a))^((mass2dens_c - mass2dens_b)/mass2dens_a)*x^(2-mass2dens_c)
END
;------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
FUNCTION CLUSTER_MASS2DENS, z_array, M_array, r_cut=r_cut, beta=beta, $
                            cosmo=cosmo, cluster=cluster, nfw_a=nfw_a, $
                            nfw_b=nfw_b, nfw_c=nfw_c, chandra=chandra, xmm=xmm, norm=norm, scale_radius=scale_radius, $
                            M500_array=M500_array, c500=c500, P0=P0, cm500=cm500, nstd=nstd, cco=cco, mdi=mdi
;------------------------------------------------------------------------------
; Compute the central densities of the clusters from z and M arrays
;------------------------------------------------------------------------------

vir_rad = CLUSTER_MASS2VIR_RAD(z_array, M_array, cosmo=cosmo, cluster=cluster)

IF KEYWORD_SET(chandra) OR KEYWORD_SET(xmm) THEN BEGIN
  IF NOT KEYWORD_SET(scale_radius) THEN BEGIN
    scale_rad = CLUSTER_MASS2RS(M_array, z_array, cosmo=cosmo, cluster=cluster, M500_array=M500_array, xmm=xmm, c500=c500, cm500=cm500, nstd=nstd, cco=cco, mdi=mdi)
  ENDIF ELSE BEGIN
    scale_rad = scale_radius
  ENDELSE
  IF NOT KEYWORD_SET(r_cut) THEN BEGIN
    r_cut = vir_rad/scale_rad
  ENDIF
  IF NOT KEYWORD_SET(nfw_a) THEN BEGIN
    ;IF KEYWORD_SET(chandra) THEN nfw_a = 1.3 ELSE nfw_a = 2.4312589
    ;IF KEYWORD_SET(chandra) THEN nfw_a = 0.9 ELSE nfw_a = 2.1981497
    ;IF KEYWORD_SET(chandra) THEN nfw_a = 0.9 ELSE nfw_a = 1.8818421
    ;IF KEYWORD_SET(chandra) THEN nfw_a = 0.9 ELSE nfw_a = 1.8651357
    ;IF KEYWORD_SET(chandra) THEN nfw_a = 0.9 ELSE nfw_a = 1.7518179
    ;IF KEYWORD_SET(chandra) THEN nfw_a = 0.9 ELSE nfw_a = 1.7000839
    ;IF KEYWORD_SET(chandra) THEN nfw_a = 0.9 ELSE nfw_a = 1.06201
    IF KEYWORD_SET(chandra) THEN nfw_a = 0.9 ELSE BEGIN
      IF KEYWORD_SET(nstd) THEN BEGIN
        nfw_a = 1.0510
        IF KEYWORD_SET(cco) THEN nfw_a = 1.2223
        IF KEYWORD_SET(mdi) THEN nfw_a = 1.4063
      ENDIF ELSE nfw_a = 1.06201
    ENDELSE
  ENDIF
  IF NOT KEYWORD_SET(nfw_b) THEN BEGIN
    ;IF KEYWORD_SET(chandra) THEN nfw_b = 4.3 ELSE nfw_b = 4.3
    ;IF KEYWORD_SET(chandra) THEN nfw_b = 5.0 ELSE nfw_b = 5.0
    ;IF KEYWORD_SET(chandra) THEN nfw_b = 5.0 ELSE nfw_b = 4.4220213
    ;IF KEYWORD_SET(chandra) THEN nfw_b = 5.0 ELSE nfw_b = 4.5794125
    ;IF KEYWORD_SET(chandra) THEN nfw_b = 5.0 ELSE nfw_b = 4.4667822
    ;IF KEYWORD_SET(chandra) THEN nfw_b = 5.0 ELSE nfw_b = 4.6496147
    ;IF KEYWORD_SET(chandra) THEN nfw_b = 5.0 ELSE nfw_b = 5.48074
    IF KEYWORD_SET(chandra) THEN nfw_b = 5.0 ELSE BEGIN
      IF KEYWORD_SET(nstd) THEN BEGIN
        nfw_b = 5.4905
        IF KEYWORD_SET(cco) THEN nfw_b = 5.49
        IF KEYWORD_SET(mdi) THEN nfw_b = 5.49
      ENDIF ELSE nfw_b = 5.48074
    ENDELSE
  ENDIF
  IF NOT KEYWORD_SET(nfw_c) THEN BEGIN
    ;IF KEYWORD_SET(chandra) THEN nfw_c = 0.7 ELSE nfw_c = 0.93300709
    ;IF KEYWORD_SET(chandra) THEN nfw_c = 0.4 ELSE nfw_c = 0.92395885
    ;IF KEYWORD_SET(chandra) THEN nfw_c = 0.4 ELSE nfw_c = 0.83999495
    ;IF KEYWORD_SET(chandra) THEN nfw_c = 0.4 ELSE nfw_c = 0.83541297
    ;IF KEYWORD_SET(chandra) THEN nfw_c = 0.4 ELSE nfw_c = 0.81210425
    ;IF KEYWORD_SET(chandra) THEN nfw_c = 0.4 ELSE nfw_c = 0.80279422
    ;IF KEYWORD_SET(chandra) THEN nfw_c = 0.4 ELSE nfw_c = 0.32924
    IF KEYWORD_SET(chandra) THEN nfw_c = 0.4 ELSE BEGIN
      IF KEYWORD_SET(nstd) THEN BEGIN
        nfw_c = 0.3081
        IF KEYWORD_SET(cco) THEN nfw_c = 0.7736
        IF KEYWORD_SET(mdi) THEN nfw_c = 0.3798
      ENDIF ELSE nfw_c = 0.32924
    ENDELSE
  ENDIF
  COMMON MASS2DENS_NFW_PARAMETER, mass2dens_a, mass2dens_b, mass2dens_c
  mass2dens_a = nfw_a
  mass2dens_b = nfw_b
  mass2dens_c = nfw_c
  IF NOT KEYWORD_SET(P0) THEN BEGIN
    ;IF KEYWORD_SET(chandra) THEN P0=5.35 ELSE P0=1.8173360
    ;IF KEYWORD_SET(chandra) THEN P0=12.2 ELSE P0=1.5998933
    ;IF KEYWORD_SET(chandra) THEN P0=12.2 ELSE P0=2.2766410
    ;IF KEYWORD_SET(chandra) THEN P0=12.2 ELSE P0=2.2942053
    ;IF KEYWORD_SET(chandra) THEN P0=12.2 ELSE P0=2.4601768
    ;IF KEYWORD_SET(chandra) THEN P0=12.2 ELSE P0=2.3714193
    ;IF KEYWORD_SET(chandra) THEN P0=12.2 ELSE P0=8.12955*(cosmo.h/0.7)^(-3/2.)
    IF KEYWORD_SET(chandra) THEN P0=12.2 ELSE BEGIN
      IF KEYWORD_SET(nstd) THEN BEGIN
        P0=8.403*(cosmo.h/0.7)^(-3/2.)
        IF KEYWORD_SET(cco) THEN P0=3.249
        IF KEYWORD_SET(mdi) THEN P0=3.202
      ENDIF ELSE P0=8.12955*(cosmo.h/0.7)^(-3/2.)
    ENDELSE
  ENDIF
ENDIF ELSE BEGIN
  core_rad = VIR_RAD2CORE_RAD(z_array, M_array, vir_rad)
  IF NOT KEYWORD_SET(r_cut) THEN r_cut=10.
  IF NOT KEYWORD_SET(beta) THEN beta=2./3
  COMMON MASS2DENS_PARAMETER, mass2dens_beta
  mass2dens_beta = beta
ENDELSE

;gaz_fraction = cosmo.gf
gaz_fraction = cluster.gf

mu = 0.6 ; mean molecular weight

proton_mass = 1.67262158d-27 ; in kg
M_sun = 1.9889*1d30 ; in kg
Mpc2m = 3.0856775807d+22 ; in m

cluster_dens = DBLARR(N_ELEMENTS(z_array))

; compute the central densities
IF KEYWORD_SET(chandra) OR KEYWORD_SET(xmm) THEN BEGIN
  IF KEYWORD_SET(norm) THEN BEGIN
    Te = CLUSTER_MASS2TEMP(z_array, M_array, cosmo=cosmo, cluster=cluster, M500_array=M500_array, xmm=xmm, chandra=chandra, norm=norm)
    Te_erg = Te / 6.2415d8
    P500 = CLUSTER_MASS2P500(M500_array, z_array, cosmo=cosmo, nstd=nstd)
    cluster_dens = P0 * P500 / Te_erg * (Mpc2m^3.*1d6) * mu * (1.+2./mu)/5.
  ENDIF ELSE BEGIN
    integ = DBLARR(N_ELEMENTS(z_array))
    so = SORT(r_cut)
    integ[so[0]] = QROMB('subfunc_mass2dens_nfw',0.,r_cut[so[0]])
    FOR i = 1L, N_ELEMENTS(cluster_dens)-1 DO BEGIN
      integ[so[i]]=integ[so[i-1]]+QROMB('subfunc_mass2dens_nfw',r_cut[so[i-1]],r_cut[so[i]])
    ENDFOR
    cluster_dens = gaz_fraction*M_array*(1d15*M_sun)*(1.+2./mu)/5./proton_mass/ $
                   (4*!pi*scale_rad^3.)/integ
  ENDELSE
ENDIF ELSE BEGIN
    integ = QROMB('subfunc_mass2dens',0.,r_cut)
    cluster_dens = gaz_fraction*M_array*(1d15*M_sun)*(1.+2./mu)/5./proton_mass/ $
    (4*!pi*core_rad^3.)/integ
ENDELSE
  RETURN, cluster_dens/(Mpc2m^3*1e6) ; in cm^(-3)
END
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
FUNCTION SUBFUNC_MODEL, x
;------------------------
COMMON CLUSTER_MODEL_PARAMETER, cluster_model_beta
  RETURN, (1+x^2)^(-3*cluster_model_beta/2.)
END
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
FUNCTION SUBFUNC_MODEL_NFW, x
;--------------------------------
COMMON CLUSTER_MODEL_NFW_PARAMETER, cluster_model_a, cluster_model_b, cluster_model_c
  RETURN,  (1+x^(cluster_model_a))^((cluster_model_c - cluster_model_b)/cluster_model_a)*x^(-cluster_model_c)
END
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
FUNCTION CLUSTER_MODEL, z_array, M_array, cosmo=cosmo, cluster=cluster, $
                        Te=Te, ne0=ne0, rc=rc, rs=rs, tau0=tau0, y0=y0, $
                        r_cut=r_cut, beta=beta, chandra=chandra, norm=norm, xmm=xmm, $
                        nfw_a=nfw_a, nfw_b=nfw_b, nfw_c=nfw_c, c500=c500, P0=P0, cm500=cm500, nstd=nstd, cco=cco, mdi=mdi
;------------------------------------------------------------------------------

vir_rad = CLUSTER_MASS2VIR_RAD(z_array, M_array, cosmo=cosmo, cluster=cluster)

IF KEYWORD_SET(chandra) OR KEYWORD_SET(xmm) THEN BEGIN  
  scale_rad = CLUSTER_MASS2RS(M_array, z_array, cosmo=cosmo, cluster=cluster, M500_array=M500_array, xmm=xmm, c500=c500, cm500=cm500, nstd=nstd, cco=cco, mdi=mdi)
  IF NOT KEYWORD_SET(r_cut) THEN BEGIN
    r_cut = vir_rad/scale_rad
  ENDIF
  IF NOT KEYWORD_SET(nfw_a) THEN BEGIN
    ;IF KEYWORD_SET(chandra) THEN nfw_a = 1.3 ELSE nfw_a = 2.4312589
    ;IF KEYWORD_SET(chandra) THEN nfw_a = 0.9 ELSE nfw_a = 2.1981497
    ;IF KEYWORD_SET(chandra) THEN nfw_a = 0.9 ELSE nfw_a = 1.8818421
    ;IF KEYWORD_SET(chandra) THEN nfw_a = 0.9 ELSE nfw_a = 1.8651357
    ;IF KEYWORD_SET(chandra) THEN nfw_a = 0.9 ELSE nfw_a = 1.7518179
    ;IF KEYWORD_SET(chandra) THEN nfw_a = 0.9 ELSE nfw_a = 1.7000839
    ;IF KEYWORD_SET(chandra) THEN nfw_a = 0.9 ELSE nfw_a = 1.06201
    IF KEYWORD_SET(chandra) THEN nfw_a = 0.9 ELSE BEGIN
      IF KEYWORD_SET(nstd) THEN BEGIN
        nfw_a = 1.0510
        IF KEYWORD_SET(cco) THEN nfw_a = 1.2223
        IF KEYWORD_SET(mdi) THEN nfw_a = 1.4063
      ENDIF ELSE nfw_a = 1.06201
    ENDELSE
  ENDIF
  IF NOT KEYWORD_SET(nfw_b) THEN BEGIN
    ;IF KEYWORD_SET(chandra) THEN nfw_b = 4.3 ELSE nfw_b = 4.3
    ;IF KEYWORD_SET(chandra) THEN nfw_b = 5.0 ELSE nfw_b = 5.0
    ;IF KEYWORD_SET(chandra) THEN nfw_b = 5.0 ELSE nfw_b = 4.4220213
    ;IF KEYWORD_SET(chandra) THEN nfw_b = 5.0 ELSE nfw_b = 4.5794125
    ;IF KEYWORD_SET(chandra) THEN nfw_b = 5.0 ELSE nfw_b = 4.4667822
    ;IF KEYWORD_SET(chandra) THEN nfw_b = 5.0 ELSE nfw_b = 4.6496147
    ;IF KEYWORD_SET(chandra) THEN nfw_b = 5.0 ELSE nfw_b = 5.48074
    IF KEYWORD_SET(chandra) THEN nfw_b = 5.0 ELSE BEGIN
      IF KEYWORD_SET(nstd) THEN BEGIN
        nfw_b = 5.4905
        IF KEYWORD_SET(cco) THEN nfw_b = 5.49
        IF KEYWORD_SET(mdi) THEN nfw_b = 5.49
      ENDIF ELSE nfw_b = 5.48074
    ENDELSE
  ENDIF
  IF NOT KEYWORD_SET(nfw_c) THEN BEGIN
    ;IF KEYWORD_SET(chandra) THEN nfw_c = 0.7 ELSE nfw_c = 0.93300709
    ;IF KEYWORD_SET(chandra) THEN nfw_c = 0.4 ELSE nfw_c = 0.92395885
    ;IF KEYWORD_SET(chandra) THEN nfw_c = 0.4 ELSE nfw_c = 0.83999495
    ;IF KEYWORD_SET(chandra) THEN nfw_c = 0.4 ELSE nfw_c = 0.83541297
    ;IF KEYWORD_SET(chandra) THEN nfw_c = 0.4 ELSE nfw_c = 0.81210425
    ;IF KEYWORD_SET(chandra) THEN nfw_c = 0.4 ELSE nfw_c = 0.80279422
    ;IF KEYWORD_SET(chandra) THEN nfw_c = 0.4 ELSE nfw_c = 0.32924
    IF KEYWORD_SET(chandra) THEN nfw_c = 0.4 ELSE BEGIN
      IF KEYWORD_SET(nstd) THEN BEGIN
        nfw_c = 0.3081
        IF KEYWORD_SET(cco) THEN nfw_c = 0.7736
        IF KEYWORD_SET(mdi) THEN nfw_c = 0.3798
      ENDIF ELSE nfw_c = 0.32924
    ENDELSE
  ENDIF
  COMMON CLUSTER_MODEL_NFW_PARAMETER, cluster_model_a, cluster_model_b, cluster_model_c
  cluster_model_a = nfw_a
  cluster_model_b = nfw_b
  cluster_model_c = nfw_c
ENDIF ELSE BEGIN
  core_rad = VIR_RAD2CORE_RAD(z_array, M_array, vir_rad)
  IF NOT KEYWORD_SET(r_cut) THEN r_cut=10.
  IF NOT KEYWORD_SET(beta) THEN beta=2./3
  COMMON CLUSTER_MODEL_PARAMETER, cluster_model_beta
  cluster_model_beta = beta
ENDELSE

Mpc2m = 3.0856775807d+22 ; in m
sigma_thomson = 0.665245854d-28 ; in m^2
electron_mass = 0.51099906*1d3 ; in keV

IF ( N_PARAMS() EQ 0 ) THEN BEGIN
 print,'CALL IS : result=CLUSTER_MODEL(z_array, M_array)'
 print,'Keywords : Te=Te, ne0=ne0, rc=rc, rs=rs, tau0=tau0, y0=y0'
 print,'cosmo=cosmo, cluster=cluster,'
 print,'chandra=chandra, xmm=xmm, norm=norm'
 print,'nfw_a=nfw_a, nfw_b=nfw_b, nfw_c=nfw_c'
 print,'c500=c500, P0=P0, cm500=cm500, nstd=nstd, cco=cco, mdi=mdi'
 GOTO, closing
ENDIF

count_fields = 0
command = 'amas = {'
IF KEYWORD_SET(Te) OR KEYWORD_SET(y0) THEN BEGIN
	IF count_fields NE 0 THEN command=command+','
	command = command+'Te:0d'
    count_fields = count_fields+1
ENDIF
IF KEYWORD_SET(ne0) OR KEYWORD_SET(y0) OR KEYWORD_SET(tau0) THEN BEGIN
	IF count_fields NE 0 THEN command=command+','
	command = command+'ne0:0d'
	count_fields = count_fields+1
ENDIF
IF KEYWORD_SET(chandra) OR KEYWORD_SET(xmm) THEN BEGIN
  IF KEYWORD_SET(rs) OR KEYWORD_SET(y0) OR KEYWORD_SET(tau0) THEN BEGIN
	IF count_fields NE 0 THEN command=command+','
	command = command+'rs:0d,rv:0d'
	count_fields = count_fields+2
   ENDIF
ENDIF ELSE BEGIN
  IF KEYWORD_SET(rc) OR KEYWORD_SET(y0) OR KEYWORD_SET(tau0) THEN BEGIN
	IF count_fields NE 0 THEN command=command+','
	command = command+'rc:0d,rv:0d'
	count_fields = count_fields+2
  ENDIF
ENDELSE
IF KEYWORD_SET(tau0) OR KEYWORD_SET(y0) THEN BEGIN
	IF count_fields NE 0 THEN command=command+','
	command = command+'tau0:0d'
	count_fields = count_fields+1
ENDIF
IF KEYWORD_SET(y0) THEN BEGIN
	IF count_fields NE 0 THEN command=command+','
	command = command+'y0:0d'
	count_fields = count_fields+1
ENDIF
command=command+'}'
IF count_fields NE 0 THEN BEGIN
	res = EXECUTE(command)
ENDIF ELSE BEGIN
	PRINT, '  NO CLUSTER PARAMETER REQUIRED, RETURNING'
        GOTO, closing
ENDELSE

catalog = REPLICATE(amas,N_ELEMENTS(z_array))

;help, catalog, /str

;COMMON CONTRAST_PARAMETER, density_contrast
;density_contrast = 178.
;density_contrast = BIG_DELTA_C(z_array, cosmo=cosmo)

; the temperatures
IF KEYWORD_SET(Te) THEN BEGIN
  catalog.Te = REFORM([CLUSTER_MASS2TEMP(z_array, M_array, cosmo=cosmo, cluster=cluster, M500_array=M500_array, xmm=xmm, chandra=chandra, norm=norm)])
ENDIF

; the scale radii
IF KEYWORD_SET(chandra) OR KEYWORD_SET(xmm) THEN BEGIN
  IF KEYWORD_SET(rs) THEN BEGIN
    catalog.rs = scale_rad
    catalog.rv = vir_rad
  ENDIF
ENDIF

; the central densities
IF KEYWORD_SET(ne0) THEN BEGIN
  catalog.ne0 = REFORM([CLUSTER_MASS2DENS(z_array, M_array, r_cut=r_cut, beta=beta, $
                            cosmo=cosmo, cluster=cluster, nfw_a=nfw_a, $
                            nfw_b=nfw_b, nfw_c=nfw_c, chandra=chandra, xmm=xmm, norm=norm, scale_radius=scale_rad, $
                            M500_array=M500_array, c500=c500, P0=P0, cm500=cm500, nstd=nstd, cco=cco, mdi=mdi)])
ENDIF

; the core radii
IF NOT KEYWORD_SET(chandra) OR NOT KEYWORD_SET(xmm) THEN BEGIN
  IF KEYWORD_SET(rc) THEN BEGIN
    catalog.rc = core_rad
    catalog.rv = vir_rad
  ENDIF
ENDIF

; the central optical depth
IF KEYWORD_SET(tau0) THEN BEGIN
  IF NOT KEYWORD_SET(ne0) THEN BEGIN
    catalog.ne0 = REFORM([CLUSTER_MASS2DENS(z_array, M_array, r_cut=r_cut, beta=beta, $
                          cosmo=cosmo, cluster=cluster, nfw_a=nfw_a, $
                          nfw_b=nfw_b, nfw_c=nfw_c, chandra=chandra, xmm=xmm, norm=norm, scale_radius=scale_rad, $
                          M500_array=M500_array, c500=c500, P0=P0, cm500=cm500, nstd=nstd, cco=cco, mdi=mdi)])
  ENDIF
  IF KEYWORD_SET(chandra) OR KEYWORD_SET(xmm) THEN BEGIN
    IF NOT KEYWORD_SET(rs) THEN BEGIN
      catalog.rs = scale_rad
      catalog.rv = vir_rad
    ENDIF
  ENDIF ELSE BEGIN
    IF NOT KEYWORD_SET(rc) THEN BEGIN
      catalog.rc = core_rad
      catalog.rv = vir_rad
    ENDIF
  ENDELSE
  IF KEYWORD_SET(chandra) OR KEYWORD_SET(xmm) THEN BEGIN
     integrale = 0
     ; The profile is very cuspy cf. integrals below. Does it worth to define tau0 or y0 ?
     ;print, QROMB('subfunc_model_nfw', 1d-2 ,MAX(r_cut), jmax=40) ;*catalog.rs
     ;print, QROMB('subfunc_model_nfw', 1d-3 ,MAX(r_cut), jmax=40) ;*catalog.rs
     ;print, QROMB('subfunc_model_nfw', 1d-4 ,MAX(r_cut), jmax=40) ;*catalog.rs
     ;print, QROMB('subfunc_model_nfw', 1d-5 ,MAX(r_cut), jmax=40) ;*catalog.rs
  ENDIF ELSE BEGIN
     integrale = QROMB('subfunc_model', 0.,r_cut)*catalog.rc
  ENDELSE
  catalog.tau0 = 2*catalog.ne0*integrale*sigma_thomson*Mpc2m*1e6
ENDIF

; the central Compton parameter
IF KEYWORD_SET(y0) THEN BEGIN
  IF NOT KEYWORD_SET(tau0) THEN BEGIN
    IF NOT KEYWORD_SET(ne0) THEN BEGIN
      catalog.ne0 = REFORM([CLUSTER_MASS2DENS(z_array, M_array, r_cut=r_cut, beta=beta, $
                            cosmo=cosmo, cluster=cluster, nfw_a=nfw_a, $
                            nfw_b=nfw_b, nfw_c=nfw_c, chandra=chandra, xmm=xmm, norm=norm, scale_radius=scale_rad, $
                            M500_array=M500_array, c500=c500, P0=P0, cm500=cm500, nstd=nstd, cco=cco, mdi=mdi)])
    ENDIF
    IF KEYWORD_SET(chandra) OR KEYWORD_SET(xmm) THEN BEGIN
      IF NOT KEYWORD_SET(rs) THEN BEGIN
        catalog.rs = scale_rad
        catalog.rv = vir_rad
      ENDIF
    ENDIF ELSE BEGIN
      IF NOT KEYWORD_SET(rc) THEN BEGIN
        catalog.rc = core_rad
        catalog.rv = vir_rad
      ENDIF
    ENDELSE
    IF KEYWORD_SET(chandra) OR KEYWORD_SET(xmm) THEN BEGIN
      integrale = 0
      ; The profile is very cuspy cf. integrals below. Does it worth to define tau0 or y0 ?
      ;print, QROMB('subfunc_model_nfw', 1d-2 ,MAX(r_cut), jmax=40) ;*catalog.rs
      ;print, QROMB('subfunc_model_nfw', 1d-3 ,MAX(r_cut), jmax=40) ;*catalog.rs
      ;print, QROMB('subfunc_model_nfw', 1d-4 ,MAX(r_cut), jmax=40) ;*catalog.rs
      ;print, QROMB('subfunc_model_nfw', 1d-5 ,MAX(r_cut), jmax=40) ;*catalog.rs
    ENDIF ELSE BEGIN
      integrale = QROMB('subfunc_model', 0.,r_cut)*catalog.rc
    ENDELSE
    catalog.tau0 = 2*catalog.ne0*integrale*sigma_thomson*Mpc2m*1e6
  ENDIF
  IF NOT KEYWORD_SET(Te) THEN BEGIN
    catalog.Te = REFORM([CLUSTER_MASS2TEMP(z_array, M_array, cosmo=cosmo, cluster=cluster, M500_array=M500_array, xmm=xmm, chandra=chandra, norm=norm)])
  ENDIF
  catalog.y0 = catalog.tau0*catalog.Te/electron_mass
ENDIF

RETURN, catalog

closing:
END
