; -----------------------------------------------------------------------------
;
; Copyright (C) 2008, 2010
;   Alexandre Boucaud
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
;-
;+
; NAME:
;  NFW_PROFILE
; ONE LINE HELP:
;  compute the y and tau values at distances r_array*rs from the centers
; PURPOSE:
;  compute the y and tau values at distances r_array*scale radii
;  from the cluster centers
; CALLING SEQUENCE:
;  result = NFW_PROFILE(z_array, M_array, r_array,
;           cosmo=cosmo, cluster=cluster, r_cut=r_cut,
;           chandra=chandra, norm=norm, xmm=xmm,
;           nfw_a=nfw_a, nfw_b=nfw_b, nfw_c=nfw_c, c500=c500, P0=P0,
;           cm500=cm500, nr500=nr500, nstd=nstd, cco=cco, mdi=mdi)
;
; INPUT:
;  z_array : the array containing cluster redshifts z
;  M_array : the array containing cluster masses M (Mpc)
;  r_array : the points (in rs) where you want to compute tau and y
;
; OUTPUT:
;  The catalog of clusters with
;      result(i).tau_array[j] : tau computed for the cluster i
;                               at distance r_array[j]*rs[i] from the center
;      result(i).y_array[j] : y computed for the cluster i
;                             at distance r_array[j]*rs[i] from the center
;      result(i).rad_array[j] : r_array[j]*rs[i] (in Mpc)
;
; KEYWORDS:
;  cosmo : the structure containing the cosmological parameters.
;  cluster : the structure containing the structure formation parameters (cf. DEFINE_CLUSTER)
;  r_cut : the radius (in number of rs) where you stop your nfw profile
;            default : r_cut = r_vir/rs
;  chandra : set this keyword if you want to use the nfw-profile fitted on chandra observations (Nagai et al 2007, ApJ 668:1-14)
;            There are some bugs in that paper : for example, P0=12.2 and not 3.3 for Chandra Obs. in Table A1 -> see erratum
;  xmm : set this keyword if you want the to use teh nfw-profile fitted on xmm observations (Arnaud et al. 2008)
;        Fits XMM, courtesy of M. Arnaud on the REXCESS sample 
;  norm : set this keyword if you want to normalise the profile on the chandra or xmm observations and not on the model.
;  nfw_a, nfw_b, nfw_c : parameters of the NFW generalized profile (you must set xmm or chandra keyword to use these keywords)
;  c500 : concentration rs/r500 of the pressure profile (you must set xmm or chandra keyword to use this keyword)
;  P0 : normalisation of the pressure profile (you must set xmm or chandra keyword to use this keyword)
;  cm500 : concentration rs/r500 of the matter profile (you must set xmm or chandra keyword to use this keyword)
;  nr500 : integration is performed up to n*r500 instead of rvir (you must set the xmm or chandra keyword to use this keyword)
;  nstd : non standard XMM profile (you must set the xmm to use this keyword)
;  cco : cool core profile (if nstd keyword is set)
;  mdi : morphologically disturbed profile (if nstd keyword is set)
; EXAMPLES:
;
; RESTRICTIONS:
;  Not fully tested yet. Report any bugs to Jean-Baptiste Melin
;  jean-baptiste.melin@cea.fr
;
; PROCEDURES CALLED:
;  NFW_FUNCTION, CLUSTER_MODEL
; REVISION HISTORY
;  Written, Jean-Baptiste Melin & Alexandre Boucaud,  June 2008
;  cosmetic improvements, JBM, June 27 2008
;  bug in integration fixed, JBM, July 24 2008
;  c500 and P0 implemented, JBM, Sept. 30 2008
;  cm500 keyword added, JBM, Oct. 6 2008
;  nr500 keyword added, JBM,  Dec 4 2008
;  nstd keyword added, JBM, Sept 13 2010
;  cco and mdi keyword added, JBM, Oct 20 2010
;-

;-----------------------------------------------------------------------------
FUNCTION NFW_FUNCTION, theta
;----------------------------
 COMMON NFW_FUNCTION_PARAMETERS, nfw_function_delta, $
                                 nfw_function_a, nfw_function_b, nfw_function_c

 RETURN, (1+(nfw_function_delta/sin(theta))^nfw_function_a)^((nfw_function_c - nfw_function_b)/nfw_function_a) * $
         ((nfw_function_delta/sin(theta))^(-nfw_function_c)) / (sin(theta))^2

END
;------------------------------------------------------------------------------

FUNCTION NFW_PROFILE, z_array, M_array, r_array, cosmo=cosmo, $
                      cluster=cluster, r_cut=r_cut, $
                      chandra=chandra, norm=norm, xmm=xmm, $
                      nfw_a=nfw_a, nfw_b=nfw_b, nfw_c=nfw_c, c500=c500, P0=P0, $
                      cm500=cm500, nr500=nr500, nstd=nstd, cco=cco, mdi=mdi

IF ( N_PARAMS() EQ 0 ) THEN BEGIN
 PRINT,'CALL IS : result = NFW_PROFILE(z_array, M_array, r_array)'
 PRINT,'Keywords : cosmo=cosmo, cluster=cluster, r_cut=r_cut, chandra=chandra, norm=norm, xmm=xmm'
 PRINT,'           nfw_a=nfw_a, nfw_b=nfw_b, nfw_c=nfw_c, c500=c500, P0=P0, cm500=cm500, nr500=nr500'
 PRINT,'           nstd=nstd, cco=cco, mdi=mdi'
 GOTO, closing
ENDIF

Mpc2m = 3.0856775807d+22 ; in m
c_light = 2.9979246d+08 ; in m/s
electron_mass = 0.51099906*1d3 ; in keV
sigma_thomson = 0.665245854d-28 ; in m^2

; format the result
step_nb = N_ELEMENTS(r_array)
cluster_nb = N_ELEMENTS(z_array)
amas = {y_array:DBLARR(step_nb),tau_array:DBLARR(step_nb), $
        rad_array:DBLARR(step_nb)}
result = REPLICATE(amas,cluster_nb)

IF KEYWORD_SET(chandra) OR KEYWORD_SET(xmm) THEN BEGIN
  catalog = CLUSTER_MODEL(z_array, M_array, cosmo=cosmo, cluster=cluster, $
                          /Te, /ne0, /rs, r_cut=r_cut, $
                          chandra=chandra, xmm=xmm, norm=norm, $
                          nfw_a=nfw_a, nfw_b=nfw_b, nfw_c=nfw_c, c500=c500, P0=P0, cm500=cm500, nstd=nstd, cco=cco, mdi=mdi)

  IF KEYWORD_SET(nr500) THEN BEGIN
    save_r_cut = r_cut
    r_cut=REPLICATE(nr500*c500, N_ELEMENTS(z_array))
  ENDIF

ENDIF ELSE BEGIN
  PRINT, 'Keyword chandra or xmm is needed'
  GOTO, closing
ENDELSE

COMMON NFW_FUNCTION_PARAMETERS, nfw_function_delta, $
                                nfw_function_a, nfw_function_b, nfw_function_c

nfw_function_a = nfw_a
nfw_function_b = nfw_b
nfw_function_c = nfw_c

FOR i=0L, step_nb-1 DO BEGIN
  delta = r_array[i]
  integrale = DBLARR(N_ELEMENTS(z_array))
  so = SORT(r_cut)
  theta_min = REPLICATE(!pi/2., N_ELEMENTS(z_array))
  whs = WHERE(delta/r_cut LT 1., nwhs)
  IF nwhs GE 1 THEN theta_min[whs]=ASIN(delta/r_cut[whs])
  nfw_function_delta = delta
  integrale[so[0]] = QROMB('nfw_function', theta_min[so[0]], !pi/2.)*delta
  FOR j = 1L, N_ELEMENTS(z_array)-1 DO BEGIN
    integrale[so[j]]=integrale[so[j-1]]+QROMB('nfw_function', theta_min[so[j]], theta_min[so[j-1]])*delta
  ENDFOR
  result.tau_array[i] = 2*catalog.ne0*integrale*catalog.rs*sigma_thomson*Mpc2m*1d6
  result.y_array[i] = result.tau_array[i]*catalog.Te/electron_mass
  result.rad_array[i] = delta*catalog.rs
ENDFOR

IF KEYWORD_SET(nr500) THEN BEGIN
 r_cut = save_r_cut
ENDIF

RETURN, result

closing:
END
