; -----------------------------------------------------------------------------
;
; Copyright (C) 2007-2012
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;     CLUSTER_FIR_SOURCE_MAPS
; ONE LINE HELP:
;     Produce fir source maps at specified frequencies
; PURPOSE:
;     Produce fir source maps at specified frequencies
; CALLING SEQUENCE:
;     CLUSTER_FIR_SOURCE_MAP, catalog, frequencies, flim=flim, cosmo=cosmo, iseed=iseed, dir=dir, map_dir=map_dir, chandra=chandra, xmm=xmm
; INPUT:
;     catalog : the catalogue of clusters
;     frequencies : the frequencies in GHz
; OUTPUT:
;     fir_sources : a structure with
;        * fir_sources[i].cluster : the number of the cluster (cluster parent number) that contains the radio source i
;        * fir_sources[i].lum : the luminosity of the radio source i (10^12 Lsol unit) 
;        * fir_sources[i].type : 'normal' or 'starburst'
;        * fir_sources[i].flux : the flux (in MJy) of the radio source i for each PSM frequency
;        * fir_sources[i].lon : the longitude of the radio source i
;        * fir_sources[i].lat : the latitude of the radio source i
; KEYWORDS:
;     cosmo : the cosmology
;     iseed : the random init seed
;     dir : the directory where the luminosity functions and the spectral energy distributions are (Lagache, Dole & Puget)
;     flim : the flux limit (MJy) for the far infrared sources
;     map_dir : directory where the point source maps are created
;     chandra/xmm : to be set if the cluster modelization rely on Chandra or XMM
; EXAMPLES:
;     
; ALGORITHM:
;     
; SIDE EFFECTS:
;     
; RESTRICTIONS:
;     Report any bugs to Jean-Baptiste Melin
;     jean-baptiste.melin@cea.fr
; PROCEDURES CALLED:
;     CLUSTER_FIR_SOURCES_LUMINOSITY
; REVISION HISTORY
;     Written from cluster_fir_sources.pro, Jean-Baptiste Melin, October 21 2008
;     frequencies are now an input, JBM, October 28 2008
;-
FUNCTION CLUSTER_FIR_SOURCE_MAPS, catalog, frequencies, cosmo=cosmo, iseed=iseed, dir=dir, flim=flim, map_dir=map_dir, nside=nside, chandra=chandra, xmm=xmm, verbose=verbose


IF NOT KEYWORD_SET(dir) THEN dir = dialog_pickfile(/dir)
dir = FIX_SEPARATOR(dir)

z_array = catalog.z
M_array = catalog.mass
fir_sources_ltf = CLUSTER_FIR_SOURCES_LTF(z_array, M_array, frequencies, fir_total_flux, cosmo=cosmo, iseed=iseed, dir=dir, flim=flim, verbose=verbose)
nsources = N_ELEMENTS(fir_sources_ltf)

clusternum = fir_sources_ltf.cluster
uclusternum = clusternum[UNIQ(clusternum, SORT(clusternum))]
r = FLTARR(nsources)

IF KEYWORD_SET(verbose) THEN print, 'Drawing the source position...'

IF KEYWORD_SET(xmm) OR KEYWORD_SET(chandra) THEN BEGIN
  res=CLUSTER_MODEL(0., 1., /rs, cosmo=cosmo, cluster=DEFINE_CLUSTER(cosmo=cosmo, /silent), chandra=chandra, xmm=xmm, $
                    nfw_a=nfw_a, nfw_b=nfw_b, nfw_c=nfw_c)  
  scale_radii = catalog[uclusternum].rs
  nsr_max = 100.
  step_r = 0.01
  radii_max = nsr_max*scale_radii
  FOR i=0, N_ELEMENTS(uclusternum)-1 DO BEGIN
    wh = WHERE(fir_sources_ltf.cluster EQ uclusternum[i], nwh)
    radius = (FINDGEN(radii_max[i]/step_r)+1)*step_r
    x = radius/scale_radii[i]
    proba = (1+x^(nfw_a))^((nfw_c - nfw_b)/nfw_a)*x^(-nfw_c)
    pix = RANDOM_DRAW(proba, nwh, /silent, iseed=iseed)
    r[wh] = radius[pix]
  ENDFOR
ENDIF ELSE BEGIN
  core_radii = catalog[uclusternum].rc
  ncr_max = 100.
  step_r = 0.01
  radii_max = ncr_max*core_radii
  FOR i=0, N_ELEMENTS(uclusternum)-1 DO BEGIN
    wh = WHERE(fir_sources_ltf.cluster EQ uclusternum[i], nwh)
    radius = FINDGEN(radii_max[i]/step_r)*step_r
    proba = (1.+(radius/core_radii[i])^2.)^(-3./2.)
    pix = RANDOM_DRAW(proba, nwh, /silent, iseed=iseed)
    r[wh] = radius[pix]
  ENDFOR
ENDELSE

theta = ACOS(2*RANDOMU(iseed, nsources)-1)
phi = 2.*!pi*RANDOMU(iseed, nsources)
x = r*SIN(theta)*COS(phi)
y = r*SIN(theta)*SIN(phi)
redshift = catalog[clusternum].z
ANG2SIZE, dlon, x, redshift, /reverse, cosmo=cosmo
ANG2SIZE, dlat, y, redshift, /reverse, cosmo=cosmo
longitudes = FLTARR(nsources)
latitudes = FLTARR(nsources)
longitudes = catalog[clusternum].lon + dlon
latitudes = catalog[clusternum].lat + dlat

nband=N_ELEMENTS(frequencies)
node = {cluster:0L, $
        lum:0d, $
        type:'normal', $
        flux:DBLARR(nband), $
        lon:0.0, $
        lat:0.0}
result = REPLICATE(node,nsources)
result.cluster = fir_sources_ltf.cluster
result.lum = fir_sources_ltf.lum
result.type = fir_sources_ltf.type
result.flux = fir_sources_ltf.flux
result.lon = longitudes
result.lat = latitudes

new_fir_total_flux = fir_total_flux
FOR i=0, nsources-1 DO BEGIN
  new_fir_total_flux[*,result[i].cluster]=new_fir_total_flux[*,result[i].cluster]-result[i].flux
ENDFOR

whlim = WHERE(new_fir_total_flux GE flim, nwhlim) ; selection of clusters to be included in the map as diffuse component
indices = ARRAY_INDICES(new_fir_total_flux, whlim)
reform_indices = REFORM(indices[1,*])
uind = reform_indices[UNIQ(reform_indices,SORT(REFORM(reform_indices)))]
nuind = N_ELEMENTS(uind)

subcat = catalog[uind]

IF NOT KEYWORD_SET(nside) THEN nside=2048

map_dir = FIX_SEPARATOR(map_dir)

IF KEYWORD_SET(verbose) THEN print, 'Creating the source maps...'

FOR i=0, nband-1 DO BEGIN
  ;print, i
  subcat.cy = REFORM(new_fir_total_flux[i,uind])*(60./!dtor)^2. ; convert MJy in MJyarcmin^2/sr
  CREATE_Y_TAUBETA_MAP_ALLSKY, nside, subcat, y_map, taubeta_map, $
                               taubeta2_q_map, taubeta2_u_map, $
                               cosmo=cosmo, cluster=DEFINE_CLUSTER(cosmo=cosmo, /silent), chandra=chandra, xmm=xmm, $
                               polar=0
  name_map = 'cluster_faint_fir_source_map_'+STRTRIM(STRING(FIX(frequencies[i])),1)+'GHz.fits'
  ; Units MJy/sr
  WRITE_FITS_MAP, map_dir+name_map, y_map, /Ring
  ;npix=NSIDE2NPIX(nside)
  ;spix = 4.*!pi/npix ; sr
  ;theta = !pi/2-latitudes*!dtor
  ;phi = longitudes*!dtor
  ;ANG2PIX_RING, nside, theta, phi, ipix
  ;new_y_map = 0*y_map
  ;FOR j=0, nsources-1 DO BEGIN
  ;  new_y_map[ipix[j]]=new_y_map[ipix[j]]+(result[j].flux)[i]/spix
  ;ENDFOR
  ;new_name_map = 'cluster_strong_fir_source_map_'+STRTRIM(STRING(FIX(central_frequency[i])),1)+'GHz.fits'
  ; Units MJy/sr
  ;WRITE_FITS_MAP, map_dir+new_name_map, new_y_map, /Ring
ENDFOR

RETURN, result

END
