; -----------------------------------------------------------------------------
;
; Copyright (C) 2008
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;     CLUSTER_FIR_SOURCES_TAB_FLUX
; ONE LINE HELP:
;     Compute the tabulated flux (MJy) of cluster far infrared sources as a function of their luminosities,
;     redshifts, types and PSM frequencies.
; PURPOSE:
;     Compute the tabulated flux (MJy) of cluster far infrared sources.
; CALLING SEQUENCE:
;     fir_sources_flux = CLUSTER_FIR_SOURCES_TAB_FLUX(frequencies, cosmo=cosmo, dir=dir, planck_dir=planck_dir)
; INPUT:
;     frequencies : PSM frequencies in GHz
; OUTPUT:
;     fir_sources_flux : the structure containing flux (MJy), luminosities, redshifts, types and PSM frequencies
; KEYWORDS:
;     cosmo : the cosmology
;     dir : the directory where the luminosity functions are (Lagache, Dole & Puget)
;     planck_dir : the directory where the Planck band model is located
; EXAMPLES:
;     
; ALGORITHM:
;     
; SIDE EFFECTS:
;     
; RESTRICTIONS:
;     Report any bugs to Jean-Baptiste Melin
;     jean-baptiste.melin@cea.fr
; PROCEDURES CALLED:
;     ANG_DIST
; REVISION HISTORY
;     Written from cluster_fir_sources_flux.pro, Jean-Baptiste Melin, October 21 2008
;     frequencies added as an input, JBM, October 28 2008
;-
FUNCTION CLUSTER_FIR_SOURCES_TAB_FLUX, frequencies, cosmo=cosmo, dir=dir, planck_dir=planck_dir


; PLANCK_MODEL and PSM_BANDINTEG are commented with tripe odot = ";;;"
; If you want to reinclude them, remove the triple odot and comment line having triple odot at the end 

IF NOT KEYWORD_SET(dir) THEN dir = dialog_pickfile(/dir)
dir = FIX_SEPARATOR(dir)

sed = MRDFITS(dir+'sed.fits', 1, /DSCALE, /SILENT)
lum_array = 10.^(sed.loglum-12.)
nl = N_ELEMENTS(lum_array)

fctlum = MRDFITS(dir+'fctlum.fits', 1, /DSCALE, /SILENT)
z_array = fctlum.z
nz = N_ELEMENTS(z_array)

ntype = 2 ; 'normal' and 'straburst'

;central_frequency = [30.,44.,70.,100.,143.,217.,353.,545.,857.]
central_frequency = FLOAT(frequencies)
nband = N_ELEMENTS(central_frequency)
;;;CREATE_PLANCK_TOYMODEL1, planck_dir, /normalise_band

Mpc2m = 3.0856775807d+22 ; in m
lum_dist = (1.+z_array)^2.*ANG_DIST(z_array, cosmo=cosmo) ; luminosity distance in Mpc
lum_dist_array = lum_dist*Mpc2m ; luminosity distance in m

lambda = sed.lam ; m
c_light = 299792458d0 ; m/s
;;;nu = c_light/lambda ; Hz
;;;nmax=9999
top_hat = 1/3. ; Planck transmission filter is considered as a top-hat filter with delta_lambda/lambda=1/3 ;;;
n_lam = N_ELEMENTS(sed.lam) ;;;
int_lam = DBLARR(n_lam) ;;;

fir_flux = DBLARR(ntype,nband,nz,nl)

FOR i=0, nband-1 DO BEGIN
  ;print, i
  ;;;filter = MRDFITS(planck_dir+'planck_toymodel1_band_'+STRTRIM(STRING(FIX(central_frequency[i])),1)+'GHz.fits', 1, hd, /SILENT)
  lam_0 = c_light/(central_frequency[i]*1d9) ;;;
  FOR j=0, nz-1 DO BEGIN
    ;;;usenu = nu[0:nmax]/(1+z_array[j])
    lam=(1+z_array[j])*sed.lam ;;;
    wha = WHERE(lam GE lam_0-top_hat*lam_0/2. AND lam LE lam_0+top_hat*lam_0/2., nwha) ;;;
    dlam = lam[wha[1:nwha-1]]-lam[wha[0:nwha-2]] ;;;
    dlam = [dlam, dlam[nwha-2]] ;;;
    FOR k=0, nl-1 DO BEGIN
      ;;;usevalue = DOUBLE(REFORM((sed.sed_normal)[k,0:nmax]))
      ;;;spectrum = {nu:usenu,value:usevalue}
      ;;;fir_flux[0,i,j,k] = PSM_BANDINTEG(filter, spectrum)
      ;;;usevalue = DOUBLE(REFORM((sed.sed_sb)[k,0:nmax]))
      ;;;spectrum = {nu:usenu,value:usevalue}
      ;;;fir_flux[1,i,j,k] = PSM_BANDINTEG(filter, spectrum)
      int_lam = (sed.sed_normal)[k,*] ;;;
      fir_flux[0,i,j,k]=TOTAL(int_lam[wha]*dlam) ;;;
      int_lam = (sed.sed_sb)[k,*] ;;;
      fir_flux[1,i,j,k]=TOTAL(int_lam[wha]*dlam) ;;;
    ENDFOR
    ;;;fir_flux[*,i,j,*]=fir_flux[*,i,j,*]*(1+z_array[j])/4/!pi/lum_dist_array[j]^2. ; in Wm^-2Hz^-1
    fir_flux[*,i,j,*]=fir_flux[*,i,j,*]/TOTAL(dlam)*(1+z_array[j])/4/!pi/lum_dist_array[j]^2. ; in Wm^-2Hz^-1
  ENDFOR
ENDFOR

res = {type:['normal', 'starburst'], band:central_frequency, $
       z:z_array, lum:lum_array, flux:fir_flux/1d-20} ; in MJy i.e 1d6 Jy

RETURN, res

END
