; -----------------------------------------------------------------------------
;
; Copyright (C) 2006-2008
;   Jean-Baptiste Melin
;   Etienne Pointecouteau
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME: hod.pro
;
; PURPOSE: compute the average Halo Occupation Number (or Halo Occupation Density)
;          for the input mass(es)
;          Default output is an average of Nocc estimated from
;          Lin et al 2004 and Popesso et al 2007
;
; CATEGORY: astrophysics
;
; CALLING SEQUENCE:
;      result=hod(mass)
;
; INPUTS:
;      mass -- float, scalar of array, masses in Msol
;
;
; KEYWORD PARAMETERS:
;      redshift -- float, scalar of array, redshifts
;      lin -- boolean, use Lin et al 2004 HOD
;      popesso -- boolean, use Popesso et al 2006 HOD
;      lanoux -- boolean, use Lanoux 2007 from SDSS Koester at al. 2007 catalogue
;      zheng3p -- boolean, use Zheng et al 2006 HOD: 3-parametres
;      zheng5p -- boolean, use Zheng et al 2006 HOD: 5-parametres
;      plot -- boolean, get a plot
;      verbose -- boolean, chatty programme
;      cosmo -- cosmo structure
;      reverse -- boolean, reverse the relation: derive mass from n_200
;
; OUTPUTS:
;      occupation number for the input masses
;
; RESTRICTIONS:
;      See the involved papers !
;
; MODIFICATION HISTORY:
;      2006.12.19 -- Etienne Pointecouteau, CESR
;                    (pointeco@cesr.fr)
;      2007.08.27 -- z added as an input to take into account the evolution
;                 (self similar expectation: Nocc \propto E(z)^-1 M)
;      2007.09.07 -- z transformed into a keyword as the Nocc should remain
;            constant for a given mass whatever the redshift. Self-similar
;            evolution can still be forces by using the keyword z.
;      2008.10.16 -- adapted to the PSM by Jean-Baptiste Melin (jbmelin@cea.fr)
;                    bug in z keyword fixed : z-> redshift
;                    h0 keyword removed : now h0 is in the cosmo structure
;                    
;-
function hod, mass, redshift=redshift $
              ,lin=lin,popesso=popesso,lanoux=lanoux,zheng3p=zheng3p,zheng5p=zheng5p $
              ,plot=plot,verbose=verbose $
              ,cosmo=cosmo,silent=silent,reverse=reverse


IF n_params() LT 1 THEN BEGIN
   print
   print,' ERROR - Number of input parameter is wrong !'
   print,' resu=hod(mass)'
   return,-1
ENDIF

; Reduce Hubble constant (H0/[100km/s/Mpc])
;  if not keyword_set(h0) then BEGIN
;     cosmo={name:'LCDM',h0:0.7,omega_m:0.3,omega_l:0.70,omega_b:0.045,n:0.953,sigma8:0.9}
;     hh=cosmo.h0
;  ENDIF ELSE   hh=h0
IF NOT KEYWORD_SET(cosmo) THEN BEGIN ; JBM adapted to the PSM
   cosmo=DEFINE_COSMO(h=0.7,omega_m=0.3,omega_de=0.70,omega_b=0.045,/silent) ; JBM adapted to the PSM
   hh=cosmo.h ; JBM adapted to the PSM
ENDIF ELSE hh=cosmo.h ; JBM adapted to the PSM

; Check mass boundaries
  IF keyword_set(verbose) THEN BEGIN
     mima=minmax(mass)
     IF mima[0] LT 1.e14 THEN BEGIN
        print
        print,'WARNING - reliability of Nocc for M < 1.e-14 Msol is low !'
     ENDIF
  ENDIF

; Implemented HOD
  name=['Lin et al. 2004, ApJ 650, L99' $
        ,'Popesso et al. 2006, astroph/0606260' $
        ,'Lanoux 2007 on SDSS Koester et al. 2007' $
        ,'Zheng et al., 2006, ApJ 633, 791: 3 param' $
        ,'Zheng et al., 2006, ApJ 633, 791: 5 param']

  nname=n_elements(name)

; Angular distances & self similar evolution factor
  IF keyword_set(redshift) THEN BEGIN
     ;da=dist_ang(redshift,cosmo=cosmo,ez=ez,/silent)
     da=ANG_DIST(redshift,cosmo=cosmo) ; JBM adapted to the PSM
     ez=HUBBLE_AT_Z(redshift,cosmo=cosmo)/100./cosmo.h ; JBM adapted to the PSM
     IF keyword_set(verbose) THEN BEGIN
        PRINT,'WARNING -  Forcing evolution to self-similar'
        print,'           However, Nocc is the same for a given mass whatever the redshift'
        print,'           Shoud not need added evolution'
     ENDIF
  ENDIF ELSE ez=mass*0.+1

  hz=hh*ez

; Mass in 10^14Msol units
  IF keyword_set(reverse) THEN N200=mass ELSE   M14=mass/1.e14

; Lin et al 2004, 610, 745
;;; 93 2MASS clusters in K-band
;;; z < 0.1
;;; X-ray temperatures and mass
;;; including BCG
;;; Work in h_70
  zmed=0.0416
  ;cosmorel={h0:0.7,omega_m:0.3,omega_l:0.7}
  ;darel=dist_ang(zmed,cosmo=cosmorel,dl=dl,/silent)
  ;da0=dist_ang(zmed,cosmo=cosmo,dl=dl,/silent)
  ;corcosmo=da0/darel
  cosmorel=DEFINE_COSMO(h=0.7, omega_m=0.3, omega_de=0.7, omega_b=0.04, /silent) ; JBM adapted to the PSM
  darel=ANG_DIST(zmed,cosmo=cosmorel) ; JBM adapted to the PSM
  da0=ANG_DIST(zmed,cosmo=cosmo) ; JBM adapted to the PSM
  corcosmo=da0/darel ; JBM adapted to the PSM
;  h0rel=0.7
  IF keyword_set(reverse) THEN BEGIN
     Nlin=N200
     M14lin=(Nlin/37.)^(1./0.85)
     M14lin_p=(Nlin/(37.-3.))^(1./(0.85-0.04))
     M14lin_m=(Nlin/(37.+3.))^(1./(0.85+0.04))
     M14=M14lin/corcosmo       ; correct from cosmo
     nocc=M14*1.e14
  ENDIF ELSE BEGIN
     M14lin=M14*corcosmo        ; correct from cosmo
     Nlin=37.*(M14lin)^0.85
     nlin_p=(37+3.)*M14lin^(0.85+0.04)
     nlin_m=(37.-3)*M14lin^(0.85-0.04)
     nlin=nlin                    ;*h0rel/hh              ; H0 dependency
     nocc=nlin                    ; Nocc at z=0
  ENDELSE


; Popesso et al 2007, 464, 451
;;; 217 SDSS clusters in R-band
;;; z < 0.3
;;; using M-L relation to get the mass
  zmed=0.1
  ;cosmorel={h0:0.7,omega_m:0.3,omega_l:0.7}
  ;darel=dist_ang(zmed,cosmo={h0:0.7,omega_m:0.3,omega_l:0.7},dl=dl,/silent)
  ;da0=dist_ang(zmed,cosmo=cosmo,dl=dl,/silent)
  ;corcosmo=da0/darel
  cosmorel=DEFINE_COSMO(h=0.7, omega_m=0.3, omega_de=0.7, omega_b=0.04, /silent) ; JBM adapted to the PSM
  darel=ANG_DIST(zmed,cosmo=cosmorel) ; JBM adapted to the PSM
  da0=ANG_DIST(zmed,cosmo=cosmo) ; JBM adapted to the PSM
  corcosmo=da0/darel ; JBM adapted to the PSM
;  h0rel=0.7
  IF keyword_set(reverse) THEN BEGIN
     Npop=N200
     M14pop=(Npop/20.4174)^(1./0.91)
     M14pop_p=(Npop/5.24808)^(1./(0.91-0.04))
     M14pop_m=(Npop/79.4329)^(1./(0.91+0.04))
     M14=M14pop/corcosmo        ; correct from cosmo
     npop=M14*1.e14
  ENDIF ELSE BEGIN
     M14pop=M14*corcosmo        ;hz/h0rel*da0/darel ; correct from cosmo
     npop=20.4174*M14pop^(0.91) ;10.^(-11.43+14*0.91)
     npop_p= 79.4329*M14pop^(0.92+0.04) ;10.^(-11.43+0.59+14*0.91)
     npop_m=5.24808*M14pop^(0.92-0.04)  ;10.^(-11.43-0.59+14*0.91)
     npop=npop                          ;*h0rel/hh              ; H0 dependency
  ENDELSE

  nocc=[[nocc],[npop]]

; Lanoux 2007 [Stage de M1]
;;; 13807 amas
;;; 0.1 < z < 0.3
;;; Masses from L_I & L_R
;;; including BCG
;;; Work in h_70
  zmed=0.2295
  ;cosmorel={h0:0.7,omega_m:0.3,omega_l:0.7}
  ;darel=dist_ang(zmed,cosmo=cosmorel,dl=dl,/silent)
  ;da0=dist_ang(zmed,cosmo=cosmo,dl=dl,/silent)
  ;corcosmo=da0/darel
  cosmorel=DEFINE_COSMO(h=0.7, omega_m=0.3, omega_de=0.7, omega_b=0.04, /silent) ; JBM adapted to the PSM
  darel=ANG_DIST(zmed,cosmo=cosmorel) ; JBM adapted to the PSM
  da0=ANG_DIST(zmed,cosmo=cosmo) ; JBM adapted to the PSM
  corcosmo=da0/darel ; JBM adapted to the PSM
;  h0rel=0.7
  IF keyword_set(reverse) THEN BEGIN
     Njl=alog10(N200)
     M14jl=10.^((Njl-1.53)/0.84)
     M14jl_p=10.^(Njl-(1.53-0.01)/(0.84-0.004))
     M14jl_m=10.^((Njl-(1.53+0.01))/(0.84+0.004))
     M14=M14jl/corcosmo       ; correct from cosmo
     njl=M14*1.e14
  ENDIF ELSE BEGIN
     M14jl=alog10(M14*corcosmo)        ; correct from cosmo
     Njl=1.53+0.84*(M14jl)
     njl_p=10.^((1.53+0.01)+M14jl*(0.84+0.004))
     njl_m=10.^((1.53-0.01)+M14jl*(0.84-0.004))
     njl=10.^njl                    ;*h0rel/hh              ; H0 dependency
  ENDELSE 

  nocc=[[nocc],[njl]]


; Zheng et al 2005: SPH & Semi Analytical models, 3 parameters
;;; 3-parametres
  zmed=0.1
  ;cosmorel={h0:1.0,omega_m:0.3,omega_l:0.7}
  ;darel=dist_ang(zmed,cosmo={h0:0.7,omega_m:0.3,omega_l:0.7},dl=dl,/silent)
  ;da0=dist_ang(zmed,cosmo=cosmo,dl=dl,/silent)
  ;corcosmo=da0/darel
  cosmorel=DEFINE_COSMO(h=0.7, omega_m=0.3, omega_de=0.7, omega_b=0.04, /silent) ; JBM adapted to the PSM
  darel=ANG_DIST(zmed,cosmo=cosmorel) ; JBM adapted to the PSM
  da0=ANG_DIST(zmed,cosmo=cosmo) ; JBM adapted to the PSM
  corcosmo=da0/darel ; JBM adapted to the PSM
;  h0rel=1.0

  MassZhg=mass*corcosmo;hz/h0rel
  logMass=alog10(MassZhg)

  siglogm=0.32
  logmin=11.77
  logm1prime3par=12.96
  alpha3par=1.04
  ncen=0.5*(1.+erf((logMass-logmin)/siglogm))
  nsat=(MassZhg/10.^logm1prime3par)^alpha3par
  nzheng = ncen+nsat
  nzheng3p = nzheng;*h0rel/hh    ; H0 dependency

  nocc=[[nocc],[nzheng3p]]      ; Nocc at z=0

;;; 5-parametres
  logmin=11.73
  siglogm=0.32
  logm0=12.09
  logm1prime3par=12.87
  alpha3par=0.96
  ncen=0.5*(1.+erf((logMass-logmin)/siglogm))
  nsat=((MassZhg-10.^logm0)/10.^logm1prime3par)^alpha3par
  nzheng = ncen+nsat
  nzheng5p = nzheng;*1./hh         ; H0 dependency

  nocc=[[nocc],[nzheng5p]]      ; Nocc at z=0


; Plot of occupation numbers @ z=0
  if keyword_set(plot) then begin
     vcol=80+findgen(nname)*(254-80)/(nname-1)
     plot_oo,m14,nlin,yr=[1,2000],xr=minmax(m14),/xsty,/ysty,/nodata $
             ,xtitle='M14 [Msol]',ytitle=' E(z) Nocc',charsize=1.5
     oplot,m14,nlin,col=vcol[0]
     oplot,m14,nlin_m,line=2,col=vcol[0]
     oplot,m14,nlin_p,line=2,col=vcol[0]
     
     oplot,m14,npop,col=vcol[1]
     oplot,m14,npop_m,line=2,col=vcol[1]
     oplot,m14,npop_p,line=2,col=vcol[1]
     
     oplot,m14,njl,col=vcol[2]
     oplot,m14,njl_m,line=2,col=vcol[2]
     oplot,m14,njl_p,line=2,col=vcol[2]

     oplot,m14,nzheng3p,col=vcol[3]
     oplot,m14,nzheng5p,col=vcol[4]

     legend,name,/top,/left,textcol=vcol,box=0
     legend,'h0='+strtrim(hh,2),/bottom,/right,box=0
  endif

;; Apply self similar evolution - Nocc ~ E(z)^-1
  si=size(nocc)
  FOR i=0,si[2]-1 DO nocc[*,i]=nocc[*,i]/ez


; Which Nocc
  case 1 of
     keyword_set(lin): begin
        resu=(reform(nocc[*,0]))
        ref=name[0]
     end
     keyword_set(popesso): begin
        resu=(reform(nocc[*,1]))
        ref=name[1]
     end
     keyword_set(lanoux): begin
        resu=(reform(nocc[*,2]))
        ref=name[2]
     end
     keyword_set(zheng3p): begin
        resu=(reform(nocc[*,3]))
        ref=name[3]
     end
     keyword_set(zheng5p): begin
        ref=name[4]
        resu=(reform(nocc[*,4]))
     end
     else: begin
        ; Default output is a composite from Lin et al. 2004 & Popesso et al 2007
        resu= (reform((nocc[*,0]+nocc[*,1])/2.))
        ref='average of '+name[0]+' and '+name[1]
     end
  endcase

  IF NOT keyword_set(reverse) THEN resu=round(resu)

  if keyword_set(verbose) then begin
     print
     print,' --> Occupation number from: '+ref
     ;print,'          with h0 =',cosmo.h0
     ;print,'          omega_m =',cosmo.omega_m
     ;print,'          omega_l =',cosmo.omega_l
     print,'          with h0 =',cosmo.h
     print,'          omega_m =',cosmo.omega_m
     print,'          omega_l =',cosmo.omega_de
     print
  endif


return, float(resu)
end
