; -----------------------------------------------------------------------------
;
; Copyright (C) 2007-2009, 2011
;   Jacques Delabrouille
;   Marc Betoule
;   Maude Le Jeune
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; Provides a wrapper to the sz simulation code
;
; @categories pipe
;
; @param gp {in}{required}{type=global_parameters} general parameters of the simulation
; @param cp {in}{required}{type=cosmological_parameters} cosmological parameters 
; @param szp {in}{required}{type=sz_parameters} sz code specific
; @param seed {in}{required}{type=int} seed value (only use in simulation case)
; parameters
; @keyword test {in}{optional}{type=boolean} If set, the code just
; check the set of parameters, no calculation is done.
; @keyword debug {in}{optional}{type=boolean} If set no catch on error
; done, subdirectory components/sz is not removed when an error occurs
;
;
; @history <p>Created january 2007, Marc Betoule</p>
; @history <p>Changed units for the thermal SZ effect (now y_sz) and sign of kinetic SZ (now T/TCMB), JD, 01 feb 2007</p>
; @history <p>Implementation of the nbody+hydro model in the PSM pakage, JD, 02 feb 2007</p>
; @history <p>Pipe modification : split prediction and simulation, February 5 2007, Marc Betoule </p>
; @history <p>Add the test keyword, february 6 2007, Marc</p>
; @history <p>Fix T_star to 1.5 (should be fix by sigma_8 later), 26/05/2007
; @history <p>Seed is an input parameter and is set @ psm_main july 12 2007, Maude </p>
; @history <p>Header of the SZ output catalog changed july 17 2007, JB Melin</p>
; @history <p>New modification of the header july 18 2007, JB Melin</p>
; @history <p>Added cluster contamination by radiosources in DMB_201 model, july 18 2007, Jacques Delabrouille</p>
; @history <p>Added tunable limit cluster mass in DMB_2001 model, july 18 2007, Jacques Delabrouille</p>
; @history <p>Fixed bug for nbody+hydro model, JD+MB, 04 september 2007</p>
; @history <p>NFW profile implemented, Jean-Baptiste Melin, July 24 2008</p>
; @history <p>name changed from nfw_profile to cluster_profile, Jean-Baptiste Melin, Sept 12 2008</p>
; @history <p>Cluster far infrared sources included, Jean-Baptiste Melin, October 22 2008</p>
; @history <p>Frequencies are now an input, JBM, October 28 2008</p>
; @history <p>Bug in cluster catalog header when beta and nr500 keywords are set is now fixed, JBM, January 5 2009</p>
; @history <p>08/04/2009: Now takes all PSM params as an input, Jacques Delabrouille</p>
; @history <p>Names of Ancillary data changed, JBM, June 12 2009</p>
; @history <p>Code updated to last catalogues and models, JBM, February 1st 2011</p>
; @history <p>Hydro catalogue added to the hydro+dmb model, JBM, November 24th 2011</p>
; @history <p>NSTD keyword added, JBM, June 14th 2012</p>
;-

PRO SZ_PIPE, debug=debug

;;****************************************************************************************************
;; Params processing
;;****************************************************************************************************

  COMMON psm, param
  COMMON sky, model

  ;; error handling
  ;;----------------
  IF PSM_DEBUG_VALUE() EQ 0 THEN BEGIN
     CATCH, error_status
     IF error_status NE 0 THEN BEGIN
        PSM_INFO_MESSAGE, 'An error was encountered while computing SZ map.'
        CATCH, /CANCEL
        MESSAGE, !ERR_STRING
     ENDIF
  ENDIF 

  ;; Definitions and initialisation
  ;;--------------------------------
  gp = param.global
  pixp = SKY_PIX()
  names=param.names
  szp = model.sz
  runp = param.run
  cp = model.cosmo
  cosmo = cp
  ;cosmo = DEFINE_COSMO(h=cp.h, omega_m=cp.Omega_m, omega_b=cp.Omega_b, lambda=cp.Omega_de, /silent)  
  componentdir = GETPSMDIR('sz')

  sz_cat_file = GETPSMDIR('sz')+names.sz_basename+'_cat.fits'

  SET_PSM_SEEDCHAIN, 'sz-pipe'
 
  ;; Model-dependent parameters common to more than one model (cluster profile)
  ;;----------------------------------------------------------------------------
  sztags = TAG_NAMES(szp)

    ;; Cluster profile
    wh_cluster_profile = WHERE(sztags EQ 'CLUSTER_PROFILE',nwh)
    IF nwh NE 0 THEN BEGIN
       CASE STRLOWCASE(szp.cluster_profile) OF
          'xmm': BEGIN
             xmm=1 & chandra=0
          END
          'chandra': BEGIN
             xmm=0 & chandra=1
          END
          'beta': BEGIN
             xmm=0 & chandra=0
          END
       ENDCASE    
    ENDIF

    ;; NSTD profile
    wh_cluster_nstd_profile = WHERE(sztags EQ 'NSTD_PROFILE',nwh)
    IF nwh NE 0 THEN BEGIN
       CASE STRLOWCASE(szp.nstd_profile) OF
          'no': nstd=0
          'yes': nstd=1
       ENDCASE
    ENDIF

    ;; Norm profile
    wh_cluster_norm_profile = WHERE(sztags EQ 'NORM_PROFILE',nwh)
    IF nwh NE 0 THEN BEGIN
       CASE STRLOWCASE(szp.norm_profile) OF
          'no': norm=0
          'yes': norm=1
       ENDCASE
    ENDIF

    ;; Profile bounds
    wh_cluster_profile_bounds = WHERE(sztags EQ 'PROFILE_BOUNDS',nwh)
    IF nwh NE 0 THEN BEGIN
       CASE STRLOWCASE(szp.profile_bounds) of
          '5r500': nr500=5.
          'rvir':               ; default case
       ENDCASE
    ENDIF
  
    ;; Create the SZ directory
    ;;-------------------------
    componentdir = GETPSMDIR('sz',exists=exists)
    IF ((NOT exists) AND SZ_MODEL() ne 'no_sz') THEN FILE_MKDIR, componentdir
   
    IF runp.verbosity GE 1 THEN PSM_INFO_SUBTITLE, "MODELING SUNYAEV-ZEL'DOVICH EMISSIONS"

  ;; Error handling : remove this directory when catching an error
  ;;---------------------------------------------------------------
  IF PSM_DEBUG_VALUE() EQ 0 THEN BEGIN
     catch, error_status
     IF error_status ne 0 THEN BEGIN
        print, 'An error was encoutered while computing SZ maps.'
        print, 'Removing subdirectory ' +componentdir
        spawn, 'rm -r '+componentdir
        catch, /CANCEL
        message, !ERR_STRING
     ENDIF
  ENDIF

  ;; Sky beam
  ;;----------
  sky_beam=GET_BEAM_STRUCT('GAUSSIAN',beamsize=SKY_RESOLUTION(),sizeunits='arcmin')


;;****************************************************************************************************
;; Main processing of the various SZ models
;;****************************************************************************************************

  CASE SZ_MODEL() OF
     ;;-------------------------------------------------------------------------------------------------
     'no_sz': RETURN

     ;;-------------------------------------------------------------------------------------------------
     'prediction': BEGIN
      
        ;;cluster = DEFINE_CLUSTER(cosmo=cosmo,sigma_8=cp.Sigma_8,t_star=0.95*cp.Sigma_8^(-1.49))
        cluster = DEFINE_CLUSTER(cosmo=cosmo, t_star=szp.cluster_t_star, /silent)
 
        cats=WHERE(STRLEN(szp.sz_input_cat) NE 0, ncats)
        acats = szp.sz_input_cat[0:ncats-1]
        ucats = acats[UNIQ(acats, SORT(acats))]
        nucats = N_ELEMENTS(ucats)

        priority_ranking = ['rosat', 'sdss']
        icats = INTARR(nucats)
        icats[*]=N_ELEMENTS(priority_ranking)
        FOR i=0, nucats-1 DO BEGIN
          wh = WHERE(STRMATCH(priority_ranking, ucats[i]) EQ 1, nwh)
          IF nwh EQ 1 THEN BEGIN
            icats[i]=wh
          ENDIF
        ENDFOR
        indcats = SORT(icats)

        name_model = 'model'

        FOR numcat=0, nucats-1 DO BEGIN
          
           CASE ucats[indcats[numcat]] OF
             'rosat': BEGIN
                        ancillary_dir = GETPSMDIR('psmdata') + 'ancillary/observations/ROSAT/'
                        ancillary_cat = 'MCXC_for_PSM_v0.save' ;'rosat_cluster_catalog.fits'
                        optical=0
                        add_model = 'xray'
                      END
             'sdss': BEGIN
                       ancillary_dir = GETPSMDIR('psmdata') + 'ancillary/observations/SDSS/'
                       ancillary_cat = 'maxbcg_cluster_catalog.fits'
                       optical=1
                       add_model = 'optical'
                     END
           END

           name_model = add_model+'_'+name_model

           ANCILLARY2INTERMEDIATE_CLUSTER_CATALOG, ancillary_dir, ancillary_cat, $
                                                   intermediate_dir, intermediate_cat, $
                                                   cosmo=cosmo, cluster=cluster, catalog=sub_catalog, /nodisk, optical=optical, mcxc=1, nstd=1
  
           IF DEFINED(local_catalog) EQ 0 THEN local_catalog=sub_catalog ELSE local_catalog=MERGE_INTERMEDIATE_CLUSTER_CATALOGS(local_catalog, sub_catalog)

        ENDFOR

        CREATE_DETERMINISTIC_SZ_SIMULATION_ALLSKY, head, local_catalog, $
                                                   y_map, taubeta_map, taubeta2_q_map, taubeta2_u_map, $
                                                   cosmo=cosmo, cluster=cluster, $
                                                   polar=polar, xmm=xmm, chandra=chandra, $
                                                   norm=norm, nr500=nr500, nstd=nstd, /nomap
        
        nclusters = N_ELEMENTS(local_catalog)
        sz_cat = CREATE_SZ_MODEL_CAT(name_model,nclusters,szp=szp,/usenan,header=sz_header)        
        FILL_SZ_MODEL_CAT, sz_cat, local_catalog, szp=szp        
        WRITE_SZ_MODEL_CAT, sz_cat_file, sz_cat, sz_header=sz_header, psm_header=psm_header        
        y_map = PTR_NEW(/alloc)
        taubeta_map = PTR_NEW(/alloc)        
        CREATE_SZ_MAP_ALLSKY, local_catalog, *y_map, *taubeta_map, tb2q, tb2u, ythetae_map=ythetae_map, ythetae2_map=ythetae2_map, ythetae3_map=ythetae3_map, ythetae4_map=ythetae4_map, relativistic=szp.sz_relativistic
        CHANGE_MAP_RESOLUTION, *y_map, *y_map,  beam_out=sky_beam, nside_out= pixp.nside, ordering_in='RING', ordering_out= pixp.ordering, /overwrite, lmax=SKY_LMAX()
        IF (STRLOWCASE(szp.sz_relativistic) GE 1) THEN BEGIN
           CHANGE_MAP_RESOLUTION, ythetae_map, ythetae_map,  beam_out=sky_beam, nside_out= pixp.nside, ordering_in='RING', ordering_out= pixp.ordering, /overwrite, lmax=SKY_LMAX()
        ENDIF
        IF (STRLOWCASE(szp.sz_relativistic) GE 2) THEN BEGIN
           CHANGE_MAP_RESOLUTION, ythetae2_map, ythetae2_map,  beam_out=sky_beam, nside_out= pixp.nside, ordering_in='RING', ordering_out= pixp.ordering, /overwrite, lmax=SKY_LMAX()
        ENDIF
        IF (STRLOWCASE(szp.sz_relativistic) GE 3) THEN BEGIN
           CHANGE_MAP_RESOLUTION, ythetae3_map, ythetae3_map,  beam_out=sky_beam, nside_out= pixp.nside, ordering_in='RING', ordering_out= pixp.ordering, /overwrite, lmax=SKY_LMAX()
        ENDIF
        IF (STRLOWCASE(szp.sz_relativistic) GE 4) THEN BEGIN
           CHANGE_MAP_RESOLUTION, ythetae4_map, ythetae4_map,  beam_out=sky_beam, nside_out= pixp.nside, ordering_in='RING', ordering_out= pixp.ordering, /overwrite, lmax=SKY_LMAX()
        ENDIF

        IF (STRLOWCASE(szp.sz_include_kinetic) eq 'yes') THEN BEGIN
           *taubeta_map = -1d*temporary(*taubeta_map)
           CHANGE_MAP_RESOLUTION, *taubeta_map, *taubeta_map,  beam_out=sky_beam, nside_out= pixp.nside, ordering_in='RING', ordering_out= pixp.ordering, /overwrite, lmax=SKY_LMAX()
        ENDIF
        IF (STRLOWCASE(szp.sz_include_polarised) eq 'yes') THEN BEGIN
           IF (STRLOWCASE(pixp.pixtype) EQ 'healpix') THEN npix = NSIDE2NPIX(pixp.nside)
           szpolar = PTR_NEW(/alloc)
           (*szpolar) = DBLARR(npix,3)
           (*szpolar)[*,1] = TEMPORARY(tb2q)
           (*szpolar)[*,2] = TEMPORARY(tb2u)
           CHANGE_MAP_RESOLUTION, *szpolar, *szpolar,  beam_out=sky_beam, nside_out= pixp.nside, ordering_in='RING', ordering_out= pixp.ordering, /overwrite, lmax=SKY_LMAX()
        ENDIF
     END

     ;;-------------------------------------------------------------------------------------------------
     'dmb': BEGIN

        useseed = GETSEED()

        ;;cluster = DEFINE_CLUSTER(cosmo=cosmo,sigma_8=cp.Sigma_8,t_star=0.95*cp.Sigma_8^(-1.49))
        cluster = DEFINE_CLUSTER(cosmo=cosmo, t_star=szp.cluster_t_star, /silent)
        mass_function = szp.mass_function

        IF STRLOWCASE(szp.sz_constrained) EQ 'yes' THEN BEGIN

          cats=WHERE(STRLEN(szp.sz_input_cat) NE 0, ncats)
          acats = szp.sz_input_cat[0:ncats-1]
          ucats = acats[UNIQ(acats, SORT(acats))]
          nucats = N_ELEMENTS(ucats)

          priority_ranking = ['rosat', 'sdss']
          icats = INTARR(nucats)
          icats[*]=N_ELEMENTS(priority_ranking)
          FOR i=0, nucats-1 DO BEGIN
            wh = WHERE(STRMATCH(priority_ranking, ucats[i]) EQ 1, nwh)
            IF nwh EQ 1 THEN BEGIN
              icats[i]=wh
            ENDIF
          ENDFOR
          indcats = SORT(icats)

          FOR numcat=0, nucats-1 DO BEGIN
          
             CASE ucats[indcats[numcat]] OF
               'rosat': BEGIN
                          ancillary_dir = GETPSMDIR('psmdata') + 'ancillary/observations/ROSAT/'
                          ancillary_cat = 'MCXC_for_PSM_v0.save' ;'rosat_cluster_catalog.fits'
                          optical=0
                         END
               'sdss': BEGIN
                         ancillary_dir = GETPSMDIR('psmdata') + 'ancillary/observations/SDSS/'
                         ancillary_cat = 'maxbcg_cluster_catalog.fits'
                         optical=1
                      END
             END

             ANCILLARY2INTERMEDIATE_CLUSTER_CATALOG, ancillary_dir, ancillary_cat, $
                                                     intermediate_dir, intermediate_cat, $
                                                     cosmo=cosmo, cluster=cluster, catalog=sub_catalog, /nodisk, optical=optical, mcxc=1, nstd=1
  
             IF DEFINED(local_catalog) EQ 0 THEN local_catalog=sub_catalog ELSE local_catalog=MERGE_INTERMEDIATE_CLUSTER_CATALOGS(local_catalog, sub_catalog)

          ENDFOR

        ENDIF

        IF szp.sz_include_polarised EQ 'no' THEN polar=0 ELSE polar=1

        CREATE_SZ_SIMULATION_ALLSKY, head, catalog, y_map, taubeta_map,$
                tb2q, tb2u, m_inf = szp.cluster_m_inf, map_type='all-sky', $
                cosmo=cosmo, cluster=cluster, mass_function=mass_function, $
                iseed=useseed,  /nocorrelation, input_catalog=local_catalog, $
                polar=polar, xmm=xmm, chandra=chandra, norm=norm, nr500=nr500, nstd=nstd, /nomap

        nclusters = N_ELEMENTS(catalog)
        sz_cat = CREATE_SZ_MODEL_CAT('dmb_model',nclusters,szp=szp,/usenan,header=sz_header)

        FILL_SZ_MODEL_CAT, sz_cat, catalog, szp=szp

        WRITE_SZ_MODEL_CAT, sz_cat_file, sz_cat, sz_header=sz_header, psm_header=psm_header
        y_map = PTR_NEW(/alloc)
        taubeta_map = PTR_NEW(/alloc)
        CREATE_SZ_MAP_ALLSKY, catalog, *y_map, *taubeta_map, tb2q, tb2u, ythetae_map=ythetae_map, ythetae2_map=ythetae2_map, ythetae3_map=ythetae3_map, ythetae4_map=ythetae4_map, relativistic=szp.sz_relativistic
        CHANGE_MAP_RESOLUTION, *y_map, *y_map,  beam_out=sky_beam, nside_out= pixp.nside, ordering_in='RING', ordering_out= pixp.ordering, /overwrite, lmax=SKY_LMAX()

        IF (STRLOWCASE(szp.sz_include_kinetic) EQ 'yes') THEN BEGIN
           *taubeta_map = -1d*TEMPORARY(*taubeta_map)
           CHANGE_MAP_RESOLUTION, *taubeta_map, *taubeta_map,  beam_out=sky_beam, nside_out= pixp.nside, ordering_in='RING', ordering_out= pixp.ordering, /overwrite, lmax=SKY_LMAX()
        ENDIF
        IF (STRLOWCASE(szp.sz_relativistic) GE 1) THEN BEGIN
           CHANGE_MAP_RESOLUTION, ythetae_map, ythetae_map,  beam_out=sky_beam, nside_out= pixp.nside, ordering_in='RING', ordering_out= pixp.ordering, /overwrite, lmax=SKY_LMAX()
        ENDIF
        IF (STRLOWCASE(szp.sz_relativistic) GE 2) THEN BEGIN
           CHANGE_MAP_RESOLUTION, ythetae2_map, ythetae2_map,  beam_out=sky_beam, nside_out= pixp.nside, ordering_in='RING', ordering_out= pixp.ordering, /overwrite, lmax=SKY_LMAX()
        ENDIF
        IF (STRLOWCASE(szp.sz_relativistic) GE 3) THEN BEGIN
           CHANGE_MAP_RESOLUTION, ythetae3_map, ythetae3_map,  beam_out=sky_beam, nside_out= pixp.nside, ordering_in='RING', ordering_out= pixp.ordering, /overwrite, lmax=SKY_LMAX()
        ENDIF
        IF (STRLOWCASE(szp.sz_relativistic) GE 4) THEN BEGIN
           CHANGE_MAP_RESOLUTION, ythetae4_map, ythetae4_map,  beam_out=sky_beam, nside_out= pixp.nside, ordering_in='RING', ordering_out= pixp.ordering, /overwrite, lmax=SKY_LMAX()
        ENDIF
        IF (STRLOWCASE(szp.sz_include_polarised) EQ 'yes') THEN BEGIN
           IF (pixp.pixtype EQ 'HEALPIX') THEN npix = pixp.npix
           szpolar = PTR_NEW(/alloc)
           (*szpolar) = DBLARR(npix,3)
           (*szpolar)[*,1] = TEMPORARY(tb2q)
           (*szpolar)[*,2] = TEMPORARY(tb2u)
           CHANGE_MAP_RESOLUTION, *szpolar, *szpolar,  beam_out=sky_beam, nside_out= pixp.nside, ordering_in='RING', ordering_out= pixp.ordering, /overwrite, lmax=SKY_LMAX()
        ENDIF

        PSM_ADD2CITE, 'The SZ model is based on a random draw of a cluster catalogue according to the method of Delabrouille, Melin & Bartlett (2002).'
        PSM_ADD2BIB, '2002ASPC..257...81D'

        IF szp.sz_constrained EQ 'yes' THEN BEGIN
           whrosat = WHERE(szp.sz_input_cat EQ 'rosat', nwhrosat)
           whsdss = WHERE(szp.sz_input_cat EQ 'sdss', nwhsdss)
           IF nwhrosat EQ 1 THEN BEGIN
              IF nwhsdss EQ 1 THEN BEGIN
                 PSM_ADD2CITE, 'The SZ catalogue is constrained to include clusters corresponding to the MCXC compilation of ROSAT clusters by Piffaretti et al. (2010),'
                 PSM_ADD2CITE, 'and the MaxBCG galaxy clusters from Koester et al. (2007).'
                 PSM_ADD2BIB, '2010arXiv1007.1916P'
                 PSM_ADD2BIB, '2007ApJ...660..239K'
              ENDIF ELSE BEGIN
                 PSM_ADD2CITE, 'The SZ catalogue is constrained to include clusters corresponding to the MCXC compilation of ROSAT clusters by Piffaretti et al. (2010).'
                 PSM_ADD2BIB, '2010arXiv1007.1916P'
              ENDELSE
           ENDIF ELSE BEGIN
                 PSM_ADD2CITE, 'The SZ catalogue is constrained to include clusters corresponding to the MaxBCG galaxy clusters from Koester et al. (2007).'
                 PSM_ADD2BIB, '2007ApJ...660..239K'
           ENDELSE
        ENDIF

     END

     ;-------------------------------------------------------------------------------------------------
     'nbody+hydro':BEGIN


          IF STRLOWCASE(szp.sz_include_thermal) EQ 'yes' THEN BEGIN
             y_map = PTR_NEW(/alloc)

             ;;Read input maps : Hubble volume simulation, thermal effect
             ;;-----------------------------------------------------------
             file_to_read = GETPSMDIR('psmdata')+'ancillary/simulations/sz/hydro-nbody/HubbleSZTherm512.fits'             
             IF (pixp.nside GT 512) THEN BEGIN
                file_to_read = GETPSMDIR('psmdata')+'ancillary/simulations/sz/hydro-nbody/HubbleSZTherm2048.fits'
             ENDIF
             IF (FILE_TEST(file_to_read) EQ 0) THEN GET_PSM_DATA, datafile=file_to_read
             READ_FITS_MAP, file_to_read, ythermal_hubble, hdr, exthdr
             ;; Put it to proper format (nside, ordering)
             ordering_in = STRTRIM(SXPAR(exthdr,'ORDERING'),2)
             CHANGE_MAP_RESOLUTION, TEMPORARY(ythermal_hubble), y_map_hubble,  beam_out=sky_beam, nside_out= pixp.nside, $
                                    ordering_in=ordering_in, ordering_out= pixp.ordering, lmax=SKY_LMAX()

             ;;Read input maps : Local cluster simulation, thermal effect
             ;;------------------------------------------------------------
             file_to_read = GETPSMDIR('psmdata')+'ancillary/simulations/sz/hydro-nbody/LocalSZTherm.gal512.fits'
             IF (pixp.nside GT 512) THEN BEGIN
               file_to_read = GETPSMDIR('psmdata')+'ancillary/simulations/sz/hydro-nbody/LocalSZTherm.gal1024.fits'
             ENDIF
             IF (FILE_TEST(file_to_read) EQ 0) THEN GET_PSM_DATA, datafile=file_to_read
             READ_FITS_MAP, file_to_read, y_map_1024, hdr2, exthdr2
             ;; Put it to proper format (nside, ordering)
             ordering_in = STRTRIM(SXPAR(exthdr2,'ORDERING'),2)
             IF SKY_LMAX() GT 2000 THEN iter=1 ELSE iter=0
             CHANGE_MAP_RESOLUTION, y_map_1024, *y_map,  beam_out=sky_beam, nside_out= pixp.nside, $
                                    ordering_in=ordering_in, ordering_out= pixp.ordering, lmax=SKY_LMAX(), iter=iter

             ;; Add both maps up
             ;;------------------
             *y_map = TEMPORARY(*y_map) + TEMPORARY(y_map_hubble)

             ;; Make figures
             ;;--------------
             IF param.run.visu GE 2 THEN BEGIN

                ;; Figures for the Hubble volume
                ;;-------------------------------
                file_to_plot1 = GETPSMDIR('psmdata')+'ancillary/simulations/sz/hydro-nbody/HubbleSZTherm2048.fits'
                IF (FILE_TEST(file_to_plot1) EQ 0) THEN GET_PSM_DATA, datafile=file_to_plot1
                file_to_plot2 = GETPSMDIR('psmdata')+'ancillary/simulations/sz/hydro-nbody/HubbleSZKin2048.fits'
                IF (FILE_TEST(file_to_plot2) EQ 0) THEN GET_PSM_DATA, datafile=file_to_plot2
                READ_FITS_MAP, file_to_plot2, map_to_plot2, hd2, xhd2
                CASE param.run.output_visu OF
                   'png': BEGIN
                      GNOMVIEW, file_to_plot1, $
                                rot=[0,90], reso=.5, max=2e-5, min=0, title='Thermal SZ, Hubble volume simulation', subtitle='Five degree patch around North Galactic Pole', $
                                units='y_sz', png=FIX_SEPARATOR(GETPSMDIR('visu'))+'gnomview_hubble_y.png', pxsize=600, pysize=600
                      GNOMVIEW, (-1.)*map_to_plot2, $
                                rot=[0,90], reso=.5, max=4e-6, min=-4e-6, title='Kinetic SZ, Hubble volume simulation', subtitle='Five degree patch around North Galactic Pole', $
                                units='K/KCMB', png=FIX_SEPARATOR(GETPSMDIR('visu'))+'gnomview_hubble_kin.png', pxsize=600, pysize=600
                   END
                   'ps': BEGIN
                      GNOMVIEW, file_to_plot1, $
                                rot=[0,90], reso=.5, max=2e-5, min=0, title='Thermal SZ, Hubble volume simulation', subtitle='Five degree patch around North Galactic Pole', $
                                units='y_sz', ps=FIX_SEPARATOR(GETPSMDIR('visu'))+'gnomview_hubble_y.ps', pxsize=600, pysize=600
                      GNOMVIEW, (-1.)*map_to_plot2, $
                                rot=[0,90], reso=.5, max=4e-6, min=-4e-6, title='Kinetic SZ, Hubble volume simulation', subtitle='Five degree patch around North Galactic Pole', $
                                units='K/KCMB', ps=FIX_SEPARATOR(GETPSMDIR('visu'))+'gnomview_hubble_kin.ps', pxsize=600, pysize=600
                   END
                   'screen': BEGIN
                      GNOMVIEW, file_to_plot1, $
                                rot=[0,90], reso=.5, max=2e-5, min=0, title='Thermal SZ, Hubble volume simulation', subtitle='Five degree patch around North Galactic Pole', $
                                units='y_sz', pxsize=600, pysize=600
                      GNOMVIEW, (-1.)*map_to_plot2, $
                                rot=[0,90], reso=.5, max=4e-6, min=-4e-6, title='Kinetic SZ, Hubble volume simulation', subtitle='Five degree patch around North Galactic Pole', $
                                units='K/KCMB', pxsize=600, pysize=600
                   END
                   ELSE: ;; Do nothing
                ENDCASE

                ;; Figures for the Local universe
                ;;--------------------------------
                file_to_plot1 = GETPSMDIR('psmdata')+'ancillary/simulations/sz/hydro-nbody/LocalSZTherm.gal1024.fits'
                IF (FILE_TEST(file_to_plot1) EQ 0) THEN GET_PSM_DATA, datafile=file_to_plot1
                file_to_plot2 = GETPSMDIR('psmdata')+'ancillary/simulations/sz/hydro-nbody/LocalSZKin.gal1024.fits'
                IF (FILE_TEST(file_to_plot2) EQ 0) THEN GET_PSM_DATA, datafile=file_to_plot2
                READ_FITS_MAP, file_to_plot2, map_to_plot2, hd2, xhd2
                CASE param.run.output_visu OF
                   'png': BEGIN
                      GNOMVIEW, file_to_plot1, $
                                rot=[0,90], reso=.5, max=2e-5, min=0, title='Thermal SZ, local universe', subtitle='Five degree patch around North Galactic Pole', $
                                units='y_sz', png=FIX_SEPARATOR(GETPSMDIR('visu'))+'gnomview_local_y.png', pxsize=600, pysize=600
                      GNOMVIEW, (-1.)*map_to_plot2, $
                                rot=[0,90], reso=.5, max=4e-6, min=-4e-6, title='Kinetic SZ, local universe', subtitle='Five degree patch around North Galactic Pole', $
                                units='K/KCMB', png=FIX_SEPARATOR(GETPSMDIR('visu'))+'gnomview_local_kin.png', pxsize=600, pysize=600, /nest
                   END
                   'ps': BEGIN
                      GNOMVIEW, file_to_plot1, $
                                rot=[0,90], reso=.5, max=2e-5, min=0, title='Thermal SZ, local universe', subtitle='Five degree patch around North Galactic Pole', $
                                units='y_sz', ps=FIX_SEPARATOR(GETPSMDIR('visu'))+'gnomview_local_y.ps', pxsize=600, pysize=600
                      GNOMVIEW, (-1.)*map_to_plot2, $
                                rot=[0,90], reso=.5, max=4e-6, min=-4e-6, title='Kinetic SZ, local universe', subtitle='Five degree patch around North Galactic Pole', $
                                units='K/KCMB', ps=FIX_SEPARATOR(GETPSMDIR('visu'))+'gnomview_local_kin.ps', pxsize=600, pysize=600, /nest
                   END
                   'screen': BEGIN
                      GNOMVIEW, file_to_plot1, $
                                rot=[0,90], reso=.5, max=2e-5, min=0, title='Thermal SZ, local universe', subtitle='Five degree patch around North Galactic Pole', $
                                units='y_sz', pxsize=600, pysize=600
                      GNOMVIEW, (-1.)*map_to_plot2, $
                                rot=[0,90], reso=.5, max=4e-6, min=-4e-6, title='Kinetic SZ, local universe', subtitle='Five degree patch around North Galactic Pole', $
                                units='K/KCMB', pxsize=600, pysize=600, /nest
                   END
                   ELSE: ;; Do nothing
                ENDCASE

             ENDIF

          ENDIF

          IF (STRLOWCASE(szp.sz_include_kinetic) EQ 'yes') THEN BEGIN

             taubeta_map = PTR_NEW(/alloc)
             
             ;; Read input maps : Hubble volume simulation, kinetic effect
             ;;------------------------------------------------------------
             file_to_read = GETPSMDIR('psmdata')+'ancillary/simulations/sz/hydro-nbody/HubbleSZKin512.fits'
             IF (pixp.nside GT 512) THEN BEGIN
                file_to_read = GETPSMDIR('psmdata')+'ancillary/simulations/sz/hydro-nbody/HubbleSZKin2048.fits'
             ENDIF
             IF (FILE_TEST(file_to_read) EQ 0) THEN GET_PSM_DATA, datafile=file_to_read
             READ_FITS_MAP, file_to_read, szkin_hubble, hdr, exthdr
             ;; Put it to proper format (nside, ordering)
             ordering_in = STRTRIM(SXPAR(exthdr,'ORDERING'),2)
             CHANGE_MAP_RESOLUTION, TEMPORARY(szkin_hubble), szkin_map_hubble,  beam_out=sky_beam, nside_out= pixp.nside, $
                                    ordering_in=ordering_in, ordering_out= pixp.ordering, lmax=SKY_LMAX()
             ;UD_GRADE, TEMPORARY(szkin_hubble), szkin_map_hubble, nside_out = pixp.nside, order_in = 'RING', order_out = pixp.ordering
             
             ;; Read input maps : Local cluster simulation, kinetic effect
             ;;------------------------------------------------------------
             file_to_read = GETPSMDIR('psmdata')+'ancillary/simulations/sz/hydro-nbody/LocalSZKin.gal512.fits'
             IF (pixp.nside GT 512) THEN BEGIN
                file_to_read = GETPSMDIR('psmdata')+'ancillary/simulations/sz/hydro-nbody/LocalSZKin.gal1024.fits'
             ENDIF
             IF (FILE_TEST(file_to_read) EQ 0) THEN GET_PSM_DATA, datafile=file_to_read
             READ_FITS_MAP, file_to_read, szkin_map_1024, hdr2, exthdr2
             ;; Put it to proper format (nside, ordering)
             ordering_in = STRTRIM(SXPAR(exthdr2,'ORDERING'),2)
             IF SKY_LMAX() GT 2000 THEN iter=1 ELSE iter=0
             CHANGE_MAP_RESOLUTION, TEMPORARY(szkin_map_1024), *taubeta_map,  beam_out=sky_beam, nside_out= pixp.nside, $
                                    ordering_in=ordering_in, ordering_out= pixp.ordering, lmax=SKY_LMAX(), iter=iter

             ;; Add both maps up
             ;;------------------
             *taubeta_map = TEMPORARY(*taubeta_map) + TEMPORARY(szkin_map_hubble)
             ;MESSAGE, 'WARNING: check sign of map of kinetic SZ for model nbody+hydro', /info
             ;;; Sign checked by JBM on March 18 2011
             ;;; The HubbleSZKin and LocalSZKin.gal maps and are in w unit with DeltaT/T=-w
             ;;; w is beta*tau where beta is the line of sight velocity (positive if the cluster
             ;;; is moving away from the observer). So DeltaT/T>0 if the cluster is moving towards
             ;;; the observer and DeltaT/T<0 if the cluster is moving away from the observer.
             ;;; DeltaT/T has the opposite sign of w.

          ENDIF

          IF (STRLOWCASE(szp.sz_include_polarised) EQ 'yes') THEN BEGIN
             IF PSM_CAREFULNESS() GE 2 THEN BEGIN
                MESSAGE, 'Cannot include polarised SZ effect for model nbody+hydro'
             ENDIF 
             IF runp.carefulness EQ 1 THEN BEGIN
                PSM_INFO_MESSAGE, 'WARNING: Cannot include polarised SZ effect for model nbody+hydro'
                PSM_INFO_MESSAGE, 'Assuming vanishing polarisation'
                szp.sz_include_polarised = 'no'
             ENDIF 
             IF ((PSM_CAREFULNESS() EQ 0) AND (PSM_VERBOSITY GE 1)) THEN BEGIN
                PSM_INFO_MESSAGE, 'WARNING: Cannot include polarised SZ effect for model nbody+hydro'
                PSM_INFO_MESSAGE, 'Assuming vanishing polarisation'
                szp.sz_include_polarised = 'no'
             ENDIF 
          ENDIF
          cat_file = GETPSMDIR('psmdata')+'ancillary/simulations/sz/hydro-nbody/nbody_hydro_catalog.fits'
          ok = TEST_PSM_DATA(cat_file, /get)
          cmd = 'cp '+cat_file+' '+sz_cat_file
          SPAWN, cmd

        PSM_ADD2CITE, 'The SZ model maps are obtained from a simulation of large scale structures by Dolag et al. (2005),'
        PSM_ADD2CITE, 'and hydrodynamical simulations of cluster formation by Schaefer et al. (2006).'
        PSM_ADD2BIB, '2005MNRAS.363...29D'
        PSM_ADD2BIB, '2006MNRAS.370.1309S'

       END

     ;-------------------------------------------------------------------------------------------------
     'hydro+dmb': BEGIN
        ;; hydro: Da Silva and Dolag 
        ;; + dmb: Delabrouille, Melin and Bartlett
      
        ;; Read input maps : Da Silva and Dolag hydro simulation, thermal effect
        ;;-----------------------------------------------------------------------
        file_to_read = GETPSMDIR('psmdata')+'ancillary/simulations/sz/hydro-dmb/tsz_hydro_Rmax_1000Mpc_nside_512_v2.fits'
        IF (pixp.nside GT 512) THEN BEGIN
           file_to_read = GETPSMDIR('psmdata')+'ancillary/simulations/sz/hydro-dmb/tsz_hydro_Rmax_1000Mpc_nside_1024_v2.fits'
        ENDIF
        IF (FILE_TEST(file_to_read) EQ 0) THEN GET_PSM_DATA, datafile=file_to_read
        ok = TEST_PSM_DATA(file_to_read,/get)
        READ_FITS_MAP, file_to_read, ythermal_1024, hdr, exthdr
        ;;Put it to proper format (nside, ordering)

        IF SKY_LMAX() GT 2000 THEN iter=1 ELSE iter=0
        CHANGE_MAP_RESOLUTION, ythermal_1024, ythermal,  beam_out=sky_beam, nside_out= pixp.nside, ordering_in='RING', ordering_out= pixp.ordering, lmax=SKY_LMAX(), iter=iter
        ;UD_GRADE, ythermal_1024, ythermal, nside_out = pixp.nside, order_in = 'RING', order_out = pixp.ordering
        ythermal_1024 = 0
        
        IF (STRLOWCASE(szp.sz_include_kinetic) eq 'yes') THEN BEGIN
           szkin_map = PTR_NEW(/alloc)
           ;;Read input maps : Da Silva and Dolag hydro simulation, kinetic effect
           file_to_read = GETPSMDIR('psmdata')+'ancillary/simulations/sz/hydro-dmb/ksz_hydro_Rmax_1000Mpc_nside_512_v2.fits'
           IF (pixp.nside GT 512) THEN BEGIN
              file_to_read = GETPSMDIR('psmdata')+'ancillary/simulations/sz/hydro-dmb/ksz_hydro_Rmax_1000Mpc_nside_1024_v2.fits'
           ENDIF
           IF (FILE_TEST(file_to_read) EQ 0) THEN GET_PSM_DATA, datafile=file_to_read
           READ_FITS_MAP, file_to_read, szkin_1024, hdr, exthdr
           ;;Put it to proper format (nside, ordering)
           ;; JBM on March 18th 2011 : the header indicates that units of
           ;; ksz_hydro_Rmax_1000Mpc_nside_1024.fits is DeltaT/T but the sign is not in agreement
           ;; with the LocalSZKin.gal1024.fits map for the Coma cluster (glon=0, glat=90).
           ;; So I decided to change the sign of ksz_hydro_Rmax_1000Mpc_nside_1024.fits to be in agreement
           ;; with LocalSZKin.gal1024.fits for the local objects

           CHANGE_MAP_RESOLUTION, szkin_1024, szkin,  beam_out=sky_beam, nside_out= pixp.nside, ordering_in='RING', ordering_out= pixp.ordering, lmax=SKY_LMAX(), iter=iter
           ;UD_GRADE, szkin_1024, szkin, nside_out = pixp.nside, order_in = 'RING', order_out = pixp.ordering
           szkin_1024 = 0
        ENDIF
        
        cosmo1 = DEFINE_COSMO(h=SXPAR(exthdr,'HUBBLEPA'), omega_m=SXPAR(exthdr,'OMEGA_M0'), $
                             omega_b=SXPAR(exthdr,'OMEGA_B0'), omega_de=SXPAR(exthdr,'OMEGA_L0'), sigma_8=SXPAR(exthdr,'SIGMA_8'), n_scal=SXPAR(exthdr,'P_INDEX'), /silent)
        cluster = DEFINE_CLUSTER(cosmo=cosmo1, t_star=szp.cluster_t_star, /silent)
        
        mass_function = szp.mass_function
        
        CREATE_SZ_SIMULATION_ALLSKY, head, catalog, y_map, taubeta_map, tb2q, tb2u, $
                                     m_inf = szp.cluster_m_inf, z_inf = SXPAR(exthdr,'Z_MAX'),$
                                     map_type='all-sky', cosmo=cosmo, cluster=cluster, $
                                     mass_function=mass_function, iseed=useseed, $
                                     /nocorrelation, input_catalog=local_catalog, $
                                     polar=polar, xmm=xmm, chandra=chandra, norm=norm, $
                                     nr500=nr500, nstd=nstd, /nomap
        
        nclusters = N_ELEMENTS(catalog)
        sz_cat = CREATE_SZ_MODEL_CAT('dmb_model',nclusters,szp=szp,/usenan,header=sz_header)
        
        FILL_SZ_MODEL_CAT, sz_cat, catalog, szp=szp
        
        WRITE_SZ_MODEL_CAT, sz_cat_file, sz_cat, sz_header=sz_header, psm_header=psm_header

        cat_file = GETPSMDIR('psmdata')+'ancillary/simulations/sz/hydro-dmb/hydro_catalog.fits'
        ok = TEST_PSM_DATA(cat_file, /get)
        hydro_cat = MRDFITS(cat_file, 1, hydro_hd, /silent)
        MWRFITS, hydro_cat, sz_cat_file, hydro_hd, /silent
       
        y_map = ptr_new(/alloc)
        taubeta_map = ptr_new(/alloc)

        CREATE_SZ_MAP_ALLSKY, catalog, *y_map, *taubeta_map, tb2q, tb2u, ythetae_map=ythetae_map, ythetae2_map=ythetae2_map, ythetae3_map=ythetae3_map, ythetae4_map=ythetae4_map, relativistic=szp.sz_relativistic
        CHANGE_MAP_RESOLUTION, *y_map, *y_map,  beam_out=sky_beam, nside_out= pixp.nside, ordering_in='RING', ordering_out= pixp.ordering, /overwrite, lmax=SKY_LMAX()
        CHANGE_MAP_RESOLUTION, *taubeta_map, *taubeta_map,  beam_out=sky_beam, nside_out= pixp.nside, ordering_in='RING', ordering_out= pixp.ordering, /overwrite, lmax=SKY_LMAX()
        IF (STRLOWCASE(szp.sz_relativistic) GE 1) THEN BEGIN
           CHANGE_MAP_RESOLUTION, ythetae_map, ythetae_map,  beam_out=sky_beam, nside_out= pixp.nside, ordering_in='RING', ordering_out= pixp.ordering, /overwrite, lmax=SKY_LMAX()
        ENDIF
        IF (STRLOWCASE(szp.sz_relativistic) GE 2) THEN BEGIN
           CHANGE_MAP_RESOLUTION, ythetae2_map, ythetae2_map,  beam_out=sky_beam, nside_out= pixp.nside, ordering_in='RING', ordering_out= pixp.ordering, /overwrite, lmax=SKY_LMAX()
        ENDIF
        IF (STRLOWCASE(szp.sz_relativistic) GE 3) THEN BEGIN
           CHANGE_MAP_RESOLUTION, ythetae3_map, ythetae3_map,  beam_out=sky_beam, nside_out= pixp.nside, ordering_in='RING', ordering_out= pixp.ordering, /overwrite, lmax=SKY_LMAX()
        ENDIF
        IF (STRLOWCASE(szp.sz_relativistic) GE 4) THEN BEGIN
           CHANGE_MAP_RESOLUTION, ythetae4_map, ythetae4_map,  beam_out=sky_beam, nside_out= pixp.nside, ordering_in='RING', ordering_out= pixp.ordering, /overwrite, lmax=SKY_LMAX()
        ENDIF

        ;; Compute component maps if needed
        ;;----------------------------------
        IF (STRLOWCASE(szp.sz_include_thermal) EQ 'yes') THEN BEGIN
           *y_map = TEMPORARY(*y_map) + TEMPORARY(ythermal)
        ENDIF

        IF (STRLOWCASE(szp.sz_include_kinetic) EQ 'yes') THEN BEGIN
           *taubeta_map = TEMPORARY(*taubeta_map) + TEMPORARY(szkin)
        ENDIF

        IF (STRLOWCASE(szp.sz_include_polarised) EQ 'yes') THEN BEGIN
           IF (strlowcase(pixp.pixtype) EQ 'healpix') THEN npix=NSIDE2NPIX(pixp.nside)
           szpolar = ptr_new(/alloc)
           (*szpolar)=dblarr(npix,3)
           (*szpolar)[*,1] = TEMPORARY(tb2q)
           (*szpolar)[*,2] = TEMPORARY(tb2u)
        CHANGE_MAP_RESOLUTION, *szpolar, *szpolar,  beam_out=sky_beam, nside_out= pixp.nside, ordering_in='RING', ordering_out= pixp.ordering, /overwrite, lmax=SKY_LMAX()
        ENDIF
        
        PSM_ADD2CITE, 'The SZ model maps are obtained from a combination of local constrained large scale structures from Dolag et al. (2005),'
        PSM_ADD2CITE, 'complemented up to z=0.25 with Lambda-CDM simulations from de Boni et al. (2010),'
        PSM_ADD2CITE, 'and at higher redshift by a random realisation of a cluster catalogue following the method of Delabrouille, Melin & Bartlett (2002).'
        PSM_ADD2BIB, '2005MNRAS.363...29D'
        PSM_ADD2BIB, '2010arXiv1008.5376D'
        PSM_ADD2BIB, '2002ASPC..257...81D'

     END
  ENDCASE


;;****************************************************************************************************
;; Creating the SZ structures and writing SZ maps
;;****************************************************************************************************

;  PSM_ID_KEYGEN, id
;  sz = COMPONENT_STRUCT('thermalsz','cmb',name=param.names.sz_basename,id=id,comp_param=szp)
;  sz.polarised = (model.sz.sz_include_polarised EQ 'yes')

  IF (STRLOWCASE(szp.sz_include_thermal) EQ 'yes') THEN BEGIN
     PSM_ID_KEYGEN, id
     CASE szp.sz_relativistic OF
        0: thermalsz = COMPONENT_STRUCT('thermalsz',name=param.names.thermalsz_basename,id=id)
        1: thermalsz = COMPONENT_STRUCT('thermalsz','relatsz1',name=param.names.thermalsz_basename,id=id) 
        2: thermalsz = COMPONENT_STRUCT('thermalsz','relatsz1','relatsz2',name=param.names.thermalsz_basename,id=id) 
        3: thermalsz = COMPONENT_STRUCT('thermalsz','relatsz1','relatsz2','relatsz3',name=param.names.thermalsz_basename,id=id) 
        4: thermalsz = COMPONENT_STRUCT('thermalsz','relatsz1','relatsz2','relatsz3','relatsz4',name=param.names.thermalsz_basename,id=id) 
     ENDCASE
     thermalsz.polarised = 0B
     PSM_ID_KEYGEN, thermalsz_id

     ;; First emission law
     ;;--------------------
     ;; thermalsz.e1.ampl.form = 'map'
     thermalsz.e1.ampl.unit = 'y_sz'
     IF thermalsz.e1.ampl.unit NE model.units.thermalsz THEN STOP
     IF szp.sz_relativistic EQ 0 THEN BEGIN
        *thermalsz.e1.ampl.value = GETPSMDIR('sz') + param.names.thermalsz_basename + '_map.fits'
     ENDIF ELSE BEGIN
        *thermalsz.e1.ampl.value = GETPSMDIR('sz') + 'relatsz0_map.fits'
     ENDELSE
     *thermalsz.e1.numin.value = 0.
     *thermalsz.e1.numax.value = FLOAT('INF')
     xhdr = PSM_COMP_HDR(compname=param.names.thermalsz_basename,compid=thermalsz_id)
     WRITE_HPXMAP, (*thermalsz.e1.ampl.value), tstokes=TEMPORARY(*y_map), datatype='COMP', psm_dtid=thermalsz_id, $
                   ordering='RING', units=thermalsz.e1.ampl.unit, lmax=SKY_LMAX(), beam=SKY_RESOLUTION(), xhdr=xhdr, coordsys='G'

     ;; Second emission law (first relativistic correction)
     ;;-----------------------------------------------------
     IF szp.sz_relativistic GE 1 THEN BEGIN
        *thermalsz.e2.numin.value = 0.
        *thermalsz.e2.numax.value = FLOAT('INF')
        thermalsz.e2.ampl.unit = 'y_sz'
        ;; Get conversion factor to compute SZ relativistic correction at 300 GHz in y_sz
        c = PSM_RELATSZ1(3d11) * CONVERSION_FACTOR('W/m2/sr/Hz','y_sz', 3d11)
        IF thermalsz.e2.ampl.unit NE model.units.thermalsz THEN STOP
        *thermalsz.e2.ampl.value = GETPSMDIR('sz') + 'relatsz1_map.fits'
        xhdr = PSM_COMP_HDR(compname=param.names.thermalsz_basename,compid=thermalsz_id)
        WRITE_HPXMAP, (*thermalsz.e2.ampl.value), tstokes=c*TEMPORARY(ythetae_map), datatype='COMP', psm_dtid=thermalsz_id, $
                      ordering='RING', units=thermalsz.e2.ampl.unit, lmax=SKY_LMAX(), beam=SKY_RESOLUTION(), xhdr=xhdr, coordsys='G'
        *thermalsz.e2.nuref.value = 3e11
     END

     ;; Third emission law (second relativistic correction)
     ;;-----------------------------------------------------
     IF szp.sz_relativistic GE 2 THEN BEGIN
        *thermalsz.e3.numin.value = 0.
        *thermalsz.e3.numax.value = FLOAT('INF')
        thermalsz.e3.ampl.unit = 'y_sz'
        ;; Get conversion factor to compute SZ relativistic correction at 300 GHz in y_sz
        c = PSM_RELATSZ2(3d11) * CONVERSION_FACTOR('W/m2/sr/Hz','y_sz', 3d11)
        IF thermalsz.e3.ampl.unit NE model.units.thermalsz THEN STOP
        *thermalsz.e3.ampl.value = GETPSMDIR('sz') + 'relatsz2_map.fits'
        xhdr = PSM_COMP_HDR(compname=param.names.thermalsz_basename,compid=thermalsz_id)
        WRITE_HPXMAP, (*thermalsz.e3.ampl.value), tstokes=c*TEMPORARY(ythetae2_map), datatype='COMP', psm_dtid=thermalsz_id, $
                      ordering='RING', units=thermalsz.e3.ampl.unit, lmax=SKY_LMAX(), beam=SKY_RESOLUTION(), xhdr=xhdr, coordsys='G'
        *thermalsz.e3.nuref.value = 3e11
     END

     ;; Fourth emission law (third relativistic correction)
     ;;-----------------------------------------------------
     IF szp.sz_relativistic GE 3 THEN BEGIN
        *thermalsz.e4.numin.value = 0.
        *thermalsz.e4.numax.value = FLOAT('INF')
        thermalsz.e4.ampl.unit = 'y_sz'
        ;; Get conversion factor to compute SZ relativistic correction at 300 GHz in y_sz
        c = PSM_RELATSZ3(3d11) * CONVERSION_FACTOR('W/m2/sr/Hz','y_sz', 3d11)
        IF thermalsz.e4.ampl.unit NE model.units.thermalsz THEN STOP
        *thermalsz.e4.ampl.value = GETPSMDIR('sz') + 'relatsz3_map.fits'
        xhdr = PSM_COMP_HDR(compname=param.names.thermalsz_basename,compid=thermalsz_id)
        WRITE_HPXMAP, (*thermalsz.e4.ampl.value), tstokes=c*TEMPORARY(ythetae3_map), datatype='COMP', psm_dtid=thermalsz_id, $
                      ordering='RING', units=thermalsz.e4.ampl.unit, lmax=SKY_LMAX(), beam=SKY_RESOLUTION(), xhdr=xhdr, coordsys='G'
        *thermalsz.e4.nuref.value = 3e11
     END

     ;; Fifth emission law (fourth relativistic correction)
     ;;-----------------------------------------------------
     IF szp.sz_relativistic GE 4 THEN BEGIN
        *thermalsz.e5.numin.value = 0.
        *thermalsz.e5.numax.value = FLOAT('INF')
        thermalsz.e5.ampl.unit = 'y_sz'
        ;; Get conversion factor to compute SZ relativistic correction at 300 GHz in y_sz
        c = PSM_RELATSZ4(3d11) * CONVERSION_FACTOR('W/m2/sr/Hz','y_sz', 3d11)
        IF thermalsz.e5.ampl.unit NE model.units.thermalsz THEN STOP
        *thermalsz.e5.ampl.value = GETPSMDIR('sz') + 'relatsz4_map.fits'
        xhdr = PSM_COMP_HDR(compname=param.names.thermalsz_basename,compid=thermalsz_id)
        WRITE_HPXMAP, (*thermalsz.e5.ampl.value), tstokes=c*TEMPORARY(ythetae4_map), datatype='COMP', psm_dtid=thermalsz_id, $
                      ordering='RING', units=thermalsz.e5.ampl.unit, lmax=SKY_LMAX(), beam=SKY_RESOLUTION(), xhdr=xhdr, coordsys='G'
        *thermalsz.e5.nuref.value = 3e11
     END

     ;; Save the component
     ;;--------------------
     SAVE_COMP, thermalsz
     PRTSTRUCT, thermalsz, file=GETPSMDIR('sz')+'thermalsz.txt', name='THERMALSZ'
     DESTROY_DATA, thermalsz
  ENDIF

  IF (STRLOWCASE(szp.sz_include_kinetic) EQ 'yes') THEN BEGIN
     PSM_ID_KEYGEN, id
     kineticsz = COMPONENT_STRUCT('cmb',name=param.names.kineticsz_basename,id=id)
     *kineticsz.e1.numin.value = 0.
     *kineticsz.e1.numax.value = FLOAT('INF')
     kineticsz.polarised = 0B
     PSM_ID_KEYGEN, kineticsz_id
     ;kineticsz.e1.ampl.form = 'map'

     ;; map originally in K/KCMB, converted into kinetic SZ units
     kineticsz.e1.ampl.unit = model.units.kineticsz
     fac = CONVERSION_FACTOR('K/KCMB',model.units.kineticsz, double = PRECISION_ISDOUBLE())
     *kineticsz.e1.ampl.value = GETPSMDIR('sz') + param.names.kineticsz_basename + '_map.fits'
     xhdr = PSM_COMP_HDR(compname=param.names.kineticsz_basename,compid=kineticsz_id)
     WRITE_HPXMAP, (*kineticsz.e1.ampl.value), tstokes=TEMPORARY(*taubeta_map)*fac*(-1.), datatype='COMP', psm_dtid=kineticsz_id, $
                   ordering='RING', units=kineticsz.e1.ampl.unit, lmax=SKY_LMAX(), beam=SKY_RESOLUTION(), xhdr=xhdr, coordsys='G'
     SAVE_COMP, kineticsz
     PRTSTRUCT, kineticsz, file=GETPSMDIR('sz')+'kineticsz.txt', name='KINETICSZ'
     DESTROY_DATA, kineticsz
  ENDIF


  ;; Reminder of how the polarised SZ map is produced above
  ;;--------------------------------------------------------
        ;; IF (STRLOWCASE(szp.sz_include_polarised) EQ 'yes') THEN BEGIN
        ;;    IF (pixp.pixtype EQ 'HEALPIX') THEN npix = pixp.npix
        ;;    szpolar = PTR_NEW(/alloc)
        ;;    (*szpolar) = DBLARR(npix,3)
        ;;    (*szpolar)[*,1] = TEMPORARY(tb2q)
        ;;    (*szpolar)[*,2] = TEMPORARY(tb2u)
        ;;    CHANGE_MAP_RESOLUTION, *szpolar, *szpolar,  beam_out=sky_beam, nside_out= pixp.nside, ordering_in='RING', ordering_out= pixp.ordering, /overwrite, lmax=SKY_LMAX()
        ;; ENDIF



  IF (STRLOWCASE(szp.sz_include_polarised) EQ 'yes') THEN BEGIN
     
     ;STOP
     PSM_DEBUG_MESSAGE, 'Polarised SZ map to be written', routine='SZ_PIPE', infolimit=1, stoplimit=3
     ;fitsfile = componentdir + param.names.sz_polarised_map_file
     ;WRITE_SIMPLE_PSM_MAP, fitsfile, szpolar, pix_param=pixp, psm_id=runp.psm_id, psm_vers=runp.psm_version, datatype='COMP_MODEL_MAP'
     ;PTR_FREE, szpolar
     ;PSM_UPDATE_HEADER_MAP, psm_beam='NO_BEAM', units='taubetat2', in_file=fitsfile, exten=1 
  ENDIF

  
  IF SZ_MODEL() NE 'no_sz' THEN BEGIN
     SAVE, szp, filename=GETPSMDIR('sz')+'sz_model_param.sav'
  ENDIF


;;****************************************************************************************************
;; Visualisation of SZ maps and catalogue histogrammes, if needed
;;****************************************************************************************************

  IF param.run.visu GE 1 THEN BEGIN

     ;; Visu of the SZ maps
     ;;---------------------
     VISU_COMP, 'sz', level=level, output=output, psm_output_dir=psm_output_dir

  ENDIF

  IF param.run.visu GE 2 THEN BEGIN

  ENDIF


GOTO, closing

;;****************************************************************************************************
;;*** Contamination by FIRPS in clusters need to be rewritten ***
;;****************************************************************************************************

          ;Contamination by cluster far infrared sources
          ;------------------------------

          ;; IF STRLOWCASE(szp.sz_firs_contamination) EQ 'yes' THEN BEGIN
          ;;    print, 'COMPUTING CLUSTER FAR-INFRARED SOURCES CATALOGS...'

          ;;    dir_model = fix_separator(!PSMROOT) + 'data/Intermediate/SZ/CFirPS/LDP/'

          ;;    Slim = param.sky.ps.ps_flux_limit*1d-9 ; from mJy to MJy

          ;;    map_dir = componentdir

          ;;    verbose=0

          ;;    freq = param.observation_parameters.obs_frequencies
          ;;    whf = WHERE(freq GE 30., nwhf)
          ;;    ; The LDP model is not given for nu LT 30GHz
          ;;    frequencies = (freq)[whf]

          ;;    scat = CLUSTER_FIR_SOURCE_MAPS(catalog, frequencies, cosmo=cosmo, iseed=useseed, dir=dir_model, flim=Slim, $
          ;;                                   map_dir=map_dir, nside=pixp.nside, chandra=chandra, xmm=xmm, $
          ;;                                   verbose=verbose)

          ;;    IF N_ELEMENTS(scat) GE 1 THEN BEGIN

          ;;    cmd = "newnode = {glon:0d, glat:0d, lum:0d, type:''"
          ;;    FOR i=0, nwhf-1 DO BEGIN
          ;;      cmd=cmd+',FLUX_'+STRTRIM(STRING(FIX(frequencies[i])),1)+'GHz:0d'
          ;;    ENDFOR
          ;;    cmd=cmd+'}'
          ;;    ok=EXECUTE(cmd)
          ;;    newscat = REPLICATE(newnode, N_ELEMENTS(scat))
          ;;    newscat.glon = scat.lon
          ;;    newscat.glat = scat.lat
          ;;    newscat.lum = scat.lum
          ;;    newscat.type = scat.type
          ;;    FOR i=0, nwhf-1 DO BEGIN
          ;;      cmd='newscat.FLUX_'+STRTRIM(STRING(FIX(frequencies[i])),1)+'GHz = REFORM((scat.flux)[i,*])*1d9'
          ;;      ok=EXECUTE(cmd)
          ;;    ENDFOR

          ;;    scatalogname = componentdir+'firps_catalog.fits'
          ;;    MWRFITS, newscat, scatalogname, /CREATE
          ;;    hd=HEADFITS(scatalogname, exten=1)
          ;;    SXADDPAR, hd, 'TTYPE1', SXPAR(hd,'TTYPE1'), 'Galactic longitude [deg]'
          ;;    SXADDPAR, hd, 'TTYPE2', SXPAR(hd,'TTYPE2'), 'Galactic latitude [deg]'
          ;;    SXADDPAR, hd, 'TTYPE3', SXPAR(hd,'TTYPE3'), 'Luminosity [10^12 Lsol unit]'
          ;;    SXADDPAR, hd, 'TTYPE4', SXPAR(hd,'TTYPE4'), 'Type [normal or starburst]'
          ;;    FOR i=0, nwhf-1 DO BEGIN
          ;;      cmd="SXADDPAR, hd, 'TTYPE"+STRTRIM(STRING(i+5),1)+"', SXPAR(hd,'TTYPE"+STRTRIM(STRING(i+5),1)+"'), 'Flux@"+STRTRIM(STRING(FIX(frequencies[i])),1)+"GHz [milliJy]'"
          ;;      ok=EXECUTE(cmd)
          ;;    ENDFOR
          ;;    MODFITS, scatalogname, 0, hd, exten=1

          ;;    ENDIF

          ;; ENDIF
          ;; ;********************************************************************************************

          ;; ;*** Contamination by RS in clusters also need to be rewritten ***

          ;; ;Contamination by radio sources
          ;; ;------------------------------
          ;; IF STRLOWCASE(szp.sz_radio_contamination) EQ 'yes' THEN BEGIN
             
          ;;    freq_list = gp.frequencies
          ;;    freq_list = freq_list(WHERE(freq_list NE 0))
          ;;    nfr = N_ELEMENTS(freq_list)
          ;;    FOR fr=0, nfr-1 DO cluster_rs_contamination, catalog, pixp.nside, freq_list(fr), param_cp = cosmo, componentdir=componentdir

          ;; ENDIF

          ;********************************************************************************************


closing:
END
