; -----------------------------------------------------------------------------
;
; Copyright (C) 2007, 2009-2012
;   Jacques Delabrouille
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	ANCILLARY2INTERMEDIATE_CLUSTER_CATALOG
;
; PURPOSE:
;	Compute an intermediate cluster catalog from an ancillary
;	cluster catalog (e.g. ROSAT ancillary catalog: the
;	intermediate catalog will include the mass derived from the
;	observed luminosity). 
;
; CATEGORY:
;	Sky model (clusters)
;
; CALLING SEQUENCE:
;	ANCILLARY2INTERMEDIATE_CLUSTER_CATALOG, Dir_anc, Name_anc, Dir_int, Name_int
;
; INPUTS:
;	Dir_anc:	Directory where the ancillary catalog is located.
;	Name_anc:	Name of the ancillary catalog (fits format).
;	Dir_int:	Directory where the intermediate catalog will be located.
;	Name_anc:	Name of the intermediate catalog.
;
; KEYWORD PARAMETERS:
;	COSMO:   	Structure containing the cosmological parameters (cf. DEFINE_COSMO)
;	CLUSTER:	Structure containing the structure formation parameters (cf. DEFINE_CLUSTER)
;	CATALOG:	Structure containing the output intermediate
;	                catalog (This is the output that is written to file).
;	NODISK: 	If set, don't write the result on the disk. The output is then in the catalog keyword.
;	NSTD:   	Set this keyword if you want to use the non standard Lx-M relation.
;	MB:     	Set this keyword if you want to use the the Malmquist bias corrected relation.
;	OPTICAL:     	To be set if the ancillary data is the optical MaxBCG catalogue and not the X-ray catalogue.
;	MCXC:     	To be set if the ancillary data is the MCXC catalogue of Rocco Piffaretti et al..
;
; OUTPUTS:
;	None. The output intermediate catalog is written to file (fits format).
;
; SIDE EFFECTS:
;	A file is written to disk, unless the /NODISK keyword is set
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille and Jean-Baptiste Melin
;	delabrouille@apc.univ-paris7.fr
;	jean-baptiste.melin@cea.fr
;
; MODIFICATION HISTORY:
;	08/03/2007:	Jean-Baptiste Melin, First version.
;	18/01/2009:	JBM, Lx[ROSAT]-M500 relation of Gabriel Pratt included.
;	21/01/2009:	JBM, Lx[ROSAT]-M500 relation of Gabriel Pratt
;	                removed temporarily (to be replaced later on).
; 	14/04/2009:	JBM, CATALOG and NODISK keywords added
; 	17/05/2010:	JBM, Recent Lx[ROSAT]-M500 relation of Gabriel Pratt coded.
; 	31/01/2011:	JBM, NSTD, MB and OPTICAL keywords added.
; 	18/03/2011:	JBM, MCXC keyword added.
; 	08/01/2012:	JD, moved away two (unused) subroutines (TEMP2MASS and
;	                LUMX2TEMP) into separate functions, renamed
;	                CLUSTER_TEMP2MASS and CLUSTER_LUMX2TEMP. Added
;	                doc and PSM license header.
;-
PRO ANCILLARY2INTERMEDIATE_CLUSTER_CATALOG, dir_anc, name_anc, dir_int, name_int, $
                                            cosmo=cosmo, cluster=cluster, catalog=catalog, nodisk=nodisk, $
                                            nstd=nstd, mb=mb, optical=optical, mcxc=mcxc


anc_catalogfile = dir_anc+name_anc
ok = TEST_PSM_DATA(anc_catalogfile,/get)

IF NOT KEYWORD_SET(optical) THEN BEGIN

;** xray part **

IF NOT KEYWORD_SET(mcxc) THEN BEGIN

;* rosat_cluster_catalog.fits *

anc_catalog = MRDFITS(anc_catalogfile, 1, hd, /SILENT)

whx = WHERE(anc_catalog.lx GT 0 AND anc_catalog.z GT 0, nwhx)
subcat = anc_catalog[whx]

;tx = CLUSTER_LUMX2TEMP(subcat.lx)
;mass = CLUSTER_TEMP2MASS(tx, subcat.z, cosmo=cosmo, cluster=cluster)

IF KEYWORD_SET(nstd) THEN BEGIN
  IF KEYWORD_SET(mb) THEN BEGIN
    cfit = 10.^(0.193)
    afit = 1.76
  ENDIF ELSE BEGIN
    cfit = 10.^(0.274)
    afit = 1.64
  ENDELSE
ENDIF ELSE BEGIN
  IF KEYWORD_SET(mb) THEN BEGIN
    cfit = 10.^(0.2150)
    afit = 1.6075
  ENDIF ELSE BEGIN
    cfit = 10.^(0.2949)
    afit = 1.5017
  ENDELSE
ENDELSE

cosmo_ref = DEFINE_COSMO(h=0.7, omega_m=0.3, omega_de=0.7, omega_b=0.04, /silent)

fact_h=cosmo.h/cosmo_ref.h

ez = (HUBBLE_AT_Z(subcat.z, cosmo=cosmo)/100./cosmo.h)

; h dependance included in the equation below but cosmology dependance (omega_m, lambda) still need to be written
; correctly
mass500 = fact_h^(-1.)*0.3*(ez^(-7/3.)*subcat.lx*fact_h^2./cfit)^(1./afit)

; use c500 matter = 3.0 to convert m500 into mvir
FAST_CONVERT_MASS_RADIUS, mass500, subcat.z, REPLICATE(500., N_ELEMENTS(subcat)), $
                          INTERPOL(cluster.big_delta_c_array, cluster.z_array, subcat.z), $
                          REPLICATE(3.0,N_ELEMENTS(subcat)) , $
                          r_in, r_out, mass, $
                          cosmo=cosmo, /five_hundred

clus = CREATE_STRUCT('name','','z',0.,'mass',0.,'lon',0.,'lat',0.)
new_cat = REPLICATE(clus,N_ELEMENTS(subcat))

new_cat.name=subcat.rxc
whname = WHERE(STRCOMPRESS(subcat.name,/REMOVE_ALL) NE '', nwhname)
new_cat[whname].name=STRTRIM(subcat[whname].name,2)
new_cat.z = subcat.z
new_cat.mass = mass

EULER, subcat.raj2000, subcat.dej2000, catlon, catlat, 1
new_cat.lon = catlon
new_cat.lat = catlat

catalog=new_cat

ENDIF ELSE BEGIN

;* MCXC_for_PSM_v0.save *

RESTORE, anc_catalogfile

whx = WHERE(mcxc.l_500 GT 0 AND mcxc.z GT 0, nwhx)
subcat = mcxc[whx]

IF KEYWORD_SET(nstd) THEN BEGIN
  IF KEYWORD_SET(mb) THEN BEGIN
    cfit = 10.^(0.193)
    afit = 1.76
  ENDIF ELSE BEGIN
    cfit = 10.^(0.274)
    afit = 1.64
  ENDELSE
ENDIF ELSE BEGIN
  IF KEYWORD_SET(mb) THEN BEGIN
    cfit = 10.^(0.2150)
    afit = 1.6075
  ENDIF ELSE BEGIN
    cfit = 10.^(0.2949)
    afit = 1.5017
  ENDELSE
ENDELSE

cosmo_ref = DEFINE_COSMO(h=0.7, omega_m=0.3, omega_de=0.7, omega_b=0.04, /silent)

fact_h=cosmo.h/cosmo_ref.h

ez = (HUBBLE_AT_Z(subcat.z, cosmo=cosmo)/100./cosmo.h)

; h dependance included in the equation below but cosmology dependance (omega_m, lambda) still need to be written
; correctly
mass500 = fact_h^(-1.)*0.3*(ez^(-7/3.)*subcat.l_500*fact_h^2./cfit)^(1./afit)

;plot, mcxc.z, mass500*10./mcxc.m_500, psym=4
;print, MEDIAN(mass500*10./mcxc.m_500)

; use c500 matter = 3.0 to convert m500 into mvir
FAST_CONVERT_MASS_RADIUS, mass500, subcat.z, REPLICATE(500., N_ELEMENTS(subcat)), $
                          INTERPOL(cluster.big_delta_c_array, cluster.z_array, subcat.z), $
                          REPLICATE(3.0,N_ELEMENTS(subcat)) , $
                          r_in, r_out, mass, $
                          cosmo=cosmo, /five_hundred

clus = CREATE_STRUCT('name','','z',0.,'mass',0.,'lon',0.,'lat',0.)
new_cat = REPLICATE(clus,N_ELEMENTS(subcat))

new_cat.name=subcat.name
whname = WHERE(STRCOMPRESS(subcat.name,/REMOVE_ALL) NE '', nwhname)
new_cat[whname].name=STRTRIM(subcat[whname].name,2)
new_cat.z = subcat.z
new_cat.mass = mass

EULER, subcat._raj2000, subcat._dej2000, catlon, catlat, 1
new_cat.lon = catlon
new_cat.lat = catlat

catalog=new_cat

ENDELSE

ENDIF ELSE BEGIN

;** optical part **

anc_catalog = MRDFITS(anc_catalogfile, 1, hd, /SILENT)

who = WHERE(anc_catalog.nbgal200 GT 0 AND anc_catalog.z GT 0, nwho)
subcat = anc_catalog[who]

;** Scaling from Planck data **
y20 = 7.4d-5 ; arcmin^2 (Planck Early Paper January 2011 : optical scaling)
alpha = 2.03
cy500_tilde = y20*(anc_catalog.nbgal200/20.)^alpha

; Normalisation of the Y500-M500 relation
factmass500=0.3
uredshift = [0.5, 0.7, 1.]
ez = (HUBBLE_AT_Z(uredshift, cosmo=cosmo)/100./cosmo.h)
ad = ANG_DIST(uredshift, cosmo=cosmo)
fadez = (ad/500.)^(-2.)
umass500 = [0.2, 0.5, 1.]
nclu = N_ELEMENTS(umass500)
FAST_CONVERT_MASS_RADIUS, umass500, uredshift, REPLICATE(500., nclu), $
                          INTERPOL(cluster.big_delta_c_array, cluster.z_array, uredshift), $
                          REPLICATE(3.0,nclu) , $
                          r_in, r_out, mass_vir, $
                          cosmo=cosmo, /five_hundred
IF KEYWORD_SET(nstd) THEN alp = 1./0.561 ELSE alp = 5/3.
xmm=1
chandra=0
norm=1
nr500=1
cco=0
mdi=0
cy500_model = FAST_INT_Y(uredshift, mass_vir, cosmo=cosmo, cluster=cluster, xmm=xmm, chandra=chandra, norm=norm, nr500=nr500,nstd=nstd,cco=cco,mdi=mdi)
;print, cy500_model/(umass500/factmass500)^alp/ez^(2/3.)/fadez
cy_star = MEAN(cy500_model/(umass500/factmass500)^alp/ez^(2/3.)/fadez)
;print, cy_star

mass500 = factmass500*(cy500_tilde/cy_star)^(1./alp)

; Scaling from the SDSS team
;bmn = 0.95 ; Rozo
;amn = 1.06 ; Rozo
;bmn = 0.724 ; Johnston
;amn = 1.25 ; Johnston
;mass500 = 0.1*EXP(bmn)*(anc_catalog.nbgal200/40.)^amn ; in 10^15 Msol

; use c500 matter = 3.0 to convert m500 into mvir
FAST_CONVERT_MASS_RADIUS, mass500, subcat.z, REPLICATE(500., N_ELEMENTS(subcat)), $
                          INTERPOL(cluster.big_delta_c_array, cluster.z_array, subcat.z), $
                          REPLICATE(3.0,N_ELEMENTS(subcat)) , $
                          r_in, r_out, mass, $
                          cosmo=cosmo, /five_hundred

clus = CREATE_STRUCT('name','','z',0.,'mass',0.,'lon',0.,'lat',0.)
new_cat = REPLICATE(clus,N_ELEMENTS(subcat))

new_cat.name = 'MaxBCG'
new_cat.z = subcat.z
new_cat.mass = mass

EULER, subcat.raj2000, subcat.dej2000, catlon, catlat, 1
new_cat.lon = catlon
new_cat.lat = catlat

catalog=new_cat

ENDELSE

IF NOT KEYWORD_SET(nodisk) THEN BEGIN

  MWRFITS, new_cat, dir_int+name_int, /CREATE
  hd = HEADFITS(dir_int+name_int,EXTEN=1)

  SXADDPAR, hd, 'TTYPE1', SXPAR(hd,'TTYPE1'), 'Name'
  SXADDPAR, hd, 'TTYPE2', SXPAR(hd,'TTYPE2'), 'Redshift'
  SXADDPAR, hd, 'TTYPE3', SXPAR(hd,'TTYPE3'), 'Mass [10^15 Msol unit]'
  SXADDPAR, hd, 'TTYPE4', SXPAR(hd,'TTYPE4'), 'Galactic longitude [deg]'
  SXADDPAR, hd, 'TTYPE5', SXPAR(hd,'TTYPE5'), 'Galactic latitude [deg]'

  MODFITS,dir_int+name_int,0,hd,EXTEN=1

ENDIF

END
