; -----------------------------------------------------------------------------
;
; Copyright (C) 2001, 2002, 2006, 2008, 2009
;   Jacques Delabrouille
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;  CREATE_Y_TAUBETA_MAP
; ONE LINE HELP:
;  create y map, tau*beta map and polarised maps
; PURPOSE:
;  create a Compton parameter map,
;  an optical depth * radial_velocity/c_light map,
;  an optical depth * (transverse_velocity/c_light)^2 map
;
; CALLING SEQUENCE:
;  CREATE_Y_TAUBETA_MAP, map_hd, catalog, y_map, taubeta_map, $
;                        taubeta2_q_map, taubeta2_u_map, $
;                        cosmo=cosmo,cluster=cluster, $
;                        r_cut=r_cut, beta=beta, polar=polar, $
;                        chandra=chandra, xmm=xmm, norm=norm, nr500=nr500, nstd=nstd
;
; INPUT:
;  map_hd : the header of the map
;  catalog : the catalog of clusters
; OUTPUT:
;  y_map : the y cluster map
;  taubeta_map : the tau*beta cluster map
;  If keyword polar is set,
;   taubeta2_q_map : the Stockes parameter Q tau*beta^2 map (ref. x,y axis)
;   taubeta2_u_map : the Stockes parameter U tau*beta^2 map (ref. x,y axis)
;  We have chosen :
;    Q = Iy-Ix
;    U = Iy'-Ix' where (x',y') = ROT((x,y), !pi/4, anticlockwise)
; KEYWORDS:
;  cosmo : the structure containing the cosmological parameters
;  cluster : the structure containing the structure formation parameters (cf. DEFINE_CLUSTER)
;  r_cut : the radius (in number of rc) where you stop your beta model
;          default : r_cut = 10.
;          OR
;          the radius (in number of rs) where you stop your nfw profile if candra or xmm is set.
;          default : r_cut = r_vir/rs
;  beta : the factor beta in the beta model (e.g. 0.66)
;         default : beta = 2./3
;  polar : set this keyword to compute polarized maps
;  chandra, xmm : if set, don't use the beta profile but the nfw profile of Chandra or XMM
;  norm : if set, use the normalisation found in the observations for cluster profiles
;  nr500 : integration is performed up to n*r500 instead of rvir (you must set the xmm or chandra keyword to use this keyword)
;  nstd : non standard XMM profile (you must set the xmm to use this keyword)
;  
; EXAMPLES:
;
; RESTRICTIONS:
;  Not fully tested yet. Report any bugs to Jacques Delabrouille
;  j.delabrouille@cdf.in2p3.fr
;
; PROCEDURES CALLED:
;  SXPAR, CLUSTER_MODEL, ANG2SIZE,
;  BETA_PROFILE, BETA_MODEL_MAP, PLACE_CLUSTER, NFW_PROFILE
; REVISION HISTORY
;  Written, Jean-Baptiste Melin April&May 2001
;  input or output keyword 'catalog' added, JD, May 2001
;  calls generate_header if header not defined,
;  M_inf=1e14 Msun by default, JBM & JD, 14 May 2001
;  change in number of steps computed for y and tau (nb_steps),JBM,May 17 2001
;  keyword polar added, JBM, May 21 2001
;  bug fixed in call to ang2size; check DEFINED(M_inf) instead of KEYWORD_SET, JD, 28 May 2001
;  small bug fixed for polarisation keyword, JBM, June 6 2001
;  Some polarised maps and no_more_than removed, calling sequence has changed, JBM, Sept 11 2001
;  Keyword cosmo introduced, JBM, Sept 27 2001
;  Changes in this routine due to changes in PLACE_CLUSTER, JBM, Aug 6 2002
;  cluster keyword added, JBM, Feb 6 2006
;  chandra, xmm and norm keywords added, JBM, June 23 2008
;  cosmetic improvements, JBM, July 1 2008
;  nr500 keyword added, JBM, Dec 8 2008
;  bug in call with beta profile and nr500 keyword fixed [Thank you Maude!], JBM, Jan 6 2009
;  bug in call with r_cut and chandra/xmm profile fixed, JBM, Jan 6 2009
;  nstd keyword added, JBM, June 13 2012
;-
PRO CREATE_Y_TAUBETA_MAP, map_hd, catalog, y_map, taubeta_map, $
                          taubeta2_q_map, taubeta2_u_map, $
                          cosmo=cosmo, cluster=cluster, $
                          r_cut=r_cut, beta=beta, polar=polar, $
                          chandra=chandra, xmm=xmm, norm=norm, nr500=nr500, nstd=nstd

IF ( N_PARAMS() EQ 0 ) THEN BEGIN
  print,'CALL IS : CREATE_Y_TAUBETA_MAP, map_hd, catalog, y_map, taubeta_map,'
  print,'                                taubeta2_q_map, taubeta2_u_map,'
  print,'Keywords : cosmo=cosmo, cluster=cluster'
  print,'r_cut=r_cut, beta=beta, polar=polar'
  print,'chandra=chandra, xmm=xmm, norm=norm, nr500=nr500, nstd=nstd'
  GOTO, closing
ENDIF

;useful physical units
c_light = 2.9979246d+08 ; in m/s
c_light = c_light/1e3 ; in km/s
electron_mass = 0.51099906*1d3

; create maps nine times larger than needed to avoid problems when placing the clusters
npix_map_x = SXPAR(map_hd,'NAXIS1')
npix_map_y = SXPAR(map_hd,'NAXIS2')
ang_map_x = ABS(SXPAR(map_hd,'CDELT1')*npix_map_x)
ang_map_y = ABS(SXPAR(map_hd,'CDELT2')*npix_map_y)
pixdim = ABS(SXPAR(map_hd,'CDELT1')) ; in deg
pixsurface = (pixdim*60.)^2. ; in arcmin^2
y_map = fltarr(3*npix_map_x, 3*npix_map_y)
taubeta_map = fltarr(3*npix_map_x, 3*npix_map_y)
IF KEYWORD_SET(polar) THEN BEGIN
	taubeta2_q_map = fltarr(3*npix_map_x, 3*npix_map_y)
	taubeta2_u_map = fltarr(3*npix_map_x, 3*npix_map_y)
ENDIF

; define r_cut and beta default values
IF KEYWORD_SET(chandra) OR KEYWORD_SET(xmm) THEN BEGIN
  ; If r_cut is defined, calling cluster_model.pro does not change the value 
  clus = CLUSTER_MODEL(catalog.z,catalog.mass, /rs, $
                       cosmo=cosmo, cluster=cluster, r_cut=r_cut, $
                       chandra=chandra, xmm=xmm, norm=norm, c500=c500, nstd=nstd)
ENDIF ELSE BEGIN
  IF NOT KEYWORD_SET(r_cut) THEN r_cut=10.
  IF NOT KEYWORD_SET(beta) THEN beta=2./3
ENDELSE

IF DEFINED(catalog) THEN cluster_nb = N_ELEMENTS(catalog) ELSE cluster_nb=0

IF (cluster_nb GT 0) THEN BEGIN
  PRINT, cluster_nb, ' cluster(s) in the catalog.'

  ;************************************************************
  ;* Compute the y and tau profiles for the cluster catalogue *
  ;************************************************************

  nb_steps = 100.
  IF (KEYWORD_SET(chandra) OR KEYWORD_SET(xmm)) AND KEYWORD_SET(nr500) THEN BEGIN
    r_array = nr500*c500/FLOAT(nb_steps)*(FINDGEN(nb_steps)+1)
  ENDIF ELSE BEGIN
    r_array = MAX(r_cut)/FLOAT(nb_steps)*(FINDGEN(nb_steps)+1)
  ENDELSE
  nelts = LONG(5000.)
  npack = cluster_nb/nelts
  IF npack GE 1 THEN BEGIN
  FOR i=0,npack-1 DO BEGIN
    jstart = i*nelts
    cat = catalog[jstart:jstart+nelts-1]
    IF (KEYWORD_SET(chandra) OR KEYWORD_SET(xmm)) THEN BEGIN
      res = NFW_PROFILE([cat.z], [cat.mass], r_array, $
                         cosmo=cosmo, cluster=cluster, r_cut=r_cut[jstart:jstart+nelts-1], $
                         chandra=chandra, xmm=xmm, norm=norm, nr500=nr500, nstd=nstd)
    ENDIF ELSE BEGIN
      res = BETA_PROFILE([cat.z], [cat.mass], r_array, $
                          cosmo=cosmo, cluster=cluster, $
                          beta=beta, r_cut=r_cut)
    ENDELSE
    IF i EQ 0 THEN y_and_tau = res ELSE y_and_tau = [y_and_tau,res]
  ENDFOR
  ENDIF
  IF NOT (npack EQ cluster_nb/FLOAT(nelts)) THEN BEGIN
    cat = catalog[npack*nelts:cluster_nb-1]
    IF (KEYWORD_SET(chandra) OR KEYWORD_SET(xmm)) THEN BEGIN
      res = NFW_PROFILE([cat.z], [cat.mass], r_array, $
                         cosmo=cosmo, cluster=cluster, r_cut=r_cut[npack*nelts:cluster_nb-1], $
                         chandra=chandra, xmm=xmm, norm=norm, nr500=nr500, nstd=nstd)
    ENDIF ELSE BEGIN
      res = BETA_PROFILE([cat.z], [cat.mass], r_array, $
                           cosmo=cosmo, cluster=cluster, $
                           beta=beta, r_cut=r_cut)
    ENDELSE
    IF NOT DEFINED(y_and_tau) THEN y_and_tau=res ELSE y_and_tau=[y_and_tau,res]
  ENDIF

  ;*********************************
  ;* Place each cluster on the map *
  ;*********************************

    FOR i=0L, cluster_nb-1 DO BEGIN
    IF (i/1000 EQ i/1000.) AND (i GT 0) THEN print, i, ' clusters placed on the map'

    ;--------------------------------
    ;- cluster center in pixel unit -
    ;--------------------------------

    xpix = npix_map_x+catalog[i].lon*npix_map_x/ang_map_x+(npix_map_x-1)/2.
    ypix = npix_map_y+catalog[i].lat*npix_map_y/ang_map_y+(npix_map_y-1)/2.

    ;---------------------------------
    ;- Estimate the cluster map size -
    ;---------------------------------

    r_sup = MAX(y_and_tau[i].rad_array)
    ANG2SIZE, ang_cluster_x, r_sup, catalog[i].z, /reverse, $
              cosmo=cosmo
    nb_pix = MAX([11,2*CEIL(npix_map_x*ang_cluster_x/ang_map_x)+1])
    IF (nb_pix LE 11) THEN d_pix = 2*r_sup/nb_pix ELSE d_pix = r_sup/(npix_map_x*ang_cluster_x/ang_map_x)

    ;--------------------------------
    ;- Create cluster y and tau map -
    ;--------------------------------

    cluster_y_map = BETA_MODEL_MAP(nb_pix, d_pix, y_and_tau[i].y_array, $
        y_and_tau[i].rad_array, hd_cluster, cosmo=cosmo)
    cluster_tau_map = BETA_MODEL_MAP(nb_pix, d_pix, y_and_tau[i].tau_array, $
      y_and_tau[i].rad_array, hd_cluster, cosmo=cosmo)
    spix_cluster = (SXPAR(hd_cluster,'raxis1')/ANG_DIST(catalog[i].z,cosmo=cosmo)/(!dtor)*60./ $
                    SXPAR(hd_cluster,'naxis1')/cosmo.h)^2.
    IF (KEYWORD_SET(chandra) OR KEYWORD_SET(xmm)) AND KEYWORD_SET(nr500) THEN BEGIN
      cluster_y_map = cluster_y_map*catalog[i].cyn500/(TOTAL(cluster_y_map)*spix_cluster)
      cluster_tau_map = cluster_tau_map*catalog[i].cyn500/(TOTAL(cluster_tau_map)*spix_cluster)*electron_mass/catalog[i].Te
    ENDIF ELSE BEGIN
      cluster_y_map = cluster_y_map*catalog[i].cy/(TOTAL(cluster_y_map)*spix_cluster)
      cluster_tau_map = cluster_tau_map*catalog[i].cy/(TOTAL(cluster_tau_map)*spix_cluster)*electron_mass/catalog[i].Te
    ENDELSE
    cluster_taubeta_map = (catalog[i].v_radial)/c_light*cluster_tau_map

    ;---------------------------------------
    ;- Paste cluster maps on the main maps -
    ;---------------------------------------

     PLACE_CLUSTER, cluster_y_map, y_map, xpix, ypix, $
		catalog[i].z, 0., hd_cluster, map_hd
     PLACE_CLUSTER, cluster_taubeta_map, taubeta_map, xpix, ypix, $
		catalog[i].z, 0., hd_cluster, map_hd
     IF KEYWORD_SET(polar) THEN BEGIN
		cluster_taubeta2_map=((catalog[i].v_transverse)/c_light)^2*cluster_tau_map
		cluster_taubeta2_q=cluster_taubeta2_map*COS(2*catalog[i].angle)
		cluster_taubeta2_u=cluster_taubeta2_map*SIN(2*catalog[i].angle)
		PLACE_CLUSTER, cluster_taubeta2_q, taubeta2_q_map, xpix, ypix, $
			catalog[i].z, 0., hd_cluster, map_hd
		PLACE_CLUSTER, cluster_taubeta2_u, taubeta2_u_map, xpix, ypix, $
			catalog[i].z, 0., hd_cluster, map_hd
	ENDIF
  ENDFOR
ENDIF

; keep only the central part of the map
y_map = y_map[npix_map_x:2*npix_map_x-1,npix_map_y:2*npix_map_y-1]
taubeta_map = taubeta_map[npix_map_x:2*npix_map_x-1,npix_map_y:2*npix_map_y-1]
IF KEYWORD_SET(polar) THEN BEGIN
taubeta2_q_map= taubeta2_q_map[npix_map_x:2*npix_map_x-1,npix_map_y:2*npix_map_y-1]
taubeta2_u_map= taubeta2_u_map[npix_map_x:2*npix_map_x-1,npix_map_y:2*npix_map_y-1]
ENDIF
closing:
END
