; -----------------------------------------------------------------------------
;
; Copyright (C) 2009, 2010
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	EMLAW_PARAMSTRUCT
;
; PURPOSE:
;	Function to create a structure with tags those corresponding
;	to the parameters of a given emission law.
;
; CATEGORY:
;	Emission law
;
; CALLING SEQUENCE:
;	result = EMLAW_PARAMSTRUCT(Law)
;
; INPUTS:
;	Law:	Name of the emission considered (Case
;	        insensitive). Can be any among the following: 'CMB',
;	        'BLACKBODY', 'GREYBODY', 'POWERLAW', 'CURVPOWERLAW',
;	        'FREEFREE', 'SPINDUST', 'THERMALSZ', 'DIRAC', 
;	        'RELATSZ1', 'RELATSZ2', 'RELATSZ3', 'RELATSZ4'
;	        'SUBMMLAPI', 'SPIRALGAL', 'STARBURSTGAL'. 
; 
;
; KEYWORD PARAMETERS:
;	INCLUDE_NURANGE:	If set, include NUMIN and NUMAX among
;	                        structure tags.
;
;	INCLUDE_AMPL:   	If set, include AMPL among
;	                        structure tags.
;
; OUTPUTS:
;	A structure with appropriate tags for the given emission law.
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;	23/09/2009:	Jacques Delabrouille, First version.
;	11/06/2010:	JD, Implemented DIRAC emission law.
;	05/01/2012:	JD, general clean-up, replaced old 'AMPL' tag
;	                by 'VALUEATNUREF', updated doc header.
;-
FUNCTION EMLAW_PARAMSTRUCT, law, include_nurange=include_nurange, include_ampl=include_ampl, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'EMLAW_PARAMSTRUCT: function which returns a structure containing parameters of an emission law'
     PSM_INFO_MESSAGE, 'SYNTAX: result = EMLAW_PARAMSTRUCT(law, /include_nurange, /include_ampl)'
     PSM_INFO_MESSAGE, "law can be:"
     PSM_INFO_MESSAGE, "CMB, BLACKBODY, GREYBODY, POWERLAW, CURVPOWERLAW, FREEFREE, SPINDUST, THERMALSZ, DIRAC"
     PSM_INFO_MESSAGE, "RELATSZ1, RELATSZ2, RELATSZ3, RELATSZ4, SUBMMLAPI, SPIRALGAL, STARBURSTGAL, SUBMMCAI"
     PSM_INFO_MESSAGE, "set /include_nurange to include 'NUMIN' and 'NUMAX' tags in structure"
     PSM_INFO_MESSAGE, "set /include_ampl to include amplitude 'AMPL' tag in parameter structure"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF
  
  tags = EMLAW_PARAMLIST(law, include_nurange=include_nurange, include_ampl=include_ampl, units=units, info=info)

  creation_string = "paramstruct = {law:law"
  FOR i=0, N_ELEMENTS(tags)-1 DO BEGIN
     value = PTR_NEW(/allocate_heap)
     ok = EXECUTE(tags[i]+"={value:value,unit:units[i],info:info[i]}")
     creation_string = creation_string +','+tags[i]+':'+tags[i]
  ENDFOR
  creation_string = creation_string+ '}'
  
  ok = EXECUTE(creation_string)
  IF NOT ok THEN MESSAGE, 'An error occured creating emission law parameter structure'
  RETURN, paramstruct
  
END
