; -----------------------------------------------------------------------------
;
; Copyright (C) 1999, 2008, 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	PLANCK_BNU
;
; PURPOSE:
;	Function to compute the blackbody emission.
;
; CATEGORY:
;	Emission law
;
; CALLING SEQUENCE:
;	result = PLANCK_BNU(Nu)
;
; INPUTS:
;	Nu:	Input frequency in Hz.
;
; KEYWORD PARAMETERS:
;	TEMP:   	Temperature at which the blackbody emission
;	                law is computed (default=CMB_TEMPERATURE()).
;
;	MEGAJYSR:	The output (B_nu) is in W/m2/sr/Hz, unless
;	                this keyword is set, in which case it is in MJy/sr. 
;
; OUTPUTS:
;	Blackbody emission. The output (B_nu) is in W/m2/sr/Hz, or
;	MJy/sr if MEGAJYSR is set.
;
; RESTRICTIONS:
;	Calculation is done in double precision
;
;	Report any bugs to Jacques Delabrouille 
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;	July 1999:	Jacques Delabrouille, First version.
;	02/10/2008:	JD, now default temperature !cosmo_param.t_cmb
;	05/12/2008:	JD, Bug corrected for test of existence of !cosmo_param
;	18/12/2009:	JD, Implemented MEGAJYSR keyword.
;	16/06/2010:	JD, Use CMB_TEMPERATURE() for default temperature.
;	January 2012:	JD, General documentation and clean-up.
;-
FUNCTION PLANCK_BNU, nu, temp=temp, megajysr=megajysr, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'PLANCK_BNU: function returning the Planck blackbody spectrum B_nu(T)'
     PSM_INFO_MESSAGE, 'SYNTAX: result = PLANCK_BNU(nu, temp=, /megajysr)'
     PSM_INFO_MESSAGE, "nu is the input frequency in Hz (may be an array)"
     PSM_INFO_MESSAGE, "temp is the temperature in K [default = 2.725K (CMB temperature)]"
     PSM_INFO_MESSAGE, "Set /megajysr for an output in MJy/sr [default W/m2/sr/Hz]"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  c = 299792458d
  h = 6.62607554d-34
  htimes10e20 = 6.62607554d-14
  k = 1.38065812d-23
  hoverk = h/k
  IF KEYWORD_SET(temp) EQ 0 THEN temp = CMB_TEMPERATURE()
  IF KEYWORD_SET(megajysr) NE 0 THEN RETURN, 2*(htimes10e20*nu)*(nu/c)^2 / (EXP(hoverk*nu/temp) - 1d) $
  ELSE RETURN, 2*(h*nu)*(nu/c)^2 / (EXP(hoverk*nu/temp) - 1d)
  
END
