; -----------------------------------------------------------------------------
;
; Copyright (C) 1999, 2008, 2009, 2010, 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	PLANCK_DBNU_DT
;
; PURPOSE:
;	Function to compute the derivative with respect to temperature
;	of a blackbody emission.
;
; CATEGORY:
;	Emission law
;
; CALLING SEQUENCE:
;	result = PLANCK_DBNU_DT(Nu)
;
; INPUTS:
;	Nu:	Input frequency in Hz.
;
; KEYWORD PARAMETERS:
;	TEMP:	Temperature T_0 of the blackbody, at which the
;	        derivative is computed (default=CMB_TEMPERATURE()). 
;
; OUTPUTS:
;	Derivative with respect to temperature of a blackbody
;	emission. The output (dB_nu/dT) is in W/m2/sr/Hz/K
;
; RESTRICTIONS:
;	Calculation is done in double precision
;
;	Report any bugs to Jacques Delabrouille 
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;	July, 1999:	Jacques Delabrouille, First version.
;	02/10/2008:	JD, now default temperature !cosmo_param.t_cmb
;	05/12/2008:	JD, Bug corrected for test of existence of !cosmo_param
;	05/01/2012:	JD, General documentation and clean-up
;-
FUNCTION planck_dBnu_dT, nu, temp=temp

c = 299792458d
h = 6.62607554d-34
k = 1.38065812d-23
IF KEYWORD_SET(temp) EQ 0 THEN temp = CMB_TEMPERATURE()

b_nu = planck_bnu(nu,temp=temp)

RETURN, (b_nu*c/nu/temp)^2 / 2d0 * EXP(h*nu/k/temp) / k
END
