; -----------------------------------------------------------------------------
;
; Copyright (C) 2011-2012
;   Jacques Delabrouille
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	PSM_THERMALSZ
;
; PURPOSE:
;	Implements the relativistic thermal SZ effect function, as
;	computed in Nozawa, Itoh and Koyama (1998), ApJ, 508, 17.
;
; CATEGORY:
;	Emission Law
;
; CALLING SEQUENCE:
;	result = PSM_THERMALSZ(nu)
;
; INPUTS:
;	nu:	Frequency in Hz
;
; OPTIONAL INPUTS:
;	
; KEYWORD PARAMETERS:
;	ORDER:  	Order of the expansion used for computing the 
;	                relativistic Sunyaev Zel'dovich effect (if 
;	                omitted, calculation is made to order 0, which
;	                corresponds to non-relativistic approximation). 
;
;	THETA_E:	Electron temperature, in units of electron
;	                mass (i.e. T_e(keV) / 511).
;
; OUTPUTS:
;	The result of this function is the SZ thermal emission for a
;	Compton parameter y of 1, at the frequency nu. Multiply by y
;	to get the total emission, in W/m2/sr/Hz.
;
; OPTIONAL OUTPUTS:
;	None
;
; COMMON BLOCKS:
;	None
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;	Physical constants must have been loaded using DEFINE_PHYSICAL_CONSTANTS
;
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;	Straightforward.
;
; EXAMPLE:
;	Plot the non-relativistic approximation of the SZ emission law
;	of a 10 keV cluster as follows: 
;	
;	IDL> nu = (FINDGEN(1000)+1)*1e9
;	IDL> PLOT, PSM_THERMALSZ(nu)
;	
;	Overplot the relativistic SZ, compute with corrections up to
;	order 4 in theta_e:
;	
;	IDL> OPLOT, PSM_THERMALSZ(nu, order=4, theta_e=10./511.)
;	
; MODIFICATION HISTORY:
; 	08/03/2012:	Jacques Delabrouille, First version, adapted
; 		        from script written by Jean-Baptiste Melin.
;-
FUNCTION PSM_THERMALSZ, freq, order=order, theta_e=theta_e

  IF NOT DEFINED(order) THEN order=0

  h = !phyconst.h
  k = !phyconst.k
  c = !phyconst.c
  T = CMB_TEMPERATURE()
  
  x = h*freq/k/T
  ex = exp(x)
  
  xt = x*(ex+1d)/(ex-1d)
  st = 2*x/(exp(x/2.)-exp(-x/2.))
  
  y0 = -4d +xt
  
  y1 = -10d +47d/2*xt -42d/5*xt^2. +7d/10*xt^3. +st^2.*(-21d/5 +7d/5*xt)
  
  y2 = -15d/2 +1023d/8*xt -868d/5*xt^2. +329d/5*xt^3. -44d/5*xt^4. + 11d/30*xt^5. $
       +st^2.*(-434d/5 +658d/5*xt -242d/5*xt^2. +143d/30*xt^3.) $
       +st^4.*(-44d/5 +187d/60*xt)
  
  y3 = 15d/2 +2505d/8*xt -7098d/5*xt^2. +14253d/10*xt^3. -18594d/35.*xt^4. $
       +12059d/140*xt^5. -128d/21*xt^6. +16d/105*xt^7. $
       +st^2.*(-7098d/10 +14253d/5*xt -102267d/35*xt^2. +156767d/140*xt^3. -1216d/7*xt^4. +64d/7*xt^5.) $
       +st^4.*(-18594d/35 +205003d/280*xt -1920d/7*xt^2. +1024d/35*xt^3.) $
       +st^6.*(-544d/21 +992d/105*xt)
  
  y4 = -135d/32 +30375d/128*xt -62391d/10*xt^2. +614727d/40*xt^3. -124389d/10*xt^4. $
       +355703d/80*xt^5. -16568d/21*xt^6. +7516d/105*xt^7. -22d/7*xt^8. +11d/210*xt^9. $
       +st^2.*(-62391d/20 +614727d/20*xt -1368279d/20*xt^2. +4624139d/80*xt^3. -157396d/7*xt^4. $
               +30064d/7*xt^5. -2717d/7*xt^6. +2761d/210*xt^7.) $
       +st^4.*(-124389d/10 +6046951d/160*xt -248520d/7*xt^2. +481024d/35*xt^3. -15972d/7*xt^4. $
               +18689d/140*xt^5.) $
       +st^6.*(-70414d/21 +465992d/105*xt -11792d/7*xt^2. +19778d/105*xt^3.) $
       +st^8.*(-682d/7 +7601d/210*xt)
    
  CASE order OF
     0: RETURN, x * ex / (ex-1d) * y0 * planck_bnu(freq,temp=T)
     1: RETURN, x * ex / (ex-1d) * (y0+theta_e*y1) * planck_bnu(freq,temp=T)
     2: RETURN, x * ex / (ex-1d) * (y0+theta_e*y1+theta_e^2.*y2) * planck_bnu(freq,temp=T)
     3: RETURN, x * ex / (ex-1d) * (y0+theta_e*y1+theta_e^2.*y2+theta_e^3.*y3) * planck_bnu(freq,temp=T)
     4: RETURN, x * ex / (ex-1d) * (y0+theta_e*y1+theta_e^2.*y2+theta_e^3.*y3+theta_e^4.*y4) * planck_bnu(freq,temp=T) 
  ENDCASE

END
