; -----------------------------------------------------------------------------
;
; Copyright (C) 2001
;   Jacques Delabrouille
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	DIST_TO_CENTER
;
; PURPOSE:
;	Make an array with element value the exact distance in pixels to the center.
;
; CATEGORY:
;	Array handling
;
; CALLING SEQUENCE:
;	Result = DIST_TO_CENTER(Nside)
;
; INPUTS:
;	Nside:	The number of pixels on each side. Note that this
;	        parameter is not connected with any HEALPix map here.
;
; KEYWORD PARAMETERS:
;	LINEAR:	Set this keyword for a 1D output array.
;
;	SQUARE:	Set this keyword for a 2D square output array (default).
;
;	CUBIC:	Set this keyword for a 3D cubic output array.
;
; OUTPUTS:
;	A 1D, 2D (default) or 3D array with pixel value the exact
;	distance in pixels to the center of the map. 
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
; 	January 2001:	Jacques Delabrouille, First version
;	May 2001;	Jean-Batiste Melin and JD, bug corrected,
;	                /cubic and /linear keywords added.
; 	January 2012:	JD, updated doc header + some clean-up
;-
FUNCTION DIST_TO_CENTER, nside, linear=linear, square=square, cubic=cubic

  IF ((KEYWORD_SET(linear) + KEYWORD_SET(square) + KEYWORD_SET(cubic)) NE 1) THEN BEGIN
     linear=0
     square=1
     cubic=0
  ENDIF ELSE BEGIN
     IF KEYWORD_SET(linear) EQ 0 THEN linear=0
     IF KEYWORD_SET(square) EQ 0 THEN square=0
     IF KEYWORD_SET(cubic) EQ 0 THEN cubic=0
  ENDELSE
  
  IF linear THEN BEGIN
     IF (nside MOD 2) THEN res = shift(DIST(nside,1),nside/2) ELSE BEGIN
        res = (FINDGEN(nside/2)+.5)
        res = [ROTATE(res, 2),res]
     ENDELSE
  ENDIF
  
  IF square THEN BEGIN
     res=FLTARR(nside,nside)
     IF (nside MOD 2) THEN res = shift(DIST(nside),nside/2,nside/2) ELSE BEGIN
        pave = FLTARR(nside/2,nside/2)
        line = (FINDGEN(nside/2)+.5)^2
        FOR i=0,nside/2-1 DO pave(i,*) = line
        pave=pave+transpose(pave)
        res(0:nside/2-1,0:nside/2-1) = ROTATE(pave,2)
        res(0:nside/2-1,nside/2:nside-1) = ROTATE(pave,1)
        res(nside/2:nside-1,nside/2:nside-1) = pave
        res(nside/2:nside-1,0:nside/2-1) = TRANSPOSE(res(0:nside/2-1,nside/2:nside-1))
        res=SQRT(res)
     ENDELSE
  ENDIF 
  
  IF cubic THEN BEGIN
     res=FLTARR(nside,nside,nside)
     IF (nside MOD 2) THEN BEGIN
        line = (shift(DIST(nside,1),nside/2))^2
        plan = (shift(DIST(nside),nside/2,nside/2)^2)
        FOR i=0,nside-1 DO res(i,*,*) = plan+line(i)
        res = SQRT(res)
     ENDIF ELSE BEGIN
        res2 = FLTARR(nside,nside)
        pave = FLTARR(nside/2,nside/2)
        line = (FINDGEN(nside/2)+.5)^2
        FOR i=0,nside/2-1 DO pave(i,*) = line
        pave=pave+transpose(pave)
        res2(0:nside/2-1,0:nside/2-1) = ROTATE(pave,2)
        res2(0:nside/2-1,nside/2:nside-1) = ROTATE(pave,1)
        res2(nside/2:nside-1,nside/2:nside-1) = pave
        res2(nside/2:nside-1,0:nside/2-1) = TRANSPOSE(res2(0:nside/2-1,nside/2:nside-1))
        line = [ROTATE(line, 2),line]
        FOR i=0,nside-1 DO res(i,*,*) = res2+line(i)
        res=SQRT(res)
     ENDELSE
  ENDIF
  
  
  RETURN, res
END
