; -----------------------------------------------------------------------------
;
; Copyright (C) 2009, 2010, 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	ISPOLARISED
;
; PURPOSE:
;	...
;
;-
FUNCTION ISPOLARISED, input, alm=alm, map=map, cl=cl, component=component, help=help

  IF ((KEYWORD_SET(help) NE 0) OR (N_PARAMS() EQ 0)) THEN BEGIN
     PRINT, ''
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'ISPOLARISED: function to check whether input data is polarised'
     PSM_INFO_MESSAGE, 'SYNTAX: result = ISPOLARISED(input, alm=, map=, cl=, component=)'
     PSM_INFO_MESSAGE, 'set /alm if the input is an array of alm values.'
     PSM_INFO_MESSAGE, 'set /map if the input is a healpix map in array format.'
     PSM_INFO_MESSAGE, 'set /cl if the input is a cl structure.'
     PSM_INFO_MESSAGE, 'set /component if the input is a component structure.'
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     GOTO, closing
  ENDIF

  ;ON_ERROR, 2

  s = SIZE(input,/type)

  
  ;;-------------------------------------------------------------------------------------------------
  ;; If the input is a string (i.e. the name of a file or the name of a component)
  ;;-------------------------------------------------------------------------------------------------
  IF s EQ 7 THEN BEGIN

     ;; Check the content of a fits file, if the input is such
     ;;--------------------------------------------------------
     IF ISFITSFILE(input) THEN BEGIN
        h = HEADFITS(input, exten=1)
        psm_dtfm = SXPAR(h,'PSM_DTFM', count=count)
        IF count EQ 0 THEN BEGIN
           IF KEYWORD_SET(map) NE 0 THEN BEGIN
              psm_dtfm = 'MAP'
              count=1
           ENDIF ELSE BEGIN
              IF KEYWORD_SET(alm) NE 0 THEN BEGIN
                 psm_dtfm = 'ALM'
                 count=1
              ENDIF
           ENDELSE
        ENDIF
        
        ;; If PSM data format written in the header, or passed by keyword
        ;;----------------------------------------------------------------
        IF count EQ 1 THEN BEGIN
           CASE STRTRIM(psm_dtfm,2) OF
              'MAP':BEGIN
                 IF (KEYWORD_SET(alm) OR KEYWORD_SET(cl)) THEN MESSAGE, 'Unexpected data format'
                 tfields = SXPAR(h,'TFIELDS')
                 IF tfields EQ 3 THEN RETURN, 1B
                 IF tfields EQ 1 THEN RETURN, 0B
              END
              'ALM':BEGIN
                 IF (KEYWORD_SET(map) OR KEYWORD_SET(cl)) THEN MESSAGE, 'Unexpected data format'
                 FITS_INFO, input, n_ext=n_ext, /silent
                 IF n_ext EQ 3 THEN RETURN, 1B
                 IF n_ext EQ 1 THEN RETURN, 0B
              END
              'CL':BEGIN
                 IF (KEYWORD_SET(alm) OR KEYWORD_SET(map)) THEN MESSAGE, 'Unexpected data format'
              END
           ENDCASE

        ;; If PSM data format NOT written in the header nor passed by keyword
        ;;--------------------------------------------------------------------
        ENDIF ELSE BEGIN           
           IF PSM_CAREFULNESS() GE 1 THEN MESSAGE, 'Polarisation information not found in file '+input 
        ENDELSE
        
     ENDIF
     
     ;; Check whether the corresponding component is polarised, if the input is the name of a known PSM component
     ;;-----------------------------------------------------------------------------------------------------------
     possible_comp = PSM_ALLCOMP()
     wh = WHERE(possible_comp EQ input, nwh)
     IF nwh EQ 0 THEN BEGIN
        IF KEYWORD_SET(component) NE 0 THEN MESSAGE, 'input is not a valid PSM component'
     ENDIF 
     IF nwh EQ 1 THEN BEGIN
        RESTORE_COMP, comp, compname=possible_comp[wh[0]]
        result = comp.polarised
        DESTROY_DATA, comp
        RETURN, result
     ENDIF

  ENDIF                         ; (case where the input is a string)

  ;;-------------------------------------------------------------------------------------------------
  ;; If the input is an array (i.e. an array of healpix pixels or an array of alm)
  ;;-------------------------------------------------------------------------------------------------
  IF s LE 5 THEN BEGIN

     ndim = SIZE(input,/n_dimensions)
     dims = SIZE(input,/dimensions)
     IF ndim LE 1 AND dims[0] NE 3 THEN RETURN, 0B

     IF KEYWORD_SET(map) NE 0 THEN BEGIN
        IF ndim LT 2 THEN RETURN, 0B
        IF ndim EQ 2 THEN BEGIN
           IF dims[1] EQ 3 THEN RETURN, 1B
           IF dims[1] EQ 1 THEN RETURN, 0B
        ENDIF
     ENDIF

     IF KEYWORD_SET(alm) NE 0 THEN BEGIN
        IF ndim LE 2 THEN RETURN, 0B
        IF ndim EQ 3 THEN BEGIN
           IF ((dims[2] EQ 3) AND (dims[1] EQ 2)) THEN RETURN, 1B
           IF ((dims[2] EQ 1) AND (dims[1] EQ 2)) THEN RETURN, 0B
        ENDIF
     ENDIF

  ENDIF

  ON_ERROR, 2
  MESSAGE, 'Cannot determine whether input is polarised'

closing:
END
