; -----------------------------------------------------------------------------
;
; Copyright (C) 2009-2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	HEALPIX_DENSITY
;
; PURPOSE:
;	Function to compute a density of events on the sphere
;	(e.g. point source density on the sky for a given flux range). 
;
; CATEGORY:
;	Map tools
;
; CALLING SEQUENCE:
;	result = HEALPIX_DENSITY(Theta_i_rad, Phi_i_rad, Nside)
;
; INPUTS:
;	Theta_i_rad:	A vector of colatitudes for the events, in radian.
;
;	Phi_i_rad:	A vector of corresponding longitudes for the
;	                events, in radian.
;
;	Nside:  	HEALPix nside for the output density
;	                function (the result will be average number of
;	                events per pixel at this value of nside).
;
; KEYWORD PARAMETERS:
;	THRESHOLD:	the minimum number of events in a region to
;                	consider the density in that region well measured (default 10).
;
;	VISU:   	visualize number of events and density map if set.
;
; OUTPUTS:
;	A map of density of events/objects.
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
; 	02/04/2009: 	Jacques Delabrouille, First version
;	05/01/2012: 	JD, updated header doc
;-
FUNCTION HEALPIX_DENSITY_NUMBERCOUNT, ip, ns

  map = LONARR(12L*LONG(ns)*LONG(ns))
  FOR i=0L, N_ELEMENTS(ip)-1 DO map(ip[i]) = map(ip[i]) + 1L
  RETURN, map

END


FUNCTION HEALPIX_DENSITY, theta_i_rad, phi_i_rad, nside, threshold=threshold, visu=visu

  IF NOT KEYWORD_SET(threshold) THEN threshold=10

  ni = N_ELEMENTS(theta_i_rad)
  map = FLTARR(12L*LONG(nside)*LONG(nside))
  filled = BYTARR(12L*LONG(nside)*LONG(nside))

  ;; Initialize for first scale
  ;;----------------------------
  ns = nside
  t = theta_i_rad
  p = phi_i_rad

  it = 0

  WHILE ns GE 1 DO BEGIN

     ;compute number density at nside = ns for remaining events
        ANG2PIX_NEST, nside, t, p, ip
        dmap = HEALPIX_DENSITY_NUMBERCOUNT(ip, nside)
        IF KEYWORD_SET(visu) NE 0 THEN IF it EQ 0 THEN MOLLVIEW, dmap, /online, /nested, title = 'NUMBER PER PIXEL AT NSIDE='+NUMBER2STRING(nside)
        UD_GRADE, FLOAT(dmap), newdmap, nside_out=ns, order_in='NESTED', order_out='NESTED'
        UD_GRADE, newdmap, dmap, nside_out=nside, order_in='NESTED', order_out='NESTED'
        mapok = dmap GE threshold * (FLOAT(ns)/FLOAT(nside))^2
        dmap = dmap * mapok
        
     ;continue only if mapok is not zero everywhere in not yet covered pixels
        new_regions = mapok GT filled
        ;;mollview, new_regions, /online, /nested, title='new regions at iteration '+NUMBER2STRING(it)
        whok = WHERE((new_regions) GE 1,nwh)
        IF nwh NE 0 THEN BEGIN
           ;find how to rescale number density to take into account pixels for which density is already calculated
              UD_GRADE, FLOAT(new_regions), w, nside_out=ns, order_in='NESTED', order_out='NESTED'
              UD_GRADE, w, ww, nside_out=nside, order_in='NESTED', order_out='NESTED'
           ;update the final density map and the map of filled pixels
              map[whok] = dmap[whok] / ww[whok]
              filled[whok] = 1
           ;update t, p
              wh_tbd = WHERE(filled[ip] EQ 0, nwh_tbd)
              IF nwh_tbd NE 0 THEN BEGIN
                 t = t[wh_tbd]
                 p = p[wh_tbd]
              ENDIF ELSE BEGIN
                 ns = 0 ;;; end the loop, all pixels are filled
              ENDELSE
        ENDIF

     ;update ns
        ns = ns/2
        it = it+1

  ENDWHILE

  ;; Fill unfilled pixels with the AVERAGE of the remaining events

  wh_tbd = WHERE(filled[ip] EQ 0, nwh_tbd)
  IF nwh_tbd NE 0 THEN BEGIN
     wh0 = WHERE(map EQ 0, nwh0)
     map(wh0) = FLOAT(nwh_tbd)/FLOAT(nwh0)
  ENDIF

  IF KEYWORD_SET(visu) NE 0 THEN mollview, map, /online, /nested, title='ESTIMATED DENSITY AT NSIDE='+NUMBER2STRING(nside)


RETURN, map

END
