; -----------------------------------------------------------------------------
;
; Copyright (C) 2010, 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	HEALPIX_NEIGHBOURS
;
; PURPOSE:
;	Finds the direct neighbours of a healpix pixel.
;
; CATEGORY:
;	Map tools
;
; CALLING SEQUENCE:
;	HEALPIX_NEIGHBOURS, Pixnum, Neighbours
;
; INPUTS:
;	Pixnum:	Pixel number.
;
; KEYWORD PARAMETERS:
;	NSIDE:  	HEALPix nside used for pixel numbering.
;
;	ORDERING:	HEALPix ordering used for pixel numbering
;	(default='RING', unless /NESTED is set).
;
;	NESTED: 	Set for nested'NESTED' HEALPix ordering.
;
; OUTPUTS:
;	Neighbours:	Pixel number of the neighbours of the input pixel.
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
; 	22/01/2010:	Jacques Delabrouille, First version.
; 	12/02/2010:	JD, now calls Healpix routines.
; 	January 2012:	JD, updated header doc.
;
;-
PRO HEALPIX_NEIGHBOURS, pixnum, neighbours, nside=nside, ordering=ordering, nested=nested
  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'HEALPIX_NEIGHBOURS: procedure to get the direct neighbours of a given healpix pixel'
     PSM_INFO_MESSAGE, 'SYNTAX: HEALPIX_NEIGHBOURS, pixnum, neighbours, nside=, ordering=, nested='
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF

  ;; Default ordering
  ;;------------------
  IF NOT KEYWORD_SET(ordering) THEN BEGIN
     IF NOT KEYWORD_SET(nested) THEN ordering='RING' ELSE ordering='NESTED'
  ENDIF

  ;; Check nside
  ;;-------------
  IF NOT KEYWORD_SET(nside) THEN MESSAGE, 'nside keyword must be set'

  ;; Find number of pixels
  ;;-----------------------
  npix = N_ELEMENTS(pixnum)
  IF npix NE 1 THEN MESSAGE, 'One and only one input pixel is allowed in call to HEALPIX_NEIGHBOURS'

  CASE STRUPCASE(ordering) OF
     'RING': NEIGHBOURS_RING, nside, pixnum, neighbours
     'NESTED': NEIGHBOURS_NEST, nside, pixnum, neighbours
     ELSE: BEGIN
        MESSAGE, "HEALPix ordering should be 'RING' or 'NESTED'"
     ENDELSE
  ENDCASE

END
