; -----------------------------------------------------------------------------
;
; Copyright (C) 2010, 2011
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	HPX_LOCALMEAN
;
; PURPOSE:
;	...
;
;-
FUNCTION HPX_LOCALMEAN, map, ordering=ordering, degrade=degrade, niter=niter, max_dist=max_dist, min_dist=min_dist, $
                   nocentralpix=nocentralpix, minpts=minpts, help=help
                   

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'HPX_LOCALMEAN: function to calculate local average value (mean) of a healpix map'
     PSM_INFO_MESSAGE, 'SYNTAX: result = HPX_LOCALMEAN(map, ordering=, degrade=, niter=, max_dist=, nocentralpix=, minpts=)'
     PSM_INFO_MESSAGE, "'map' is the input healpix map"
     PSM_INFO_MESSAGE, "'ordering' is the ordering of map, and of the output"
     PSM_INFO_MESSAGE, "The procedure first finds the mean in pixels larger by the factor degrade (must be a power of 2, "
     PSM_INFO_MESSAGE, "default is 8), and then uses HPX_PIXSMOOTHING to average the stats over all neighbouring pixels"
     PSM_INFO_MESSAGE, "at degraded resolution"
     PSM_INFO_MESSAGE, "'niter' sets the number of iterations of the HPX_PIXSMOOTHING procedure (done 1+niter times)"
     PSM_INFO_MESSAGE, "Set /nocentralpix for discarding the (degrade x degrade) central pixels in mean calculation"
     PSM_INFO_MESSAGE, "If the keyword 'max_dist' is set to a non-zero number, then HPX_PIXSMOOTHING uses pixels"
     PSM_INFO_MESSAGE, "within a distance of 'max_dist' superpixels of the central superpixel (degrade x degrade times bigger)"
     PSM_INFO_MESSAGE, "If 'max_dist' is set to a non-zero number AND 'min_dist' is set also, the code uses pixels"
     PSM_INFO_MESSAGE, "between 'min_dist' and 'max_dist' of the center of the central one (in units of average pixel size)."
     PSM_INFO_MESSAGE, "rather than using its direct neighbours."
     PSM_INFO_MESSAGE, "Set minpts [default = 1] to the minimum number of points required to return the mean."
     PSM_INFO_MESSAGE, "pixels for which the minimum is not reached are flagged as bad."
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  IF NOT DEFINED(minpts) THEN minpts=1

  npix = N_ELEMENTS(map)
  IF NOT DEFINED(ordering) THEN ordering='RING'
  nside = NPIX2NSIDE(npix)

  CASE STRUPCASE(ordering) OF
     'RING': sigma = REORDER(map,/r2n)
     'NESTED': sigma = map
  ENDCASE

  good = (sigma NE !healpix.bad_value)
  IF NOT KEYWORD_SET(degrade) THEN degrade=8L ELSE degrade = 2L^FIX(ALOG2(degrade))
  nnn = LONG(degrade)^2
  sigma = REFORM(TEMPORARY(sigma), nnn, npix/nnn) 
  good = REFORM(TEMPORARY(good), nnn, npix/nnn) 
  denom = TOTAL(good,1) & whbad = WHERE(denom EQ 0, nwhbad, complement=whgood, ncomplement=nwhgood)

  mean = TOTAL(TEMPORARY(sigma)*TEMPORARY(good),1)
  IF nwhbad NE 0 THEN BEGIN
     mean[whbad] = !healpix.bad_value
     denom[whbad] = !healpix.bad_value
  ENDIF
;stop
  HPX_PIXSMOOTHING, TEMPORARY(mean), mean, ordering='NESTED', max_dist=max_dist, min_dist=min_dist, niter=niter, nocentralpix=nocentralpix, /total
  HPX_PIXSMOOTHING, TEMPORARY(denom), meandenom, ordering='NESTED', max_dist=max_dist, min_dist=min_dist, niter=niter, nocentralpix=nocentralpix, /total
;stop
  whok = WHERE(meandenom GE minpts, nwhok, complement=whnotok, ncomplement=nwhnotok)
  IF nwhok NE 0 THEN mean[whok] = mean[whok]/meandenom[whok]
  IF nwhnotok NE 0 THEN mean[whnotok] = !healpix.bad_value
  UD_GRADE, TEMPORARY(mean), mean, nside_out=nside, order_in='NESTED', order_out=ordering

  RETURN, mean

END
