; -----------------------------------------------------------------------------
;
; Copyright (C) 2010
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	HPX_LOCALMEDIAN
;
; PURPOSE:
;	...
;
;-
FUNCTION HPX_LOCALMEDIAN, map, ordering=ordering, nocentralpix=nocentralpix, minpts=minpts, help=help
                   

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'HPX_LOCALMEDIAN: function to calculate local median value of a healpix map'
     PSM_INFO_MESSAGE, 'SYNTAX: result = HPX_LOCALMEDIAN(map, ordering=, nocentralpix=, minpts=)'
     PSM_INFO_MESSAGE, "The HPX_LOCALMEDIAN function computes the median of a pixel and its direct neighbours."
     PSM_INFO_MESSAGE, "'map' is the input healpix map"
     PSM_INFO_MESSAGE, "'ordering' is the ordering of map, and of the output."
     PSM_INFO_MESSAGE, "Set /nocentralpix for discarding the central pixel in the calculation"
     PSM_INFO_MESSAGE, "Set minpts [default = 1] to the minimum number of points required to return the median."
     PSM_INFO_MESSAGE, "Pixels for which this minimum is not reached are flagged as bad."
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  IF NOT DEFINED(minpts) THEN minpts=1

  npix = N_ELEMENTS(map)

  IF NOT DEFINED(ordering) THEN ordering='RING'
  nside = NPIX2NSIDE(npix)

  CASE STRUPCASE(ordering) OF
     'RING': map = REORDER(TEMPORARY(map),/r2n)
     'NESTED': ;; do nothing
  ENDCASE

  ;;good = (map NE !healpix.bad_value)

  CASE SIZE(map[0],/type) OF
     5: med = DBLARR(npix)
     4: med = FLTARR(npix)
     3: med = LONARR(npix)
     2: med = INTARR(npix)
     1: med = BYTARR(npix)
     ELSE: MESSAGE, 'Case not implemented'
  ENDCASE

  ;; loop on pixels
  ;;---------------------
  t=systime(2)
  FOR i=0L, npix-1 DO BEGIN
     IF (i/1000000L) EQ (i/1000000d0) THEN PRINT, 'i='+NUMBER2STRING(i), '   '+NUMBER2STRING(LONG(100*FLOAT(i)/npix))+'%', systime(2)-t, ' seconds elapsed'

     ;; Find list of pixels in the neighborhood
     HEALPIX_NEIGHBOURS, i, pixlist, nside=nside, ordering='NESTED'
;     IF NOT KEYWORD_SET(nocentralpix) THEN pixlist = [pixlist,i]

;     IF NOT KEYWORD_SET(nocentralpix) THEN BEGIN
;        ;; add the central pixel if necessary
;        pixlist = [pixlist,i]
;     ENDIF ELSE BEGIN
;        ;; remove the central pixel
;        wh = WHERE(pixlist NE i, nwh)
;        pixlist = pixlist[wh]
;     ENDELSE

     ;; Find corresponding list of pixels of the original map
     ;;nlist = N_ELEMENTS(pixlist)
     ;;wh = WHERE(map(nlist) NE !healpix.bad_value, nwh)

     ;; Compute median of pixels, excluding those with bad value
     ;;IF nwh GE minpts THEN BEGIN
     ;;   med[i] = MEDIAN(map[pixlist[wh]]) ELSE med[i] = !healpix.bad_value
     ;;ENDIF
        med[i] = MEDIAN(map[pixlist])

  ENDFOR


  CASE STRUPCASE(ordering) OF
     'RING': BEGIN
        map = REORDER(TEMPORARY(map),/n2r)
        med = REORDER(TEMPORARY(med),/n2r)
     END
     'NESTED': ;; do nothing
  ENDCASE

  RETURN, med

END
