; -----------------------------------------------------------------------------
;
; Copyright (C) 2010, 2011
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	HPX_LOCALSIGMA
;
; PURPOSE:
;	...
;
;-
FUNCTION HPX_LOCALSIGMA, map, usemean=usemean, mean=mean, ordering=ordering, degrade=degrade, niter=niter, $
                   nocentralpix=nocentralpix, max_dist=max_dist, minpts=minpts, help=help
                   

  IF ((KEYWORD_SET(help) NE 0) OR (N_PARAMS() EQ 0)) THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'HPX_LOCALSIGMA: function to calculate local standard deviation (sigma) of a healpix map'
     PSM_INFO_MESSAGE, 'SYNTAX: result = HPX_LOCALSIGMA(map, mean=, ordering=, degrade=, niter=, nocentralpix=, max_dist=, minpts=)'
     PSM_INFO_MESSAGE, "'map' is the input healpix map"
     PSM_INFO_MESSAGE, "'usemean' is the (optional) input local average value (needed to compute local sigma)"
     PSM_INFO_MESSAGE, "'mean' is the (optional) output local average value, (needed to compute local sigma), only if 'usemean' not defined in input"
     PSM_INFO_MESSAGE, "'ordering' is the ordering of map, and of the output"
     PSM_INFO_MESSAGE, "The procedure computes averages over pixels larger by the factor degrade (must be a power of 2, "
     PSM_INFO_MESSAGE, "default is 8), and uses HPX_PIXSMOOTHING to average the over all neighbouring pixels"
     PSM_INFO_MESSAGE, "at degraded resolution"
     PSM_INFO_MESSAGE, "'niter' sets the number of iterations of the HPX_PIXSMOOTHING procedure (done 1+niter times)"
     PSM_INFO_MESSAGE, "Set /nocentralpix for discarding the (degrade x degrade) central pixels in sigma calculation"
     PSM_INFO_MESSAGE, "If the keyword 'max_dist' is set to a non-zero number, HPX_PIXSMOOTHING used pixels"
     PSM_INFO_MESSAGE, "within a distance of 'max_dist' superpixels of the central superpixel (degrade x degrade times bigger)."
     PSM_INFO_MESSAGE, "Set minpts [default = 1] to the minimum number of points required to return the sigma."
     PSM_INFO_MESSAGE, "pixels for which the minimum is not reached are flagged as bad."
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  IF NOT DEFINED(minpts) THEN minpts=1
  IF NOT DEFINED(ordering) THEN ordering='RING'

  IF NOT DEFINED(usemean) THEN BEGIN
     domean = 1B
     usemean = HPX_LOCALMEAN(map, ordering=ordering, degrade=degrade, niter=niter, nocentralpix=nocentralpix, max_dist=max_dist, minpts=minpts)
  ENDIF ELSE BEGIN
     domean = 0B
     ;; do nothing, usemean already provided
  ENDELSE

  whok = WHERE((map NE !healpix.bad_value) AND (usemean NE !healpix.bad_value), nwhok, complement=whbad, ncomplement=nwhbad)
  sigma = map
  IF nwhbad NE 0 THEN sigma[whbad] = !healpix.bad_value
  IF nwhok NE 0 THEN sigma[whok] = (sigma[whok]-usemean[whok])^2

  sigma = HPX_LOCALMEAN(TEMPORARY(sigma), ordering=ordering, degrade=degrade, niter=niter, nocentralpix=nocentralpix, max_dist=max_dist, minpts=minpts)
  sigma = SQRT(TEMPORARY(sigma))

  IF domean THEN mean = TEMPORARY(usemean)

  RETURN, sigma

END
