; -----------------------------------------------------------------------------
;
; Copyright (C) 2010-2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	LOCALCOVAR
;
; PURPOSE:
;	...
;
;-
FUNCTION LOCALCOVAR, map1, map2, ordering=ordering, pixperscale=pixperscale, nsidecovar=nsidecovar, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'LOCALCOVAR: function to calculate local covariance of two maps'
     PSM_INFO_MESSAGE, 'SYNTAX: result = LOCALCOVAR(map1, map2, ordering=, pixperscale=, nsidecovar=)'
     PSM_INFO_MESSAGE, "'map1' and 'map2' are input healpix maps"
     PSM_INFO_MESSAGE, "'ordering' is the ordering of map1 and map2, and of the output"
     PSM_INFO_MESSAGE, "'pixperscale' sets the number of pixels per scale width on which the covariance is computed"
     PSM_INFO_MESSAGE, "'pixperscale' is given in units of pixels of the original map"
     PSM_INFO_MESSAGE, "default for 'pixperscale' is 16, i.e. about 250 neighbouring pixels are used for computing local covariance"
     PSM_INFO_MESSAGE, "'nsidecovar' is the healpix nside of the output covariance map"
     PSM_INFO_MESSAGE, "default for 'nsidecovar' is the nside of the original map(s)"
     PSM_INFO_MESSAGE, "See also HPX_LOCALCOVAR for an alternate version that uses pixel space smoothing"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  ON_ERROR, 2
  ok = ISANUMBER(map1)
  IF DEFINED(map2) THEN ok=ok*ISANUMBER(map2)
  IF NOT ok THEN MESSAGE, 'Input map type not implemented'

  IF NOT KEYWORD_SET(pixperscale) THEN pixperscale=16

  IF NOT DEFINED(ordering) THEN ordering='RING'
  IF DEFINED(map2) THEN map = map1*map2 ELSE map = map1*map1
  npix = N_ELEMENTS(map)
  nside = NPIX2NSIDE(npix)
  IF NOT KEYWORD_SET(nsidecovar) THEN nsidecovar = nside

  ;; First degrade a bit to speed-up smoothing
  nside_out = (nside/4) > 1
  UD_GRADE, TEMPORARY(map), stat, nside_out=nside_out, order_in=ordering, order_out='RING'

  ;; Compute alm
  lmax = 2*nside_out
  PSM_MAP2ALM, TEMPORARY(stat), alm, lmax=lmax

  ;; Find smoothing size
  pixsize = SQRT(4.*!pi/npix)
  fwhm = pixperscale*pixsize
  bl = PSM_GET_BL(fwhm/!dtor*60)

  ;; Smooth the alm
  index = ALM_INDEX(0,lmax)
  INDEX2L_ONLY, index, l
  alm[*,0] = alm[*,0] * bl[l]
  alm[*,1] = alm[*,1] * bl[l]

  ;; Back to pixel space
  PSM_ALM2MAP, TEMPORARY(alm), stat, hpx_nside=nsidecovar, /nohdr, ordering=ordering

  RETURN, stat
END
