; -----------------------------------------------------------------------------
;
; Copyright (C) 2007, 2010, 2012
;   Jacques Delabrouille
;   Guillaume Castex
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	POLAR_NEARBY_ANGLE_ON_SPHERE
;
; PURPOSE:
;	Program to compute the polarisation angle in nearby pixels for
;	an extended source, e.g. a galaxy cluster (solves the problem
;	of extended sources near the poles, where angles rotate
;	fast). The frame for Q and U is taken from the HEALPix
;	convention: (x,y)	where x is along the local cluster meridian towards south
;                           	and y is along the local cluster parallel towards east
;	Note : For each cluster of a generated cluster catalog,
;	catalog[cluster].angle gives the angle between south (x axis)
;	and the transverse velocity towards east (y axis)
;	(i.e. counterclockwise, z pointing outwards the sphere) Q and
;	U are defined as: Q = Ix-Iy and U = Ix'-Iy' 
;	where (x',y') = ROT((x,y), !pi/4, counterclockwise).

; CATEGORY:
;	Polarisation
;
; CALLING SEQUENCE:
;	POLAR_NEARBY_ANGLE_ON_SPHERE, Theta_in, Phi_in, Angle_in, Theta_out, Phi_out, Angle_out
;
; INPUTS:
;	Teta_in:	theta of position of the center (where the
;	                polarisation angle is defined) in radians.
; 
;	Phi_in: 	phi of position of the center (where the
;	                polarisation angle is defined) in radians.
;
;	Angle_in: 	polarisation angle measured from meridian.
;
; KEYWORD PARAMETERS:
;	OTHERMETHOD:	Use second of the two implemented methods.
;
; OUTPUTS:
;	Theta_out: 	theta of position of the point of interest in radians.
;
;	Phi_out: 	phi of position of the point of interest in radians.
;
;	Angle_out: 	polarisation angle at the point of interest  measured from meridian.
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;	April 2007:	Jacques Delabrouille, First version.
;	02/06/2010:	Guillaume Castex, Corrected normalisation of
;	                vectors yyy_in and yyy_out.
;	09/06/2010:	JD, implemented alternate method, somewhat faster.
;	10/01/2012:	JD, doc and PSM licnece headers.
;-
PRO POLAR_NEARBY_ANGLE_ON_SPHERE, theta_in, phi_in, angle_in, theta_out, phi_out, angle_out, othermethod=othermethod, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'POLAR_NEARBY_ANGLE_ON_SPHERE: procedure to parallel-transport a vector on the sphere.'
     PSM_INFO_MESSAGE, 'SYNTAX: POLAR_NEARBY_ANGLE_ON_SPHERE, theta_in, phi_in, angle_in, theta_out, phi_out, $'
     PSM_INFO_MESSAGE, '                                      angle_out, /othermethod'
     PSM_INFO_MESSAGE, "'theta_in' and 'phi_in' are the input coordinates of the reference point, in radians;"
     PSM_INFO_MESSAGE, "'angle_in' is the input reference polarisation angle at the reference point;"
     PSM_INFO_MESSAGE, "'theta_out' and 'phi_out' are the input coordinates of nearby points, in radians;"
     PSM_INFO_MESSAGE, "'theta_out' and 'phi_out' can be arrays."
     PSM_INFO_MESSAGE, "'angle_out' is the angle of the direction parallel, at the nearby points,"
     PSM_INFO_MESSAGE, "to that defined at the reference point by 'angle_in'."
     PSM_INFO_MESSAGE, "by default, the output direction is the intersection of the sphere and the plane which is"
     PSM_INFO_MESSAGE, "parallel to that defined by the input direction and the radius at the reference point."
     PSM_INFO_MESSAGE, "set /othermethod to use an alternate method (slightly faster, but less accurate?) which uses"
     PSM_INFO_MESSAGE, "the tangent approximation." 
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF
  
  ;; basis vectors for polarisation at center (reference) point
  ;;-----------------------------------------------------------
  xxx_in_x = COS(theta_in)*COS(phi_in)
  xxx_in_y = COS(theta_in)*SIN(phi_in)
  xxx_in_z = -SIN(theta_in)

  ;; vector along polarisation at center (reference) point
  ;;------------------------------------------------------
  IF NOT KEYWORD_SET(othermethod) THEN BEGIN
   ;;yyy_in = [-SIN(phi_in), COS(phi_in), 0] ;d_phi
   yyy_in_x = -SIN(phi_in)
   yyy_in_y = COS(phi_in)
   ;yyy_in_z = 0.

   ;;angle_vec = COS(angle_in)*xxx_in + SIN(angle_in)*yyy_in
   angle_vec_x = COS(angle_in)*xxx_in_x + SIN(angle_in)*yyy_in_x
   angle_vec_y = COS(angle_in)*xxx_in_y + SIN(angle_in)*yyy_in_y
   angle_vec_z = COS(angle_in)*xxx_in_z ;+ SIN(angle_in)*yyy_in_z

   ENDIF ELSE BEGIN
      angle_vec_x = xxx_in_x
      angle_vec_y = xxx_in_y
      angle_vec_z = xxx_in_z  
   ENDELSE
   
   
   ;; basis vectors for polarisation at point of interest
   ;;----------------------------------------------------
   ;;xxx_out = [COS(theta_out)*COS(phi_out), COS(theta_out)*SIN(phi_out), -SIN(theta_out)] ;d_theta
   xxx_out_x = COS(theta_out)*COS(phi_out)
   xxx_out_y = COS(theta_out)*SIN(phi_out)
   xxx_out_z = -SIN(theta_out)

   ;;yyy_out = [-SIN(phi_out), COS(phi_out), 0] ;d_phi
   yyy_out_x = -SIN(phi_out)
   yyy_out_y = COS(phi_out)
   yyy_out_z = 0.

   ;; compute scalar product with x and y at point of interest
   ;;---------------------------------------------------------
   propto_cos_angle_out = xxx_out_x * angle_vec_x + xxx_out_y * angle_vec_y + xxx_out_z * angle_vec_z
   propto_sin_angle_out = yyy_out_x * angle_vec_x + yyy_out_y * angle_vec_y + yyy_out_z * angle_vec_z
   
   ;; compute polarisation angle at point of interest
   ;;------------------------------------------------
   angle_out = ATAN(propto_sin_angle_out, propto_cos_angle_out)

   IF KEYWORD_SET(othermethod) THEN angle_out = (angle_out+angle_in)+2*!pi MOD (2*!pi)

END
