; -----------------------------------------------------------------------------
;
; Copyright (C) 2001, 2002, 2012
;   Jacques Delabrouille
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	GAUSSIAN_FIELD
;
; PURPOSE:
;	Generate a Gaussian random field with (isotropic) spectrum
;	P(k). The convention is that k is in units of
;	2!pi/(max-box-size), i.e. a mode for which |k|=1 corresponds
;	to a wavelength equal to the size of the box along its biggest
;	dimension. The pixels of the box are assumed to be cubic (same
;	sampling along all axes). 
;
; CATEGORY:
;	Random draws
;
; CALLING SEQUENCE:
;	result = GAUSSIAN_FIELD(Dimensions, K_values, Pk_values)
;
; INPUTS:
;	Dimensions:	Dimensions of the box to be generated, e.g. 100, or [10,10,20].
;
; OPTIONAL INPUTS:
;	k_values:	Values of k for a tabulated input spectrum.
;	Pk_values:	Corresponding values of Pk.
;	
; KEYWORD PARAMETERS:
;	USE_FUNCTION:	Set this keyword to a string expression to use
;	                a function of k, in which case k_values and
;	                Pk_values are ignored and need not (should
;	                not) be given in input. 
;
;	ISEED:  	Set this keyword to the value of the seed used
;	                for random generation.
;
; OUTPUTS:
;	A randomly drawn Gaussian random field of dimensions specified by 'dimensions'.
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;
; EXAMPLE:
;	result = GAUSSIAN_FIELD([100,150],use_function='k^2*EXP(-k/5)') 
;
; MODIFICATION HISTORY:
;	20/05/2001:	Jacques Delabrouille, First version.
;	30/10/2002:	Jean-Baptiste Melin, Tabulated P(k) implemented.
;	04/01/2012:	JD, Some cleaning + updated documentation header.
;-
FUNCTION GAUSSIAN_FIELD, dimensions, k_values, Pk_values, use_function=use_function, $
	iseed = iseed

  IF ( N_PARAMS() EQ 0 ) THEN BEGIN
     PRINT,'  CALL IS : '
     PRINT,'  Keywords :'
     GOTO, closing
  ENDIF
  
  ndim = N_ELEMENTS(dimensions)             ;number of dimensions of the result
  kinc = MAX(dimensions)/(dimensions>1.)    ;increment in k for each dimension

  ;; Generate the output box ('result')
  ;;----------------------------------
  creation_string = 'result = RANDOMN(seed'
  IF KEYWORD_SET(iseed) NE 0 THEN creation_string = 'result = RANDOMN(iseed'
  FOR i=0, ndim-1 DO $
     creation_string = creation_string+','+STRTRIM(STRING(FIX(dimensions(i))),1)
  creation_string = creation_string+')'
  ok = EXECUTE(creation_string)
  
  ;; Generate k values
  ;;-----------------
  k = result
  IF ndim EQ 0 THEN k=0
  IF ndim EQ 1 THEN k=DIST(dimensions(0),1)
  IF ndim EQ 2 THEN BEGIN
     k = (DIST(dimensions(0),1)#REPLICATE(1,dimensions(1))*kinc(0))^2 $
         + (REPLICATE(1,dimensions(0))#DIST(dimensions(1),1)*kinc(1))^2
     k = SQRT(k)
  ENDIF                         ;(end 'IF ndim EQ 2')
  IF ndim EQ 3 THEN BEGIN
     k(*,*,0) = (DIST(dimensions(0),1)#REPLICATE(1,dimensions(1))*kinc(0))^2 $
                + (REPLICATE(1,dimensions(0))#DIST(dimensions(1),1)*kinc(1))^2
     IF dimensions(2) GT 1 THEN BEGIN
        distz2 = (DIST(dimensions(2),1)*kinc(2))^2
        FOR iz=1, dimensions(2)-1 DO k(*,*,iz) = k(*,*,0) + distz2(iz)
        k = SQRT(k)
     ENDIF
  ENDIF                         ;(end 'IF ndim EQ 3')
  IF ndim GT 3 THEN BEGIN
     PRINT, '  SORRY, more than 3 dimensions not allowed yet!'
     PRINT, '  RETURNING'
     GOTO, closing
  ENDIF                         ;(end 'IF ndim GT 3')
  
  ;; Compute FFT and multiply by spectrum in k values
  ;;------------------------------------------------

  result = FFT(result,/overwrite) ;overwrite to save memory space

  IF KEYWORD_SET(use_function) NE 0 THEN BEGIN
     sqrt_Pk_expression = 'SQRT('+use_function+')'
     ok = EXECUTE('result=result*'+sqrt_Pk_expression)
  ENDIF ELSE BEGIN
     sqrt_Pk_expression = SQRT(INTERPOL(Pk_values,k_values,k))
     result=result*sqrt_Pk_expression
  ENDELSE
  
  result = FFT(result,1,/overwrite)
  result = FLOAT(result)
  RETURN, result

closing:
  
END
