; -----------------------------------------------------------------------------
;
; Copyright (C) 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	PSM_MRANDOMN
;
; PURPOSE:
;	Function to draw NRAND random deviates from a multivariate normal
;	distribution with zero mean and covariance matrix COVAR.
;
; CATEGORY:
;	Math
;
; CALLING SEQUENCE:
;	Result = PSM_MRANDOMN(seed, covar, nrand, /svd)
;
; INPUTS:
;	Seed:	Input seed for random number generation
;	Covar:	Input covariance matrix (must be definite
;		non-negative), dimension [n, n]
;	Nrand:	Number of random numbers generated
;
; OPTIONAL INPUTS:
;	NONE	...
;	
; KEYWORD PARAMETERS:
;	SVD		Set this keyword to use SVD when Cholesky
;			decomposition fails. This solves the issue when Cholesky
;			does not work by reason of numerical errors
;
;	COVAR_TOLERANCE	Set this keyword to specify the acceptable
;			difference between the input covariance and
;			the covariance actually used for generating
;			the random varianles (obtained by SVD when the
;			Cholesky decomposition fails)
;
; OUTPUTS:
;	Result: a [nrand, n] array of random numbers
;
; OPTIONAL OUTPUTS:
;	NONE
;
; COMMON BLOCKS:
;	NONE
;
; SIDE EFFECTS:
;	NONE
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;	Adapted from MRANDOMN in astron library, by Brandon C. Kelly. 
;	However, the CHOLDC has been replaced by LA_CHOLDC, and can be
;	replaced by an SVD to handle exceptions due to numerical errors.
;
; EXAMPLE:
;	... 
;
; MODIFICATION HISTORY:
; 	12/07/2012:	Jacques Delabrouille, First version.
;-
FUNCTION PSM_MRANDOMN, seed, covar, nrand, /svd, covar_tolerance=covar_tolerance

if n_params() lt 2 then begin
    print, 'Syntax- Result = mrandomn( seed, covar, [nrand] )'
    return, 0
endif

;check inputs and set up defaults
if n_elements(nrand) eq 0 then nrand = 1
if size(covar, /n_dim) ne 2 then begin
    print, 'COVAR must be a matrix.'
    return, 0
endif

np = (size(covar))[1]
if (size(covar))[2] ne np then begin
    print, 'COVAR must be a square matrix.'
    return, 0
endif

diag = lindgen(np) * (np + 1L)
epsilon = randomn(seed, nrand, np) ;standard normal random deviates (NP x NRAND matrix)

A = covar  ;store covariance into dummy variable
U = covar  ;store covariance into dummy variable

CHOLDC, A, P, /double           ;do Cholesky decomposition
LA_CHOLDC, U, status=status

STOP

for j = 0, np - 1 do for i = j, np - 1 do A[i,j] = 0d

A[diag] = P

;transform standard normal deviates so they have covariance matrix COVAR
epsilon = A ## epsilon

return, epsilon
end
