; -----------------------------------------------------------------------------
;
; Copyright (C) 2001, 2012
;   Jacques Delabrouille
;   Guillaume Castex
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;       RANDOM_DRAW
;
; PURPOSE:
;       Draw a random number according to a given probablity density.
;
; CATEGORY:
;       Random draws
;
; CALLING SEQUENCE:
;       result = RANDOM_DRAW(Proba, Ndraw)
;
; INPUTS:
;       Proba:       The probability density (a vector).
;
; OPTIONAL INPUTS:
;       Ndraw:       The number of random numbers to be generated (default=1).
;
; KEYWORDS:
;       ISEED:       To give a seed value for the random draw.
;
; OUTPUTS:
;       The result is an index in the input probability density vector.
;
; PROCEDURE:
;       Calculate integrated probablility, draws a random number using RANDOMU
;       and check in which bin it falls.
;
; RESTRICTIONS:
;       Report any bugs to Jacques Delabrouille
;       delabrouille@apc.univ-paris7.fr
;
; EXAMPLES:
;       Assume a discrete probablity density distributed on 3 possible values
;       with probability density defined as follows:
;       > proba = [0.2,0.3,0.5]. 
;
;       Draw a number according to this probability density:
;       > result = RANDOM_DRAW(proba) 
;
;       The command returns
;       0 : 20% of the time
;       1 : 30% of the time
;       2 : 50% of the time
;
; REVISION HISTORY
;       19/05/2001:       Jacques Delabrouille, First version
;       20/09/2001:       JD, Keyword ISEED added
;       04/01/2012:       JD, Documentation header updated
;-      06/02/2012        GC, New algorithm, walues and draw keyword added, updated doc

FUNCTION RANDOM_DRAW, proba, ndraw, values=values, draw=draw, silent=silent, iseed=iseed, help=help
   t0 = SYSTIME(1)
  
  IF ( (N_PARAMS() EQ 0 ) OR KEYWORD_SET(help) ) THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'RANDOM_DRAW: function to draw a random number according to a given probablity density'
     PSM_INFO_MESSAGE, 'SYNTAX: result = RANDOM_DRAW(proba, ndraw, values=, draw=, nonorm=, silent=, iseed=)'
     PSM_INFO_MESSAGE, "'proba' is the input probability density (a vector)."
     PSM_INFO_MESSAGE, "'ndraw' is the size of the expected output vector of random numbers."
     PSM_INFO_MESSAGE, "if 'draw' is selected, the function returns a vector filled with ndraw points"
     PSM_INFO_MESSAGE, "'values' is the vector of values to fill in the vector, if 'draw' keyword selected (default 1)"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     GOTO, closing
  ENDIF
  
  IF MIN(proba) LT 0 THEN BEGIN
     PRINT, '  NEGATIVE PROBABILITY DETECTED IN INPUT, RETURNING...'
     GOTO, closing
  ENDIF
  
  IF defined(values) eq 1 THEN BEGIN
     IF DEFINED(ndraw) eq 1 THEN BEGIN
        IF ndraw NE n_elements(values) THEN BEGIN
           PRINT, 'number of values must match ndraw'
           GOTO, closing
        ENDIF
     ENDIF
     ndraw=n_elements(values)
     quick=0
  ENDIF ELSE BEGIN
        IF defined(draw) EQ 1 THEN BEGIN
		IF DEFINED(ndraw eq 1) THEN BEGIN
        	  values=1.
		ENDIF ELSE BEGIN
		  values=1.
		ENDELSE
		quick=1
        ENDIF
   ENDELSE

  asort=SORT(proba)
  ;; t1 = systime(1)
  ;; PRINT, 'Map sorted in '+NUMBER2STRING(t1-t0)+' seconds'
  
  sum_proba = proba[asort]
  nel = N_ELEMENTS(sum_proba)
  IF nel EQ 1 THEN BEGIN
     RETURN, 0
     GOTO, closing
  ENDIF ELSE BEGIN
     FOR i=1L, nel-1 DO sum_proba(i) = sum_proba(i-1)+sum_proba(i)
  ENDELSE
  
  
  tot = DOUBLE(sum_proba(nel-1))
  
  IF tot NE 1 THEN BEGIN
     IF NOT KEYWORD_SET(silent) THEN BEGIN
        PRINT, '  WARNING, INTEGRATED PROBABILITY DIFFERENT FROM 1'
        PRINT, '  RENORMALIZING...'
     ENDIF
     sum_proba = TEMPORARY(sum_proba)/tot
     tot = DOUBLE(sum_proba(nel-1))
  ENDIF
  ;; t2 = systime(1)
  ;; PRINT, 'Map renormalised in '+NUMBER2STRING(t2-t1)+' seconds'
  
  IF DEFINED(ndraw) EQ 0 THEN ndraw=1
  
  IF NOT DEFINED(iseed) THEN result = RANDOMU(GETSEED(), ndraw)*tot $
  ELSE result = RANDOMU(iseed, ndraw)*tot
  ind=sort(result)
  result=TEMPORARY(result(ind))
  
  ;; FOR i=0L, ndraw-1 DO result(i) = MIN(WHERE(result(i) LE sum_proba))
  IF DEFINED(draw) EQ 1 THEN BEGIN
     outarr=proba*0.
  ENDIF ELSE BEGIN
     draw=0
  ENDELSE
   j=0L
   IF draw eq 1 THEN BEGIN
	IF quick eq 0 THEN BEGIN
	  values=values[ind]
   	  FOR i=0L, long(long(ndraw)-1) DO BEGIN
        	While result[i] gt sum_proba[j] DO BEGIN
                	j=j+1L
        	ENDWHILE
        	outarr[asort[j]]=outarr[asort[j]]+values[i]
   	  ENDFOR
	     IF NOT KEYWORD_SET(silent) THEN PRINT,'TOTAL  time=', round(systime(1)-t0), ' sec'
          RETURN, outarr
	ENDIF ELSE BEGIN
;PRINT,'B-F  time=', round(systime(1)-t0), ' sec'
          FOR i=0L, long(long(ndraw)-1) DO BEGIN
                While result[i] gt sum_proba[j] DO BEGIN
                        j=j+1L
                ENDWHILE
                outarr[asort[j]]=outarr[asort[j]]+values
          ENDFOR
             IF NOT KEYWORD_SET(silent) THEN PRINT,'TOTAL  time=', round(systime(1)-t0), ' sec'
          RETURN, outarr
	ENDELSE
   ENDIF ELSE BEGIN
   	FOR i=0L, long(long(ndraw)-1) DO BEGIN
        	While result[i] gt sum_proba[j] DO BEGIN
                	j=j+1L
        	ENDWHILE
        	result[i]=asort[j]
   	ENDFOR
  	RETURN, LONG(result)

  ENDELSE	
  
  
closing:
  
END
