; -----------------------------------------------------------------------------
;
; Copyright (C) 2001, 2012
;   Jacques Delabrouille
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	ARRAY2STRING
;
; PURPOSE:
;	Write an array of numbers of dimension 1 (or 2) in string
;	format. This routine is useful for printing purposes or for
;	writing IDL lines of code to be executed with the EXECUTE command.
;
; CATEGORY:
;	String handling
;
; CALLING SEQUENCE:
;	Result = ARRAY2STRING(Array)
;
; INPUTS:
;	Array:	The array of numbers to be written/printed.
;
; KEYWORDS:
;	COMPRESS:	Set this keyword to remove extra spaces added
;	                by the formatting of input numbers when
;	                converted to strings.
;
; OUTPUTS:
;	The result is a string that writes the array definition in
;	proper IDL syntax.
;
; RESTRICTIONS:
;	Input arrays of numbers must have at most two
;	dimensions. Beware that the type of the IDL variables may not
;	be properly taken care of by this routine.
;
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;	Simple string manipulation.
;
; EXAMPLE:
;	Create an array in IDL:
;	> array = FINDGEN(3)
;
;	View the string array that defines that array in IDL syntax:
;	> PRINT, ARRAY2STRING(array, /compress)
;
;	(IDL prints '[ 0.00000, 1.00000, 2.00000]').
;
; MODIFICATION HISTORY
;	28/11/2001	Jean-Baptiste Melin,  First version.
;	10/01/2012	Jacques Delabrouille, COMPRESS keyword added,
;	                doc and PSM license headers added/completed.
;-
FUNCTION ARRAY2STRING, array, compress=compress


  IF NOT ISARRAY(array) THEN BEGIN
     PRINT, 'The input has to be an array !'
     GOTO, closing
  ENDIF
  
  s = SIZE(array)
  
  IF s[0] GE 3 THEN BEGIN
     PRINT, 'Too many dimensions !'
     GOTO, closing
  ENDIF
  
  IF s[0] EQ 1 THEN BEGIN
     sarray = STRING(array)
     result = '[' + sarray[0]
     IF s[1] GT 1 THEN BEGIN
        FOR i=1, s[1]-1 DO BEGIN
           result = result+ ',' + sarray[i]
        ENDFOR
     ENDIF
     result = result + ']'
  ENDIF
  
  IF s[0] EQ 2 THEN BEGIN
     sarray = STRING(array)
     result = '['
     FOR i=0, s[2]-1 DO BEGIN
        result = result + '[' + sarray[0,i]
        IF s[1] GT 1 THEN BEGIN
           FOR j=1, s[1]-1 DO BEGIN
              result = result + ',' + sarray[j,i] 
           ENDFOR
        ENDIF
        result = result + ']'
        IF  i EQ s[2]-1 THEN result = result + ']' ELSE result = result + ','
     ENDFOR
  ENDIF
  
  IF NOT KEYWORD_SET(compress) THEN RETURN, result ELSE RETURN, STRCOMPRESS(result)
  
closing:
END
