; -----------------------------------------------------------------------------
;
; Copyright (C) 2010-2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	VISU_MAP
;
; PURPOSE:
;	Wrapper to Healpix MOLLVIEW and GNOMVIEW routines to visualise
;	a sky map and avoid modifying the IDL plotting DEVICE.
;
; CATEGORY:
;	Visualisation
;
; CALLING SEQUENCE:
;	VISU_MAP, Map, Select_in, PS=, PNG=, _EXTRA=, MOLL=, GNOM=
;
; INPUTS:
;	Map:		The input healpix map to be visualised.
;
; OPTIONAL INPUTS:
;	Select_in:	The extension or field of the map to be visualised.
;			e.g., use select_in=1, 2 or 3... for T, Q, or U.
;			Default is 1 (T).
;	
; KEYWORD PARAMETERS:
;	PS:		Set this keyword to a filename for postscript output.
;	PNG:		Set this keyword to a filename for png output.
;	_EXTRA:		Use this to pass extra keywords to MOLLVIEW or GNOMVIEW.
;	MOLL:		Set this keyword to do MOLLVIEW (only).
;	GNOM:		Set this keyword to do GNOMVIEW (only).
;
; OUTPUTS:
;	None
;
; SIDE EFFECTS:
;	Windows are open for display, or files are written.
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; MODIFICATION HISTORY:
; 	12/02/2010:	Jacques Delabrouille, First version.
; 	24/01/2011:	JD, Added Select_in optional input.
; 	02/03/2011:	JD, Added PS and PNG keywords.
; 	20/09/2012:	JD, Added an EXECUTE in call to DEVICE (for
; 	 	 	old versions of IDL). Now do both GNOMVIEW and
; 	 	 	MOLLVIEW only if none of PS or PNG keywords is set.
; 	 	 	Doc header completed.
;-
PRO VISU_MAP, map, select_in, ps=ps, png=png, _extra=_extra, moll=moll, gnom=gnom, help=help

  IF ((KEYWORD_SET(help) NE 0) OR (N_PARAMS() LT 1)) THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'VISU_MAP: procedure to visualize a PSM map'
     PSM_INFO_MESSAGE, 'SYNTAX: VISU_MAP, map, [select_in], title=, /moll, /gnom, _extra=_extra'
     PSM_INFO_MESSAGE, "Set /gnom or /moll for GNOMVIEW or MOLLVIEW only"
     PSM_INFO_MESSAGE, "Use select_in=1, 2 or 3... for T, Q, or U"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF

  IF NOT KEYWORD_SET(ps) THEN DEVICE, get_decomposed=d
  done=0

  IF KEYWORD_SET(moll) THEN BEGIN
     MOLLVIEW, map, select_in, ps=ps, png=png, _extra=_extra
     done=1
  ENDIF
  IF KEYWORD_SET(gnom) THEN BEGIN
     GNOMVIEW, map, select_in, ps=ps, png=png, _extra=_extra
     done=1
  ENDIF
  IF NOT done THEN BEGIN
     IF NOT KEYWORD_SET('ps') AND NOT KEYWORD_SET('png') THEN BEGIN
        MOLLVIEW, map, select_in, ps=ps, png=png, _extra=_extra
        GNOMVIEW, map, select_in, ps=ps, png=png, _extra=_extra
     ENDIF
  ENDIF

  IF NOT KEYWORD_SET(ps) THEN ok=EXECUTE('DEVICE, decomposed=d')
     

END
