; -----------------------------------------------------------------------------
;
; Copyright (C) 2009, 2010, 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	WDEL
;
; PURPOSE:
;	Quick way to delete open windows.
;	Deletes the open window specified by user.
;	Can delete all windows by calling WDEL with no parameters.
;	Can just print open windows by setting the /open keyword.
;     
; CATEGORY:
;	Visualisation
;
; CALLING SEQUENCE:
;	WDEL, N
;
; OPTIONAL INPUTS:
;	N:	Then number of the window to be deleted. If omitted,
;	        all active windows will be closed.
;
; OUTPUTS:
;	None.
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; MODIFICATION HISTORY:
;	29/10/2009:   JD, First version, adapted from the WD routine by Tim Robishaw.
;	22/01/2010:   JD, general clean-up.
;	04/01/2012:   JD, added doc header.
;-
PRO WDEL, n, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'WDEL: procedure to delete open windows'
     PSM_INFO_MESSAGE, 'SYNTAX: WDEL, n'
     PSM_INFO_MESSAGE, "n is the number of the window(s) to be closed (if absent, procedure closes all windows)"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF
  
  ;; Check that X windows device is used
  ;;-------------------------------------
  IF (!d.name NE 'X') THEN BEGIN
     MESSAGE, 'DEVICE not set to X Windows.', /info
     RETURN
  ENDIF

  ;; Find open windows
  ;;-------------------
  DEVICE, window_state = openwindows
  openwindows = WHERE(openwindows,Nopen)
  
  ;; Delete requested windows
  ;;--------------------------

  IF (N_PARAMS() EQ 0) THEN BEGIN

     ;; If no input, delete all windows
     FOR i = 0, Nopen-1 do WDELETE, openwindows[i]
     RETURN
     
  ENDIF ELSE BEGIN
     
     ;; Else delete only the requested window(s)
     FOR i = 0, N_ELEMENTS(n)-1 DO BEGIN
        IF (total(openwindows EQ n[i]) GT 0) $
        THEN WDELETE, n[i]  $
        ELSE MESSAGE, 'Window '+NUMBER2STRING(n[i])+' not open.', /info
     ENDFOR
  ENDELSE
     
END
