; -----------------------------------------------------------------------------
;
; Copyright (C) 2010-2012
;   Soumen Basak
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	COSINE_ELL_BANDS
;
; PURPOSE:
;	This function generates cosine-shaped windows in harmonic mode
;	(ell) space, used for wavelet (needlet) analysis of spherical maps.
;
; CATEGORY:
;	Wavelet (needlet) processing.
;
; CALLING SEQUENCE:
;	Result = COSINE_ELL_BANDS()
;
; INPUTS:
;	None (all parameters are passed by keywords)
;
; KEYWORD PARAMETERS:
;	BANDCENTERS:	Harmonic mode (ell) values where bands will be
;	                centred (actually, these values will be used
;	                for borders and for maxima of the cosine
;	                bands). An array of Nband numbers.
;	SINGLE: 	By default, the bands are in double precision.
;	                Set this keyword for single floating point precision.
;
; OUTPUTS:
;	The result is an array of dimension
;	[MAX(bandcenters)+1,Nband], that gives the shapes of the
;	spectral bands. These bands are such that the sum of their
;	squares is equal to 1 for all ell.
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; EXAMPLE:
;	
;	Create a set of bands:
;	b = COSINE_ELL_BANDS(bandcenters=[2,10,20,40,65,100], /single) 
;	
;	View them using the PLOT_BANDS routine:
;	PLOT_ELL_BANDS, b, yrange=[0,1.2]
;
; MODIFICATION HISTORY:
; 	08/12/2011:	Jacques Delabrouille, First version (name
; 	                changed from earlier version COSINE_BANDS, by
; 	                JD and Soumen Basak).
; 	08/01/2012:	JD, General clean-up + doc and PSM headers.
;-
FUNCTION COSINE_ELL_BANDS, bandcenters=bandcenters, single=single, help=help

  IF ((N_PARAMS() NE 0) OR (KEYWORD_SET(help) NE 0)) THEN BEGIN
     PRINT, 'Syntax : bands = COSINE_ELL_BANDS( [bandcenters=, single=] )'
     RETURN, 0
  ENDIF
  
  IF NOT KEYWORD_SET(bandcenters) THEN BEGIN
     bandcenters = [0,20,40,70,100,140,180,240,320,450,600,800,1000,1300,1600,2000,2500,3000,4000,6000]
  ENDIF
  
;analysis and synthesis windows are the same 
;-------------------------------------------
  lmax = MAX(bandcenters)
  nbands=N_ELEMENTS(bandcenters)
  bands = DBLARR(lmax+1,nbands) 

  mypi=!dpi

  b=bandcenters(0)
  c=bandcenters(1)

  ;; Left part
  ;;-----------
  ell_left = DINDGEN(b+1) 
  IF b GT 0 THEN BEGIN
     bands(ell_left,0)=1.d0
  ENDIF

  ;; Right part
  ;;------------
  ell_right = b+DINDGEN(c-b+1) 
  bands(ell_right,0) = (COS((ell_right-b)/(c-b)*mypi/2D0))
  
  
  IF nbands GE 3 THEN BEGIN
     FOR i=1, nbands-2 DO BEGIN
        a=bandcenters(i-1)
        b=bandcenters(i)
        c=bandcenters(i+1)
        ;; Left part
        ell_left = a+DINDGEN(b-a+1)
        bands(ell_left,i) = (COS((b-ell_left)/(b-a)*mypi/2D0))
        ;; Right part
        ell_right = b+DINDGEN(c-b+1)
        bands(ell_right,i) = (COS((ell_right-b)/(c-b)*mypi/2D0))
     ENDFOR
     a=bandcenters(nbands-2)
     b=bandcenters(nbands-1)
     ;; Left part
     ell_left = a+DINDGEN(b-a+1)
     bands(ell_left,i) = (COS((b-ell_left)/(b-a)*mypi/2D0))
  ENDIF
  
IF ((NOT KEYWORD_SET(double)) AND (KEYWORD_SET(single))) THEN RETURN, FLOAT(bands) ELSE RETURN, bands
  
END


