!-----------------------------------------------------------------------------!
!-----------------------------------------------------------------------------!
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc !
! Program to simulate lensed CMB anisotropies in HEALPix pixelization either  !
! from anguar power spectrum of unlesned CMB  and lensing potential  or       !
! from unlesned CMB  and lensing potential map in harmonic space              !
! Written by Soumen Basak (basak@iap.fr)                                      !
!=============================================================================!
!     version 2.0                                                             !
!=============================================================================!
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc!
!-----------------------------------------------------------------------------!
!-----------------------------------------------------------------------------!
program getlensedmap

  use healpix_types
  use myhealmodule

  use paramfile_io
  use extension
  use pix_tools
  use misc_utils
  use fitstools

  implicit none

  integer(i4b)  :: n,status, pol
  integer(i4b)  :: nside, nlmax
  integer(i4b)  :: sigma, kappa
  character(len=filenamelen) :: cambscalclfile
  character(len=filenamelen) :: ulcmbmapfile, gradpotmapfile, lcmbmapfile
  character(len=filenamelen) :: ulcmbclfile, lcmbclfile
  character(len=filenamelen) :: cmbalmfile,potalmfile
  character(len=80) :: input, method  
  real(dp) :: time_finish, time_start, time_f, time_s
  real(dp), dimension(:,:), allocatable :: cmbcl, potcl
  real(dp), dimension(:,:), allocatable :: ultqutp,ltqutp,gradtp
  complex(dpc), dimension(:,:,:), allocatable :: alm,plm

  type(paramfile_handle) :: handle
  character(len=filenamelen) :: description, parafile
  character(len=*), parameter :: code = "GETLENSEDMAP"
  character(len=FILENAMELEN)  :: lcode
  character(len=*), PARAMETER :: version = "2.0"


  !-----------------------------------------------------------------------
  !                    get input parameters 
  !-----------------------------------------------------------------------
  parafile = ''
  if (nArguments() == 1) call getArgument(1, parafile)
  lcode = trim(strlowcase(CODE))
  !     --- read parameters interactively if no command-line arguments
  !     --- are given, otherwise interpret first command-line argument
  !     --- as parameter file name and read parameters from it:

  !     --- announces program, default banner ***
  PRINT *, " "
  PRINT *,"***************************************************************************************************"
  PRINT *,"                    "//code//" "//version
  PRINT *,"***************************************************************************************************"
  write(*,'(a)') &
       & " Synthesis of a Lensed Temperature (and Polarisation) map either" 
  write(*,'(a)') &
       & " from angular power spectrum of unlensed CMB and lensing potential" 
  write(*,'(a)') &
       & " or from unlensed CMB  and lensing potential map in harmonic space"
  PRINT *, " "
  PRINT *, "---------------------------------------------------------------------------------------------------"
  handle = parse_init(parafile)
  PRINT *, "---------------------------------------------------------------------------------------------------"
12    continue
  description = concatnl(&
       & "", &
       & " Do you want to simulate from ", &
       & " 'ps') angular power spectrum ", &
       & " 'map') map in harmonic space ") 
  input = parse_string(handle, 'input', default="ps", descr=description)
  ! --- check that some form of input has been given ---
  if ((input.ne."ps").and.(input.ne."map")) then
     print *,"ERROR: input should be either 'ps' or 'map' "
     if (handle%interactive) goto 12
     call fatal_error("Abort execution")
  endif

  if(input.eq.'ps') then
     !     --- gets the file name for input power spectrum ---
7    continue
     description = concatnl(&
        & "", &
        & " Enter input CMB angular power spectrum filename")
     cambscalclfile = parse_string(handle, 'cambscalclfile', default='cmb_Scalcl.dat', &
        & descr=description, filestatus='old')
     if (cambscalclfile.eq.'')  then
        print *, " Error: input CMB angular power spectrum filename is empty"
        if (handle%interactive) goto 7
        call fatal_error(code)
     endif
     !     --- gets the file name for the CMB alm ---
     description = concatnl(&
        & "", &
        & " Enter file name in which to write CMB alm used to synthesize the map : ", &
        & " (eg cmbalm.fits or !cmbalm.fits to overwrite an existing file): ", &
        & " (If '', the alms are not written to a file) ")
     cmbalmfile = parse_string(handle, 'cmbalmfile', &
        & default="''", descr=description, filestatus='new')
     !     --- gets the file name for the potential alm ---
     description = concatnl(&
        & "", &
        & " Enter file name in which to write Potential alm used to synthesize the map : ", &
        & " (eg potalms.fits or !potalms.fits to overwrite an existing file): ", &
        & " (If '', the alms are not written to a file) ")
     potalmfile = parse_string(handle, 'potalmfile', &
        & default="''", descr=description, filestatus='new')
  else if (input .eq. 'map') then
     !     --- constrained realisations ---
9    continue
     !     --- gets the file name for the CMB alm ---
     description = concatnl(&
        & "", &
        & " Enter file name of input CMB alm to synthesize the map : ", &
        & " (eg cmbalm.fits ): ", &
        & " These should be totally free of beam and pixel window.")
     cmbalmfile = parse_string(handle, 'cmbalmfile', &
        & default='cmbalm.fits', descr=description, filestatus='old')
     if (cmbalmfile.eq.'')  then
        print *, " Error: input CMB alm filename is empty"
        if (handle%interactive) goto 9
        call fatal_error(code)
     endif
10    continue
     !     --- gets the file name for the potential alm ---
     description = concatnl(&
        & "", &
        & " Enter file name of Potential alm  to synthesize the map : ", &
        & " (eg potalms.fits ): ", &
        & " These should be totally free of beam and pixel window.")
     potalmfile = parse_string(handle, 'potalmfile', &
        & default='potalm.fits', descr=description, filestatus='old')
     if (potalmfile.eq.'')  then
        print *, " Error: input Potential alm filename is empty"
        if (handle%interactive) goto 10
        call fatal_error(code)
     endif
  end if
  !     --- gets the effective resolution of the sky map ---
3 continue
  description = concatnl( &
       & "", &
       & " Enter the resolution parameter (Nside) for the simulated skymap: ",&
       & " (Npix = 12*Nside**2, where Nside HAS to be a power of 2, eg: 32, 512, ...)" )
  nside = parse_int(handle, 'nside', default=1024, descr=description)
  if (nside2npix(nside) < 0) then
     print *, " Error: nside is not a power of two."
     if (handle%interactive) goto 3
     call fatal_error(code)
  endif

  !     --- gets the L range for the simulation ---
  description = concatnl(&
       & "", &
       & " Enter the maximum l range (l_max) for the simulation. ")
  nlmax = parse_int(handle, 'nlmax', default=2*nside, vmin=0, vmax=3*nside-1,descr=description)

  !     --- choose temp. only or  temp. + pol. ---
11 continue
  description = concatnl( &
       & " Do you want to simulate", &
       & " 0) Temperature only", &
       & " 1) Temperature + polarisation")
  pol = parse_int(handle, 'pol', default=1, vmin=0, vmax=1, descr=description)
  if (pol < 0) then
    print *, " Error: pol is less than 0."
    if (handle%interactive) goto 11
    call fatal_error(code)
  else if (pol > 1 ) then
    print *, " Error: pol is greater than 1."
    if (handle%interactive) goto 11
    call fatal_error(code)
  endif

  !     --- inputs the random seed ---
4 continue
  if(input.eq.'ps') then
    description = concatnl(&
       & "", &
       & " Enter the index (positive integer) of realization. ")
    n = parse_int(handle, 'n', default=1, descr=description)
    if (n < 1) then
      print *, " Error: index of realization is not a positive interger."
      if (handle%interactive) goto 4
      call fatal_error(code)
    endif
  endif

  !     --- gets the accuracy parameter sigma(oversampling factor) of NFFT ---
5 continue
  description = concatnl( &
       & "", &
       & " Enter accuracy parameter (sigma) of NFFT for the simulated skymayp: ",&
       & " (sigma has to be greater than 1)" )
  sigma = parse_int(handle, 'sigma', default=2, descr=description)
  if (sigma < 2) then
     print *, " Error: sigma is less than 2"
     if (handle%interactive) goto 5
     call fatal_error(code)
  endif

  !     --- gets the accuracy parameter kappa(convolution length) of NFFT ---
6 continue
  description = concatnl( &
       & "", &
       & " Enter accuracy parameter (kappa) of NFFT for the simulated skymayp: ",&
       & " (kappa has to be greater than 3)" )
  kappa = parse_int(handle, 'kappa', default=4, descr=description)
  if (kappa < 4) then
     print *, " Error: kappa is less than 4"
     if (handle%interactive) goto 6
     call fatal_error(code)
  endif

  !     --- gets file name  for output map ---
  description = concatnl(&
       & "", &
       & " Enter output unlensed CMB map file name (eg, ulmap.fits) :", &
       & "  (or !ulmap.fits to overwrite an existing file)", &
       & " (If '', no map is created) ")
  ulcmbmapfile = parse_string(handle, "ulcmbmapfile", &
       & default='!ulmap.fits', descr=description, filestatus="new")

  description = concatnl(&
       & "", &
       & " Enter output lensing potential and deflection field map file name (eg, gmap.fits) :", &
       & "  (or !gmap.fits to overwrite an existing file)", &
       & " (If '', no map is created) ")
  gradpotmapfile = parse_string(handle, "gradpotmapfile", &
       & default='!gmap.fits', descr=description, filestatus="new")

  description = concatnl(&
       & "", &
       & " Enter output lensed CMB map file name (eg, lmap.fits) :", &
       & "  (or !lmap.fits to overwrite an existing file)", &
       & " (If '', no map is created) ")
  lcmbmapfile = parse_string(handle, "lcmbmapfile", &
       & default='!lmap.fits', descr=description, filestatus="new")

  !     --- gets file name  for output cl ---
  description = concatnl(&
       & "", &
       & " Enter output unlensed CMB cl file name (eg, ulcl.fits) :", &
       & " (If '', no cl is created) ")
  ulcmbclfile = parse_string(handle, "ulcmbclfile", &
       & default='!ulcl.fits', descr=description, filestatus="new")

  description = concatnl(&
       & "", &
       & " Enter output lensed CMB cl file name (eg, lcl.fits) :", &
       & " (If '', no cl is created) ")
  lcmbclfile = parse_string(handle, "lcmbclfile", &
       & default='!lcl.fits', descr=description, filestatus="new")

  !---------------choose method------------------------
120    continue
  description = concatnl(&
       & "", &
       & " Do you want to simulate unlensed cmb ", &
       & " lensing potential and deflection map using ", &
       & " 'hpix') HEALPix ", &
       & " 'nfft') NFFT ") 
  method = parse_string(handle, 'method', default="nfft", descr=description)
  ! --- check that some form of method has been given ---
  if ((method.ne."hpix").and.(method.ne."nfft")) then
     print *,"ERROR: input should be either 'hpix' or 'nfft' "
     if (handle%interactive) goto 120
     call fatal_error("Abort execution")
  endif

  PRINT *, "*********************************************************************"
  call parse_summarize(handle,code=lcode)
  call parse_finish (handle)
  PRINT *, "*********************************************************************"
  !******************************************************************************!

  !-----------------------------------------------------------------------
  ! allocates space for unlesned CMB and lensing potential in harmonic space
  !-----------------------------------------------------------------------
  allocate(alm(1:2*pol+1,0:nlmax,0:nlmax),stat = status)
  if (status /= 0) stop 
  alm(:,:,:)=0.d0

  allocate(plm(1:1,0:nlmax,0:nlmax),stat = status)
  if (status /= 0) stop 
  plm(:,:,:)=0.d0

  allocate(cmbcl(0:nlmax,1:3*pol+1),stat = status)
  if (status /= 0) stop 
  cmbcl(:,:)=0.d0

  allocate(potcl(0:nlmax,1:2),stat = status)
  if (status /= 0) stop 
  potcl(:,:)=0.d0

  call cpu_time(time_start)

  if(input.eq.'ps') then
    call readcambscalcl(pol,nlmax,cambscalclfile,cmbcl,potcl)
    call cambscalcl2asciialm(pol,nlmax,cmbcl,potcl,n,alm,plm)
    if(cmbalmfile.ne.'') then
      call asciialm2fitsalm(pol,nlmax,alm,cmbalmfile)
    endif
    if(potalmfile.ne.'') then
      call asciialm2fitsalm(0,nlmax,plm,potalmfile)
    endif
  elseif(input.eq.'map') then
    call fitsalm2asciialm(pol,nlmax,cmbalmfile,alm)
    call fitsalm2asciialm(0,nlmax,potalmfile,plm)
  endif

  call cpu_time(time_finish)

  if (input.eq.'ps') print *, "time to compute alms from cl =",time_finish-time_start
  if (input.eq.'map') print *, "time to read alms =",time_finish-time_start

  deallocate(cmbcl)
  deallocate(potcl)

  !---------------------------------------------------------------------------
  ! allocates space for CMB map, lensing potential and deflection field
  !---------------------------------------------------------------------------
  allocate(ultqutp(0:12*nside**2-1,1:2*pol+1),stat=status)
  if (status /= 0) stop
  allocate(gradtp(0:12*nside**2-1,1:3),stat=status)
  if (status /= 0) stop
  allocate(ltqutp(0:12*nside**2-1,1:2*pol+1),stat=status)
  if (status /= 0) stop

  ultqutp(:,:)=0.d0
  gradtp(:,:)=0.d0
  ltqutp(:,:)=0.d0

  call cpu_time(time_s) 

  print *, "*********************************************************************"
  call simul_all_map(pol,nlmax,alm,plm,nside,sigma,kappa,ultqutp,gradtp,ltqutp, method)
  print *, "*********************************************************************"

  call cpu_time(time_f) 

  print *, "time to simulate maps from harmonic modes using nfft =",time_f-time_s
  print *, "*********************************************************************"
  !-----------------------------------------------------------------------
  ! frees space for unlensed CMB and lensing potential in harmonic space
  !-----------------------------------------------------------------------
  deallocate(alm)
  deallocate(plm)
  !-----------------------------------------------------------------------
  !  write map to FITS file and cl to scaii file
  !-----------------------------------------------------------------------
  call cpu_time(time_start)

  if(ulcmbmapfile.ne.'') then
    call asciimap2fitsmap(pol,nside,ultqutp,ulcmbmapfile)
  endif

  if(ulcmbclfile.ne.'') then
    call asciimap2fitscl(pol,nside,ultqutp,nlmax,ulcmbclfile)
  endif

  call cpu_time(time_finish)

  print *, "time to store unlensed map and cl into a fits file =",time_finish-time_start

  !---------------------------------------------------------------------------
  ! frees space for unlensed CMB map
  !---------------------------------------------------------------------------
  deallocate(ultqutp)

  call cpu_time(time_start)

  if(gradpotmapfile.ne.'') then
    call asciimap2fitsmap(1,nside,gradtp,gradpotmapfile)
  endif

  call cpu_time(time_finish)

  print *, "time to store gradient map into a fits file =",time_finish-time_start

  !---------------------------------------------------------------------------
  ! frees space for lensing potential and deflection field
  !---------------------------------------------------------------------------
  deallocate(gradtp)


  call cpu_time(time_start)

  if(lcmbmapfile.ne.'') then
    call asciimap2fitsmap(pol,nside,ltqutp,lcmbmapfile)
  endif

  if(lcmbclfile.ne.'') then
    call asciimap2fitscl(pol,nside,ltqutp,nlmax,lcmbclfile)
  endif

  call cpu_time(time_finish)

  print *, "time to store lensed map and cl into a fits file =",time_finish-time_start

  !---------------------------------------------------------------------------
  ! frees space for lensed CMB map
  !---------------------------------------------------------------------------
  deallocate(ltqutp)

  PRINT *,"***************************************************************************************************"
  !-----------------------------------------------------------------------
  !                       end of routine
  !-----------------------------------------------------------------------
  write(*,'(a)') " "//code//"> normal completion"
  PRINT *,"***************************************************************************************************"

end program getlensedmap














!!$  call cpu_time(time_start)
!!$
!!$  call fitsmap2fitscl(pol,nside,ulcmbmapfile,nlmax,ulcmbclfile,0)
!!$
!!$  call cpu_time(time_finish)
!!$  print *, "time to compute unlensed cl from unlensed map =",time_finish-time_start
!!$
!!$  call cpu_time(time_start)
!!$
!!$  call fitsmap2fitscl(pol,nside,lcmbmapfile,nlmax,lcmbclfile,0)
!!$
!!$  call cpu_time(time_finish)
!!$  print *, "time to compute lensed cl from lensed map =",time_finish-time_start









  !******************************************************************************!
  !write(*,*) "If input is cl then input='ps'"
  !write(*,*) "If input is alm then  input='map'"
  !read(*,'(a)') input
  !if(input.eq.'ps') then
  !  write(*,*) "Input filename for cmb power spectrum (in fits)"
  !  read(*,'(a)') cmbclfile
  !  write(*,*) "Input filename for lensing potential power spectrum (in fits)"
  !  read(*,'(a)') potclfile
  !  write(*,*) "Output filename for cmb map (in harmonic space) (in fits)"
  !  read(*,'(a)') cmbalmfile
  !  write(*,*) "Output filename for potential map (in harmonic space) (in fits)"
  !  read(*,'(a)') potalmfile
  !elseif(input.eq.'map') then
  !  write(*,*) "Input filename for cmb map (in harmonic space) (in fits)"
  !  read(*,'(a)') cmbalmfile
  !  write(*,*) "Input filename for potential map (in harmonic space) (in fits)"
  !  read(*,'(a)') potalmfile
  !endif
  !write(*,*) "Healpix pixelization parameter(nside) of output map"
  !read(*,*) nside
  !write(*,*) "Bandlimit of output map"
  !read(*,*) nlmax
  !write(*,*) "For temperature only pol=0"
  !write(*,*) "For temperature and polarization for pol=1"
  !read(*,*) pol
  !if(input.eq.'ps') then
  !  write(*,*) "Index of realization (n)"
  !  read(*,*) n
  !endif
  !write(*,*) "Oversampling factor"
  !read(*,*) sigma
  !write(*,*) "Convolution length"
  !read(*,*) kappa
  !write(*,*) "Output filename for unlensed cmb  map (in fits)"
  !read(*,'(a)') ulcmbmapfile
  !write(*,*) "Output filename for unlensed cmb  map (in fits)"
  !read(*,'(a)') gradpotmapfile
  !write(*,*) "Output filename for lensed cmb  map (in fits)"
  !read(*,'(a)') lcmbmapfile
  !write(*,*) "Output filename for unlensed cmb power spectrum (in ascii)"
  !read(*,'(a)') ulcmbclfile
  !write(*,*) "Output filename for lensed cmb power spectrum (in ascii)"
  !read(*,'(a)') lcmbclfile
  !******************************************************************************!
